/* $Id: dma.h,v 1.3 2003/05/13 17:36:57 cochran Exp $ */
#ifndef _HAVE_DMA_H
#define _HAVE_DMA_H

/* private audio format extensions */
#define AFMT_STEREO 0x80000000
#define AFMT_ISSTEREO(x) ((x) & AFMT_STEREO)

#define AFMT_IS16BIT(x) ((x) & (AFMT_S16_LE|AFMT_S16_BE|AFMT_U16_LE|AFMT_U16_BE))
#define AFMT_ISUNSIGNED(x) ((x) & (AFMT_U8|AFMT_U16_LE|AFMT_U16_BE))

#define AFMT_BYTESSHIFT(x) ((AFMT_ISSTEREO(x) ? 1 : 0) + (AFMT_IS16BIT(x) ? 1 : 0))

#define AFMT_BYTES(x)      (1<<AFMT_BYTESSHFIT(x))

#define DMABUFSHIFT 17	/* 128k worth of DMA buffer */
#define NRSGBUF     (1U<<(DMABUFSHIFT-PAGE_SHIFT))

struct dmabuf {
  unsigned int format; /* buffer data format */
  unsigned int srate;
  unsigned char *sgbuf[NRSGBUF]; /* physical buffer */
  unsigned bufsize;
  unsigned numfrag;
  unsigned fragshift;
  unsigned wrptr, rdptr;
  unsigned total_bytes;
  int count;
  unsigned error; /* over/underrun */
  wait_queue_head_t wait;
  unsigned fragsize; /* redundant, but makes calculations easier */
  unsigned dmasize;
  unsigned mapped:1; /* OSS stuff */
  unsigned ready:1;
  unsigned ossfragshift;
  int ossmaxfrags;
  unsigned subdivision;
};

void dmabuf_copyin(struct dmabuf *db, const void *buffer, unsigned int size);

int dmabuf_copyin_user(struct dmabuf *db, unsigned int ptr, const void *buffer,
		       unsigned int size);

void dmabuf_copyout(struct dmabuf *db, void *buffer, unsigned int size);

int dmabuf_copyout_user(struct dmabuf *db, unsigned int ptr, void *buffer,
			unsigned int size);

int dmabuf_init(struct dmabuf *db);

int dmabuf_mmap(struct dmabuf *db, struct vm_area_struct *vma); 

void dmabuf_release(struct dmabuf *db);

#endif
