/* $Id: exaudio.h,v 1.10 2003/06/23 18:19:05 cochran Exp $
 *
 *  Copyright (c) 2003 Richard Cochran (cochran@cs.umass.edu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 */
#ifndef _HAVE_EXAUDIO_H
#define _HAVE_EXAUDIO_H

#define ERR(format,arg...)  printk(KERN_ERR "exaudio: " format "\n" , ## arg)
#define INFO(format,arg...) printk(KERN_INFO "exaudio: " format "\n" , ## arg)

#define SET_CUR    0x01   /* Audio Class specific Request Codes */
#define GET_CUR    0x81
#define SET_MIN    0x02
#define GET_MIN    0x82
#define SET_MAX    0x03
#define GET_MAX    0x83
#define SET_RES    0x04
#define GET_RES    0x84
#define SET_MEM    0x05
#define GET_MEM    0x85
#define GET_STAT   0xff

#define COPY_PROTECT_CONTROL       0x01   /* Terminal Control Selectors */
#define MUTE_CONTROL               0x01   /* Feature Unit Control Selectors */
#define VOLUME_CONTROL             0x02
#define BASS_CONTROL               0x03
#define MID_CONTROL                0x04
#define TREBLE_CONTROL             0x05
#define GRAPHIC_EQUALIZER_CONTROL  0x06
#define AUTOMATIC_GAIN_CONTROL     0x07
#define DELAY_CONTROL              0x08
#define BASS_BOOST_CONTROL         0x09
#define LOUDNESS_CONTROL           0x0a
#define SAMPLING_FREQ_CONTROL      0x01   /* Endpoint Control Selectors */
#define PITCH_CONTROL              0x02

#define N_MINORS 3

/*  A "struct extigy" is created for each extigy connected to the bus.
 *  Of course you are going to use more than one at the same time...
 */
struct extigy {
  struct list_head   devices; /* Points to next (!) extigy. */
  struct usb_device* usbdev;
  struct list_head   audiolist;
  struct list_head   mixerlist;
  unsigned           count; /* Usage count (including USB stack) */
  mode_t             modes_in_use; /* Tracks which IO directions are in use. */
  int                ossdev[N_MINORS]; /* Remembers which minors registered. */
};

/*  release()
 *
 *  Should be called with open_sem held, so that no new processes
 *  look at the audio device to be destroyed.
 *
 *  This decrements exy->count, and calls up()
 *
 *  I (RPC) don't like this function. It is from the original audio.c and
 *  it will have to stay, for now.
 */
void release(struct extigy *s);

#endif
