#ifndef _MGACLK_H
#define _MGACLK_H

#define CLK_PIXEL	0
#define CLK_MEMORY	1
#define CLK_WARP	2
#define CLK_GFXENG	3
#define CLK_CODEC	4


#pragma pack(1)

typedef struct {
	/* Length of this structure in bytes */
	u16 StructLen;

	/*
	 * Unique number identifying the product type
	 * 0 : MGA-S1P20 (2MB base with 175MHz Ramdac)
	 * 1 : MGA-S1P21 (2MB base with 220MHz Ramdac)
	 * 2 : Reserved
	 * 3 : Reserved
	 * 4 : MGA-S1P40 (4MB base with 175MHz Ramdac)
	 * 5 : MGA-S1P41 (4MB base with 220MHz Ramdac)
	 */
	u16 ProductID;

	/* Serial number of the board */
	u8 SerNo[ 10 ];

	/*
	 * Manufacturing date of the board (at product test)
	 * Format: yyyy yyym mmmd dddd
	 */
	u16 ManufDate;

	/* Identification of manufacturing site */
	u16 ManufId;

	/*
	 * Number and revision level of the PCB
	 * Format: nnnn nnnn nnnr rrrr
	 *         n = PCB number ex:576 (from 0->2047)
	 *         r = PCB revision      (from 0->31)
	 */
	u16 PCBInfo;

	/* Identification of any PMBs */
	u16 PMBInfo;

	/*
	 * Bit  0-7  : Ramdac speed (0=175MHz, 1=220MHz)
	 * Bit  8-15 : Ramdac type  (0=TVP3026, 1=TVP3027)
	 */
	u16 RamdacType;

	/* Maximum PCLK of the ramdac */
	u16 PclkMax;

	/* Maximum LDCLK supported by the WRAM memory */
	u16 LclkMax;

	/* Maximum MCLK of base board */
	u16 ClkBase;

	/* Maximum MCLK of 4Mb board */
	u16 Clk4MB;

	/* Maximum MCLK of 8Mb board */
	u16 Clk8MB;

	/* Maximum MCLK of board with multimedia module */
	u16 ClkMod;

	/* Diagnostic test pass frequency */
	u16 TestClk;

	/* Default VGA mode1 pixel frequency */
	u16 VGAFreq1;

	/* Default VGA mode2 pixel frequency */
	u16 VGAFreq2;

	/* Date of last BIOS programming/update */
	u16 ProgramDate;

	/* Number of times BIOS has been programmed */
	u16 ProgramCnt;

	/* Support for up to 32 hardware/software options */
	u32 Options;

	/* Support for up to 32 hardware/software features */
	u32 FeatFlag;

	/* Definition of VGA mode MCLK */
	u16 VGAClk;

	/* Indicate the revision level of this header struct */
	u16 StructRev;

	u16 Reserved[ 3 ];
} MGABiosInfo;


/* from the PINS structure, refer pins info from MGA */
typedef struct tagParamMGA {
	u16 	PinID;		/* 0 */
	u8	StructLen;	/* 2 */
	u8	Rsvd1;		/* 3 */
	u16	StructRev;	/* 4 */
	u16	ProgramDate;	/* 6 */
	u16	ProgramCnt;	/* 8 */
	u16	ProductID;	/* 10 */
	u8	SerNo[16];	/* 12 */
	u8	PLInfo[6];	/* 28 */
	u16	PCBInfo;	/* 34 */
	u32	FeatFlag;	/* 36 */
	u8	RamdacType;	/* 40 */
	u8	RamdacSpeed;	/* 41 */
	u8	PclkMax;	/* 42 */
	u8	ClkGE;		/* 43 */
	u8   ClkMem;		/* 44 */
	u8	Clk4MB;		/* 45 */
	u8	Clk8MB;		/* 46 */
	u8	ClkMod;		/* 47 */
	u8	TestClk;	/* 48 */
	u8	VGAFreq1;	/* 49 */
	u8	VGAFreq2;	/* 50 */
	u8	MCTLWTST;	/* 51 */
	u8	VidCtrl;	/* 52 */
	u8	Clk12MB;	/* 53 */
	u8	Clk16MB;	/* 54 */
	u8	Reserved[8];	/* 55-62 */
	u8	PinCheck;	/* 63 */
}	MGABios2Info;

#pragma pack()


int	mga_get_clk( struct mga_dev_s *mga, int clk_id, int *ret_clk );

int	mga_mod_clkdiv_g400( struct mga_dev_s *mga, int clk_id, int div_delta );

int	mga_get_bios2(struct mga_dev_s *mga, MGABios2Info *biosinfo);

void	mga_check_mctlwtst(struct mga_dev_s *mga);


#endif


