/*
  GPL Driver for the Codec Interface on MGA G1/2/400 cards
  (requires mgacore.c)

  (c) 1999-2000 David Barth <dbarth@besancon.net>
*/

#include <linux/poll.h>

#define CODEC_BUFFER_SIZE	0x40000	/* 256K */
/* #define CODEC_BUFFER_SIZE	0x20000 */	/* 128K */

#define  MGACODEC_MODE_I33	2
#define  MGACODEC_MODE_VMI_A	0
#define  MGACODEC_MODE_VMI_B	16

#define	MGACODEC_START	0
#define MGACODEC_STOP	1

#define CODEC_NO_TRANSFER	0
#define CODEC_SEND_TRANSFER	1
#define CODEC_RECV_TRANSFER	2


/* moved out from mgacodec.c;
	since mgacodec_and_or_codecctl is exposed, might as well give useful
	constants for the argument --lvs */
/*
 * CODECCTL fields and commands
 */
#define codecen 	0x0001	/* CODEC enable, invert to disable */
#define codecmode 	0x0002	/* CODEC I33 mode, invert for VMI mode */
#define cmdexectrig 	0x0004	/* command execution trigger, invert to prevent execution */
#define codecdatain 	0x0008	/* compression, invert for decompression */
#define vmimode 	0x0010	/* VMI mode B, invert for mode A */
#define stopcodec 	0x0020	/* stop after current field, invert to not stop */
#define codectransen 	0x0040	/* enable transfer, invert to disable */

void mgacodec_attach(struct mga_dev_s *mga );
void mgacodec_deattach		(struct mga_codec_s *codec);
void mgacodec_init(struct mga_codec_s *codec, int mode, int reg, int xor_regs );
void mgacodec_reset		(struct mga_codec_s *codec);
u32 mgacodec_bitstream		(struct mga_codec_s *codec);
void mgacodec_and_or_codecctl	(struct mga_codec_s *codec, u8 and, u8 or);
void mgacodec_and_or_miscctl	(struct mga_codec_s *codec, u8 and, u8 or);
u8   mgacodec_get_miscctl	(struct mga_codec_s *codec);
int  mgacodec_set_misc2		(struct mga_codec_s *codec, u8 param);
int  mgacodec_set_ddc2		(struct mga_codec_s *codec, u8 param);

void mgacodec_begin_commands	(struct mga_codec_s *codec);
void mgacodec_push_readb	(struct mga_codec_s *codec, u8 reg);
void mgacodec_push_readw	(struct mga_codec_s *codec, u8 reg);
void mgacodec_push_writeb	(struct mga_codec_s *codec, u8 reg, u8 data);
void mgacodec_push_writew	(struct mga_codec_s *codec, u8 reg, u16 data);
void mgacodec_push_pause        (struct mga_codec_s *codec);
int  mgacodec_exec_commands	(struct mga_codec_s *codec, u8 and, u8 or);
u8   mgacodec_get_byte		(struct mga_codec_s *codec);
u16  mgacodec_get_word		(struct mga_codec_s *codec);
void mgacodec_end_commands	(struct mga_codec_s *codec);

int  mgacodec_stop_transfer	(struct mga_codec_s *codec);
int  mgacodec_get_transfer	(struct mga_codec_s *codec);
int  mgacodec_start_transfer	(struct mga_codec_s *codec, int direction);
int  mgacodec_is_stalled	(struct mga_codec_s *codec);

int  mgacodec_send_bitstream	(struct mga_codec_s *codec, const u8 *bitstream, unsigned int size);
int  mgacodec_send_bitstream_irq	(struct mga_codec_s *codec, const u8 *bitstream, unsigned int size);
int  mgacodec_recv_bitstream	(struct mga_codec_s *codec, u8 *addr, int start, int end, int limit);

u32  mgacodec_buffer_free	(struct mga_codec_s *codec);
void mgacodec_callback		(struct mga_codec_s *codec, void (*p)(struct mga_dev_s *mga, int state));
void mgacodec_set_transfer_limit(struct mga_codec_s *codec, u32 hostptr);
u32 mgacodec_hardptr		(struct mga_codec_s *codec);
u32 mgacodec_lcode		(struct mga_codec_s *codec);
int mgacodec_stall_transfer	(struct mga_codec_s *codec);
unsigned int mgacodec_poll(struct mga_codec_s *codec, struct file *file, poll_table *wait,
			   u32 read_threshold, u32 write_threshold);
int mgacodec_vsync_sleep	(struct mga_codec_s *codec);
