#ifndef _MGACORE_H
#define _MGACORE_H

#include <linux/version.h>
#include <asm/io.h>
#include <linux/pci.h>

#include "aiee.h"

#if (LINUX_VERSION_CODE >= 0x020300)

#include <linux/spinlock.h>

#else /* (LINUX_VERSION_CODE >= 0x020300) */

#include <asm/spinlock.h>
#define wait_queue_head_t struct wait_queue *
/*#define init_waitqueue_head(wq)    *(wq) = NULL; */

#if (LINUX_VERSION_CODE < 0x020100)
#define MODULE_AUTHOR(_author)
#define MODULE_DESCRIPTION(_description)
#endif /* (LINUX_VERSION_CODE < 0x020100) */

#endif /* (LINUX_VERSION_CODE >= 0x020300) */

#include "i2cdef.h"


#define	mgacore_printk	if (mgacore_debug) printk
extern	int	mgacore_debug;

#define MGA_ERR       KERN_INFO
#define MGA_DEBUG     KERN_INFO
#define MGA_INFO      KERN_INFO
#define MGA_WARNING   KERN_INFO
#define MGA_NOTICE    KERN_INFO


struct mga_dev_s;
typedef void (*mga_interrupt_handler_t)(struct mga_dev_s *, struct pt_regs *);

struct buf_data {
	u32 image_width;  /* Images width(before decimation) */
	u32 real_width;   /* Number of pixels actually written (decimated) */
	u32 image_height; /* Number of valid lines per field */
	int fields;       /* The source image has alternating fields */
	                  /* Full res is obtained by combining two. */
	u32 subimage_x; /* grabbed portion of the image */
	u32 subimage_y; /* (coordinates before scaling */
	u32 subimage_w; /* (y range : 0-image_height)) */
	u32 subimage_h;
	u32 grab_width;  /* Size and format of the grabbed image */
	u32 grab_height;
	u32 grab_crop_x;	/* X offset decimated for grab */
	int grab_format;
        /* Source video comes through ks0127 -> hardware horizontal scaling */
 	int decoder_hscale;
 	int scale_change_stamp; /* Image number when we changed the scaling */
#define	FIELD_TOP		0x0001
#define	FIELD_BOTTOM		0x0002
	int fields_wanted;
	u32 frame_next;
	
};

#define MGA_MAX_VIRQ	4	/* nr of video irqs */
#define MGA_MAX_IRQ	12	/* nr of core irqs */
#define MGA_MAX_ISR	4	/* how many handlers per irq */


struct mga_dev_s
{
	/*************************************************************
	 * Core parameters                                           *
	 *                                                           *
	 *************************************************************/
	struct pci_dev *dev;
	int		card_num;
	u8* fb;                 /* framebuffer address */
        u8* ctrl;	        /* control aperture address */
	u32 memory_size;	/* framebuffer memory size */
	u32 options;            /* options detected (like HAS_TUNER, ...) */
	int in_use;		/* indicates that the core driver
				   is already used by a V4L module
				   (mgadvd, mga_vid, mgavideo, etc...)
				*/

	spinlock_t malloc_lock;	/* locks { */
	u32 malloc_limit;	/* used by "memory management" routines */
				/* } locks */

       	struct proc_dir_entry *proc_directory;
	char			name[32];

	
	spinlock_t reg_lock;	/* locks { seq. ordered mga register access } */


	int mga_type;		/* Type of mga card G100, G200, G400 */


        struct buf_data v4lbuf;

	/*************************************************************
	 * Interrupt handlers specific parameters                    *
	 *                                                           *
	 *************************************************************/
	/* dynamic functions handling the different interrupt requests :
	 * the interrupt index is just associated with the array index, so
	 * not all slots will be used
	 */
	spinlock_t irq_lock;	/* locks { VICLEAR, VIEN, + */
	mga_interrupt_handler_t irq_handler[MGA_MAX_VIRQ][MGA_MAX_ISR];
	int irq_inuse;
	int enabled_irq;
	
	mga_interrupt_handler_t core_irq_handler[MGA_MAX_IRQ][MGA_MAX_ISR];
	int enabled_core_irq;
				/* } locks */
	int saved_core_irq;
				
	u8 bios_mctlwtst;

	int status;		/* interrupt status register */

	/*************************************************************
	 * Codec specific fields                                     *
	 *                                                           *
	 *************************************************************/
	struct mga_codec_s *codec;

	/*************************************************************
	 * I2C specific fields                                       *
	 *                                                           *
	 *************************************************************/
	struct i2c_adapter *i2c;
	struct i2c_client *tuner_client;
	struct i2c_client *msp_client;
	struct i2c_client *ks_client;
	struct i2c_client *maven_client;

	/*************************************************************
	 * V4L-1 params for uncompressed capture functions           *
	 *                                                           *
	 *************************************************************/
	struct mgacap_v4l1	*cap_v4l1;

	/*************************************************************
	 * BES specific parameters                                   *
	 *                                                           *
	 *************************************************************/
	struct mga_bes_s *bes;

	/*************************************************************
	 * GRABBER specific parameters                               *
	 *                                                           *
	 *************************************************************/
	struct mga_grab_s *grab;

	/*************************************************************
	 * MJPEG specific parameters                                 *
	 *                                                           *
	 *************************************************************/
	struct zr36060_dev *zr;

	/*************************************************************
	 * i33 specific parameters                                   *
	 *                                                           *
	 *************************************************************/
	struct i33_dev_s *i33;

	/*************************************************************
	 * video specific parameters                                 *
	 *                                                           *
	 *************************************************************/
	struct mga_video_s *video;

	/*************************************************************
	 * i34 specific parameters                                   *
	 *                                                           *
	 *************************************************************/
	struct i34_dev_s *i34;

	/*************************************************************
	 * administration of the busses on the board                 *
	 *                                                           *
	 *************************************************************/
	struct mgabus_admin_s *bus_admin;

	/*************************************************************
	 * the bus for transfers of coded image data                 *
	 *                                                           *
	 *************************************************************/
	struct mgabus_s *codec_bus;
	
	/*************************************************************
	 * main video bus                                            *
	 *                                                           *
	 *************************************************************/
	struct mgabus_s *vip_bus;

	/*************************************************************
	 * video out bus                                             *
	 *                                                           *
	 *************************************************************/
	struct mgabus_s *vdo_bus;
	
	/*************************************************************
	 * V4L-1 params for MJPEG functions                          *
	 *                                                           *
	 *************************************************************/
	struct mgajpg_v4l1* jpg_v4l1;
	
	/*************************************************************
	 * V4L-1 params for DVD functions                            *
	 *                                                           *
	 *************************************************************/
	struct mgadvd_v4l1* dvd_v4l1;
	
	struct mgayuv_s *yuv_v4l1;
	
	struct mgacrtc2_v4l1* crtc2_v4l1;
	
	struct maven_s * maven;	//anonymous struct
	struct ks0127 *ks0127;	//anonymous struct

	/* mutual exclusion for mjpeg/dvd */	
	int card_lock;

	/* mga video out port connection to the vdo bus */
	struct mgabus_client_vid_s *vdo_port;
	
	/* maven(TVO) video in port connection to the vdo bus */
	struct mgabus_client_vid_s *tvo_port;
	
	/* current/cached maven standard */
	int maven_std;
	
	/* current/cached msp audio channel */
	int msp_input;
};

#define MGA_G100	1
#define MGA_G200	2
#define MGA_G400	3

#define MGAVIDEO_HAS_TUNER   0x00000001
#define MGAVIDEO_HAS_AUDIO   0x00000002
#define MGAVIDEO_HAS_DECODER 0x00000004
#define MGAVIDEO_HAS_MJPEG   0x00000008
#define MGAVIDEO_HAS_TVOUT   0x00000010
#define MGAVIDEO_HAS_RRG     0x00000020
#define	MGAVIDEO_HAS_MARVEL  0x00000040
#define	MGAVIDEO_HAS_ENCODER 0x00000080
#define	MGAVIDEO_HAS_ZR36060 0x00000100

#define MGA_IRQ_VINVSYNC	0
#define MGA_IRQ_CMDCMPL		1
#define MGA_IRQ_BUFLVL		2
#define MGA_IRQ_DCMPEOI		3
#define MGA_IRQ_ALLMASK		0x0f

#define MGA_CORE_IRQ_VLINE	5
#define MGA_CORE_IRQ_EXTERNAL	6

#define PALWTADD        0x3c00
#define X_DATAREG       0x3c0a
#define VINCTL0         0x3e00
#define VINCTL1         0x3e04
#define VBIADDR0        0x3e08
#define VBIADDR1        0x3e0c
#define VINADDR0        0x3e10
#define VINADDR1        0x3e14
#define VINNEXTWIN      0x3e18
#define VBICOUNT	0x3e18	/* G400 */
#define VINCTL          0x3e1c
#define VINADDR2	0x3e20	/* G400 */
#define VINADDR3	0x3e24	/* G400 */
#define VINCTL2		0x3e28	/* G400 */
#define VINCTL3		0x3e2c	/* G400 */
#define VSTATUS		0x3e30
#define VICLEAR         0x3e34
#define VIEN            0x3e38
#define VINHEIGHT	0x3e3c	/* G400 */
#define STATUS		0x1e14
#define ICLEAR          0x1e18
#define IEN             0x1e1c
#define CRTMISC		0x1fcc
#define CRTCADD		0x1fd4
#define CRTCREG 	0x1fd5
#define CRTCEXTADD	0x1fde
#define CRTCEXTREG	0x1fdf
#define CACHEFLUSH	0x1fff


/* indexed registers */
#define	XMULCTRL	0x19
#define	XPIXCLKCTRL	0x1a
#define XMISCCTRL       0x1e
#define XGENIOCTRL      0x2a
#define XGENIODATA      0x2b
#define XKEYOPMODE      0x51
#define XCOLMSK0RED     0x52
#define XCOLMSK0GREEN   0x53
#define XCOLMSK0BLUE    0x54
#define XCOLKEY0RED     0x55
#define XCOLKEY0GREEN   0x56
#define XCOLKEY0BLUE    0x57
#define SDA             0x10
#define SCL             0x20

#ifndef	I2C_BUSID_MGA
#define I2C_BUSID_MGA 	6
#endif

/* Meaning of the VSFIELD (top field is when VS_FIELD is 0) */
#define	VS_TOPFIELD	0
#define VS_BOTFIELD	VS_FIELD

/* mga video status */
#define VS_IVSYNC    0x0001 /* input vsync interrupt */
#define VS_CMDCMPL   0x0002 /* command complete interrupt */
#define VS_BLVL      0x0004 /* buffer level interrupt */
#define VS_EOI       0x0008 /* codec end of image interrupt */
#define VS_FIELD     0x0100 /* field 0 or 1 detected */
#define VS_VIDEO     0x0200 /* indicates video was captured */
#define VS_RAWVBI    0x0400 /* indicates raw vbi was captured */
#define VS_SLICEDVBI 0x0800 /* indicates sliced vbi was captured */
#define VS_STALLED   0x1000 /* codec stalled after transfer stopped */
#define VS_TASKDETD  0x2000 /* video input task detected, G400 only */
#define VS_GRABPTR   0xb000 /* pointer to the last field grabbed, G400 only */

#define VI_IVSYNC    0x0001 /* input vsync interrupt */
#define VI_CMDCMPL   0x0002 /* command complete interrupt */
#define VI_BLVL      0x0004 /* buffer level interrupt */
#define VI_EOI       0x0008 /* codec end of image interrupt */
#define VI_STALLED   0x1000 /* codec stalled after transfer stopped */

/**
 *	mga_read -	read a register (32 bits)
 *	@mga:	mga device instance
 *	@reg:	register to read
 *	
 *	Read a 32 bits register of the MGA card.
 *	Returns the value of the register read.
 */
static __inline__ u32 mga_read(struct mga_dev_s* mga, u32 reg)
{
        return readl(mga->ctrl + reg );
}


static __inline__ u8 mga_readb(struct mga_dev_s* mga, u32 reg)
{
        return readb(mga->ctrl + reg );
}

/**
 *	mga_writeb -	write a register (8 bits)
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	data to write
 *	
 *	Write one (1) byte to a register in the MGA card.
 */
static __inline__ void mga_writeb(struct mga_dev_s* mga, u32 reg, u32 data)
{
        writeb( data, mga->ctrl + reg );
}

/**
 *	mga_writew -	write a register (16 bits)
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	data to write
 *	
 *	Write a word (16 bits).
 */
static __inline__ void mga_writew(struct mga_dev_s* mga, u32 reg, u32 data)
{
        writew( data, mga->ctrl + reg );
}

/**
 *	mga_write -	write a register (32 bits)
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	data to write
 *	
 *	Write a 32 bits value.
 */
static __inline__ void mga_write(struct mga_dev_s* mga, u32 reg, u32 data)
{
        writel( data, mga->ctrl + reg );
}

/**
 *	mga_indirect_readb -	read an indirect register (byte)
 *	@mga:	mga device instance
 *	@reg:	register to read
 *	
 *	Read a byte from an indirect register in the MGA card.
 *	Returns the byte read.
 */
static __inline__ u8 mga_indirect_readb(struct mga_dev_s *mga, u8 reg)
{
	u8	ret;

	writeb(reg, mga->ctrl + PALWTADD);
	ret =  readb(mga->ctrl + X_DATAREG);

	return ret;
}

static __inline__ u8 mga_indirect_readb_mt(struct mga_dev_s *mga, u8 reg)
{
	unsigned long flags;
	u8 rv;

	spin_lock_irqsave( &mga->reg_lock, flags );

	rv = mga_indirect_readb( mga, reg );

	spin_unlock_irqrestore( &mga->reg_lock, flags );
	return rv;
}


/**
 *	mga_indirect_writeb -	write a byte into an indirect register
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	byte to write
 *	
 *	Write a byte into an indirect register in the MGA card.
 */
static __inline__ void mga_indirect_writeb(struct mga_dev_s *mga, u8 reg, u8 data)
{
	writeb(reg, mga->ctrl + PALWTADD);
	writeb(data, mga->ctrl + X_DATAREG);
}


static __inline__ void mga_indirect_writeb_mt(struct mga_dev_s *mga, u8 reg, u8 data)
{
	unsigned long flags;

	spin_lock_irqsave( &mga->reg_lock, flags );

	mga_indirect_writeb( mga, reg, data );

	spin_unlock_irqrestore( &mga->reg_lock, flags );
}


/**
 *	mga_indirect_and_or -	apply a and/or operation on a register
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@and:	AND operand
 *	@or:	OR operand	
 *	
 *	This function reads a byte from an indexed register, applies both
 *	the OR and AND operation on the read data, and writes the result
 *	back into the register.
 */
static __inline__ void mga_indirect_and_or(struct mga_dev_s *mga, u8 reg, u8 and, u8 or)
{
	u8 data;

	data = mga_indirect_readb(mga, reg);
	data &= and;
	data |= or;
	mga_indirect_writeb(mga, reg, data);
}


static __inline__ void mga_indirect_and_or_mt(struct mga_dev_s *mga, u8 reg, u8 and, u8 or)
{
	unsigned long flags;

	spin_lock_irqsave( &mga->reg_lock, flags );

	mga_indirect_and_or( mga, reg, and, or );

	spin_unlock_irqrestore( &mga->reg_lock, flags );
}


static __inline__ void mga_flush(struct mga_dev_s* mga)
{
	mga_writeb( mga, CACHEFLUSH, 0xee );
}


static inline unsigned long uvirt_to_kva(pgd_t *pgd, unsigned long adr)
{
	unsigned long ret = 0UL;
	pmd_t *pmd;
	pte_t *ptep;
	pte_t pte;

	if (!pgd_none(*pgd)) {
		pmd = pmd_offset(pgd, adr);
		if (!pmd_none(*pmd)) {
			ptep = pte_offset(pmd, adr);
			pte = *ptep;
			if (pte_present(pte))
#if (LINUX_VERSION_CODE >= 0x020300)
				ret = (unsigned long)page_address(pte_page(pte));
				ret |= (adr & (PAGE_SIZE -1));
#else
				ret = (pte_page(pte) | (adr & (PAGE_SIZE-1)));

#endif

		}
	}
	return ret;
}

static inline unsigned long kvirt_to_pa(unsigned long adr)
{
	unsigned long va;
	unsigned long kva;

	va = VMALLOC_VMADDR(adr);
	kva = uvirt_to_kva(pgd_offset_k(va), va);
	return __pa(kva);
}



/* see mgacore.c for a detailed documentation of the public API */
void   mga_display_size(struct mga_dev_s *mga, int *width, int *height, int *depth);
int	mga_display_pitch(struct mga_dev_s *mga);
struct mga_dev_s * mga_get_card(int num);
void   mga_release_card(struct mga_dev_s *mga);
u32    mga_alloc_buffer(struct mga_dev_s *mga, unsigned long size, unsigned long alignment );
void   mga_free_buffer (struct mga_dev_s *mga, unsigned long size);
int    mga_disable_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler);
int    mga_enable_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler);
int    mga_disable_core_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler);
int    mga_enable_core_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler);
void   mga_check_irqs( struct mga_dev_s *mga );
struct proc_dir_entry	*
mga_create_proc_entry(struct mga_dev_s *mga, char *name,
	read_proc_t *rp, write_proc_t *wp, void *data);

void	mga_remove_proc_entry(struct mga_dev_s *mga, char *name);
void	*mga_rvmalloc(signed long size, unsigned long page_table[]);
void	mga_rvfree(void *mem, signed long size, unsigned long page_table[]);
void mga_test_lines(struct mga_dev_s *mga, int details);
int	mga_card_lock(struct mga_dev_s *mga);
void	mga_card_unlock(struct mga_dev_s *mga);

int	mga_i2c_control( struct mga_dev_s *mga, int driver_id, unsigned int cmd, void *arg );

int	mga_set_defaults(struct mga_dev_s *mga);
void	mga_set_audioinput(struct mga_dev_s *mga, int channel);


#endif /* _MGACORE_H */
