/* -*- linux-c -*- --------------------------------------------------------- *  
 * This file is part of the ZORAN zr367x0 driver
 *
 * Copyright (C) 1999 David Barth <dbarth@besancon.net>
 * Copyright (C) 2000,2001 Merrick Johnson <inkling@nop.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/**
 * modified and extended by:
 *   Merrick Johnson <inkling@nop.org> MAINTAINER, send queries here
 */

#ifndef _MGADVD_H
#define _MGADVD_H

#include <linux/videodev.h>

/* this is our new official hardware ID */
#define VID_HARDWARE_ZR36700	28
#define MGADVD_ID_STRING	"Matrox DVD decoder card"
#define MGADVD_VERSION		"0.5"

#define I34_NTSC_ALT	02
#define I34_PAL_ALT	04
/* deprecated: use VIDEO_MODE_ */
/* #define I34_NTSC	01 */
/* #define I34_PAL		03 */

#define BITSTREAM_VCD		0x00
#define BITSTREAM_DVD_VOB	0x02
#define BITSTREAM_CDDA_PCM	0x04
#define BITSTREAM_VCD_AUX	0x05
#define BITSTREAM_DVD_NAV	0x06
#define BITSTREAM_MPEG12_PS	0x07
#define BITSTREAM_MPEG12_V_ES	0x0C
#define BITSTREAM_MPEG12_AV_PES	0x0D
#define BITSTREAM_MPEG12_A_ES	0x0F
#define BITSTREAM_AC3_A_ES	0x16
#define BITSTREAM_LPCM_A_ES_DVD	0x18

#define DVP_MICROCODE_NONE	0
#define DVP_MICROCODE_DVD	1 /* DVD video */
#define DVP_MICROCODE_VCD	2 /* VCD video */
#define DVP_MICROCODE_MPEG	3 /* MPEG-1,2 video */
#define DVP_MICROCODE_SCAN      4 /* unknown microcode */

#define ADP_MICROCODE_NONE	0
#define ADP_MICROCODE_AC3	1 /* Dolby Digital 2.0 audio, 2 channel */
#define ADP_MICROCODE_AC36	2 /* Dolby Digital 5.1 audio, 6 channel */
#define ADP_MICROCODE_LPCM	3 /* PCM audio, 8 channel */
#define ADP_MICROCODE_MPEG	4 /* MPEG audio, 2 channel */
#define ADP_MICROCODE_PCM	5 /* CDDA PCM audio, 2 and 4 channel */


struct mgadvd_status_s
{
	unsigned short isreg;

	unsigned short status0;
	unsigned short status1;
	unsigned short status2;

	unsigned int sclk;
	unsigned int sclkirq;

	unsigned short vidbufs;
	unsigned short audbufs;
	unsigned short spbufs;

	unsigned char adp_status_reply[16];

	unsigned char nav_seg;
	unsigned short nav_buf[1024];

	unsigned short dvp_valid;
	unsigned char dvp_buf[64];

};


#define I34_IOCPRIVATE (BASE_VIDIOCPRIVATE+8)

#define I34_IOCRESET		_IO   ('v', I34_IOCPRIVATE +  0)
#define I34_IOCRESYNC		_IOW  ('v', I34_IOCPRIVATE +  1, int)
#define I34_IOCGSTATUS		_IOR  ('v', I34_IOCPRIVATE +  2, struct mgadvd_status_s)
#define I34_IOCGVSYNCSTATUS	_IOR  ('v', I34_IOCPRIVATE +  3, struct mgadvd_status_s)

#define I34_IOCSVIDEOSID	_IOW  ('v', I34_IOCPRIVATE +  4, int)
#define I34_IOCSAUDIOSID	_IOW  ('v', I34_IOCPRIVATE +  5, int)
#define I34_IOCSSUBPICSID	_IOW  ('v', I34_IOCPRIVATE +  6, int)
#define I34_IOCSADPMICROCODE	_IOW  ('v', I34_IOCPRIVATE +  7, int)
#define I34_IOCSDVPMICROCODE	_IOW  ('v', I34_IOCPRIVATE +  8, int)
#define I34_IOCSSTREAMTYPE	_IOW  ('v', I34_IOCPRIVATE +  9, int)
#define I34_IOCSVIDEOSYNC	_IOW  ('v', I34_IOCPRIVATE +  10, int)
#define I34_IOCSAUDIOSYNC	_IOW  ('v', I34_IOCPRIVATE +  11, int)
#define I34_IOCSSPPALETTE	_IOW  ('v', I34_IOCPRIVATE +  12, unsigned int *)


#ifdef DVD_AUTH	/* defined in <linux/cdrom.h> if DVD/CSS support is present */
#define CSS_SUPPORT
#define I34_IOCS_DISK_KEY	_IOW  ('v', BASE_VIDIOCPRIVATE + 32, dvd_authinfo)
#define I34_IOCS_TITLE_KEY	_IOW  ('v', BASE_VIDIOCPRIVATE + 33, dvd_authinfo)
#define I34_IOCG_CHAL_KEY	_IOR  ('v', BASE_VIDIOCPRIVATE + 34, dvd_authinfo)
#define I34_IOCS_CHAL_KEY	_IOW  ('v', BASE_VIDIOCPRIVATE + 35, dvd_authinfo)
#define I34_IOCG_RESP_KEY	_IOR  ('v', BASE_VIDIOCPRIVATE + 36, dvd_authinfo)
#define I34_IOCS_RESP_KEY	_IOW  ('v', BASE_VIDIOCPRIVATE + 37, dvd_authinfo)
#endif


/* This part is a "cut'n paste" from the new videodev header
   (from 2.3.99-pre...) to benefit from the new definitions and structures.
   It's only needed if you're not running the latest kernel
*/

#ifndef VID_TYPE_MPEG_DECODER

#define VID_TYPE_MPEG_DECODER	1024	/* Can decode MPEG streams */
#define VID_TYPE_MPEG_ENCODER	2048	/* Can encode MPEG streams */
#define VID_TYPE_MJPEG_DECODER	4096	/* Can decode MJPEG streams */
#define VID_TYPE_MJPEG_ENCODER	8192	/* Can encode MJPEG streams */

/* video_info is biased towards hardware mpeg encode/decode */
/* but it could apply generically to any hardware compressor/decompressor */
struct video_info
{
	__u32	frame_count;	/* frames output since decode/encode began */
	__u32	h_size;		/* current unscaled horizontal size */
	__u32	v_size;		/* current unscaled veritcal size */
	__u32	smpte_timecode;	/* current SMPTE timecode (for current GOP) */
	__u32	picture_type;	/* current picture type */
	__u32	temporal_reference;	/* current temporal reference */
	__u8	user_data[256];	/* user data last found in compressed stream */
	/* user_data[0] contains user data flags, user_data[1] has count */
};

/* generic structure for setting playback modes */
struct video_play_mode
{
	int	mode;
	int	p1;
	int	p2;
};

/* for loading microcode / fpga programming */
struct video_code
{
	char	loadwhat[16];	/* name or tag of file being passed */
	int	datasize;
	__u8	*data;
};

#define VIDIOCSPLAYMODE		_IOW('v',24, struct video_play_mode)	/* Set output video mode/feature */
#define VIDIOCSWRITEMODE	_IOW('v',25, int)			/* Set write mode */
#define VIDIOCGPLAYINFO		_IOR('v',26, struct video_info)		/* Get current playback info from hardware */
#define VIDIOCSMICROCODE	_IOW('v',27, struct video_code)		/* Load microcode into hardware */
#define	VIDIOCGVBIFMT		_IOR('v',28, struct vbi_format)		/* Get VBI information */
#define	VIDIOCSVBIFMT		_IOW('v',29, struct vbi_format)		/* Set VBI information */

/* VIDIOCSWRITEMODE */
#define VID_WRITE_MPEG_AUD		0
#define VID_WRITE_MPEG_VID		1
#define VID_WRITE_OSD			2
#define VID_WRITE_TTX			3
#define VID_WRITE_CC			4
#define VID_WRITE_MJPEG			5

/* VIDIOCSPLAYMODE */
#define VID_PLAY_VID_OUT_MODE		0
	/* p1: = VIDEO_MODE_PAL, VIDEO_MODE_NTSC, etc ... */
#define VID_PLAY_GENLOCK		1
	/* p1: 0 = OFF, 1 = ON */
	/* p2: GENLOCK FINE DELAY value */ 
#define VID_PLAY_NORMAL			2
#define VID_PLAY_PAUSE			3
#define VID_PLAY_SINGLE_FRAME		4
#define VID_PLAY_FAST_FORWARD		5
#define VID_PLAY_SLOW_MOTION		6
#define VID_PLAY_IMMEDIATE_NORMAL	7
#define VID_PLAY_SWITCH_CHANNELS	8
#define VID_PLAY_FREEZE_FRAME		9
#define VID_PLAY_STILL_MODE		10
#define VID_PLAY_MASTER_MODE		11
	/* p1: see below */
#define		VID_PLAY_MASTER_NONE	1
#define		VID_PLAY_MASTER_VIDEO	2
#define		VID_PLAY_MASTER_AUDIO	3
#define VID_PLAY_ACTIVE_SCANLINES	12
	/* p1 = first active; p2 = last active */
#define VID_PLAY_RESET			13
#define VID_PLAY_END_MARK		14


#endif /* VID_TYPE_MPEG_DECODER */

#ifndef VIDEO_WINDOW_CHROMAKEY
#define VIDEO_WINDOW_CHROMAKEY	16
#endif /* VIDEO_WINDOW_CHROMAKEY */

#endif /* _MGADVD_H */
