/*
 *
 * G200/G400 I2C adapter interface
 *
 * Copyright (C) 1999-2001 marvel project @ sourceforge
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/proc_fs.h>
#include <linux/spinlock.h>

#include "mgacore.h"
#include "mgai2c.h"
#include "../../drivers/media/video/tuner.h"
#include "audiochip.h"
#include "ks0127.h"
#include "maven.h"

#include <linux/i2c.h>
#include "i2c-algo-ks.h"


MODULE_PARM(mgai2c_debug,"i");

static int mgai2c_debug=0;
#define dprintk if (mgai2c_debug) printk


/****************************************************************************
 * mga i2c bus interface
 ****************************************************************************/


static int
maven_proc_read(char *buf, char **start, off_t offset, int len, int *eof, void *data)
{
	struct mga_dev_s *mga = (struct mga_dev_s*)data;

	struct mga_procread param = {
		buf, start, offset, len, eof, mga->maven };
	
	return mga_i2c_control(mga, I2C_DRIVERID_VIDEOENCODER, 
			MAVEN_READ_PROC, &param);
}


static int
maven_proc_write(struct file *file, const char *buf, unsigned long count, void *data)
{
	struct mga_dev_s *mga = (struct mga_dev_s*)data;

	struct mga_procwrite param = {
		file, buf, count, mga->maven };

	return mga_i2c_control(mga, I2C_DRIVERID_VIDEOENCODER, 
			MAVEN_WRITE_PROC, &param);
}

static int
ks_proc_read(char *buf, char **start, off_t offset, int len, int *eof, void *data)
{
	struct mga_dev_s *mga = (struct mga_dev_s*)data;

	struct mga_procread param = {
		buf, start, offset, len, eof, mga->ks0127 };
	
	return mga_i2c_control(mga, I2C_DRIVERID_VIDEODECODER, 
			KS0127_READ_PROC, &param);
}


static int
ks_proc_write(struct file *file, const char *buf, unsigned long count, void *data)
{
	struct mga_dev_s *mga = (struct mga_dev_s*)data;

	struct mga_procwrite param = {
		file, buf, count, mga->ks0127 };

	return mga_i2c_control(mga, I2C_DRIVERID_VIDEODECODER, 
			KS0127_WRITE_PROC, &param);
}



static void mga_setscl(void *data, int state)
{
	struct mga_dev_s* mga = (struct mga_dev_s*)data;
	unsigned long flags;

	spin_lock_irqsave( &mga->reg_lock, flags );

	if (state) {
		mga_indirect_and_or(mga, XGENIOCTRL, ~SCL, 0);
	}
	else {
		mga_indirect_and_or(mga, XGENIODATA, ~SCL, 0);
		mga_indirect_and_or(mga, XGENIOCTRL, ~0, SCL);
	}
	
	spin_unlock_irqrestore( &mga->reg_lock, flags );
}


static void mga_setsda(void *data, int state)
{
	struct mga_dev_s* mga = (struct mga_dev_s*)data;
	unsigned long flags;

	spin_lock_irqsave( &mga->reg_lock, flags );

	if (state) {
		mga_indirect_and_or(mga, XGENIOCTRL, ~SDA, 0);
	} else {
		mga_indirect_and_or(mga, XGENIODATA, ~SDA, 0);
		mga_indirect_and_or(mga, XGENIOCTRL, ~0, SDA);
	}
	spin_unlock_irqrestore( &mga->reg_lock, flags );
} 


static int mga_getscl(void *data)
{
	struct mga_dev_s* mga = (struct mga_dev_s*)data;

	int	rv;
	unsigned long flags;
/* todo: maybe just use mga_indirect_readb_mt? */
	spin_lock_irqsave( &mga->reg_lock, flags );

	rv = (mga_indirect_readb(mga, XGENIODATA) & SCL) ? 1 : 0;

	spin_unlock_irqrestore( &mga->reg_lock, flags );

	return rv;
}


static int mga_getsda(void *data)
{
	struct mga_dev_s* mga = (struct mga_dev_s*)data;
	int rv;
	unsigned long flags;
	
	spin_lock_irqsave( &mga->reg_lock, flags );

	rv = (mga_indirect_readb(mga, XGENIODATA) & SDA) ? 1 : 0;
	
	spin_unlock_irqrestore( &mga->reg_lock, flags );	

	return rv;
}




static int adap_reg_client(struct i2c_client *client)
{
	struct mga_dev_s *mga = (struct mga_dev_s*)
		((struct i2c_algo_ks_data*)client->adapter->algo_data)->data;
		
	void *data = client->data;

	dprintk( "adap_reg_client addr:%02x, id:%02x\n", client->addr, client->driver->id );

	switch ( client->driver->id ) {

		case I2C_DRIVERID_MSP3400:
			mga->options |= MGAVIDEO_HAS_AUDIO;
			mga->msp_client = client;
			break;

		case I2C_DRIVERID_TUNER:
			mga->options |= MGAVIDEO_HAS_TUNER;
			mga->tuner_client = client;
			break;

		case I2C_DRIVERID_KS0127:
			mga->ks0127 = (struct ks0127*)data;
			mga_create_proc_entry(mga, "ks0127", 
				ks_proc_read, ks_proc_write, mga);	      
			mga->options |= MGAVIDEO_HAS_DECODER;
			if (client->addr == 0xD8>>1) {
				dprintk("%s: Detected KS0127 in RRG fashion\n", __FUNCTION__);
				mga->options |= MGAVIDEO_HAS_RRG;
			}
			else if (client->addr == 0xDA>>1) {
				dprintk("%s: Detected KS0127 in Marvel fashion\n", __FUNCTION__);
				mga->options |= MGAVIDEO_HAS_MARVEL;
			}
			mga->ks_client = client;
			break;

		case I2C_DRIVERID_MGATVO:
			mga->maven = (struct maven_s*)data;
			mga_create_proc_entry(mga, "maven", 
				maven_proc_read, maven_proc_write, mga);
			
			mga->options |= MGAVIDEO_HAS_ENCODER;
			mga->maven_client = client;
			break;

		default:
			printk("mgai2c: unknown device %x ", client->driver->id);
			break;
	}

	return 0;
}


static int adap_unreg_client(struct i2c_client *client)
{
	struct mga_dev_s *mga = (struct mga_dev_s*)
		((struct i2c_algo_ks_data*)client->adapter->algo_data)->data;

	dprintk( "adap_unreg_client %02x\n", client->addr );

	switch ( client->driver->id ) {
		case I2C_DRIVERID_MSP3400:
			mga->options &= ~MGAVIDEO_HAS_AUDIO;
			break;

		case I2C_DRIVERID_TUNER:
			mga->options &= ~MGAVIDEO_HAS_TUNER;
			break;
	
		case I2C_DRIVERID_KS0127:
			mga_remove_proc_entry(mga, "ks0127");
			mga->options &= ~MGAVIDEO_HAS_RRG;
			mga->options &= ~MGAVIDEO_HAS_MARVEL;
			mga->options &= ~MGAVIDEO_HAS_DECODER;
			break;
	
		case I2C_DRIVERID_MGATVO:
			mga_remove_proc_entry(mga, "maven");
			mga->options &= ~MGAVIDEO_HAS_ENCODER;
			mga->maven = NULL;
			break;
		
		default:
			printk("mgai2c: unknown device %x ", client->driver->id);
			break;
	}
	return 0;
}


static struct i2c_algo_ks_data mga_i2c_bit_ops_tmpl = {
	data:		NULL,
	setsda:		mga_setsda,
	setscl:		mga_setscl,
	getsda:		mga_getsda,
	getscl:		mga_getscl,
	udelay:		I2C_DELAY,
	mdelay:		1,
	timeout:	HZ/5
};


static struct i2c_adapter mga_i2c_adap_tmpl = {
	THIS_MODULE,
	"mgai2c",
	I2C_HW_B_BT848, /* needed for 2.4 tuner/msp3400*/	/* I2C_HW_B_G400 fixme */
	NULL,
	NULL,
	adap_reg_client,
	adap_unreg_client,
};


EXPORT_SYMBOL(mgai2c_init);
EXPORT_SYMBOL(mgai2c_cleanup);


void mgai2c_init(struct mga_dev_s *mga)
{
	struct i2c_algo_ks_data *ops;
	struct i2c_adapter *adap;

	ops = kmalloc( sizeof(*ops), GFP_KERNEL );
	if (ops == NULL) {
		return;//
	}
	memcpy( ops, &mga_i2c_bit_ops_tmpl, sizeof(*ops) );
	ops->data = mga;
	
	
	adap = kmalloc( sizeof(*adap), GFP_KERNEL );
	if (adap == NULL) {
		kfree( ops );
		return; //
	}
	memcpy( adap, &mga_i2c_adap_tmpl, sizeof(*adap) );
	adap->algo_data = ops;
	
	mga->i2c = adap;

#ifndef DVD
	if (i2c_ks_add_bus(adap) < 0)
		return;// -ENODEV;
#endif /* -DVD */
}



void mgai2c_cleanup(struct mga_dev_s *mga)
{
	if ( mga->i2c != NULL ) {

#ifndef DVD
		i2c_ks_del_bus( mga->i2c );
#endif /* -DVD */

		kfree( mga->i2c->algo_data );

		kfree( mga->i2c );
		mga->i2c = NULL;
	}
}

