#ifndef	_ZR36060
#define _ZR36060 1

#include "aiee.h"

/* these constants define the JPEG codec internal state, 
   the codec struct can be found in mgavideo.h */

#define	ZR36060_ID	1

#define	BUZ_MAX_FRAME	256	/* Must be a power of 2 */
#define	BUZ_MASK_FRAME	255	/* Must be BUZ_MAX_FRAME-1 */

#define XAWTV_HACK

#ifdef XAWTV_HACK
#define   BUZ_MAX_WIDTH   768   /* never display more than 768 pixels */
#else
#define   BUZ_MAX_WIDTH   720   /* never display more than 720 pixels */
#endif
#define   BUZ_MAX_HEIGHT  576   /* never display more than 576 rows */
#define   BUZ_MIN_WIDTH    32   /* never display less than 32 pixels */
#define   BUZ_MIN_HEIGHT   24   /* never display less than 24 rows */

struct buz_requestbuffers
{
   unsigned long count;      /* Number of buffers for MJPEG grabbing */
   unsigned long size;       /* Size PER BUFFER in bytes */
};

struct buz_sync
{
   unsigned long frame;      /* number of buffer that has been free'd */
   unsigned long length;     /* number of code bytes in buffer (capture only) */
   unsigned long seq;        /* frame sequence number */
   struct timeval timestamp; /* timestamp */
};

struct buz_status
{
   int input;                /* Input channel, has to be set prior to BUZIOC_G_STATUS */
   int signal;               /* Returned: 1 if valid video signal detected */
   int norm;                 /* Returned: VIDEO_MODE_PAL or VIDEO_MODE_NTSC */
   int color;                /* Returned: 1 if color signal detected */
};

struct buz_params
{

   /* The following parameters can only be queried */

   int major_version;            /* Major version number of driver */
   int minor_version;            /* Minor version number of driver */

   /* Main control parameters */

   int input;                    /* Input channel: 0 = Composite, 1 = S-VHS */
   int norm;                     /* Norm: VIDEO_MODE_PAL or VIDEO_MODE_NTSC */
   int decimation;               /* decimation of captured video,
                                    enlargement of video played back.
                                    Valid values are 1, 2, 4 or 0.
                                    0 is a special value where the user
                                    has full control over video scaling */

   /* The following parameters only have to be set if decimation==0,
      for other values of decimation they provide the data how the image is captured */

   int HorDcm;                    /* Horizontal decimation: 1, 2 or 4 */
   int VerDcm;                    /* Vertical decimation: 1 or 2 */
   int TmpDcm;                    /* Temporal decimation: 1 or 2,
                                     if TmpDcm==2 in capture every second frame is dropped,
                                     in playback every frame is played twice */
   int field_per_buff;            /* Number of fields per buffer: 1 or 2 */
   int img_x;                     /* start of image in x direction */
   int img_y;                     /* start of image in y direction */
   int img_width;                 /* image width BEFORE decimation,
                                     must be a multiple of HorDcm*16 */
   int img_height;                /* image height BEFORE decimation,
                                     must be a multiple of VerDcm*8 */

   /* --- End of parameters for decimation==0 only --- */

   /* JPEG control parameters */

   int  quality;                  /* Measure for quality of compressed images.
                                     Scales linearly with the size of the compressed images.
                                     Must be beetween 0 and 100, 100 is a compression
                                     ratio of 1:4 */

   int  odd_even;                 /* Which field should come first ??? */

   int  APPn;                     /* Number of APP segment to be written, must be 0..15 */
   int  APP_len;                  /* Length of data in JPEG APPn segment */
   char APP_data[60];             /* Data in the JPEG APPn segment. */

   int  COM_len;                  /* Length of data in JPEG COM segment */
   char COM_data[60];             /* Data in JPEG COM segment */

   unsigned long jpeg_markers;    /* Which markers should go into the JPEG output.
                                     Unless you exactly know what you do, leave them untouched.
                                     Inluding less markers will make the resulting code
                                     smaller, but there will be fewer aplications
                                     which can read it.
                                     The presence of the APP and COM marker is
                                     influenced by APP0_len and COM_len ONLY! */
#define JPEG_MARKER_DHT (1<<3)    /* Define Huffman Tables */
#define JPEG_MARKER_DQT (1<<4)    /* Define Quantization Tables */
#define JPEG_MARKER_DRI (1<<5)    /* Define Restart Interval */
#define JPEG_MARKER_COM (1<<6)    /* Comment segment */
#define JPEG_MARKER_APP (1<<7)    /* App segment, driver will allways use APP0 */

   int  VFIFO_FB;                 /* Flag for enabling Video Fifo Feedback.
                                     If this flag is turned on and JPEG decompressing
                                     is going to the screen, the decompress process
                                     is stopped every time the Video Fifo is full.
                                     This enables a smooth decompress to the screen
                                     but the video output signal will get scrambled */

   /* Misc */

   char reserved[312];            /* Makes 512 bytes for this structure */
};


enum buz_codec_mode
{
   BUZ_MODE_IDLE,                 /* nothing going on */
   BUZ_MODE_MOTION_COMPRESS,      /* grabbing frames */
   BUZ_MODE_MOTION_DECOMPRESS,    /* playing frames */
   BUZ_MODE_STILL_COMPRESS,       /* still frame conversion */
   BUZ_MODE_STILL_DECOMPRESS      /* still frame conversion */
};

enum buz_buffer_state
{
   BUZ_STATE_USER,                /* buffer is owned by application */
   BUZ_STATE_PEND,                /* buffer is queued in pend[] ready to feed to I/O */
   BUZ_STATE_DMA,                 /* buffer is queued in dma[] for I/O */
   BUZ_STATE_DONE                 /* buffer is ready to return to application */
};

struct buz_gbuffer
{
   u8	*virt;			/* virt address/size pairs */
   enum buz_buffer_state state;   /* non-zero if corresponding buffer is in use in grab queue */
   int  times_done;
   int  fields_seen;
   struct buz_sync bs;            /* DONE: info to return to application */
   u32	offset;
   u32  end[ 2 ]; /* endpoint of each field in buffer; max 2 fields at the moment */
};

/* This is the JPEG codec (also known as Zoran 36060, short zr36060)
   and _not_ the MGA codec !  */
struct zr36060_dev {
//	int	codec_id;
	char	device_name[32];
	void	*cte_private;
        int	state; /* defining the internal state, 
			  for valid constants see zr36060.h */ 

	int	restart;
	int	inv_vclk;

	int	initialized;	/* flag if buz has been correctly initalized */
	int	user;		/* number of current users (0 or 1) */

	/* MJPEG parameters */
	int	jpg_nbufs;		/* Number of buffers */
	int	jpg_bufsize;		/* size of mjpeg buffers in bytes */
	int	jpg_buffers_allocated;	/* flag if buffers are allocated */
	int	need_contiguous;	/* flag if contiguous buffer are needed */

	enum   buz_codec_mode codec_mode;	/* status of codec */
        struct buz_params params;    /* Defining the detailed JPEG parameters the JPEG codec has */ 

	int			field_time;
	
	WAIT_QUEUE		jpg_capq;	/* wait here for grab to finish */

	/* grab queue counts/indices, mask with BUZ_MASK_STAT_COM before using as index */
	/* (dma_head - dma_tail) is number active in DMA, must be <= BUZ_NUM_STAT_COM */
	/* (value & BUZ_MASK_STAT_COM) corresponds to index in stat_com table */
	unsigned int jpg_que_head;     /* Index where to put next buffer which is queued */
	unsigned int jpg_dma_head;     /* Index of next buffer which goes into stat_com  */
	unsigned int jpg_dma_tail;     /* Index of last buffer in stat_com               */
	unsigned int jpg_que_tail;     /* Index of last buffer in queue                  */
	unsigned int jpg_seq_num;      /* count of frames since grab/play started */

	unsigned int field_delete;	/* used at start of compress to delete 2 buffer */

	/* (value & BUZ_MASK_FRAME) corresponds to index in pend[] queue */
	int jpg_pend[BUZ_MAX_FRAME];

	/* array indexed by frame number */
	struct buz_gbuffer jpg_gbuf[BUZ_MAX_FRAME];   /* MJPEG buffers' info */

	spinlock_t lock;             /* Spinlock */

	struct	tvnorm	*norms;

	int	input_count;
	int	video_bus_size;
	int	code_master; /* not used by zr */
	int	memory_size; /* not used by zr */

	int	max_image;	/* Max size of a single image */

        /* Mike's CTE handling routines */
	int	(*read_memory)(void *cte_private, void *data, int start, int size);
	int	(*write_memory)(void *cte_private, void *data, int start, int size);
	void	(*reset)(void *cte_privater, int sleep_flag);
	void	(*start_transfer)(void *cte_private, enum buz_codec_mode mode);

	int	(*codec_set_jpg)(void *cte_private, enum buz_codec_mode mode);
	int	(*codec_set_video)(void *cte_private, enum buz_codec_mode mode);
};


/* these functions are to be called from the mgavideo.c "public" interface,
   _not_ from the API interface (if some calls here don't have an equivalent
   in mgavideo.c, you will have to invent new ones, and then use these 
   mgavideo calls in your API interface) */ 

int     zr36060_attach(struct zr36060_dev *zr, void *cte_private, int inv_vclk);
int	zr36060_deattach(struct zr36060_dev *codec);

int	zr36060_current_last(struct zr36060_dev *zr);
u8	*zr36060_current_buffer(struct zr36060_dev *zr, u32 *left, int field);
int	zr36060_current_get(struct zr36060_dev *zr, u32 size);
int	zr36060_current_put(struct zr36060_dev *zr, u32 size, int *pfield, struct timeval *stamp);
int	zr36060_current_flush(struct zr36060_dev *zr);
int	zr36060_done(struct zr36060_dev *codec, struct timeval *timestamp);

int	zr36060_ioctl(struct zr36060_dev *zr36060, int command, void *arg);
int	zr36060_mmap_bufs(struct zr36060_dev *codec, unsigned long adr, unsigned long size);

int	zr36060_proc_read(char *buf, char **start, off_t offset, int len, int *eof, void *unused);
int	zr36060_proc_write(struct file *file, const char *buf, unsigned long count, void *data);



#define	ZR36060_DEF_PARAMS	0x0000
#define	ZR36060_GET_PARAMS	0x0001
#define	ZR36060_SET_PARAMS	0x0002
#define	ZR36060_SYNC		0x0003
#define ZR36060_REQBUFS		0x0004
#define ZR36060_CANBUFS		0x0005
#define	ZR36060_GET_MODE	0x0006
#define	ZR36060_SET_MODE	0x0008
#define	ZR36060_QBUF_CAPT	0x0009
#define	ZR36060_QBUF_PLAY	0x000a
#define	ZR36060_RESTART		0x000b
#define	ZR36060_SET_INPUT	0x000c
#define	ZR36060_SET_NORM	0x000d

#endif

