/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/****************************************************************************
*
*  *****  ******  *****      ***     *****
* **   ** * ** * **   **    ****    **   **
* **   **   **        **   ** **    **
* *******   **       **   **  **    **
* **   **   **     **    *******    **
* **   **   **    **          **    **   **
* **   **  ****  *******      ** **  *****
*
*****************************************************************************
*
* AT24 serial device access code. Used by:
*  - ISACPNP, Big Appl
*  - PCI (bm) 1 ("PCI T") & Compaq PCI
*  - PCI (bm) 2 ("Abyss")
*  - CardBus
*
*****************************************************************************
*
* Copyright (C) 1996-98 Madge Networks Ltd.
*
* COMPANY CONFIDENTIAL
*
*****************************************************************************
*/


#define UDI_MODULE

#include "udt_driver.h"

#if defined ATULA_UDI   ||                                                  \
    defined SMART16_UDI ||                                                  \
    defined ISACPNP_UDI ||                                                  \
    defined PCIBM1_UDI  ||                                                  \
    defined PCIBM2_UDI  ||                                                  \
    defined CARDBUS_UDI ||                                                  \
    defined COMPAQ_PCI_UDI



#define AT24_ENABLE             0x04
#define AT24_DATA               0x02
#define AT24_CLOCK              0x01

#define AT24_WRITE              0xA0
#define AT24_READ               0xA1



STATIC BYTE_T UDT_INDIRECT
at24_readb(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       addr);


#ifdef UDT_SERIAL_EEPROM_WRITE_CODE

STATIC RETCODE_T UDT_INDIRECT
at24_writeb(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       addr,
          BYTE_T       data_byte);

#endif



STATIC BYTE_T
at24_in(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC VOID_T
at24_out_preserve_data(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       bits);

STATIC VOID_T
at24_out(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       bits);

STATIC VOID_T
at24_set_clock(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC VOID_T
at24_clear_clock(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC VOID_T
at24_enable(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC BYTE_T
at24_read_data_bit(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC VOID_T
at24_write_data_bit(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       data_bit);

STATIC VOID_T
at24_start(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC VOID_T
at24_stop(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC BOOLEAN_T
at24_wait_for_ack(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC BOOLEAN_T
at24_wait_for_nack(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC BYTE_T
at24_read_data_byte(
    INOUT P_UDI_DATA_T udids_ptr);

STATIC VOID_T
at24_write_data_byte(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       data);

STATIC BOOLEAN_T
at24_send_cmd(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       cmd);

STATIC BOOLEAN_T
at24_send_cmd_and_addr(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       cmd,
          BYTE_T       addr);

STATIC VOID_T
at24_delay(
    INOUT P_UDI_DATA_T udids_ptr);

DECLARE_MODULE_START_FUNCTION(at24_module_start)
DECLARE_MODULE_END_FUNCTION(at24_end)



RETCODE_T UDT_INTER_MODULE
at24_register(
    IN P_UDI_DATA_T udids_ptr)
{
    POKE_CFG(COLOUR_SUBFN, "RegAt24");

    

    SSI_MODULE_REGISTERED(
        udids_ptr,
        at24_register,
        at24_module_start,
        at24_end,
        MOD_SCOPE_INITIALISATION);

    

    SMART_INIT_FROM_UDIDS(udids_ptr).smart_at24_readb = at24_readb;

#ifdef UDT_SERIAL_EEPROM_WRITE_CODE

    SMART_INIT_FROM_UDIDS(udids_ptr).smart_at24_writeb = at24_writeb;

#endif

    

    return SUCCESS;
}

BUILD_MODULE_START_FUNCTION(at24_module_start)



STATIC BYTE_T UDT_INDIRECT
at24_readb(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       addr)
{
    BYTE_T             data       = 0xFF;

    

    at24_enable(udids_ptr);

    

    if(at24_send_cmd_and_addr(udids_ptr, AT24_WRITE, addr))
    {
        

        if(at24_send_cmd(udids_ptr, AT24_READ))
        {
            

            data = at24_read_data_byte(udids_ptr);

            

            if(!(at24_wait_for_nack(udids_ptr)))
            {
                

                data = 0xFF;
            }
        }
    }

    

    at24_stop(udids_ptr);

    

    at24_out(udids_ptr, 0);

    return data;
}




#ifdef UDT_SERIAL_EEPROM_WRITE_CODE

STATIC RETCODE_T UDT_INDIRECT
at24_writeb(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       addr,
          BYTE_T       data_byte)
{
    RETCODE_T  rc = FAILURE;

    

    at24_enable(udids_ptr);

    

    if(at24_send_cmd_and_addr(udids_ptr, AT24_WRITE, addr))
    {
        

        at24_write_data_byte(udids_ptr, data_byte);

        at24_wait_for_ack(udids_ptr);

        at24_stop(udids_ptr);

        at24_delay(udids_ptr);

        at24_send_cmd(udids_ptr, AT24_WRITE);

        rc = SUCCESS;
    }

    

    at24_out(udids_ptr, 0);

    return rc;
}

#endif


STATIC VOID_T
at24_delay(
    INOUT P_UDI_DATA_T udids_ptr)
{
    

    SSI_WAIT_AT_LEAST_USEC(udids_ptr, 20);

    return;
}



STATIC BYTE_T
at24_in(
    INOUT P_UDI_DATA_T udids_ptr)
{
    BYTE_T             bits, temp;

    

    if (GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).at24_eeprom_IO_offset > 0)
    {
        

        bits = INB(
                   udids_ptr,
                   0,
                   GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).
                       at24_eeprom_IO_offset);

        

        SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_reserved_bits =
            (BYTE_T)(bits & ~(AT24_ENABLE | AT24_DATA | AT24_CLOCK));

    }
    else
    {
        

        bits = PCI_READB(udids_ptr, PCIBM1_EEPROM_ACCESS_INDEX);

        

        SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_reserved_bits =
            (BYTE_T)(bits & 0x8F);

        

        temp = bits;         
        bits &= 0x40;        
        bits >>= 6;          
        temp &= 0x30;        
        temp >>= 3;          
        bits |= temp;        
    }

    

    bits &= (BYTE_T)(AT24_ENABLE | AT24_DATA | AT24_CLOCK);

    return bits;
}



STATIC VOID_T
at24_out_preserve_data(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       bits)
{
    BYTE_T temp;

    

    bits &= ~(AT24_DATA);

    

    bits |= SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_data_bit;

    

    bits &= (AT24_ENABLE | AT24_DATA | AT24_CLOCK);

    

    if (GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).at24_eeprom_IO_offset > 0)
    {
        

        bits |= SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_reserved_bits;

        OUTB(
            udids_ptr,
            0,
            GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).at24_eeprom_IO_offset,
            bits);
    }
    else
    {
        

        temp =  bits;              
        bits &= 0x01;              
        bits <<= 6;                
        temp &= 0x06;              
        temp <<= 3;                
        bits |= temp;              

        bits |= SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_reserved_bits;

        PCI_WRITEB(udids_ptr, PCIBM1_EEPROM_ACCESS_INDEX, bits);
    }

    

    at24_delay(udids_ptr);

    return;
}



STATIC VOID_T
at24_out(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       bits)
{
    BYTE_T temp;

    

    SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_data_bit =
        (BYTE_T)(bits & AT24_DATA);

    

    bits &= (AT24_ENABLE | AT24_DATA | AT24_CLOCK);

    

    if (GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).at24_eeprom_IO_offset > 0)
    {
        

        bits |= SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_reserved_bits;

        OUTB(
            udids_ptr,
            0,
            GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).at24_eeprom_IO_offset,
            bits);
    }
    else
    {
        

        temp =  bits;              
        bits &= 0x01;              
        bits <<= 6;                
        temp &= 0x06;              
        temp <<= 3;                
        bits |= temp;              

        bits |= SMART_RES_INFO_FROM_UDIDS(udids_ptr).at24_reserved_bits;

        PCI_WRITEB(udids_ptr, PCIBM1_EEPROM_ACCESS_INDEX, bits);
    }

    

    at24_delay(udids_ptr);

    return;
}



STATIC VOID_T
at24_set_clock(
    INOUT P_UDI_DATA_T udids_ptr)
{
    BYTE_T             bits;

    

    bits = at24_in(udids_ptr);

    

    bits |= (AT24_CLOCK);

    

    at24_out_preserve_data(udids_ptr, bits);

    return;
}



STATIC VOID_T
at24_clear_clock(
    INOUT P_UDI_DATA_T udids_ptr)
{
    BYTE_T             bits;

    

    bits = at24_in(udids_ptr);

    

    bits &= ~(AT24_CLOCK);

    

    at24_out_preserve_data(udids_ptr, bits);

    return;
}



STATIC BYTE_T
at24_read_data_bit(
    INOUT P_UDI_DATA_T udids_ptr)
{
    BYTE_T             data_bit;

    

    at24_out(udids_ptr, (AT24_ENABLE | AT24_DATA));

    

    at24_set_clock(udids_ptr);

    

    at24_out(udids_ptr, (AT24_DATA | AT24_CLOCK));

    

    data_bit = (BYTE_T)((at24_in(udids_ptr) & AT24_DATA) >> 1);

    

    at24_out(udids_ptr, (AT24_ENABLE | AT24_DATA | AT24_CLOCK));

    

    at24_clear_clock(udids_ptr);

    return data_bit;
}



STATIC VOID_T
at24_write_data_bit(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       data_bit)
{
    BYTE_T             bits;

    

    data_bit <<= 1;

    

    bits = at24_in(udids_ptr);

    

    bits &= ~(AT24_DATA);
    bits |= data_bit;

    

    at24_out(udids_ptr, bits);

    return;
}



STATIC VOID_T
at24_enable(
    INOUT P_UDI_DATA_T udids_ptr)
{
    BYTE_T             bits;

#   ifdef UDT_SERIAL_EEPROM_WRITE_CODE

    BYTE_T             reset_reg_value;

#   endif

    

#   ifdef UDT_SERIAL_EEPROM_WRITE_CODE

    if(ADAPTER_INFO_FROM_UDIDS(udids_ptr).adapter_type ==
           ADAPTER_TYPE_PCIBM2
       ||
       ADAPTER_INFO_FROM_UDIDS(udids_ptr).adapter_type ==
           ADAPTER_TYPE_CARDBUS)
    {
        reset_reg_value = INB(udids_ptr, 0, PCIBM2_RESET_REG);

        

        reset_reg_value |= PCIBM2_RESET_REG_CHIP_NRES;

        OUTB(udids_ptr, 0, PCIBM2_RESET_REG, reset_reg_value);

        

        SSI_WAIT_AT_LEAST_USEC(udids_ptr, 100);
    }

#   endif

    

    bits = at24_in(udids_ptr);

    

    bits |= AT24_ENABLE;

    

    at24_out(udids_ptr, bits);

    return;
}



STATIC VOID_T
at24_start(
    INOUT P_UDI_DATA_T udids_ptr)
{
    

    at24_out(udids_ptr, (AT24_ENABLE | AT24_DATA));

    

    at24_set_clock(udids_ptr);

    

    at24_out(udids_ptr, (AT24_ENABLE | AT24_CLOCK));

    

    at24_clear_clock(udids_ptr);

    return;
}



STATIC VOID_T
at24_stop(
    INOUT P_UDI_DATA_T udids_ptr)
{
    

    at24_out(udids_ptr, AT24_ENABLE);

    

    at24_set_clock(udids_ptr);

    

    at24_out(udids_ptr, (AT24_ENABLE | AT24_DATA | AT24_CLOCK));

    

    at24_clear_clock(udids_ptr);

    return;
}



STATIC BOOLEAN_T
at24_wait_for_ack(
    INOUT P_UDI_DATA_T udids_ptr)
{
    UINT_T             count;

    

    for(count = 0; count < 10; count++)
    {
        

        if((at24_read_data_bit(udids_ptr) & 0x01) == 0x00)
        {
            return TRUE;
        }
    }

    

    return FALSE;
}



STATIC BOOLEAN_T
at24_wait_for_nack(
    INOUT P_UDI_DATA_T udids_ptr)
{
    UINT_T             count;

    

    for(count = 0; count < 10; count++)
    {
        

        if((at24_read_data_bit(udids_ptr) & 0x01) == 0x01)
        {
            return TRUE;
        }
    }

    

    return FALSE;
}



STATIC BYTE_T
at24_read_data_byte(
    INOUT P_UDI_DATA_T udids_ptr)
{
    BYTE_T             data       = 0x00;
    UINT_T             count;

    

    for(count = 0; count < 8; count++)
    {
        data <<= 1;
        data |= at24_read_data_bit(udids_ptr);
    }

    return data;
}



STATIC VOID_T
at24_write_data_byte(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       data)
{
    UINT_T             count;

    

    for(count = 0; count < 8; count++)
    {
        at24_write_data_bit(
            udids_ptr,
            (BYTE_T)((data >> (7 - count)) & 0x01));

        

        at24_set_clock(udids_ptr);
        at24_clear_clock(udids_ptr);
    }

    return;
}



STATIC BOOLEAN_T
at24_send_cmd(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       cmd)
{
    UINT_T             count;

    

    for(count = 0; count < 10; count++)
    {
        

        at24_start(udids_ptr);

        

        at24_write_data_byte(udids_ptr, cmd);

        

        if(at24_wait_for_ack(udids_ptr))
        {
            return TRUE;
        }
    }

    return FALSE;
}



STATIC BOOLEAN_T
at24_send_cmd_and_addr(
    INOUT P_UDI_DATA_T udids_ptr,
          BYTE_T       cmd,
          BYTE_T       addr)
{
    

    if(at24_send_cmd(udids_ptr, cmd))
    {
        at24_write_data_byte(udids_ptr, addr);

        return at24_wait_for_ack(udids_ptr);
    }

    return FALSE;
}



BUILD_MODULE_END_FUNCTION(at24_end)



#endif




