/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/*****************************************************************************
*
* *****   *******  *****   *****  **   ** ******     *****
*  ** **   **   * **   ** **   ** *** ***  **  **   **   **
*  **  **  ** *   **      **   ** *******  **  **   **
*  **  **  ****   **      **   ** ** * **  *****    **
*  **  **  ** *   **      **   ** **   **  **       **
*  ** **   **   * **   ** **   ** **   **  **       **   **
* *****   *******  *****   *****  **   ** ****   **  *****
*
******************************************************************************
*
* This module provides the decompress function for data originally
* compressed using an LZ-type algorithm with Huffman tree encoding of the
* lengths and distances.
*
* This compression usees 2 Shannon-Fano trees and a 4K sliding dictionary.
*
******************************************************************************
*
* Copyright (C) 1996-98 Madge Networks Ltd.
*
* COMPANY CONFIDENTIAL
*
******************************************************************************
*/



#include "udt_driver.h"


#ifdef UDT_DECOMPRESS_BIN_FILES





#define LS_DISTANCE_BITS     6           
#define MIN_MATCH_LEN        2           
#define EXTENDED_LENGTH      8



STATIC VOID_T
load_tree(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    PCLIENT_SF_TREE_T              tree);

STATIC VOID_T
read_tree_code_lengths(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    PCLIENT_SF_TREE_T              tree);

STATIC BYTE_T
get_bits(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    UINT_T                         number_of_bits);

STATIC VOID_T
sort_by_tree_code_lengths(
    IN    PCLIENT_SF_TREE_T              tree);

STATIC VOID_T
calculate_tree_codes(
    IN    PCLIENT_SF_TREE_T              tree);

STATIC VOID_T
reverse_tree_codes(
    IN    PCLIENT_SF_TREE_T              tree);

STATIC BYTE_T
decompress_a_byte(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w);

STATIC BYTE_T
decode_using_tree(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    PCLIENT_SF_TREE_T              tree);



VOID_T UDT_INTER_MODULE
initialise_decompression(
    IN    P_UDI_DATA_T                   udids_ptr,
    IN    PCLIENT_CODE_HANDLE_T          code_handle_ptr,
    INOUT PCLIENT_DECOMPRESS_WORKSPACE_T w)
{
    UINT_T i;

    

    

    w->udids_ptr = udids_ptr;

    

    w->code_handle_ptr = code_handle_ptr;

    

    for (i=0; i<DECOMPRESS_BUFFER_SIZE; i++)
    {
        w->buffer[i] = 0;
    }

    w->least_recent_byte = 0;

    

    w->pending_bit_count = 0;
    w->copy_bytes_left = 0;

    

    get_bits(w, 8);
    get_bits(w, 8);


    

    load_tree(w, (PCLIENT_SF_TREE_T)&(w->len_tree));

    load_tree(w, (PCLIENT_SF_TREE_T)&(w->dist_tree));

    return;
}



STATIC VOID_T
load_tree(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    PCLIENT_SF_TREE_T              tree)
{
    

    read_tree_code_lengths(w, tree);

    

    sort_by_tree_code_lengths(tree);

    

    calculate_tree_codes(tree);

    

    reverse_tree_codes(tree);

    

    return;
}



STATIC VOID_T
read_tree_code_lengths(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    PCLIENT_SF_TREE_T              tree)
{
    BYTE_T compressed_length;
    BYTE_T num_values_at_length;
    BYTE_T length;
    BYTE_T value;

    

    compressed_length = (BYTE_T) (get_bits(w, 8) + 1);


    

    value = 0;

    for ( ; compressed_length!=0; compressed_length--)
    {
        length = (BYTE_T) (get_bits(w, 4) + 1);

        num_values_at_length = (BYTE_T) (get_bits(w, 4) + 1);

        

        for ( ; num_values_at_length!=0; num_values_at_length--)
        {
            tree[value].value = value;
            tree[value].bit_length = length;

            value++;
        }
    }

    return;
}



STATIC BYTE_T
get_bits(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    UINT_T                         number_of_bits)
{
    BYTE_T result;
    BYTE_T next_bit;
    UINT_T i;

    result = 0;

    

    next_bit = 1;

    for (i=0; i<number_of_bits; i++)
    {
        

        if (w->pending_bit_count == 0)
        {
            w->pending_bits =
                SSI_CODE_READW(w->udids_ptr, w->code_handle_ptr);

            w->pending_bit_count = 16;
        }

        

        if (w->pending_bits & 1)
        {
            result=(BYTE_T)(result+next_bit);
        }

        w->pending_bits = w->pending_bits>>1;
        w->pending_bit_count--;

        next_bit = (BYTE_T)(next_bit << 1);
    }

    return result;
}


STATIC VOID_T
sort_by_tree_code_lengths(
    IN    PCLIENT_SF_TREE_T tree)
{
    UINT_T i,j;
    SF_TREE_T temp;

    

    for (i=0; i<DECOMPRESS_TREE_ENTRIES-1; i++)
    {
        for (j=i+1; j<DECOMPRESS_TREE_ENTRIES; j++)
        {
            if (tree[i].bit_length > tree[j].bit_length
                ||
                (
                    tree[i].bit_length == tree[j].bit_length
                    &&
                    tree[i].value > tree[j].value
                )
               )
            {
               

                temp = tree[i];
                tree[i] = tree[j];
                tree[j] = temp;
            }
        }
    }

    return;
}


STATIC VOID_T
calculate_tree_codes(
    IN    PCLIENT_SF_TREE_T tree)
{
    INT_T i;                 
                             
                             
    UINT_T last_bit_length;
    WORD_T code;
    WORD_T code_increment;

    

    code = 0;
    code_increment = 0;

    last_bit_length = 0;

    for (i=DECOMPRESS_TREE_ENTRIES-1; i>=0; i--)
    {
        tree[i].tree_code = code;

        if(tree[i].bit_length!=last_bit_length)
        {
            last_bit_length = tree[i].bit_length;
            code_increment = 1 << (16-last_bit_length);
        }

        code+=code_increment;
    }
}


STATIC VOID_T
reverse_tree_codes(
    IN    PCLIENT_SF_TREE_T tree)
{
    UINT_T i,j;
    WORD_T temp=0;

    for (i=0; i<DECOMPRESS_TREE_ENTRIES-1; i++)
    {
        for (j=0; j<16; j++)
        {
            temp = temp << 1;
            temp += tree[i].tree_code & 1;
            tree[i].tree_code = tree[i].tree_code >> 1;
        }

        tree[i].tree_code = temp;
    }

    return;
}


WORD_T UDT_INTER_MODULE
decompress_a_word(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w)
{
    BYTE_T first_byte;
    BYTE_T second_byte;

    first_byte = decompress_a_byte(w);
    second_byte = decompress_a_byte(w);

    return first_byte + (second_byte<<8);
}


STATIC BYTE_T
decompress_a_byte(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w)
{
    BYTE_T result;

    

    if (w->copy_bytes_left)
    {
        result = w->buffer[w->copy_offset];
        w->copy_offset ++;
        w->copy_offset &= DECOMPRESS_BUFFER_MASK;

        w->copy_bytes_left --;
    }
    else
    {
        

        if (get_bits(w,1))
        {
            

            result = get_bits(w,8);
        }
        else
        {
            UINT_T distance;
            UINT_T length;

            

            distance = get_bits(w, LS_DISTANCE_BITS);
            distance +=
                decode_using_tree(w, (PCLIENT_SF_TREE_T)&(w->dist_tree))
                   << LS_DISTANCE_BITS;

            length = decode_using_tree(w, (PCLIENT_SF_TREE_T)&(w->len_tree));

            if (length==63)
            {
                length += get_bits(w, 8);
            }
            length += MIN_MATCH_LEN;

            

            w->copy_offset = (w->least_recent_byte-1-distance)
                             & DECOMPRESS_BUFFER_MASK;
            w->copy_bytes_left = length;

            

            return decompress_a_byte(w);
        }
    }

    

    w->buffer[w->least_recent_byte] = result;

    w->least_recent_byte ++;
    w->least_recent_byte &= DECOMPRESS_BUFFER_MASK;

    return result;
}


STATIC BYTE_T
decode_using_tree(
    IN    PCLIENT_DECOMPRESS_WORKSPACE_T w,
    IN    PCLIENT_SF_TREE_T              tree)
{
    UINT_T bits;
    WORD_T code;
    UINT_T index;

    

    bits = 0;
    code = 0;
    index = 0;

    for (;;)
    {
        

        code += get_bits(w, 1) << bits;
        bits++;

        

        while (bits==tree[index].bit_length)
        {
            if (tree[index].tree_code==code)
            {
                return tree[index].value;
            }
            index++;
        }
    }
}

#endif 




