/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/*****************************************************************************
*
* driver.h 
*
* This is the configuration header file for the Universal Driver Toolkit.
* This file serves to configure the UDT for the driver's needs and also to
* provide type definitions and some helper macros to encapsulate platform
* specific details.
*
* Copyright (c) Madge Networks Ltd
*
*    
*****************************************************************************/

/*----------------------------------------------------------------------------
|
| Include our Linux version compatibility header file
|
|---------------------------------------------------------------------------*/ 

#include <linux/autoconf.h>

#include <linux/config.h>
#ifndef MODULE
#include <linux/version.h>
#ifndef EMBEDDED_MICRO_CODE
#define EMBEDDED_MICRO_CODE
#endif
#endif


#if defined(CONFIG_SMP) && !defined(__SMP__)
#define __SMP__
#endif

#ifdef MODULE
#if !defined(MODVERSIONS) && defined(CONFIG_MODVERSIONS)
#define MODVERSIONS
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* MODULE */


#if LINUX_VERSION_CODE < 0x20155
#define OLD_PCI
#endif


/*----------------------------------------------------------------------------
|
| Need these Linux header files for the I/O and PCI operations below.
|
|---------------------------------------------------------------------------*/

#include <linux/types.h>
#include <asm/byteorder.h> 
#include <linux/ioport.h>

#include <linux/string.h>
#include <asm/io.h>

#include "ver.h"

#ifdef __BIG_ENDIAN
#define UDT_CPU_IS_BIG_ENDIAN
#endif 
 
/*----------------------------------------------------------------------------
|
| Linux is a flat 32bit enviornment so there is no concept of FAR pointers.
|
|---------------------------------------------------------------------------*/ 

#define _FAR                       

#define FAR_DATA  
#define FAR_CLIENT
#define FAR_INIT_DATA
#define FAR_FRAME_DATA
#define FAR_IMAGE
#define FAR_IMAGE_DEF
#define FAR_FRAME_DATA


/*---------------------------------------------------------------------------
|             
| cdecl doesn't exist in Linux.
|
|---------------------------------------------------------------------------*/ 

#define cdecl


/*---------------------------------------------------------------------------
|             
| CODE_MODEL_FLAT is the default. Used in ALL non segmented architectures.
|
|---------------------------------------------------------------------------*/ 

#define CODE_MODEL_FLAT


/*----------------------------------------------------------------------------
|
| Function argument modifiers.
|
| IN     Called function 'reads' through the pointer.
| OUT    Called function 'writes' through the pointer.
| INOUT  Called function 'reads'and 'writes' through the pointer.
|
|---------------------------------------------------------------------------*/

#define IN     
#define OUT
#define INOUT



/*----------------------------------------------------------------------------
|
| Some useful macros.
|
----------------------------------------------------------------------------*/

#ifndef NULL
#define NULL      ((void *) 0)
#endif

#define VOLATILE  volatile
#define STATIC static


/*----------------------------------------------------------------------------
|
| Primitive data types. Define the following primitive types appropriately.
|
----------------------------------------------------------------------------*/

/*
 * Types used for Hardware access - must be exactly the given size
 */ 

typedef __u8  BYTE_T;     /* Unsigned 8 bit quantity */
typedef __u16 WORD_T;     /* Unsigned 16 bit quantity */
typedef __u32 DWORD_T;    /* Unsigned 32 bit quantity */


/*
 * Toolkit usage quantities - must not be less than suggested sizes.
 * May be larger where this results in more efficient code being generated.
 */

/*
 * 8 bit types.
 */
 
typedef char               CHAR_T; 
typedef unsigned char      UCHAR_T;


/*
 * 16 bit types.
 */
 
typedef short              SHORT_T; 
typedef unsigned short     USHORT_T;


/*
 * 32 bit types.
 */
 
typedef __s32               LONG_T;
typedef __u32               ULONG_T;


/*
 * Natural word size (not less than 16 bit) types.
 */
 
typedef int                 INT_T;
typedef unsigned int        UINT_T;


/*
 * Boolean type (2 values).
 */
 
typedef int                BOOLEAN_T;

#ifndef FALSE
#define FALSE              0
#define TRUE               1
#endif
#define BOOL_T_DEFINED

/*
 * Character string type.
 */
 
typedef CHAR_T           * STRING_T;


/*
 * Void type.
 */
 
#define VOID_T             void


/*----------------------------------------------------------------------------
|
| Special types.
|
----------------------------------------------------------------------------*/

/*
 * Pointer to the driver's data space
 */

typedef void    * P_DRIVER_T;

/*
 * Physical Addressess - used for DMA
 */
 
typedef DWORD_T   PHYS_ADDRESS_T;


/*----------------------------------------------------------------------------
|
| An I/O handle. This type is opaque to the UDT. It is used by the driver to 
| describe an I/O region.
|
|---------------------------------------------------------------------------*/

typedef unsigned long IO_HANDLE_T;


/*----------------------------------------------------------------------------
|
| Shared RAM handles. These types are opaque to the UDT. They are used by the 
| driver to describe regions of shared RAM.
|
|---------------------------------------------------------------------------*/

typedef char * SHARED_HANDLE_T;
typedef char * INIT_SHARED_HANDLE_T;


/*----------------------------------------------------------------------------
|
| An PCI handle. This type is opaque to the UDT. It is used by the driver to 
| describe an PCI device.
|
|---------------------------------------------------------------------------*/
 
#ifdef OLD_PCI
typedef unsigned PCI_HANDLE_T;
#else
typedef struct pci_dev * PCI_HANDLE_T;
#endif

/*---------------------------------------------------------------------------
|
| Define a fragment descriptor for SSI.
|
---------------------------------------------------------------------------*/

typedef struct
{
	__u8             * mem;  /* kmalloc'ed block */
    __u8             * data; /* Pointer to data area (aligned) */
    __u32            length; /* Fragment length. */
    PHYS_ADDRESS_T   phys;   /* Physical address of the data area. */               
} FRAG_HANDLE_T;


/*----------------------------------------------------------------------------
|
| A code handle. This type is opaque to the UDT. It is used by the 
| driver to describe a download image.
|
|---------------------------------------------------------------------------*/

typedef struct {
	WORD_T * data;
	DWORD_T offset;
} CODE_HANDLE_T;


/*----------------------------------------------------------------------------
|
| The completion structure used by the UDT for asynchronous UDI commands.
|
|---------------------------------------------------------------------------*/

typedef struct _MTOK_COMPLETION * CLIENT_ARGUMENT_T;


/*----------------------------------------------------------------------------
|
| UDT Debugging.
|
|---------------------------------------------------------------------------*/

#ifdef DEBUG
#define UDT_DEBUG
#define UDT_POKEOUTS_ON
#define DO_POKEOUTS
#endif


/*----------------------------------------------------------------------------
|
| UDT Error reporting.
|
|---------------------------------------------------------------------------*/

/* #define UDT_ERROR_REPORTING_ON */

/* #define UDT_NO_OFF_LEVEL_INTERRUPT_HANDLING */ 

#define UDT_MAINTAIN_COUNTERS
#define UDT_EXTENDED_COUNTERS

#define UDT_DYNAMIC_RINGSPEED

/*----------------------------------------------------------------------------
|
| UDT Error checking.
|
|---------------------------------------------------------------------------*/

/* #define UDT_RETURN_CHECKING_ON */


/*----------------------------------------------------------------------------
|
| UDT Sanity check failed macro.
|
|---------------------------------------------------------------------------*/

/* Do Nothing on production build */
#define UDT_INSANE



/*----------------------------------------------------------------------------
|
| Macros used by the UDT probing code for doing platform specific things.
|
| See the file pssi.h for details of what each macro should do.
|
|---------------------------------------------------------------------------*/

#define _IO_PORT(_handle, _offset) \
    (IO_HANDLE_T) ((_handle) + (IO_HANDLE_T) (_offset))
    
/*--------------------------------------------------------------------------*/
    
#define D_PSSI_DETECT_MCA(_dds_ptr) \
    (0)

#define D_PSSI_POKE(dds,chan,lvl,color,prompt,type,data) \
	poke(color,prompt,type,data) 

#define D_SSI_POKE(dds,chan,lvl,color,prompt,type,data) \
	poke(color,prompt,type,data) 

#define D_POKE(dds,chan,lvl,color,prompt,type,data) \
	poke(color,prompt,type,data)

#define D_SSI_READ_TUPLE(dds,tuple,buffer,len)  \
    ssi_read_tuple(dds,tuple,buffer,len)

/*--------------------------------------------------------------------------*/

#define D_PSSI_REGISTER_IO(_dds_ptr, _io_base, _io_range, _io_handle) \
	mtok_pssi_register_io(_dds_ptr,_io_base, _io_range, _io_handle)
	
#define D_PSSI_UNREGISTER_IO(_dds_ptr, _io_base, _io_handle)
       
#define D_PSSI_IO_READB(_dds_ptr, _io_handle, _offset) \
    inb(_IO_PORT(_io_handle, _offset))
    
#define D_PSSI_IO_READW(_dds_ptr, _io_handle, _offset) \
    inw(_IO_PORT(_io_handle, _offset))
       
#define D_PSSI_IO_WRITEB(_dds_ptr, _io_handle, _offset, _value) \
    outb((unsigned char) (_value), _IO_PORT(_io_handle, _offset))

#define D_PSSI_IO_WRITEW(_dds_ptr, _io_handle, _offset, _value) \
    outw((unsigned short) (_value), _IO_PORT(_io_handle, _offset))

/*--------------------------------------------------------------------------*/

#define D_PSSI_REGISTER_SHARED(_dds_ptr, _base, _max_offset, _shared_handle) \
    (*(_shared_handle) = (SHARED_HANDLE_T) bus_to_virt(_base), TRUE)
    
#define D_PSSI_UNREGISTER_SHARED(_dds_ptr, _base, _shared_handle)

#define D_PSSI_SHARED_READB(_dds_ptr, _shared_handle, _offset) \
    readb((_shared_handle) + (_offset))

/*--------------------------------------------------------------------------*/

#define D_PSSI_DWORD_SHIFT_L(_dds_ptr, _dword, _bit_count) \
    ((_dword) << (_bit_count))

#define D_PSSI_DWORD_SHIFT_R(_dds_ptr, _dword, _bit_count) \
    ((_dword) >> (_bit_count))


/*----------------------------------------------------------------------------
|
| Macros used by the UDT driver code for doing platform specific things.
|
| See the file ssi.h for details of what each macro should do.
|
|---------------------------------------------------------------------------*/

#define D_SSI_IO_READB(_dds_ptr, _io_handle, _offset) \
    inb(_IO_PORT(_io_handle, _offset))
    
#define D_SSI_IO_READW(_dds_ptr, _io_handle, _offset) \
    inw(_IO_PORT(_io_handle, _offset))
       
#define D_SSI_IO_READD(_dds_ptr, _io_handle, _offset) \
    inl(_IO_PORT(_io_handle, _offset))
       
#define D_SSI_IO_WRITEB(_dds_ptr, _io_handle, _offset, _value) \
    outb((unsigned char) (_value), _IO_PORT(_io_handle, _offset))

#define D_SSI_IO_WRITEW(_dds_ptr, _io_handle, _offset, _value) \
    outw((unsigned short) (_value), _IO_PORT(_io_handle, _offset))
       
#define D_SSI_IO_WRITED(_dds_ptr, _io_handle, _offset, _value) \
    outl((unsigned int) (_value), _IO_PORT(_io_handle, _offset))
    
/*--------------------------------------------------------------------------*/
        
#define D_SSI_INIT_SHARED_READB(_dds_ptr, _shared_handle, _offset) \
    readb((_shared_handle) + (_offset))

#define D_SSI_INIT_SHARED_READW(_dds_ptr, _shared_handle, _offset) \
    readw((_shared_handle) + (_offset))
                       
#define D_SSI_INIT_SHARED_READD(_dds_ptr, _shared_handle, _offset) \
    readl((_shared_handle) + (_offset))
                       
/*--------------------------------------------------------------------------*/

#define D_SSI_SHARED_READB(_dds_ptr, _shared_handle, _offset) \
    readb((_shared_handle) + (_offset))

#define D_SSI_SHARED_READW(_dds_ptr, _shared_handle, _offset) \
    readw((_shared_handle) + (_offset))
                       
#define D_SSI_SHARED_READD(_dds_ptr, _shared_handle, _offset) \
    readl((_shared_handle) + (_offset))
              
#define D_SSI_SHARED_WRITEB(_dds_ptr, _shared_handle, _offset, _value) \
    writeb((_value), (_shared_handle) + (_offset))
                       
#define D_SSI_SHARED_WRITEW(_dds_ptr, _shared_handle, _offset, _value) \
    writew((_value), (_shared_handle) + (_offset))
                       
#define D_SSI_SHARED_WRITED(_dds_ptr, _shared_handle, _offset, _value) \
    writel((_value), (_shared_handle) + (_offset))
                       
/*--------------------------------------------------------------------------*/

/*
 * FRAGMENT ACCESS MACROS
 */


#define D_SSI_FRAG_WRITEB(_dds_ptr, _frag_handle, _offset, _value) \
    (*((volatile BYTE_T *) &(_frag_handle).data[_offset]) = (_value))

#define D_SSI_FRAG_READB(_dds_ptr, _frag_handle, _offset) \
    (*((volatile BYTE_T *) &(_frag_handle).data[_offset]))


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,1,71)

/*
 * Use the leXX_to_cpu for endianness portability 
 */

#define D_SSI_FRAG_READW(_dds_ptr, _frag_handle, _offset) \
    le16_to_cpu( *((volatile WORD_T *)&_frag_handle.data[_offset]))

#define D_SSI_FRAG_READD(_dds_ptr, _frag_handle, _offset) \
    le32_to_cpu( *((volatile DWORD_T *)&_frag_handle.data[_offset]))

#define D_SSI_FRAG_WRITEW(_dds_ptr, _frag_handle, _offset, _value) \
    (*((volatile WORD_T *)&_frag_handle.data[_offset]) = cpu_to_le16(_value))

#define D_SSI_FRAG_WRITED(_dds_ptr, _frag_handle, _offset, _value) \
    (*((volatile DWORD_T *)&_frag_handle.data[_offset]) = cpu_to_le32(_value))

#else /*** Don't support big endian arch on old Linux kernels (<2.1.71) ***/

#define D_SSI_FRAG_READW(_dds_ptr, _frag_handle, _offset) \
    (*((volatile WORD_T *) &(_frag_handle).data[_offset]))
                
#define D_SSI_FRAG_READD(_dds_ptr, _frag_handle, _offset) \
    (*((volatile DWORD_T *) &(_frag_handle).data[_offset]))
                        
#define D_SSI_FRAG_WRITEW(_dds_ptr, _frag_handle, _offset, _value) \
    (*((volatile WORD_T *) &(_frag_handle).data[_offset]) = (_value))
                                
#define D_SSI_FRAG_WRITED(_dds_ptr, _frag_handle, _offset, _value) \
    (*((volatile DWORD_T *) &(_frag_handle).data[_offset]) = (_value))

#endif /****** (Version > 2.1.71) ******/


/*--------------------------------------------------------------------------*/

#define D_SSI_FRAG_PHYS_ADDRESS(_dds_ptr, _frag_handle) \
    ((_frag_handle).phys)

#define D_SSI_FRAG_LENGTH(_dds_ptr, _frag_handle) \
    ((u32) (_frag_handle).length)

/*--------------------------------------------------------------------------*/

#ifndef UDT_NOPCI
#define D_SSI_PCI_READB(_dds_ptr, _pci_handle, _offset) \
    mtok_pci_readb((_pci_handle), (unsigned) (_offset))

#define D_SSI_PCI_READW(_dds_ptr, _pci_handle, _offset) \
    mtok_pci_readw((_pci_handle), (unsigned) (_offset))

#define D_SSI_PCI_READD(_dds_ptr, _pci_handle, _offset) \
    mtok_pci_readd((_pci_handle), (unsigned) (_offset))

#define D_SSI_PCI_WRITEB(_dds_ptr, _pci_handle, _offset, _value) \
    mtok_pci_writeb((_pci_handle), (unsigned) (_offset), _value)
    
#define D_SSI_PCI_WRITEW(_dds_ptr, _pci_handle, _offset, _value) \
    mtok_pci_writew((_pci_handle), (unsigned) (_offset), _value)
        
#define D_SSI_PCI_WRITED(_dds_ptr, _pci_handle, _offset, _value) \
    mtok_pci_writed((_pci_handle), (unsigned) (_offset), _value)

#else /* UDT_NOPCI */

#define D_SSI_PCI_READB(_dds_ptr, _pci_handle, _offset) \
    0xff

#define D_SSI_PCI_READW(_dds_ptr, _pci_handle, _offset) \
    0xffff

#define D_SSI_PCI_READD(_dds_ptr, _pci_handle, _offset) \
    0xffffffff

#define D_SSI_PCI_WRITEB(_dds_ptr, _pci_handle, _offset, _value)
    
#define D_SSI_PCI_WRITEW(_dds_ptr, _pci_handle, _offset, _value)
        
#define D_SSI_PCI_WRITED(_dds_ptr, _pci_handle, _offset, _value)

#endif /* UDT_NOPCI */
        
/*--------------------------------------------------------------------------*/
        
#define D_SSI_IO_TO_FRAG_16BIT(_dds_ptr, _frag_handle, _frag_offset, \
                               _io_handle, _io_offset, _byte_count) \
    mtok_16bit_pio_from_adapter( \
        _IO_PORT(_io_handle, _io_offset), \
        &(_frag_handle).data[_frag_offset], \
        (_byte_count))
    
#define D_SSI_FRAG_TO_IO_16BIT(_dds_ptr, _io_handle, _io_offset, \
                               _frag_handle, _frag_offset, _byte_count) \
    mtok_16bit_pio_to_adapter( \
        _IO_PORT(_io_handle, _io_offset), \
        &(_frag_handle).data[_frag_offset], \
        (_byte_count))
        
#define D_SSI_IO_TO_FRAG_32BIT(_dds_ptr, _frag_handle, _frag_offset, \
                               _io_handle, _io_offset, _byte_count) \
    mtok_32bit_pio_from_adapter( \
        _IO_PORT(_io_handle, _io_offset), \
        &(_frag_handle).data[_frag_offset], \
        (_byte_count))

#define D_SSI_FRAG_TO_IO_32BIT(_dds_ptr, _io_handle, _io_offset, \
                               _frag_handle, _frag_offset, _byte_count) \
    mtok_32bit_pio_to_adapter( \
        _IO_PORT(_io_handle, _io_offset), \
        &(_frag_handle).data[_frag_offset], \
        (_byte_count))
        
/*--------------------------------------------------------------------------*/
        
#define D_SSI_MMIO_TO_FRAG(_dds_ptr, _frag_handle, _frag_offset, \
                           _shared_handle, _shared_offset, _byte_count)
        
#define D_SSI_FRAG_TO_MMIO(_dds_ptr, _shared_handle, _shared_offset, \
                           _frag_handle, _frag_offset, _byte_count)
                           
#define D_SSI_SHARED_TO_FRAG(_dds_ptr, _frag_handle, _frag_offset, \
                             _shared_handle, _shared_offset, _byte_count) \
    memcpy_fromio( \
        (void *) &(_frag_handle).data[_frag_offset], \
        (void *) (_shared_handle) + (_shared_offset), \
        (_byte_count))
        
        
#define D_SSI_FRAG_TO_SHARED(_dds_ptr, _shared_handle, _shared_offset, \
                             _frag_handle, _frag_offset, _byte_count) \
    memcpy_fromio( \
        (void *) (_shared_handle) + (_shared_offset), \
        (void *) &(_frag_handle).data[_frag_offset], \
        (_byte_count))
                           
/*--------------------------------------------------------------------------*/
                           
#define D_SSI_FRAG_TO_FRAG(_dds_ptr, _dest_handle, _dest_offset, \
                           _source_handle, _source_offset, _byte_count) \
    memcpy( \
        (void *) &(_dest_handle).data[_dest_offset], \
        (void *) &(_source_handle).data[_source_offset], \
        (_byte_count))
                           
/*--------------------------------------------------------------------------*/
                       
#define D_SSI_DWORD_SHIFT_L(_dds_ptr, _dword, _bit_count) \
    ((_dword) << (_bit_count))

#define D_SSI_DWORD_SHIFT_R(_dds_ptr, _dword, _bit_count) \
    ((_dword) >> (_bit_count))
          
/*--------------------------------------------------------------------------*/

#define D_SSI_WAIT_AT_LEAST_MSEC(_dds_ptr, _msec) 	\
	{												\
	    int i;										\
        for (i = 0; i < (_msec) * 10; i++) 			\
                udelay((unsigned long) 100);		\
	}

#define D_SSI_WAIT_AT_LEAST_USEC(_dds_ptr, _usec) \
    udelay((unsigned long) (_usec))

/*--------------------------------------------------------------------------*/
    
#define D_SSI_MODULE_REGISTERED(_dds_ptr, _module, _scope)

/*--------------------------------------------------------------------------*/

#define D_SSI_CODE_OPEN(_dds_ptr, _type, _handle) \
    mtok_code_open(_type, _handle)
    
#define D_SSI_CODE_CLOSE(_dds_ptr, _handle)	\
    mtok_code_close(_handle)
    
#define D_SSI_CODE_READW(_dds_ptr, _handle) \
	mtok_code_readw(_handle)
    
/*--------------------------------------------------------------------------*/
    
#define D_SSI_ENABLE_DMA_CHANNEL(_dds_ptr, _dma_channel) \
    mtok_enable_isa_dma(_dma_channel)
    
#define D_SSI_DISABLE_DMA_CHANNEL(_dds_ptr, _dma_channel) \
    mtok_disable_isa_dma(_dma_channel)

#define D_SSI_COMPLETE_DMA_XFER(_dds_ptr, _frag_handle)

/*-------------------------------------------------------------------------*/

#define D_SSI_PROTECT(_dds_ptr, _target, _udids_ptr, _arg1, _arg2) \
    (*(_target))((_udids_ptr), (_arg1), (_arg2))
                  
                  
/*----------------------------------------------------------------------------
|
| Extra misc defines added during UDT development.
|
----------------------------------------------------------------------------*/

/* 
 * Compress the downloads. 
 */

#define UDT_DECOMPRESS_BIN_FILES

/* 
 * Cause the adapter to die if the host doesn't respond after a certain
 * timeout period. 
 */
   
#define UDT_USE_WATCHDOG

/* 
 * Allow 4 commands to be issued simultaneously. We need this because on
 * an MPX system there could be multiple IOCTLs issued at the same time. 
 */
   
#define UDT_DRIVER_REQUIRED_COMMAND_QUEUE_ENTRIES 4

/* 
 * We want to see all error strings. 
 */

#define INTERNAL_UDT_ERRORS_IN_ERROR_TABLE

/*
 * We want to be able to kill promiscuous mode on PCI3s.
 */

#ifndef UDT_NOPCI
 
#define UDI_KILL_PROMISCUOUS_MODE

#endif

/*----------------------------------------------------------------------------
|
| Include the configuration file.
|
----------------------------------------------------------------------------*/
 
/*
 * This file specifies support for cards and can be used to override
 * any of the default macros defined above.
 * The default macros defined above and support for a different range
 * of cards is only available if the source code to the UDT is available
 * to you.
 *
 * If the source code to the UDT is not available to you then only the
 * standard configurations can be used.  (see configs/ for the standard
 * configurations).
 */

#include "udtconfig.h"


/*----------------------------------------------------------------------------
|
| Include the master UDT header file.
|
----------------------------------------------------------------------------*/

#include "udt.h"


/*----------------------------------------------------------------------------
|
| Various functions needed to be implemented by the driver.
|
|---------------------------------------------------------------------------*/
#ifndef UDT_NOPCI

/*
 * PCI Configuration Space Access SSI Functions
 */

BYTE_T mtok_pci_readb(PCI_HANDLE_T bdf, unsigned offset);
WORD_T mtok_pci_readw(PCI_HANDLE_T bdf, unsigned offset);
DWORD_T mtok_pci_readd(PCI_HANDLE_T bdf, unsigned offset);
void mtok_pci_writeb(PCI_HANDLE_T bdf, unsigned offset, BYTE_T value);
void mtok_pci_writew(PCI_HANDLE_T bdf, unsigned offset, WORD_T value);
void mtok_pci_writed(PCI_HANDLE_T bdf, unsigned offset, DWORD_T value);

#endif /* UDT_NOPCI */

inline UINT_T
mtok_16bit_pio_from_adapter(IO_HANDLE_T io_port, char * buf_ptr, UINT_T count);

inline UINT_T
mtok_16bit_pio_to_adapter(IO_HANDLE_T io_port, char * buf_ptr, UINT_T count);

inline UINT_T
mtok_32bit_pio_from_adapter(IO_HANDLE_T io_port, char * buf_ptr, UINT_T count);

inline UINT_T
mtok_32bit_pio_to_adapter(IO_HANDLE_T io_port, char * buf_ptr, UINT_T count);

BOOLEAN_T mtok_code_open(CODE_TYPE_T type, CODE_HANDLE_T * handle_ptr);
WORD_T mtok_code_readw(CODE_HANDLE_T * handlep);
VOID_T mtok_code_close(CODE_HANDLE_T * handlep);


BOOLEAN_T mtok_enable_isa_dma(unsigned channel);
void mtok_disable_isa_dma(unsigned channel);

BOOLEAN_T mtok_pssi_register_io(    P_DRIVER_T ddsptr,
                                    IO_LOCATION_T io_base,
                                    UINT_T io_range,
                                    IO_HANDLE_T * io_handle);


BOOLEAN_T mtok_pssi_register_pci(   P_DRIVER_T dds_ptr,
                                    WORD_T      pci_number,
                                    PCLIENT_PCI_HANDLE_T pci_handle);

VOID_T poke(BYTE_T,STRING_T,BYTE_T,unsigned long);

WORD_T ssi_read_tuple(P_DRIVER_T dds_ptr,WORD_T tuple,VOID_T *buffer,int len);

/**** End of driver.h *******************************************************/


