/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/*****************************************************************************
*
* **   **  *****     ***   ****       *****
* **   ** **   **   ****  **  **     **   **
* **   ** **       ** **  **         **
* ******* **      **  **  ******     **
* **   ** **      ******* **  **     **
* **   ** **   **     **  **  **     **   **
* **   **  *****     ****  ****   **  *****
*
******************************************************************************
*
* HC46 serial device access code. Used by PCMCIA and PCI Presto.
* (PCI-1b not supported, ATP only supported if UDT_AT_SERIAL_EEPROM_CODE is
* defined) plus some Olicom adapters.
*
* Note that write code is required for Presto.
*
******************************************************************************
*
* Copyright (C) 1996-98 Madge Networks Ltd.
*
* COMPANY CONFIDENTIAL
*
******************************************************************************
*/



#define UDI_MODULE

#include "udt_driver.h"

#if defined ATULA_UDI         ||                                            \
    defined PCMCIA_UDI        ||                                            \
    defined PCI1B_UDI         ||                                            \
    defined PRESTO_UDI        ||                                            \
    defined KNOSSOS_UDI       ||                                            \
    defined OLICOM_PCI_UDI    ||                                            \
    defined OLICOM_EISA_UDI   ||                                            \
    defined OLICOM_ISA_UDI    ||                                            \
    defined OLICOM_ISAPNP_UDI






STATIC WORD_T UDT_INDIRECT
hc46_read_word(
    IN P_UDI_DATA_T    udids_ptr,
    BYTE_T             addr);

STATIC RETCODE_T UDT_INDIRECT
hc46_write_word(
    IN P_UDI_DATA_T    udids_ptr,
    BYTE_T             addr,
    WORD_T             data_word);




STATIC VOID_T
hc46_write_enable_seeprom(
    IN P_UDI_DATA_T    udids_ptr);

STATIC VOID_T
hc46_write_disable_seeprom(
    IN P_UDI_DATA_T    udids_ptr);

STATIC VOID_T
hc46_start(
    IN P_UDI_DATA_T    udids_ptr);

STATIC VOID_T
hc46_stop(
    IN P_UDI_DATA_T    udids_ptr);

STATIC BYTE_T
hc46_read_data(
    IN P_UDI_DATA_T    udids_ptr);

STATIC VOID_T
hc46_write_bit(
    INOUT P_UDI_DATA_T udids_ptr,
    BOOLEAN_T          set_bit,
    BYTE_T             mask);

STATIC VOID_T
hc46_twitch_clk(
    IN P_UDI_DATA_T    udids_ptr);

STATIC WORD_T
hc46_serial_read_bit_string(
    IN P_UDI_DATA_T    udids_ptr,
    int                num_bits);

STATIC VOID_T
hc46_serial_write_bit_string(
    IN P_UDI_DATA_T    udids_ptr,
    int                num_bits,
    WORD_T             data_bits);

STATIC VOID_T
hc46_serial_send_cmd_addr(
    IN P_UDI_DATA_T    udids_ptr,
    BYTE_T             cmd_addr);

DECLARE_MODULE_START_FUNCTION(hc46_module_start)
DECLARE_MODULE_END_FUNCTION(hc46_end)



RETCODE_T UDT_INTER_MODULE
hc46_register(
    IN P_UDI_DATA_T udids_ptr)
{
    POKE(
        udids_ptr,
        PO_CHAN_CONFIG,
        PO_LEV_USER,
        PO_BROWN,
        "REGISTERING HC46");

    

    SSI_MODULE_REGISTERED(
        udids_ptr,
        hc46_register,
        hc46_module_start,
        hc46_end,
        MOD_SCOPE_RUNTIME);

    

    COMMON_RES_FROM_UDIDS(udids_ptr).hc46_read_word  = hc46_read_word;
    COMMON_RES_FROM_UDIDS(udids_ptr).hc46_write_word = hc46_write_word;

    

    return SUCCESS;
}

BUILD_MODULE_START_FUNCTION(hc46_module_start)




STATIC WORD_T UDT_INDIRECT
hc46_read_word(
    IN P_UDI_DATA_T    udids_ptr,
    BYTE_T             addr)
{
    WORD_T data_word;

    

    GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_remembered_bits = 0;

    

    hc46_start(udids_ptr);

    

    addr &= 0x3f;
    addr |= 0x80;

    

    hc46_serial_send_cmd_addr(
        udids_ptr,
        addr);

    

    data_word = hc46_serial_read_bit_string(
                    udids_ptr,
                    16);

    

    hc46_stop(udids_ptr);

    return data_word;
}





STATIC RETCODE_T UDT_INDIRECT
hc46_write_word(
    IN P_UDI_DATA_T    udids_ptr,
    BYTE_T             addr,
    WORD_T             data_word)
{
    RETCODE_T rc = FAILURE;
    UINT_T    i;

    

    GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_remembered_bits = 0;

    

    hc46_write_enable_seeprom(udids_ptr);

    

    hc46_start(udids_ptr);

    

    addr &= 0x3f;
    addr |= 0x40;

    

    hc46_serial_send_cmd_addr(
        udids_ptr,
        addr);

    

    hc46_serial_write_bit_string(
        udids_ptr,
        16,
        data_word);

    

    hc46_stop(udids_ptr);

    hc46_start(udids_ptr);

    

    for (i=0;i<4;i++)
    {
        hc46_twitch_clk(udids_ptr);
    }

    

    for (i=0;i<40;i++)
    {
        if (hc46_read_data(udids_ptr) & 0x0001)
        {
            rc = SUCCESS;
            break;
        }

        hc46_twitch_clk(udids_ptr);
    }

    

    hc46_stop(udids_ptr);

    

    hc46_write_disable_seeprom(udids_ptr);

    return rc;
}




STATIC VOID_T
hc46_write_enable_seeprom(
    IN P_UDI_DATA_T    udids_ptr)
{
    

    hc46_start(udids_ptr);

    

    hc46_serial_send_cmd_addr(udids_ptr, (BYTE_T)0x30);

    

    hc46_stop(udids_ptr);
}




STATIC VOID_T
hc46_write_disable_seeprom(
    IN P_UDI_DATA_T    udids_ptr)
{

    

    hc46_start(udids_ptr);

    

    hc46_serial_send_cmd_addr(udids_ptr, (BYTE_T)0x00);

    

    hc46_stop(udids_ptr);
}





STATIC VOID_T
hc46_start(
    IN P_UDI_DATA_T    udids_ptr)
{
    

    hc46_write_bit(
        udids_ptr,
        FALSE,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_DI_mask);

    

    hc46_write_bit(
        udids_ptr,
        TRUE,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_CS_mask);

}




STATIC VOID_T
hc46_stop(
    IN P_UDI_DATA_T    udids_ptr)
{
    

    hc46_write_bit(
        udids_ptr,
        FALSE,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_DI_mask);

    

    hc46_write_bit(
        udids_ptr,
        FALSE,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_CS_mask);

    hc46_twitch_clk(udids_ptr);
}



STATIC BYTE_T
hc46_read_data(
    IN P_UDI_DATA_T    udids_ptr)
{
    BYTE_T contents;            

    contents = INB(
                   udids_ptr,
                   0,
                   GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).
                       hc46_eeprom_DO_IO_offset);

    

    contents &= GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_DO_mask;

    

    contents >>= GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_DO_posn;

    contents &= 0x01;

    return contents;
}




STATIC VOID_T
hc46_write_bit(
    INOUT P_UDI_DATA_T udids_ptr,
    BOOLEAN_T          set_bit,
    BYTE_T             mask)

{
    BYTE_T bit_vals;

    

    

    bit_vals = INB(
                   udids_ptr,
                   0,
                   GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).
                       hc46_eeprom_IO_offset);

    bit_vals &=
        ~ GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_bits_to_remember;

    bit_vals |=
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_remembered_bits;

    

    bit_vals &= ~mask;

    if(set_bit)                   
    {
        bit_vals |= mask;  
    }

    bit_vals |=
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_bits_to_set;

    bit_vals &=
        ~ GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_bits_to_clear;

    GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_remembered_bits =
        (BYTE_T)(bit_vals &
          GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_bits_to_remember);

    

    OUTB(
        udids_ptr,
        0,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_eeprom_IO_offset,
        bit_vals);

    

    SSI_WAIT_AT_LEAST_USEC(udids_ptr, 200);

    

    INB(
        udids_ptr,
        0,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_eeprom_IO_offset);
}




STATIC VOID_T
hc46_twitch_clk(
    IN P_UDI_DATA_T    udids_ptr)
{
    hc46_write_bit(
        udids_ptr,
        TRUE,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_SK_mask);

    hc46_write_bit(
        udids_ptr,
        FALSE,
        GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_SK_mask);
}




STATIC WORD_T
hc46_serial_read_bit_string(
    IN P_UDI_DATA_T    udids_ptr,
    int                num_bits)

{
    BYTE_T read_bit;         
    WORD_T read_string = 0;  
    int i;

    for(i=num_bits; i>0; i--)
    {
        hc46_twitch_clk(udids_ptr);
        read_bit = hc46_read_data(udids_ptr);
        read_string <<= 1;
        read_string |= (WORD_T) read_bit;
    }

    return read_string;
}



STATIC VOID_T
hc46_serial_write_bit_string(
    IN P_UDI_DATA_T    udids_ptr,
    int                num_bits,
    WORD_T             data_bits)

{
    int i;
    for(i=num_bits; i>0; i--)
    {
        if((data_bits & 0x8000))
        {
            

            hc46_write_bit(
                udids_ptr,
                TRUE,
                GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_DI_mask);
        }
        else
        {
            

            hc46_write_bit(
                udids_ptr,
                FALSE,
                GENERAL_RES_INFO_FROM_UDIDS(udids_ptr).hc46_DI_mask);
        }

        hc46_twitch_clk(udids_ptr);

        data_bits <<= 1;
    }
}




STATIC VOID_T
hc46_serial_send_cmd_addr(
    IN P_UDI_DATA_T    udids_ptr,
    BYTE_T             cmd_addr)
{
    WORD_T command;

    

    command = (WORD_T) cmd_addr;
    command <<= 7;
    command |= 0x8000; 

    

    hc46_twitch_clk(udids_ptr);

    hc46_serial_write_bit_string(udids_ptr, 9, command);
}



BUILD_MODULE_END_FUNCTION(hc46_end)



#endif




