/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/****************************************************************************
*
* madgetr.c 
*
* Copyright (c) Madge Networks Ltd
*
*    
****************************************************************************/
/* Please refer to the LICENSE file supplied with this source code */

#include <linux/config.h>
#if defined(CONFIG_SMP) && !defined(__SMP__)
#define __SMP__
#endif

#ifdef MODULE
#if !defined(MODVERSIONS) && defined(CONFIG_MODVERSIONS)
#define MODVERSIONS
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#endif /* MODULE */

#ifndef EMBEDDED_MICRO_CODE

/* 
 * If the adapter micro code is not embedded in the driver we use syscalls
 * to open the download image file.
 */

#define __KERNEL_SYSCALLS__
#endif

#include <linux/autoconf.h>

#include <linux/types.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/fs.h>
#include <linux/unistd.h>
#include <linux/proc_fs.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/irq.h>

#include <linux/netdevice.h>
#include <linux/trdevice.h>

#ifndef MODULE
#include <linux/version.h>
#endif

#include <asm/byteorder.h>

#include "madgetr.h"

#ifdef DEBUG
#include "mpoke.h"
#endif

#if LINUX_VERSION_CODE > 0x20109
#include <linux/init.h>
#include <asm/uaccess.h>
#endif


#if !defined(MODULE) && !defined(EMBEDDED_MICRO_CODE)
#error You must embed the adapter micro code if not building as a module
#endif

static const char * version = 
    "Madge Token Ring Linux Driver v" MTOK_VERSION
#ifdef __SMP__
    " SMP"
#endif
    "\nCopyright (c) 1999 Madge Networks Ltd\n";

char *CardNames[] = CARD_NAME_STRINGS;

#if defined(CONFIG_PROC_FS) && defined(CONFIG_MADGE_BACK_COMPAT)

static int mtok_proc_info(  char *buffer, char **start, off_t offset,
                            int length, int dummy);

static struct proc_dir_entry proc_net_mtok = {
            0, 4, MTOK_DRIVER_NAME,
            S_IFREG | S_IRUGO,1,0,0,0,
            &proc_net_inode_operations,
            mtok_proc_info,
            NULL,
            NULL,NULL,NULL,
            NULL
            };
#endif

/* ** DRIVER ENTRY POINTS **
 * Externals only accessed through function pointers can be static. Others
 * must not be static otherwise they will not appear in the module symbol
 * table. mtok_close and mtok_isr are directly referenced by the Cardbus
 * and PCMCIA modules so must not be declared static.
 */

static int  mtok_open(LINUXDEV * dev);
       int  mtok_close(LINUXDEV * dev);
void mtok_isr IRQ_ARGS(int irq,void * dev_id,struct pt_regs * regs);
static int  mtok_transmit(struct sk_buff *skb,LINUXDEV *dev);
static struct net_device_stats *mtok_get_stats(LINUXDEV  * dev);
static void mtok_set_multicast_list(LINUXDEV * dev);
static int  mtok_set_mac_address(LINUXDEV * dev,void *p);
static int  mtok_change_mtu(LINUXDEV * dev,int new_mtu);
static int  mtok_ioctl(LINUXDEV *dev, struct ifreq *ifr, int cmd);

/* 
 * Accessed externally by the CardBus and PCMCIA helper drivers
 */

void mtok_add_dev(LINUXDEV *dev);
void mtok_remove_dev(LINUXDEV *dev);
LINUXDEV *mtok_get_dev_by_name(char * name);

/* Initialise, add us into list, display details */
LINUXDEV * mtok_probe1(LINUXDEV *dev,
                       ADAPTER_ID_T * padapter_id,
                       int irq,
                       int maxframesize,
                       int txslots,int rxslots,
                       int ringspeed,int dtr,
                       void * usrdata);

/* Deinitialise */
void mtok_cleanup(MTOK_ADAPTER * adap);

/* Probe Function */
int mtok_pci_probe(LINUXDEV *dev);

/* Adapter Initialisation */

static int mtok_update_params(MTOK_ADAPTER * adap);

static void mtok_background(unsigned long context);
static RETCODE_T mtok_await_completion( MTOK_ADAPTER * adap,
                                        MTOK_COMPLETION * comp,int timeout);
static void mtok_show_error(MTOK_ADAPTER * adap,RETCODE_T retc);

/* UDT Callback Entry Points */
static void mtok_rx_complete(P_DRIVER_T context,P_FRAME_T  frame);

static void mtok_tx_complete(P_DRIVER_T context,BYTE_T fs,
                             BOOLEAN_T status,UINT_T handle);

static void mtok_transfer_complete( P_DRIVER_T context,
                                    P_FRAME_T frame,UINT_T handle);

static void mtok_command_complete(  P_DRIVER_T context,
                                    UDT_ARGUMENT_T * argument,
                                    RETCODE_T response);

static void mtok_event_handler(     P_DRIVER_T context,
                                    P_VOID_T argument,
                                    ADAPTER_EVENT_T event);



#ifdef FIX_SR /* Older Linux kernels mangle source routing. */
inline void mtok_fix_rx_sr(struct sk_buff *skb);
inline struct sk_buff * mtok_fix_tx_sr(struct sk_buff *skb);
#endif


/* 
 * Private Functions 
 */

static int mtok_alloc_tx_descriptors(LINUXDEV * dev);
static int mtok_alloc_rx_descriptors(LINUXDEV * dev);
static void mtok_free_tx_descriptors(LINUXDEV * dev);
static void mtok_free_rx_descriptors(LINUXDEV * dev);
static int mtok_alloc_rx_skb(LINUXDEV * dev,FRAME_T *frame,int flags);
static void mtok_hardware_send_frame(MTOK_ADAPTER *Adapter,struct sk_buff *skb);
static void mtok_set_promiscuous_mode(MTOK_ADAPTER *adap,PROMISCUOUS_LEVEL_T mode);

struct dev_list_node {
    struct dev_list_node * next;
    LINUXDEV * dev;
};

#ifdef DEBUG 
#if BITS_PER_LONG == 64

#define QPOKEP(c,str,val) \
    { QPOKED(c,str,(DWORD_T)(((unsigned long)val) >> 32)); \
      QPOKED(c,"",(DWORD_T)(((unsigned long)val) & 0xFFFFFFFF)); }


#define QPOKEDD(c,str,val) QPOKEP(c,str,val)

#else

#define QPOKEP(c,str,val) QPOKED(c,str,val)
#define QPOKEDD(c,str,val) QPOKED(c,str,val)

#endif
#endif /* DEBUG */
static struct dev_list_node * mtok_dev_list = NULL;

/* Product Identification Strings */
#ifdef MVER_STRINGS
char mtok_mver_title[] = "VeRsIoN=" MTOK_PRODUCT_NAME; 
char mtok_mver_copyright[] = "VeRsIoN=" MTOK_PRODUCT_COPYRIGHT;
char mtok_mver_platform[] = "VeRsIoN=PLATFORM: " MTOK_PRODUCT_PLATFORM;
char mtok_mver_version[61] = "VeRsIoN=VERSION: v" MTOK_VERSION;
char mtok_mver_product[] = "VeRsIoN=PRODUCT-ID: " MTOK_PRODUCT_ID;
char mtok_crcp[] = "MaDgE=XXXX";
#endif

#ifdef OLICOM_PCMCIA_UDI
/* HACK job for Olicom PCMCIA cards */
struct oli_pcmcia {
    RS_TUPLE_T *  olicom_rs_tuple;
    BIA_TUPLE_T * olicom_bia_tuple;
};
#endif /* OLICOM_PCMCIA_UDI */

/*
 * MODULE PARAMETERS.
 *
 * NB. MTOK_MAX_ADAPTERS just refers the maximum number of parameters.
 * The driver will actually load on to an infinite(!!) number of Madge
 * adapters.
 */
int maxframesize[MTOK_MAX_ADAPTERS] = { 0, };
int txslots[MTOK_MAX_ADAPTERS] = {0, };
int rxslots[MTOK_MAX_ADAPTERS] = {0, };
int ringspeed[MTOK_MAX_ADAPTERS] = {0, };
int dtr[MTOK_MAX_ADAPTERS] = {0, };

#ifdef DEBUG
int mp; /* Our poke out handle */
#endif

#ifdef MODULE
int pcmcia;
#if LINUX_VERSION_CODE > 0x20115
MODULE_AUTHOR("Madge Networks Ltd (www.madge.com)");
MODULE_DESCRIPTION("Madge Token Ring Adapter driver v" MTOK_VERSION);
MODULE_PARM(ringspeed,          "1-" __MODULE_STRING(MTOK_MAX_ADAPTERS) "i");
MODULE_PARM(maxframesize,       "1-" __MODULE_STRING(MTOK_MAX_ADAPTERS) "i");
MODULE_PARM(txslots,            "1-" __MODULE_STRING(MTOK_MAX_ADAPTERS) "i");
MODULE_PARM(rxslots,            "1-" __MODULE_STRING(MTOK_MAX_ADAPTERS) "i");
MODULE_PARM(dtr,                "1-" __MODULE_STRING(MTOK_MAX_ADAPTERS) "i");
MODULE_PARM(pcmcia,             "i");
MODULE_LICENSE("Madge Open source");
#endif /* LINUX_VERSION_CODE > 0x20115 */

int init_module(void)
{
    int retval;
#ifdef PCMCIA_SET
	pcmcia = 1;	
#endif

#ifdef DEBUG
    mp = mpoke_register("mtok");
#endif

    printk(version);

#ifdef DEBUG 
    QPOKED(PO_GREEN,"int=",sizeof(int));
    QPOKED(PO_YELLOW,"long=",sizeof(long));    
#endif

    retval = mtok_pci_probe(0);
    if (retval && pcmcia==0) {
        printk(KERN_NOTICE MTOK_DRIVER_NAME ": No Madge PCI Cards found.\n");
        return retval;
    }

    if (pcmcia) {
        printk(KERN_NOTICE MTOK_DRIVER_NAME 
                ": Staying resident for PCMCIA and CardBus adapters.\n");
    }

#if defined(CONFIG_PROC_FS) && defined(CONFIG_MADGE_BACK_COMPAT)
    PROC_NET_REGISTER(&proc_net_mtok);
#endif

	return 0;	

}

void cleanup_module(void)
{
    LINUXDEV * dev;
    while(mtok_dev_list)
    {
        dev = mtok_dev_list->dev;
        /* Need to free resources associated with this adapter */
        mtok_cleanup(dev->priv);
        mtok_remove_dev(dev);
        kfree(dev);
    }
#if defined(CONFIG_PROC_FS) && defined(CONFIG_MADGE_BACK_COMPAT)
    PROC_NET_UNREGISTER(&proc_net_mtok);
#endif

#ifdef DEBUG
    mpoke_deregister(mp);
#endif

}

#else /* Not a module */
int mtok_probe(LINUXDEV * dev)
{
    static int scanned = 0;
    /* We detect all adapters in one go so only need to probe once */
    if (scanned++)
        return -ENODEV;
    printk(version);
#if defined(CONFIG_PROC_FS) && defined(CONFIG_MADGE_BACK_COMPAT)
    PROC_NET_REGISTER(&proc_net_mtok);
#endif
    return mtok_pci_probe(dev);
}
#endif /* MODULE */

/* Maintain a list of all devices the driver is currently attached to. */

void mtok_add_dev(LINUXDEV *dev)
{
    struct dev_list_node * node = kmalloc(sizeof(struct dev_list_node),GFP_KERNEL);
    node->next = mtok_dev_list;
    mtok_dev_list = node;
    node->dev = dev;
}

void mtok_remove_dev(LINUXDEV *dev)
{
    struct dev_list_node * node = mtok_dev_list;
    struct dev_list_node ** last = &mtok_dev_list;

    if (!dev)
    {
#ifdef DEBUG 
        QPOKE(PO_RED,"dev==NULL");
#endif
        return;
    }
    
    while(node)
    {
        if (node->dev == dev)
        {
            *last = node->next;
            kfree(node);
            break;
        }
        last = &(node->next);
        node = node->next;
    }
}

LINUXDEV *mtok_get_dev_by_name(char * name)
{
    struct dev_list_node * node = mtok_dev_list;
    while(node)
    {
        if (strcmp(node->dev->name,name) == 0)
            return node->dev;
        node = node->next;
    }
    return NULL;
}

char * mtok_ringspeed_to_string(int rs)
{
    char * speed;
    switch (rs)
    {
        case RS_4M: 
            speed = "4 MBit/s";
            break;
        case RS_16M: 
            speed = "16 MBit/s";
            break;
        case RS_100M:
            speed = "100 MBit/s";
            break;
        case RS_AUTO:
            speed = "Auto";
            break;
        case RS_ADAPTER:
            speed = "Adapter";
            break;
        default: 
            speed = "Unknown speed";
            break;
    }
    return speed;
}

static char * mtok_bia_to_string(unsigned char *p,char *buffer)
{
    sprintf(buffer,"%02X%02X%02X%02X%02X%02X",p[0],p[1],p[2],p[3],p[4],p[5]);
    return buffer;
}

static void mtok_display_details(LINUXDEV *dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dev->priv;
    char BIABuf[14];
    printk(KERN_NOTICE "%s: %s at %lx IRQ %d",
                            dev->name,
                            CardNames[adap->subtype],
                            dev->base_addr,dev->irq);
                            
    if (adap->busdevfn)
    {
        /* 
         * If this is a PCI card then print PCI info.
         * The only non PCI cards supported are PCMCIA
         */
        printk(" PCI Bus %d DevFn %X",(adap->busdevfn & 0xFF00) >> 8,
                                      (adap->busdevfn & 0x00FF));
    }
    
    printk("\n");
        
    printk(KERN_NOTICE "%s: BIA: %s Force Ring Speed: %s\n",
                            dev->name,mtok_bia_to_string(adap->bia.byte,BIABuf),
                            mtok_ringspeed_to_string(adap->ringspeed));

    printk(KERN_NOTICE "%s: Tx Slots: %d Rx Slots: %d MaxFrameLen: %d\n",
                            dev->name,
                            adap->txslots,adap->rxslots,adap->max_frame_size);
}

typedef struct {
    u16                 vendorid;
    u16                 deviceid;
    u8                  revision;
    u16                 subsystemid;
    ADAPTER_TYPE_T      adapter_type;
    ADAPTER_SUBTYPE_T   adapter_subtype;
} mtok_card_id;

static mtok_card_id pcicards[] = {
#ifdef PCIBM2_UDI       /*** Smart 16/4 PCI BM2 ***/
    {   
        PCI_MADGE_VENDOR_ID, PCIBM2_DEVICE_ID, 
        0xFF, 0xFFFF, 
        ADAPTER_TYPE_PCIBM2, ADAPTER_SUBTYPE_PCIBM2
    },
#endif
#ifdef PRESTO_UDI       /*** Presto PCI ***/
    {   
        PCI_MADGE_VENDOR_ID, PRESTO_DEVICE_ID, 
        0xFF, PRESTO_SUBSYSTEM_ID, 
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_PRESTO
    },
#endif
#ifdef PRESTO2_UDI      /*** Presto Plus PCI ***/
    {   
        PCI_MADGE_VENDOR_ID, K2_DEVICE_ID,
        PRESTO2_PCI_REVISION, PRESTO_SUBSYSTEM_ID,
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_PRESTO2
    },
#endif
#ifdef PRESTO2000_UDI   /*** Presto 2000 PCI ***/
    {   
        PCI_MADGE_VENDOR_ID, K2_DEVICE_ID,
        PRESTO2000_PCI_REVISION, PRESTO_SUBSYSTEM_ID,
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_PRESTO2000
    },
#endif
#ifdef PCI3_UDI         /*** Smart 16/4 PCI Mk 3 ***/
    {   
        PCI_MADGE_VENDOR_ID, K2_DEVICE_ID, 
        0xFF, PCI3_SUBSYSTEM_ID,
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_PCI3
    },
#endif
#ifdef PCI4_UDI         /*** 100/16/4 PCI ***/  
    {   
        PCI_MADGE_VENDOR_ID, PCI4_DEVICE_ID,
        0xFF, 0xFFFF,
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_PCI4
    },
                        /*** 100/16/4 PCI Mk 2 ***/
    {   
        PCI_MADGE_VENDOR_ID, PCI4_MK2_DEVICE_ID,
        0xFF, 0xFFFF,
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_PCI4_MK2
    },
#endif
#ifdef OLICOM_3140V2_UDI  /* 16/4 PCI ringnode */
    {   
        PCI_MADGE_VENDOR_ID, OLICOM_3140V2_DEVICE_ID,
        0xFF, 0xFFFF,
        ADAPTER_TYPE_KNOSSOS, ADAPTER_SUBTYPE_OLICOM_PCI_3140_V2
    },
#endif
#ifdef OLICOM_3139_UDI  /*** RapidFire 3139 16/4 PCI ***/
    {
        PCI_OLICOM_VENDOR_ID, OLICOM_PCI_3139_DEVICE_ID,
        OLICOM_PCI_3139_PCI_REVISION,0xFFFF,
        ADAPTER_TYPE_HAWKEYE_OLICOM_PCI,
        ADAPTER_SUBTYPE_OLICOM_PCI_3139
    },
#endif
#ifdef OLICOM_314X_UDI  /*** RapidFire 3140 16/4 PCI ***/
    {
        PCI_OLICOM_VENDOR_ID, OLICOM_PCI_3140_DEVICE_ID,
        OLICOM_PCI_3140_PCI_REVISION,0xFFFF,
        ADAPTER_TYPE_HAWKEYE_OLICOM_PCI,
        ADAPTER_SUBTYPE_OLICOM_PCI_3140
    },
    {                   /*** RapidFire 3141 16/4 PCI (Fiber) ***/
        PCI_OLICOM_VENDOR_ID, OLICOM_PCI_3141_DEVICE_ID,
        0xFF,0xFFFF,
        ADAPTER_TYPE_HAWKEYE_OLICOM_PCI,
        ADAPTER_SUBTYPE_OLICOM_PCI_3141
    },
#endif
#ifdef OLICOM_3540_UDI  /*** RapidFire 3540 100/16/4 PCI ***/
    {
        PCI_OLICOM_VENDOR_ID, OLICOM_PCI_3540_DEVICE_ID,
        0xFF,0xFFFF,
        ADAPTER_TYPE_BULLSEYE_OLICOM_PCI,
        ADAPTER_SUBTYPE_OLICOM_PCI_3540
    },
#endif
#ifdef OLICOM_PCI_UDI  /*** Olicom Token-Ring 16/4 PCI (OC-3136) ***/
    {
        PCI_OLICOM_VENDOR_ID, OLICOM_PCI_3136_DEVICE_ID,
        OLICOM_PCI_3136_1_PCI_REVISION,0xFFFF,
        ADAPTER_TYPE_OLICOM_PCI,
        ADAPTER_SUBTYPE_OLICOM_PCI_3136
    },
    {                  /*** Olicom Token-Ring 16/4 PCI/II (OC-3137) ***/
        PCI_OLICOM_VENDOR_ID, OLICOM_PCI_3137_DEVICE_ID,
        OLICOM_PCI_3137_PCI_REVISION,0xFFFF,
        ADAPTER_TYPE_OLICOM_PCI,
        ADAPTER_SUBTYPE_OLICOM_PCI_3137
    },
#endif
};

#define NUM_ADAPTERS (sizeof(pcicards)/sizeof(mtok_card_id))
/*
 * Convert a PCI Vendor, Device, Sub Vendor, Sub Device and Revision into a UDT
 * type and subtype.
 */
int mtok_pci_to_udt_type(int ven,int dev,int sven,int sdev,int rev,unsigned int * type,unsigned int *subtype)
{
    int i;
    int found = 0;
    for(i=0;i<NUM_ADAPTERS;i++)
    {

        if (ven != pcicards[i].vendorid) 
            continue;
                    
        if (dev != pcicards[i].deviceid)
            continue;
            
        /* Check Sub System ID if its relevent */
            
        if (pcicards[i].subsystemid != 0xffff &&
            pcicards[i].subsystemid != sdev)
                continue;
                
        /*
         * If the card looks like an Abyss, it may be one of the early
         * CardBus cards which erroneously had a device ID of 0002h (the
         * same as the Abyss). They can be distinguished by looking at
         * the subsystem adapter ID, which is correctly 0006h (as the
         * device ID should have been).
         */
        if (pcicards[i].adapter_subtype == ADAPTER_SUBTYPE_PCIBM2 &&
            sdev == CARDBUS_SUBSYSTEM_ID)
            continue;

        if (pcicards[i].revision != 0xFF &&
            pcicards[i].revision != rev)
            continue;

        *subtype = pcicards[i].adapter_subtype;
        *type = pcicards[i].adapter_type;
        found = 1;
        break;
    }
    return found;
}

#ifndef OLD_PCI
/*
 * Returns: 0 if successfully found and initialised one or more Madge PCI cards
 *          Otherwise returns -ENODEV
 */
int mtok_pci_probe(LINUXDEV *dev)
{
    int found = 0,card_idx=0;
    struct pci_dev * pdev = NULL;

#ifdef DEBUG 
    QPOKE(PO_GREEN,"[pci_probe");
#endif

    if (!pci_present())
        return -ENODEV;

    while ((pdev = pci_find_class(PCI_CLASS_NETWORK_TOKEN_RING<<8,pdev))!=NULL)
    {
        u16 ssysid;
        u8  rev;
        long io_base;
        ADAPTER_ID_T    adapter_id;
        
        pci_read_config_word(pdev,PCI_SUBSYSTEM_ID,&ssysid);
        pci_read_config_byte(pdev,PCI_REVISION_ID,&rev);

        if (!mtok_pci_to_udt_type(  pdev->vendor,pdev->device,
                                    0,ssysid,rev,
                                    &adapter_id.adapter_type,&adapter_id.adapter_subtype) )
            continue;

        io_base = MTOK_PCI_IO_BASE(pdev);

#ifdef DEBUG 
        QPOKEDD(PO_CYAN,"IOBASE:",io_base);
        QPOKEDD(PO_YELLOW,"IRQ:",pdev->irq);
#endif
        
        if (check_region(io_base,4))
        {
            printk(KERN_WARNING MTOK_DRIVER_NAME ": Adapter at IO %lX already in use\n",io_base);
            continue;
        }

        /* Store the low 32 bits of IO base address in the UDT adapter ID structure */
        adapter_id.io_base = (unsigned int)(io_base & 0xFFFFFFFF);
        adapter_id.pci_number = (((int)pdev->bus->number) << 8) | (pdev->devfn);

        /* Only have space for MTOK_MAX_ADAPTERS parameters but can attach driver to more adapters */

        if (card_idx < MTOK_MAX_ADAPTERS)
            dev = mtok_probe1(dev,&adapter_id,pdev->irq,
                                maxframesize[card_idx],
                                txslots[card_idx],rxslots[card_idx],
                                ringspeed[card_idx],dtr[card_idx],NULL);
        else
            dev = mtok_probe1(dev,&adapter_id,pdev->irq,0,0,0,0,0,NULL);

            
        card_idx++; 

        if (!dev)
        {
            printk(KERN_ERR MTOK_DRIVER_NAME ": Probe of PCI card at %lx failed\n",io_base);
            continue;
        }
        else
        {
            dev = NULL;
            found++;
        }
    }
#ifdef DEBUG 
    QPOKE(PO_GREEN,"]");
#endif
    return found ? 0 : -ENODEV;
}
#else /* OLD_PCI */
#include <linux/bios32.h>
int mtok_pci_probe(LINUXDEV *dev)
{
    int found = 0,card_idx=0;
    int idx = 0;
    u8 bus;
    u8 devfn;
    
    if (!pcibios_present())
        return -ENODEV;

    while (pcibios_find_class(PCI_CLASS_NETWORK_TOKEN_RING<<8,idx,&bus,&devfn)
            != PCIBIOS_DEVICE_NOT_FOUND)
    {
        u16 pciven;
        u16 pcidev;     
        u16 ssysid;
        u8  rev;
        u32 io_base;

        u8 irq;
        ADAPTER_ID_T    adapter_id;

        idx++;
        
        pcibios_read_config_word(bus,devfn,PCI_VENDOR_ID,&pciven);
        pcibios_read_config_word(bus,devfn,PCI_DEVICE_ID,&pcidev);

        /* PCI_SUBSYSTEM_ID is defined incorrectly to 0x2c for 2.0.x kernels */
        pcibios_read_config_word(bus,devfn,0x2e /* PCI_SUBSYSTEM_ID */,&ssysid);
        pcibios_read_config_byte(bus,devfn,PCI_REVISION_ID,&rev);

        if (!mtok_pci_to_udt_type(  pciven,pcidev,
                                    0,ssysid,rev,
                                    &adapter_id.adapter_type,&adapter_id.adapter_subtype) )
            continue;

        pcibios_read_config_dword(bus,devfn,PCI_BASE_ADDRESS_0,&io_base);
        io_base &= PCI_BASE_ADDRESS_IO_MASK;

        if (check_region(io_base,4))
        {
            printk(KERN_WARNING MTOK_DRIVER_NAME ": Adapter at IO %X already in use\n",io_base);
            continue;
        }

        adapter_id.io_base = io_base;
        adapter_id.pci_number = ((u16)bus << 8) | devfn;

        pcibios_read_config_byte(bus,devfn,PCI_INTERRUPT_LINE,&irq);
        
        if (card_idx < MTOK_MAX_ADAPTERS)
            dev = mtok_probe1(dev,&adapter_id,irq,
                                maxframesize[card_idx],
                                txslots[card_idx],rxslots[card_idx],
                                ringspeed[card_idx],dtr[card_idx],NULL);
        else
            dev = mtok_probe1(dev,&adapter_id,irq,0,0,0,0,0,NULL);
            
        card_idx++; 

        if (!dev)
        {
            printk(KERN_ERR MTOK_DRIVER_NAME ": Probe of PCI card at %x failed\n",io_base);
        }
        else
        {
            dev = NULL;
            found++;
        }
    }
    return found ? 0 : -ENODEV;
}
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,16)
static int mtok_dummy_probe(LINUXDEV * dev)
{
    return 0;
}

static LINUXDEV * init_trdev(LINUXDEV * dev,int size) 
{
    if (!dev) {
        dev = kmalloc(sizeof(LINUXDEV),GFP_KERNEL);
        if (!dev)
            return NULL;
        memset(dev,0,sizeof(LINUXDEV));
    }

    if (dev->name == NULL) {
        int num;
        dev->name = kmalloc(8,GFP_KERNEL);
        num = 0;
        do {
            sprintf(dev->name,"tr%d",num);
            num++;
        } while(dev_get(dev->name) != NULL);
    }
    
    tr_setup(dev);

    dev->init = mtok_dummy_probe;
    
    register_netdev(dev);
    return dev;
}
#endif

LINUXDEV * mtok_probe1(LINUXDEV *dev,ADAPTER_ID_T * padapter_id,int irq,
                            int maxframesize,int txslots,int rxslots,
                            int ringspeed,int dtr,
                            void *usrdata)
{
    MTOK_ADAPTER * adap;
    MTOK_COMPLETION comp;
    UDT_ARGUMENT_T udt_arg;
    RETCODE_T retc;
    int i;
    BOOLEAN_T done = FALSE;

    dev = init_trdev(dev,0);
    
    if (!dev) {
        printk(KERN_ERR MTOK_DRIVER_NAME "init_trdev failed\n");
        return NULL;
    }

    dev->base_addr = padapter_id->io_base; /* TODO: Needs changing */
    dev->irq = irq; /* TODO */

    dev->priv = kmalloc(sizeof(MTOK_ADAPTER),GFP_KERNEL);
    memset(dev->priv,0,sizeof(MTOK_ADAPTER));
    adap = (MTOK_ADAPTER *)dev->priv;
    adap->dev = dev;
    
    adap->busdevfn = padapter_id->pci_number;
    adap->subtype = padapter_id->adapter_subtype;
    adap->type = padapter_id->adapter_type;
    
    /* Fill in module parameters for this adapter */
    adap->max_frame_size = maxframesize;
    adap->txslots = txslots;
    adap->rxslots = rxslots;
    adap->ringspeed = ringspeed;
    adap->dtr = dtr;

#ifdef OLICOM_PCMCIA_UDI
    if (padapter_id->adapter_type == ADAPTER_TYPE_OLICOM_PCMCIA)
    {
        struct oli_pcmcia * odata = (struct oli_pcmcia *)usrdata;
        if (!usrdata)
        {
            kfree(dev->priv);
            return NULL;
        }
        adap->olicom_rs_tuple = (u8 *)odata->olicom_rs_tuple;
        adap->olicom_bia_tuple = (u8 *)odata->olicom_bia_tuple;
    }
#endif    

    /*
     * Initialise our mutex.
     */

    MTOK_INIT_MUTEX(&adap->isr_lock);

	MTOK_INIT_MUTEX(&adap->tx_lock);

    /* Tell the UDMI to create an instance of the UDI for this adapter. */

    if ((retc = UDMI_REGISTER_UDI(
                    padapter_id,
                    (P_DRIVER_T) adap,
                    &adap->udi_data,
                    &adap->udi_init_data,
                    mtok_event_handler,
                    NULL)) != SUCCESS)
    {
        mtok_show_error(adap, retc); 
        mtok_cleanup(adap);
        return NULL;
    }

    /*
     * Configure the adapter and allocate resources. To do this we repeatly
     * call the UDI's UDI_GET_PARAMETERS() function to find out what 
     * parameters need setting and/or resources allocating. We have to 
     * repeatedly call these functions as the UDT interatively discovers 
     * things about the adapter (e.g. it needs to register an I/O range 
     * before it can find out the interrupt number).
     */
     
    for (i = 0; i < MTOK_MAX_CONFIG_LOOPS; i++)
    {
        /*
         * Read the state of the adapter's configuration from the UDT.
         * The static parameters (sparams) are things the UDT is telling
         * us about the adapter (I/O location, IRQ number, possible transfer
         * modes etc). The configuration parameters (cparams) are things the 
         * UDT needs us to tell it (I/O access handles, location of memory 
         * buffers etc).
         */
        
        if ((retc = UDI_GET_PARAMETERS(&adap->udi_data, &done)) != SUCCESS)
        {
#ifdef DEBUG 
            QPOKE(PO_RED,"Error in UDI_GET_PARAMETERS");
#endif
            mtok_show_error(adap, retc); 
            mtok_cleanup(adap);
            return NULL;
        }
        
        /*
         * If the configuration process has finished then break out of the
         * loop.
         */
         
        if (done)
        {
            break;
        }
        
        /*
         * Based on the information in the static parameters allocate
         * resources and set information in the configuration parameters.
         */
         
        if ((retc = mtok_update_params(adap)) != EOK)
        {
            mtok_cleanup(adap);
            return NULL;
        }
    }

    /*
     * Make sure that adapter configuration was completed.
     */
     
    if (!done)
    {
        printk(KERN_ERR "%s: Adapter configuration could not be completed\n",dev->name);
        mtok_cleanup(adap);
        return NULL;
    }

    /*
     * Note that the adapter is in the process of being initialised.
     */
     
    adap->state = MTOK_STATE_INITIALISING;
                  
    /*
     * We now tell the UDT to commit the configuration parameters. This
     * initialises the hardware (but doesn't open the adapter onto the 
     * ring).
     */

    comp.command            = MTOK_CMD_COMMIT;
    comp.completed          = FALSE;
    udt_arg.client_argument = &comp;

#ifdef DEBUG 
    QPOKE(PO_MAGENTA,"Calling commit parameters\n");
#endif

    if ((retc = UDI_COMMIT_PARAMETERS(
                   &adap->udi_data,
                    TRUE, /* Interrupts on at host and may be used */
                    mtok_command_complete,
                    &udt_arg)) != SUCCESS)
    {
        mtok_show_error(adap, retc); 
        mtok_cleanup(adap);
        return NULL;
    }
    
    /*
     * Wait for the operation to complete.
     */
     
    if ((retc = mtok_await_completion(adap, &comp, MTOK_COMP_TIMEOUT)) != EOK)
    {
        mtok_show_error(adap, retc); 
        mtok_cleanup(adap);
        return NULL;
    }

    /*
     * Start our background timer.
     */
     
    init_timer(&adap->bkgd_timer);
    
    adap->bkgd_timer.data     = (unsigned long) adap;
        adap->bkgd_timer.function = mtok_background;
        adap->bkgd_timer.expires  = jiffies + MTOK_BACKGROUND_INTERVAL;

        adap->bkgd_runs = TRUE;    
        add_timer(&adap->bkgd_timer);


        if ((retc = UDI_REGISTER_UDTI(
                &adap->udi_data,
                mtok_tx_complete,
                NULL,
                mtok_rx_complete,
                NULL)) != SUCCESS)
    {
#ifdef DEBUG 
        QPOKE(PO_RED,"RegisterUDTI Failed\n");
#endif
        mtok_show_error(adap, retc); 
        mtok_cleanup(adap);
        return NULL;
    }


    if (mtok_alloc_rx_descriptors(dev) < 0)
    {
#ifdef DEBUG 
        QPOKE(PO_RED,"alloc_rx_descriptors_failed");
#endif
        mtok_cleanup(adap);
        return NULL;
    }

    if (mtok_alloc_tx_descriptors(dev) < 0)
    {
#ifdef DEBUG 
        QPOKE(PO_RED,"alloc_tx_descriptors_failed");
#endif
        mtok_cleanup(adap);
        return NULL;
    }

    UDTI_PROVIDE_RX_DESCRIPTORS(&adap->udi_data,adap->rx_list);

    /* Note that the adapter is now running but not yet open.*/
     
    adap->state = MTOK_STATE_READY;

    /* Set the product instance ID. */
     
    {
        /* Pad length to 18 bytes*/
        char prodid[18];
        memset(prodid,0,18);
        strcpy(prodid,MTOK_PRODUCT_ID);

        comp.command            = MTOK_CMD_PROD_ID;
        comp.completed          = FALSE;
        udt_arg.client_argument = &comp;
    
        if ((retc = UDI_SET_PRODUCT_INSTANCE_ID(
                &adap->udi_data,
                prodid,
                mtok_command_complete,
                &udt_arg)) != SUCCESS)
        {
            mtok_show_error(adap, retc); 
            mtok_cleanup(adap);
            return NULL;
        }
    }

    /*
     * Wait for the operation to complete.
     */

    mtok_await_completion(adap, &comp, MTOK_COMP_TIMEOUT);


#ifdef SLEEP_WAIT
    adap->command_wait = NULL;
#endif

    /*
     * Note our entry points.
     */

    dev->open               = mtok_open;
    dev->stop               = mtok_close;
    dev->hard_start_xmit    = mtok_transmit;
    dev->get_stats          = mtok_get_stats;

#ifdef HAVE_MULTICAST
    dev->set_multicast_list = mtok_set_multicast_list;
#endif
#ifdef HAVE_SET_MAC_ADDR
    dev->set_mac_address    = mtok_set_mac_address;
#endif
#ifdef HAVE_CHANGE_MTU
    dev->change_mtu         = mtok_change_mtu;
#endif
#ifdef HAVE_PRIVATE_IOCTL
    dev->do_ioctl           = mtok_ioctl;
#endif

    /*
     * Set up some flags.
     */
     

	MTOK_SET_TBUSY(dev);
	MTOK_RESET_INTR(dev);
	MTOK_RESET_START(dev);

    mtok_add_dev(dev);
    mtok_display_details(dev);

    return dev;
}

                                            
/*----------------------------------------------------------------------------
|
| Function    - mtok_await_completion
|
| Parameters  - adap    -> Pointer to an adapter structure.
|               comp    -> Pointer to a completion structure.
|               timeout -> Maximum time (in seconds) to wait.
|
| Purpose     - Wait for an asynchronous command to complete.
|
| Returns     - The completion code from the command.
|
---------------------------------------------------------------------------*/

static RETCODE_T mtok_await_completion(
    MTOK_ADAPTER    * adap,
    MTOK_COMPLETION * comp,
    int               timeout
    )
{
#ifdef DEBUG 
    QPOKE(PO_RED,"mtok_await_completion");
#endif

#ifdef SLEEP_WAIT
    /*
     * This function returns either when the timeout expires
     * or when wake_up_interruptible is called (in mtok_command_complete)
     *
     * The Linux Kernel interface changed sometime quite recently
     * hence the conditional compile.
     */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,130)
    current->timeout = jiffies + (timeout * HZ);
    interruptible_sleep_on(&adap->command_wait);
#else   
    interruptible_sleep_on_timeout(&adap->command_wait,timeout*HZ);
#endif

    /*
     * If the command completed (mtok_complete got called) then return
     * the command return code.
     */
    
    if (comp->completed)
        return comp->retc;


#else
{
    int i;
    int j;
    
    /*
     * Wait for the command described by comp to complete.
     */
     
    for (i = 0; i < timeout; i++)
    {
        for (j = 0; j < 10000; j++)
        {
            if (*((volatile BOOLEAN_T *) &comp->completed)==TRUE)
            {
                return comp->retc;
            }
        
            udelay(100);
        }    
    }
}

#endif

    /*
     * Otherwise the command timed out.
     */

#ifdef DEBUG 
    QPOKE(PO_RED,"TimedOut");
#endif

    return ADAPTER_TIMEDOUT;
}




/*----------------------------------------------------------------------------
|
| Function    - mtok_show_error
|
| Parameters  - adap -> Pointer to an adapter structure.
|               retc -> Error code.
|
| Purpose     - Display a UDT standard error string.
|
| Returns     - Nothing.
|
|---------------------------------------------------------------------------*/

#ifndef MTOK_TERSE_ERRORS
ERROR_TABLE_T mtok_error_table[] = {ERROR_TABLE};
#endif

static void mtok_show_error(MTOK_ADAPTER * adap,RETCODE_T retc)
{
#ifndef MTOK_TERSE_ERRORS
    ERROR_TABLE_T * etp;
    for (etp = mtok_error_table; 
         etp->adapter_error_code != ILLEGAL_ERROR_CODE; 
         etp++) {
        if (etp->adapter_error_code == retc) {
            printk(KERN_ERR "%s: %s (%x)\n", 
                adap->dev->name, etp->adapter_error_str, retc);
            return;
        }
    }
#endif /* MTOK_TERSE_ERRORS */
    printk(KERN_ERR "%s: An error occurred (%x)\n", adap->dev->name, retc);
}
                


/*----------------------------------------------------------------------------
|
| Function    - mtok_cleanup
|
| Parameters  - adap -> Pointer to an adapter structure.
|
| Purpose     - Free all of the resources associated with an adapter.
|
| Returns     - Nothing.
|               
|---------------------------------------------------------------------------*/
void mtok_cleanup(MTOK_ADAPTER * adap)
{
    int i;

    if (!adap)
    {
#ifdef DEBUG 
        QPOKE(PO_RED,"adap==NULL");
#endif
        return;
    }

    if (adap->state > MTOK_STATE_UNKNOWN)
    {
        UDI_KILL(&adap->udi_data);
    }

    /*
     * Stop the background function from running.
     */
 
    adap->bkgd_runs = FALSE;
    del_timer(&adap->bkgd_timer);

#if 0 /* Not needed for PCI adapters or PCMCIA in PIO mode */

    /*
     * Free the DMA channel.
     */
     
    if (adap->dma_channel != 0)
    {
        MDBG2(MPOKEF("Freeing DMA Channel\n");)
        free_dma(adap->dma_channel);
        adap->dma_channel = 0;
    }
#endif
    
    /*
     * Free the I/O regions.
     */
     
    for (i = 0; i < MAX_IO_REGIONS; i++)
    {
        if (adap->io_base[i] != 0)
        {
#ifdef DEBUG 
            QPOKED(PO_BLUE,"FreeingRegion:",adap->io_base[i]);
#endif
            release_region(adap->io_base[i], adap->io_range[i]);
            adap->io_base[i] = 0;
        }
    }
    
    /*
     * Free memory fragments.
     */
     
    for (i = 0; i < MAX_FRAG_REGIONS; i++)
    {
        if (adap->frags[i].mem != NULL)
        {
            kfree((void *) adap->frags[i].mem);
            adap->frags[i].mem = NULL;
        }
    }

    if (adap->irq_number) {
#ifndef SA_SHIRQ
        irq2dev_map[adap->dev->irq] = NULL;
#endif
        FREE_IRQ(adap->dev->irq,adap->dev);
        adap->irq_number = 0;
    }
    
    UNREGISTER_NETDEV(adap->dev);

    mtok_free_rx_descriptors(adap->dev);
    mtok_free_tx_descriptors(adap->dev);

    kfree(adap);
}

/*============================================================================
|
| PERIODIC BACKGROUND PROCESSING
|
|===========================================================================*/

/*----------------------------------------------------------------------------
|
| Function    - mtok_background
|
| Parameters  - context -> A pointer to an adapter structure.
|
| Purpose     - Our background proessing function called off a timer.
|
| Returns     - Nothing.
|               
|---------------------------------------------------------------------------*/

static void mtok_background(unsigned long context)
{
    MTOK_ADAPTER   * adap = (MTOK_ADAPTER *) context;
    UDT_ARGUMENT_T   udt_arg;
    RETCODE_T        retc;
    static  int lastcall = 0;
    
    /*
     * Block out our interrupt handler.
     */
     
    MTOK_LOCK(&adap->isr_lock);

#ifdef DEBUG 
    QPOKE(PO_MAGENTA,"{b");
#endif

    /*
     * If the adapter has closed then try to re-open.
     */
     
    if (adap->state == MTOK_STATE_CLOSED && adap->open_count > 0)
    {
#ifdef DEBUG 
        QPOKE(PO_WHITE,"Attempting To Reopen...\n");
#endif

        adap->state = MTOK_STATE_RE_OPENING;
        
        adap->open_comp.command   = MTOK_CMD_OPEN;
        adap->open_comp.completed = FALSE;
        udt_arg.client_argument   = &adap->open_comp;
        
        if ((retc = UDI_OPEN(
                        &adap->udi_data, 
                        OPEN_DEFAULT,
                        mtok_command_complete, 
                        &udt_arg)) != SUCCESS)
        {
            adap->state = MTOK_STATE_RUNNING;
            mtok_show_error(adap, retc);
        }
    }
         
    /*
     * Call the UDI's housekeeping routine.
     * The second argument is the number of microseconds since the 
     * last call.  This argument is ignored on the first call to this
     * function.
     */
     
    UDI_HOUSEKEEPING(
        &adap->udi_data, 
        ((jiffies - lastcall)*1000000) / HZ
        );

    lastcall = jiffies;

    /*
     * If the background timer should still be running re-arm the timer.
     */
     
    if (adap->bkgd_runs)
    {
        adap->bkgd_timer.expires = jiffies + MTOK_BACKGROUND_INTERVAL;
        add_timer(&adap->bkgd_timer);
    }
             
    /*
     * Allow our interrupt handler in.
     */
#ifdef DEBUG 
    QPOKE(PO_MAGENTA,"}");     
#endif
    MTOK_UNLOCK(&adap->isr_lock);
}

                 
/*----------------------------------------------------------------------------
|
| Function    - mtok_init_frag
|
| Parameters  - frag    -> Pointer to a fragment structure to be initialised.
|               length  -> Fragment length required.
|               for_dma -> Is the fragment to be used for DMA.
|               align   -> Alignment needed.
|               mem     -> A pointer to a block of memory to use or NULL if
|                          mtok_init_frag() should allocate the 
|                          memory.
|             
| Purpose     - Initialise a fragment descriptor. If mem is NULL then memory
|               is allocated with kmalloc(). If mem is none NULL then the
|               memory it points to is used (the amount of memory used
|               will be in frag->size on return). If memory is allocated by
|               this function it should be freed by calling kfree of
|               frag->mem.
|
| Returns     - 0 on success or an errno.h error code.
|
|---------------------------------------------------------------------------*/

static int mtok_init_frag(
    FRAG_HANDLE_T * frag,
    unsigned long length,
    BOOLEAN_T       for_dma,
    unsigned long align,
    char          * mem
    )
{
    unsigned        size;
    unsigned        phys;
    char          * data;
    
#ifdef DEBUG 
    QPOKE(PO_YELLOW,"mtok_init_frag\n");
#endif
    
    /*
     * Work out how much memory we need to allocate to meet the alignment
     * constraint.
     */
     
    size = length + align - 1;

    /*
     * If memory has not been supplied then allocate some.
     */
     
    if (mem == NULL)
    {
        mem = (char *) kmalloc(size, for_dma ? GFP_DMA : GFP_KERNEL);
    }
        
    if (mem == NULL)
    {
        return -ENOMEM;
    }

    /*
     * The data area may need to be offset into the area pointed to by
     * mem to meet the alignment constraint. If the fragment will be
     * used for DMA then align the physical memory. Otherwise align the
     * virtual memory.
     */

    if (for_dma)
    {     
        phys = virt_to_bus((void *) mem);
        phys = (phys + align - 1) & ~(align - 1);
        data = (char *) bus_to_virt(phys); 
    }
    else
    {
        phys = 0;
        data = (char *) (((unsigned long) mem + align - 1) & ~(align - 1));
    }
    
    /*
     * Fill in the fields of the fragment descriptor.
     */
     
    frag->mem    = mem;
    frag->length = length;
    frag->data   = data;
    frag->phys   = phys;
    
#ifdef DEBUG 
    QPOKED(PO_MAGENTA,"size:",size);
    QPOKED(PO_GREEN,"length:",length);
    QPOKED(PO_YELLOW,"align:",align);
 
    QPOKEP(PO_RED,"memreal",mem);
    
    QPOKEP(PO_RED,"mem:",mem);
    QPOKEP(PO_WHITE,"data:",data);
#endif
            
    return EOK;
}



/*----------------------------------------------------------------------------
|
| Function    - mtok_update_params
|
| Parameters  - adap -> Pointer to an adapter structure.
|
| Purpose     - Based on the information in the static parameters allocate
|               resources and set information in the configuration parameters.
|
| Returns     - 0 on success or an errno.h error code on failure.
|               
|---------------------------------------------------------------------------*/

static int mtok_update_params(MTOK_ADAPTER * adap)    
{    
    STATIC_PARAMETERS_T * sparams;
    CONFIG_PARAMETERS_T * cparams;
    int                   i;
    long                  io_base;
    long                  io_range;
    long                  length;
    long                  align;
    FRAG_HANDLE_T       *  frag;
    BOOLEAN_T             for_dma;
    long                  irq_number;
    long                  shared_phys;
    long                  shared_size;
    
#ifdef DEBUG 
    QPOKE(PO_GREEN,"[mtok_update_params");
    QPOKEP(PO_YELLOW,"adap:",adap);
#endif

    /*
     * Initialise pointers to the UDT parameter structures.
     */
     
    sparams = STATIC_FROM_UDIDS(&adap->udi_data);
    cparams = CONFIG_FROM_UDIDS(&adap->udi_data);
    
    /*
     * Define some macros to work out if the UDI requires a given type of
     * resource.
     */
     
#define NEED_RESOURCE(_resource, _trigger) \
    (GET_CONFIG_STATUS(cparams, _resource) == STATUS_UNKNOWN) && \
    (GET_STATIC_STATUS(sparams, _trigger) == STATUS_SET)

#define NEED_RESOURCE2(_resource, _trigger1, _trigger2) \
    (GET_CONFIG_STATUS(cparams, _resource) == STATUS_UNKNOWN) && \
    (GET_STATIC_STATUS(sparams, _trigger1) == STATUS_SET) && \
    (GET_STATIC_STATUS(sparams, _trigger2) == STATUS_SET)

#define NEED_RESOURCE3(_resource, _trigger1, _trigger2, _trigger3) \
    (GET_CONFIG_STATUS(cparams, _resource) == STATUS_UNKNOWN) && \
    (GET_STATIC_STATUS(sparams, _trigger1) == STATUS_SET) && \
    (GET_STATIC_STATUS(sparams, _trigger2) == STATUS_SET) && \
    (GET_STATIC_STATUS(sparams, _trigger3) == STATUS_SET)
             
    /*
     * First supply any PCI handle required. In Linux the PCI handle
     * is just the PCI number (i.e. bus/device/function).
     */

    if (NEED_RESOURCE(pci_handle, pci_number))
    {
#ifdef OLD_PCI
        SET_CONFIG_VALUE(
            cparams, 
            pci_handle, 
            (PCI_HANDLE_T) GET_STATIC_VALUE(sparams, pci_number));
#else
        int bdf     = GET_STATIC_VALUE(sparams, pci_number);
        int bus     = (unsigned char) ((bdf >> 8) & 0xff);
        int dev_func= (unsigned char) (bdf & 0xff);
        struct pci_dev * pdev = pci_find_slot(bus,dev_func);

        if (!pdev)
        {
            /* Could happen in the case of card bus cards */
            return ENOMSG;
        }

        SET_CONFIG_VALUE(
            cparams,
            pci_handle,
            pdev);

#endif
    }

    /*
     * Next we supply I/O handles. In Linux the I/O handle is the just the
     * I/O base address - but we do need to register the I/O region with
     * the kernel.
     */

   for (i = 0; i < MAX_IO_REGIONS; i++)
   {
        if (NEED_RESOURCE2(io_handle[i], io_base[i], io_range[i]))
        {
            io_base  = (long) GET_STATIC_VALUE(sparams, io_base[i]);
            io_range = (long) GET_STATIC_VALUE(sparams, io_range[i]);

#ifdef DEBUG 
            QPOKEDD(PO_RED,"!iobase:",io_base);
#endif

#if BITS_PER_LONG == 64
            if (i == 0)
            {
                long pci_io_base;
                struct pci_dev * pdev = (struct pci_dev *)GET_CONFIG_VALUE(cparams,pci_handle);

                pci_io_base = MTOK_PCI_IO_BASE(pdev);

                adap->io_offset_64bit = pci_io_base - io_base;

#ifdef DEBUG 
                QPOKEDD(PO_CYAN,"highiobase:",adap->io_offset_64bit);
#endif
            }

            io_base += adap->io_offset_64bit;

#endif /* BITS_PER_LONG == 64 */

#ifdef DEBUG 
            QPOKEDD(PO_CYAN,"iobase:",io_base);
            QPOKED(PO_GREEN,"i=",i);
#endif

            /*
             * Normally we check the I/O region is OK to use. However
             * if we're a PCMCIA adapter we don't because Card Services
             * seems to have grabbed the region. We know the region is
             * safe because Card Services has told us it is.
             */
             
            if (adap->type != ADAPTER_TYPE_PCMCIA &&
                adap->type != ADAPTER_TYPE_KNOSSOS_CARDBUS &&
#if defined(OLICOM_PCMCIA_UDI) || defined(OLICOM_CARDBUS_UDI)
                adap->type != ADAPTER_TYPE_OLICOM_PCMCIA &&
                adap->type != ADAPTER_TYPE_OLICOM_CARDBUS &&
#endif
                1 )
            { 
                if (check_region(io_base, io_range) != 0)
                {
                  printk(KERN_ERR "%s: I/O region %lx to %lx is already in use\n",
                        adap->dev->name, io_base, io_base + io_range - 1);
                    
                    return ENXIO;
                }
            }    

            /*
             * Keep track of the I/O regions registered so we can free
             * them later.
             */

            adap->io_base[i]  = io_base;
            adap->io_range[i] = io_range;


#ifdef DEBUG 
            QPOKEP(PO_GREEN,"iobase=",io_base);
            QPOKEP(PO_GREEN,"iorange=",io_range);

#endif
            request_region(io_base, io_range, MTOK_DRIVER_NAME);

            SET_CONFIG_VALUE(cparams, io_handle[i], (IO_HANDLE_T) io_base);
       }
   }

    /*
     * Next we need to set up memory fragments.
     */
     
    for (i = 0; i < MAX_FRAG_REGIONS; i++)
    {
        if (NEED_RESOURCE3(
                frag_handle[i], 
                frag_type[i], 
                frag_length[i], 
                frag_alignment[i]))
        {
            /*
             * Work out whether we need physical or virtual memory.
             */
             
            switch (GET_STATIC_VALUE(sparams, frag_type[i]))
            {
                case FRAG_TYPE_VIRT_LE:
                case FRAG_TYPE_VIRT_BE: for_dma = FALSE; break;
                default:                for_dma = TRUE;  break;
            }
                
            /*
             * Get the length and alignment.
             */
             
            length = GET_STATIC_VALUE(sparams, frag_length[i]);
            align  = GET_STATIC_VALUE(sparams, frag_alignment[i]);
            
            /*
             * Allocate a fragment.
             */

            frag = &adap->frags[i];
            
            if (mtok_init_frag(frag, length, for_dma, align, NULL) != EOK)
            {
                printk(KERN_ERR "%s: Insufficient kernel memory\n",adap->dev->name);
                return -ENOMEM;    
            }            

            /*
             * Set the value.
             */
             
            SET_CONFIG_VALUE(cparams, frag_handle[i], *frag);
        }
    }            
           
    /*
     * Next we'll set the number of receive slots.
     */
     
    if (NEED_RESOURCE2(rx_slots, min_rx_slots, max_rx_slots))
    {
        int min,max;
        
        min = GET_STATIC_VALUE(sparams, min_rx_slots);
        max = GET_STATIC_VALUE(sparams, max_rx_slots);

        if (adap->rxslots == 0)
            adap->rxslots = (adap->type == ADAPTER_TYPE_KNOSSOS ||
            				 adap->type == ADAPTER_TYPE_KNOSSOS_CARDBUS) ? 
					            MTOK_DEF_STREAM_RX_SLOTS : MTOK_DEF_RX_SLOTS;

        if (adap->rxslots > max || adap->rxslots < min)
        {
            printk(KERN_WARNING "%s: Warning - RX slots (%d) invalid.  Must be in the range %d - %d\n",
                        adap->dev->name,adap->rxslots,min,max);

            adap->rxslots = (adap->rxslots < min) ? min : max;

            printk(KERN_WARNING "%s: Resetting RX slots to %d\n",adap->dev->name,adap->rxslots);
        }

        SET_CONFIG_VALUE(cparams, rx_slots, (UINT_T) adap->rxslots);
    }

    /*
     * Next we'll set the number of transmit slots.
     */
     
    if (NEED_RESOURCE2(tx_slots, min_tx_slots, max_tx_slots))
    {
        int min,max;
        
        min = GET_STATIC_VALUE(sparams, min_tx_slots);
        max = GET_STATIC_VALUE(sparams, max_tx_slots);

        if (adap->txslots == 0)
        {
            if (ADAPTER_INFO_FROM_UDIDS(&adap->udi_data).adapter_ram_size == 128 &&
                GET_STATIC_VALUE(sparams,adapter_interface) == FASTMAC_PLUS)
            {
                /* 
                 * Limit number of transmit slots to 4 for FastMac adapters
                 * with only 128K memory.
                 */
                adap->txslots = 4;
                printk( KERN_WARNING 
                        "%s: Adapter only has 128K limiting to 4 TX slots\n",
                        adap->dev->name);
            }
            else
            {
                adap->txslots = MTOK_DEF_TX_SLOTS;
            }
        }
        
        if (adap->txslots > max || adap->txslots < min)
        {
            printk(KERN_WARNING "%s: Warning - TX slots (%d) invalid."
                                "  Must be in the range %d - %d\n",
                        adap->dev->name,adap->txslots,min,max);

            adap->txslots = (adap->txslots < min) ? min : max;

            printk( KERN_WARNING "%s: Resetting TX slots to %d\n",
                    adap->dev->name,adap->txslots);
        }
        SET_CONFIG_VALUE(cparams, tx_slots, (UINT_T) adap->txslots);
    }        

    /*
     * Next we'll fill in the IRQ resource. We use the UDI's preferred value.
     * We need to register the interrupt with the kernel and enable it.
     */
     
    if (NEED_RESOURCE(irq, irq_mask))
    {
        /*
         * If a none 0 IRQ was passed into mtok_register_device() then
         * adap->req_irq_number will be none 0. In such cases (probably 
         * PCMCIA) we'll use this IRQ value rather than the preferred.
         */
         
        if (adap->dev->irq != 0)
        {
            irq_number = adap->dev->irq;
        }
        else
        {
            adap->dev->irq = irq_number = 
                GET_STATIC_VALUE(sparams, preferred_irq);
        }

        /*
         * If the IRQ the adapter reports is 2 then we need to register 9
         * with the kernel.
         */
         
        if (irq_number == 2)
        {
            irq_number = 9;
        }

        /*
         * Tell the UDT that we've registered the handler.
         */
                 
        SET_CONFIG_VALUE(cparams, irq, (BYTE_T) irq_number);

        adap->dev->irq = irq_number;

        /*
         * Grab our IRQ
         */

        if (REQUEST_IRQ(    adap->dev->irq,     /* Which IRQ ?              */
                            mtok_isr,           /* Handler Routine          */
                            SA_SHIRQ,           /* Flags                    */
                            MTOK_DRIVER_NAME,   /* Driver name in proc fs   */
                            adap->dev           /* Any user parameter       */
                            ) != 0)
        {
            printk( KERN_ERR "%s: Cannot register IRQ number %d\n",
                        adap->dev->name,adap->dev->irq);
            return -EBUSY;
        }

        /*
         * Note the IRQ number to device descriptor mapping and enable
         * the interrupt.
         */
         
#ifndef SA_SHIRQ
        irq2dev_map[irq_number] = adap->dev;
#endif
        
        adap->irq_number = irq_number;
    }        
        
    /*
     * Set the transfer type to the preferred value.
     */
     
    if (NEED_RESOURCE(xfer_method, xfer_method_mask))
    {
        adap->xfer_mode = GET_STATIC_VALUE(sparams, preferred_xfer_method);
        SET_CONFIG_VALUE(cparams, xfer_method, adap->xfer_mode);
    }

#if 0 /* Dont need this for PCI cards. PCMCIA does PIO */
    /*
     * Set the DMA channel to the preferred value. We need to register the
     * DMA channel with the kernel.
     */
     
    if (NEED_RESOURCE(dma_channel, dma_channel_mask))
    {
        int dma_channel = GET_STATIC_VALUE(sparams, preferred_dma_channel);
        
        if (request_dma(dma_channel, MTOK_DRIVER_NAME) != 0)
        {
            printk(KERN_ERR "%s: Cannot register DMA channel %d\n",
                adap->dev->name, dma_channel);
                
            return ENXIO;
        }
            
        SET_CONFIG_VALUE(cparams, dma_channel, (BYTE_T) dma_channel);
        
        /*
         * Record the DMA channel so we can free it later.
         */
         
        adap->dma_channel = dma_channel;
                
        /*
         * Record our DMA channel in the device descriptor so it can be 
         * displayed in /proc files.
         */
     
        adap->dev->dma = dma_channel; 

//      adap->MaxDMAAddress = MAX_DMA_ADDRESS;
        
    }
#endif /* 0 */

    /*
     * Set the ring speed to either the value programmed on the adapter
     * or the value chosen by the user.
     */
     
    if (NEED_RESOURCE(ring_speed, ring_speed_mask))
    {
        /* Did we specify a ring speed module parameter */
        if (adap->ringspeed == 0) /* No */
            adap->ringspeed = GET_STATIC_VALUE(sparams, preferred_ring_speed);
        else /* Yes */
            switch(adap->ringspeed)
            {
                case 4: adap->ringspeed = RS_4M; break;
                case 16: adap->ringspeed = RS_16M; break;
                case 100: adap->ringspeed = RS_100M; break;
                default:
                    adap->ringspeed = GET_STATIC_VALUE(sparams,preferred_ring_speed); break;
            }

        if ((MASK(adap->ringspeed) & GET_STATIC_VALUE(sparams, ring_speed_mask)) == 0) {
            printk(
                KERN_WARNING
                "%s: Invalid ring speed (%d) specified for this adapter, "
                "using the default\n",
                adap->dev->name,
                adap->ringspeed);
            adap->ringspeed = GET_STATIC_VALUE(sparams, preferred_ring_speed);
        }
        
        SET_CONFIG_VALUE(cparams, ring_speed, adap->ringspeed);
    }

    /*
     * Set the DTR options to DTR and CLASSIC.
     */
     
    if (NEED_RESOURCE(dtr_options, dtr_options_mask))
    {
        switch(adap->dtr)
        {
            case 1:
                SET_CONFIG_VALUE(cparams, dtr_options, DTR_ONLY);
                break;
            case 2:
                SET_CONFIG_VALUE(cparams, dtr_options, CLASSIC_ONLY);
                break;
            default:
            case 0:
                SET_CONFIG_VALUE(cparams, dtr_options, DTR_AND_CLASSIC);
                break;
        }
    }

    /*
     * Set the maximum frame size.
     */
    
    if (GET_CONFIG_STATUS(cparams, requested_max_frame_size) == STATUS_UNKNOWN)
    {
        if (adap->max_frame_size < 18 ||
            adap->max_frame_size > MAX_FRAME_SIZE_100M ||
            adap->max_frame_size == 0)
                adap->max_frame_size = MTOK_DEF_MAX_FRAME_SIZE;

        SET_CONFIG_VALUE(
            cparams, 
            requested_max_frame_size,
            adap->max_frame_size 
            );  /* +4 for the CRC. */
        
    }

    /*
     * We don't need a transmit completion status.
     */      

    if (GET_CONFIG_STATUS(cparams, 
            tx_completion_status_required) == STATUS_UNKNOWN)
    {
        SET_CONFIG_VALUE(cparams, tx_completion_status_required, FALSE);
    }           
          
    /*
     * Set the shared RAM override to its preferred value.
     */
     
    if (NEED_RESOURCE(shared_override, shared_override_mask))
    {
        SET_CONFIG_VALUE(
            cparams, 
            shared_override, 
            GET_STATIC_VALUE(sparams, preferred_shared_override));
    }
          
    /*
     * The UDT may need handles for some shared RAM regions. In Linux
     * a shared RAM handle is just the base address of the virtual memory.
     */
          
    for (i = 0; i < MAX_SHARED_REGIONS; i++)
    {
        if (NEED_RESOURCE2(
                shared_handle[i], 
                shared_phys[i], 
                shared_maximum_offset[i]))
        {
    
            shared_phys = GET_STATIC_VALUE(sparams, shared_phys[i]);
            shared_size = GET_STATIC_VALUE(sparams, shared_maximum_offset[i]);

            if (i==0)
                adap->shared_ram = shared_phys;
                            
            SET_CONFIG_VALUE(
                cparams,
                shared_handle[i],
                (SHARED_HANDLE_T) bus_to_virt(shared_phys));

            SET_CONFIG_VALUE(
                cparams,
                init_shared_handle[i],
                (SHARED_HANDLE_T) bus_to_virt(shared_phys));
                
            
            if (i == 0)
            {    
                /*
                 * Record our shared memory region in the device descriptor so 
                 * it can be displayed in /proc files.
                 */
     
                adap->dev->mem_start = shared_phys;
                adap->dev->mem_end   = shared_phys + shared_size;
            }    
        }
    }
    
    /*
     * Set the watchdog timeout period.
     */
     
    if (NEED_RESOURCE(watchdog_period, preferred_watchdog_period))
    {
        SET_CONFIG_VALUE(
            cparams, 
            watchdog_period, 
            GET_STATIC_VALUE(sparams,preferred_watchdog_period)
            );
            
#ifdef DEBUG 
        QPOKEW(PO_CYAN,"WatchDog:", GET_STATIC_VALUE(sparams,preferred_watchdog_period) );    
#endif
    }
        
    /*
     * Make a note of the BIA.
     */
     
    if (GET_STATIC_STATUS(sparams, bia) == STATUS_SET)
    {
        adap->bia = GET_STATIC_VALUE(sparams, bia); 
        
#if 0
        MDBG2(MPOKEF("BIA is %02x %02x %02x %02x %02x %02x\n",
            adap->bia.byte[0], adap->bia.byte[1], adap->bia.byte[2],
            adap->bia.byte[3], adap->bia.byte[4], adap->bia.byte[5]));
#endif
            
        memcpy(adap->dev->dev_addr,adap->bia.byte,6);
    }
    return EOK;
}


/*--------------------------------------------------------------------------
|
| Function    - mtok_open
|
| Parameters  - dev -> Pointer to a device descriptor.
|
| Purpose     - Open the adapter.
|
| Returns     - 0 on success or an errno.h error code on failure.
|               
|-------------------------------------------------------------------------*/
static int mtok_open(LINUXDEV * dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *) dev->priv;
    UDT_ARGUMENT_T udt_arg;
    RETCODE_T retc;
        

    if (adap->open_count > 0)
    {
        /* We have already been opened */
        MOD_INC_USE_COUNT;
        adap->open_count++;
        return 0;
    }

    if (adap->state == MTOK_STATE_UNKNOWN)
    {
        return -EIO;
    }

    /*
     * Open the adapter.
     */
     
    adap->state = MTOK_STATE_OPENING;
    adap->open_comp.command   = MTOK_CMD_OPEN;
    adap->open_comp.completed = FALSE;
    udt_arg.client_argument   = &adap->open_comp;
     
     
    if ((retc = UDI_OPEN(   &adap->udi_data, 
                            OPEN_DEFAULT,
                            mtok_command_complete, 
                            &udt_arg)) != SUCCESS)
    {
        adap->state = MTOK_STATE_READY;
        mtok_show_error(adap, retc); 
        return -EIO;
    }


    retc = mtok_await_completion(adap, &adap->open_comp, MTOK_OPEN_TIMEOUT);
    
    if (retc != SUCCESS)
    {
		adap->state = MTOK_STATE_READY;
		mtok_show_error(adap,retc);
		return -EIO;    
    }

    /*
     * Set the start device flag to indicate that the adapter is running. 
     */
         

    /*
     * If the adapter is actually open on the ring then clear tbusy to
     * tell higher layers that we can accept transmits. If the adapter
     * is not currently open then tbusy will be cleared in 
     * mtok_command_complete() when the adapter opens.
     */
     
    if (adap->state == MTOK_STATE_OPEN)
    {
        adap->tx_full = 0;    

		MTOK_FIRST_RESET_TBUSY(adap->dev);
    }

    MOD_INC_USE_COUNT;
    adap->open_count++;
    
    return 0;
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_close
|
| Parameters  - dev -> Pointer to a device descriptor.
|
| Purpose     - Close the adapter.
|
| Returns     - 0 on success or an errno.h error code on failure.
|               
|---------------------------------------------------------------------------*/
/* Not static because CardBus helper calls it */
int mtok_close(LINUXDEV * dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *) dev->priv;
    UDT_ARGUMENT_T udt_arg;
    

    if (adap->open_count==0)
        return 0;

    /*
     * Note that the higher layers think we're closed.
     */
     
    adap->open_count--;
     
    /*
     * Set the start device flags to indicate that the adapter is not running. 
     */
         

    /*
     * Set tbusy to indicate that we can't transmit.
     */
      
	MTOK_SET_TBUSY(adap->dev);

    /*
     * Decrement the module's reference count.
     */

    MOD_DEC_USE_COUNT;

    if (adap->open_count == 0)
    {
        if (adap->state == MTOK_STATE_OPEN)
        {
            adap->state = MTOK_STATE_CLOSING;
            adap->open_comp.command   = MTOK_CMD_CLOSE;
            adap->open_comp.completed = FALSE;

            udt_arg.client_argument   = &adap->open_comp;

            MTOK_LOCK(&adap->isr_lock);   
            UDI_CLOSE(&adap->udi_data,mtok_command_complete,&udt_arg);
            MTOK_UNLOCK(&adap->isr_lock);

            /*
             * Wait for the open command to complete.
             */
     
            mtok_await_completion(adap, &adap->open_comp, MTOK_CLOSE_TIMEOUT);
        }
    }
    return 0;
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_get_stats
|
| Parameters  - dev -> Pointer to a device descriptor.
|
| Purpose     - Return the statistics.
|
| Returns     - A pointer to a token ring statistics structure.
|               
|---------------------------------------------------------------------------*/
static struct net_device_stats * mtok_get_stats(LINUXDEV  * dev)
{
    MTOK_ADAPTER    * adap = (MTOK_ADAPTER *) dev->priv;
    MTOK_COMPLETION   comp;
    UDT_ARGUMENT_T    udt_arg;
    RETCODE_T         retc;
#if 0
#if defined(LINUX21) || MPOKE_DBG_LEVEL > 0
    STATS_T         * stats = &STATS_FROM_UDIDS(&adap->udi_data);
#endif
#endif

#if 0
    MDBG1(MPOKEF("TX: %ldb %ldf %lde RX: %ldb %ldf %lde\n",
        stats->counters.tx_bytes,
        stats->counters.tx_frames,
        stats->counters.tx_no_resources,
        stats->counters.rx_bytes,
        stats->counters.rx_frames,
        stats->counters.rx_aborts + stats->counters.rx_overruns);)
#endif

    /*
     * If the adapter isn't running then just return whatever happens
     * to be in the statistics structure.
     */
     
    if (adap->state < MTOK_STATE_RUNNING)
    {
        return (struct net_device_stats *) &adap->stats; 
    }    

    /*
     * Block out our interrupt handler.
     */
     
    MTOK_LOCK(&adap->isr_lock);
    
    /*
     * Tell the UDT to update the error log.
     */
    
    comp.command            = MTOK_CMD_ERROR_LOG;
    comp.completed          = FALSE;
    udt_arg.client_argument = &comp;
     
    retc = UDI_UPDATE_ERROR_LOG(
               &adap->udi_data, 
               mtok_command_complete, 
               &udt_arg);
    
    /*
     * Let our interrupt handler back in.
     */
     
    MTOK_UNLOCK(&adap->isr_lock);
    
    /*
     * Wait for the error log read to complete.
     */
    
    if (retc == SUCCESS)
    { 
        mtok_await_completion(adap, &comp, MTOK_COMP_TIMEOUT);
    }
    
    /*
     * Copy the UDI's error log into the Linux form.
     */

#if 0
    adap->stats.line_errors         = (int) stats->error_log.line_errors;
    adap->stats.internal_errors     = (int) stats->error_log.internal_errors;
    adap->stats.burst_errors        = (int) stats->error_log.burst_errors;
    adap->stats.A_C_errors          = (int) stats->error_log.ari_fci_errors;
    adap->stats.abort_delimiters    = (int) stats->error_log.abort_delimiters;
    adap->stats.lost_frames         = (int) stats->error_log.lost_frame_errors;
    adap->stats.recv_congest_count  = (int) stats->error_log.congestion_errors;
    adap->stats.frame_copied_errors = (int) stats->error_log.frame_copied_errors;
    adap->stats.frequency_errors    = (int) stats->error_log.frequency_errors;
    adap->stats.token_errors        = (int) stats->error_log.token_errors;
#endif
    
    /*
     * Return a pointer to the statistics structure.
     */


    return (struct net_device_stats *) &adap->stats; 
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_set_multicast_list
|
| Parameters  - dev -> Pointer to a device descriptor.
|
| Purpose     - Set the multicast receive mode.
|
| Returns     - Nothing.
|               
|---------------------------------------------------------------------------*/

static void mtok_set_multicast_list(LINUXDEV * dev)
{
    MTOK_ADAPTER       * adap = (MTOK_ADAPTER *) dev->priv;
    MAC_ADDRESS_T        mask;
    struct dev_mc_list * mc;
    MTOK_COMPLETION      comp;
    UDT_ARGUMENT_T       udt_arg;
    RETCODE_T            retc;
    

    /*
     * If the adapter isn't running then give up.
     */
     
    if (adap->state < MTOK_STATE_RUNNING)
    {
        return;
    }
         
    /*
     * I'm not really sure what should happen with token ring here. What
     * we'll do is generate a functional address mask from the multicast
     * list and download this mask to the adapter. This is what 
     * multicast processing usually does on token ring.
     */

    memset((void *) &mask, 0, sizeof(MAC_ADDRESS_T));
    mask.byte[0] = 0xc0;
    
    for (mc = adap->dev->mc_list; mc != NULL; mc = mc->next)
    {
#if 0
        MDBG2(MPOKEF("MCA: %02x %02x %02x %02x %02x %02x\n",
            (unsigned) mc->dmi_addr[0] & 0xff,
            (unsigned) mc->dmi_addr[1] & 0xff,
            (unsigned) mc->dmi_addr[2] & 0xff,
            (unsigned) mc->dmi_addr[3] & 0xff,
            (unsigned) mc->dmi_addr[4] & 0xff,
            (unsigned) mc->dmi_addr[5] & 0xff));
#endif
            
        mask.byte[2] |= (BYTE_T) mc->dmi_addr[2];
        mask.byte[3] |= (BYTE_T) mc->dmi_addr[3];
        mask.byte[4] |= (BYTE_T) mc->dmi_addr[4];
        mask.byte[5] |= (BYTE_T) mc->dmi_addr[5];
    }
    
    /*
     * If the all multicast flag is set then set the receive everything
     * functional address.
     */
    
    if ((dev->flags & IFF_ALLMULTI) != 0)
    {
        
        mask.byte[2] = 0xff;
        mask.byte[3] = 0xff;
        mask.byte[4] = 0xff;
        mask.byte[5] = 0xff;
    }    
    
    mtok_set_promiscuous_mode(adap,
        dev->flags & IFF_PROMISC ?  (adap->wantmac == 1 ? ALL_LLC_AND_MAC : ALL_LLC)  : 
                                    (adap->wantmac == 1 ? LLC_AND_MAC : LLC) );

    /*
     * Block out our interrupt handler for a while.
     */
     
    MTOK_LOCK(&adap->isr_lock);

    /*
     * Set the functional address mask.
     */
     
    comp.command            = MTOK_CMD_FUNC_ADDR;
    comp.completed          = FALSE;
    udt_arg.client_argument = &comp;
     
    retc = UDI_SET_FUNCTIONAL_ADDRESS(
               &adap->udi_data, 
               &mask, 
               mtok_command_complete, 
               &udt_arg);
    
    /*
     * Let our interrupt handler back in.
     */
     
    MTOK_UNLOCK(&adap->isr_lock);
    
    /*
     * Wait for the command to complete.
     */
    
    if (retc == SUCCESS)
    { 
        mtok_await_completion(adap, &comp, MTOK_COMP_TIMEOUT);
    }
}


static void mtok_set_promiscuous_mode(MTOK_ADAPTER *adap,PROMISCUOUS_LEVEL_T mode)
{    
    MTOK_COMPLETION      comp;
    UDT_ARGUMENT_T       udt_arg;
    RETCODE_T            retc;


    if (STATS_FROM_UDIDS(&adap->udi_data).promiscuous_level == mode)
    {
        /*
         * Adapter already at the desired promiscuous level
         */
        return;
    }

    if (mode == ALL_LLC ||
        mode == ALL_LLC_AND_MAC)
    {
        if (GET_STATIC_VALUE(STATIC_FROM_UDIDS(&adap->udi_data),
                promiscuous_enabled) == FLAG_FALSE)
        {
            /*
             * This can be FLAG_TRUE, FLAG_FALSE or FLAG_ABSENT
             * Some adapters have a flag to say whether promiscuous mode is
             * allowed on that adapter.  The value of promiscuous_enabled
             * gives the state of the flag on the adapter or FLAG_ABSENT if 
             * this type of adapter does not have the flag.
             */
            printk( KERN_WARNING
                    "%s: Promiscuous Mode is disabled on this adapter\n",
                    adap->dev->name);
            return;
        }
    
        if (GET_STATIC_VALUE(STATIC_FROM_UDIDS(&adap->udi_data),
                supports_promiscuous) == FALSE)
        {
            printk( KERN_WARNING
                    "%s: Promiscuous Mode is not supported on this adapter\n",
                    adap->dev->name);
            return;
        }
    }   
    
        MTOK_LOCK(&adap->isr_lock);
        
        comp.command = MTOK_CMD_SET_PROMISC;
        comp.completed = FALSE;
        udt_arg.client_argument = &comp;
        
        retc = UDI_SET_PROMISCUOUS_MODE(&adap->udi_data,
                                        mode,
                                        mtok_command_complete,
                                        &udt_arg);
    
        /*
         * Let our interrupt handler back in.
        */
     
        MTOK_UNLOCK(&adap->isr_lock);
    
        /*
          * Wait for the command to complete.
        */
    
         if (retc == SUCCESS)
         { 
             mtok_await_completion(adap, &comp, MTOK_COMP_TIMEOUT);
        }
}

#ifdef HAVE_CHANGE_MTU
static int mtok_change_mtu(LINUXDEV * dev,int new_mtu) {
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dev->priv;
    if (new_mtu < 18 || new_mtu > adap->max_frame_size - 18)
        return -EINVAL;
    dev->mtu = new_mtu;
    return 0;
}
#endif

#ifdef HAVE_PRIVATE_IOCTL

int mtok_copy_from_user(void *dest,void *src,int len)
{
    int error;
    if ((error = verify_area(VERIFY_READ,src,len)) < 0)
        return error;

#if LINUX_VERSION_CODE > 0x20109
    if (copy_from_user(dest,src,len) < 0)
        return -EACCES;
#else
    memcpy_fromfs(dest,src,len);
#endif          
    return 0;
}

int mtok_copy_to_user(void *dest,void *src,int len)
{
    int error;
    if ((error = verify_area(VERIFY_WRITE,dest,len)) < 0)
        return error;

#if LINUX_VERSION_CODE > 0x20109
    if (copy_to_user(dest,src,len) < 0)
        return -EACCES;
#else
    memcpy_tofs(dest,src,len);
#endif          
    return 0;
}

#include "mdgioctl.h"
static int mtok_ioctl(LINUXDEV *dev, struct ifreq *ifr, int cmd)
{
    MTOK_ADAPTER    * adap = (MTOK_ADAPTER *) dev->priv;
    int error;

    switch (cmd)
    {
        /* Identify ourselves as a Madge driver and give the version number */
        case MTOK_SIOCMDGID:
            {
                MTOK_IOCTL_MDGID    mdgid;
                if ((error = mtok_copy_from_user(&mdgid,ifr->ifr_data,sizeof(MTOK_IOCTL_MDGID))) < 0)
                    return error;

                if (mdgid.len == sizeof(MTOK_IOCTL_MDGID) && mdgid.sig == MTOK_ID_IN)
                {
                    mdgid.sig = MTOK_ID_OUT;
                    mdgid.version = MTOK_VERSION_NUM;
                }
                else
                    return -EINVAL;

                if ((error = mtok_copy_to_user(ifr->ifr_data,&mdgid,sizeof(MTOK_IOCTL_MDGID))) < 0)
                    return error;
            }
            break;

        case MTOK_SIOCADAPTERINFO:
            {
                MTOK_IOCTL_INFO info;
                STATS_T         *   udtinfo;

                if ((error = mtok_copy_from_user(&info,ifr->ifr_data,sizeof(MTOK_IOCTL_INFO))) < 0)
                    return error;               
    
                if(info.len < sizeof(MTOK_IOCTL_INFO) || info.sig != MTOK_ID_IN)
                    return -EINVAL; 

                strncpy(info.name,adap->dev->name,8);

                info.busdevfn = adap->busdevfn;

                info.type=adap->type;
                info.subtype=adap->subtype;

                info.iobase = dev->base_addr;
                info.irq=adap->irq_number;
                info.xfermode = adap->xfer_mode;
                info.ringspeed = adap->ringspeed;
                info.rxslots = adap->rxslots;
                info.txslots = adap->txslots;
                info.dtr = adap->dtr;
                info.max_frame_size = adap->max_frame_size;
                info.state=adap->state;

                udtinfo = &STATS_FROM_UDIDS(&adap->udi_data);

                memcpy(info.bia,adap->bia.byte,6);
                memcpy(info.laa,dev->dev_addr,6);
                memcpy(info.group,udtinfo->group_address[0].byte,6);
                memcpy(info.func,udtinfo->functional_address.byte,6);
                memcpy(info.naun,udtinfo->upstream_node_address.byte,6);        
                memcpy(info.product_instance_id,udtinfo->product_instance_id,18);
                info.promiscuous_level = udtinfo->promiscuous_level;

#define C(field)    info.field = udtinfo->open_info.field
                info.open_ring_speed = udtinfo->open_info.ring_speed;
                C(error_code);C(fault_phase);C(max_frame_size_avail);
                C(max_rcv_buf_size);C(mode);

#undef C
#define C(field)    info.field = udtinfo->field

                C(ring_status);C(last_beacon_type);C(local_ring_number);
                C(udi_state);
#undef C
                if ((error = mtok_copy_to_user(ifr->ifr_data,&info,sizeof(MTOK_IOCTL_INFO))) < 0)
                    return error;
            }
            break;
            
        /* Get information about the adapter */ 
        case MTOK_SIOCADAPTERSTATS:
            {
                MTOK_IOCTL_STATS    stats;
                STATS_T         *   udtstats;   
                if ((error = mtok_copy_from_user(&stats,ifr->ifr_data,sizeof(MTOK_IOCTL_STATS))) < 0)
                    return error;               
    
                if(stats.len < sizeof(MTOK_IOCTL_STATS) || stats.sig != MTOK_ID_IN)
                    return -EINVAL; 

                if (adap->state >= MTOK_STATE_RUNNING)
                {
                    MTOK_COMPLETION   comp;
                    UDT_ARGUMENT_T    udt_arg;
                    RETCODE_T         retc;  

                    comp.command            = MTOK_CMD_ERROR_LOG;
                    comp.completed          = FALSE;
                    udt_arg.client_argument = &comp;

                    MTOK_LOCK(&adap->isr_lock);   
                    retc = UDI_UPDATE_ERROR_LOG(
                          &adap->udi_data, 
                          mtok_command_complete, 
                          &udt_arg);
                    MTOK_UNLOCK(&adap->isr_lock);
                
                    if (retc == SUCCESS) { 
                        mtok_await_completion(adap, &comp, MTOK_COMP_TIMEOUT);
                    }
                }

                udtstats = &STATS_FROM_UDIDS(&adap->udi_data);

#define C(field)    stats.field = udtstats->error_log.field
                C(line_errors);C(internal_errors);C(burst_errors);C(ari_fci_errors);
                C(abort_delimiters);C(lost_frame_errors);C(congestion_errors);
                C(frame_copied_errors);C(frequency_errors);C(token_errors);
                C(dma_bus_errors);C(dma_parity_errors);
#undef C
#define C(field)    stats.field = adap->stats.field;
                C(rx_packets);C(tx_packets);C(rx_errors);
                C(tx_errors);C(rx_dropped);C(tx_dropped);

#ifdef LINUX21
                C(rx_bytes);C(tx_bytes);
#else
                stats.rx_bytes=0;
                stats.tx_bytes=0;
#endif
#undef C
                if ((error = mtok_copy_to_user(ifr->ifr_data,&stats,sizeof(MTOK_IOCTL_STATS))) < 0)
                    return error;
            }
            break;
        case MTOK_SIOCGFILTERMAC:
            {
                MTOK_IOCTL_FILTERMAC    filter;
                if ((error = mtok_copy_from_user(&filter,ifr->ifr_data,sizeof(MTOK_IOCTL_STATS))) < 0)
                    return error;               
    
                if(filter.len < sizeof(MTOK_IOCTL_FILTERMAC) || filter.sig != MTOK_ID_IN)
                    return -EINVAL; 

                filter.wantmac = adap->wantmac;
            
                if ((error = mtok_copy_to_user(ifr->ifr_data,&filter,sizeof(MTOK_IOCTL_FILTERMAC))) < 0)
                    return error;
            }
            break;

        case MTOK_SIOCSFILTERMAC:
            {
                MTOK_IOCTL_FILTERMAC    filter;

                if (!suser())
                    return -EPERM;

                if ((error = mtok_copy_from_user(&filter,ifr->ifr_data,sizeof(MTOK_IOCTL_STATS))) < 0)
                    return error;               
    
                if(filter.len < sizeof(MTOK_IOCTL_FILTERMAC) || filter.sig != MTOK_ID_IN)
                    return -EINVAL; 

                adap->wantmac = filter.wantmac;

                mtok_set_promiscuous_mode(adap,
                        dev->flags & IFF_PROMISC ?  (adap->wantmac == 1 ? ALL_LLC_AND_MAC : ALL_LLC)  : 
                                                    (adap->wantmac == 1 ? LLC_AND_MAC : LLC) );

                if ((error = mtok_copy_to_user(ifr->ifr_data,&filter,sizeof(MTOK_IOCTL_FILTERMAC))) < 0)
                    return error;
            }
            break;
#ifdef UDT_TRANSMIT_PRIORITY_SUPPORT
        /* Update the frame priority information in the driver */
        case MTOK_SIOCGPRI:
            return -EINVAL;
        case MTOK_SIOCSPRI:
            if (!suser())
                return -EPERM;
            return -EINVAL;
#endif
        default:
            return -EINVAL;
    }
    return 0;
}
#endif /* HAVE_PRIVATE_IOCTL */

#ifdef HAVE_SET_MAC_ADDR
static int mtok_set_mac_address(LINUXDEV * dev,void *p)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dev->priv;
    struct sockaddr * addr = p;

#if 0
    MDBG2(MPOKEF("Trying to set LAA to %02x %02x %02x %02x %02x %02x\n",
            addr->sa_data[0], addr->sa_data[1], addr->sa_data[2],
            addr->sa_data[3], addr->sa_data[4], addr->sa_data[5]));
#endif
    
    if ((addr->sa_data[0] & 0x40) != 0x40) {
        return -EINVAL;
    }
    
	if (MTOK_TEST_START(dev)) {
        return -EBUSY;
    }
     
    if (SUCCESS != UDI_SET_NODE_ADDRESS(&adap->udi_data,(MAC_ADDRESS_T *)addr->sa_data)) {
        return -EBUSY;
    }
    
    memcpy(dev->dev_addr,addr->sa_data,dev->addr_len);
    return 0;
}
#endif


/*============================================================================
|
| UPCALLS MADE BY THE UNIVERSAL DRIVER TOOLKIT
|
|===========================================================================*/

/*----------------------------------------------------------------------------
|
| Function    - mtok_event_handler
|
| Parameters  - context  -> Pointer to an adapter structure.
|               argument -> Not used.
|               event    -> An event code.
|
| Purpose     - Upcall made by the UDT to notify that an event has happened.
|
| Returns     - Nothing.
|
| Notes       - This upcall will be made out of our interrupt handler.
|
---------------------------------------------------------------------------*/

static void mtok_event_handler(
    P_DRIVER_T      context,
    P_VOID_T        argument,
    ADAPTER_EVENT_T event
    )
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *) context;
    STATS_T      * stats = &adap->udi_data.res.stats;
    
    switch (event)
    {
        /*
         * If we get a ring status event then just tell the user that
         * something has happened.
         */
         
        case RING_STATUS:
        
            if ((stats->ring_status & MTOK_RING_STATUS_MASK) != 0)
            {
                printk(KERN_NOTICE "%s: Ring status 0x%x\n",
                    adap->dev->name, stats->ring_status & MTOK_RING_STATUS_MASK);
            }    
                
            break;
            
        /*
         * If the adapter closes we tell the user and change the adapter
         * state to closed so that an attempt gets made to re-open it.
         * We set dev->tbusy to 1 to tell higher layers that we can't
         * transmit.
         */
              
        case ADAPTER_CLOSED:
        
            adap->state      = MTOK_STATE_CLOSED;

			MTOK_SET_TBUSY(adap->dev);
            
            printk(KERN_WARNING "%s: The adapter has closed - "
                "trying to re-open\n", adap->dev->name);
                
            break;

        /*
         * If the adapter has been removed then set the adapter state to
         * removed and tell the user. We don't bother setting dev->tbusy
         * since we won't be able to transmit anything for a long
         * period of time.
         */
          
        case ADAPTER_REMOVED:
        
            adap->state = MTOK_STATE_REMOVED;
            
            printk(KERN_WARNING "%s: The adapter has been removed\n", 
                adap->dev->name);
           
            /*
             * Tell the kernel the adapter is not running
             */
            
			MTOK_RESET_START(adap->dev);
            
            break;
            
        /*
         * If the adapter has died it's all over! Set the adapter state to
         * died and tell the user. We don't bother setting dev->tbusy
         * since we won't be able to transmit anything for a long
         * period of time, if ever.
         */
          
        case ADAPTER_DEAD:
        
            adap->state = MTOK_STATE_DEAD;
            
            printk(KERN_WARNING "%s: The adapter has died\n", adap->dev->name);
            
            break;

        /*
         * Shouldn't see anything else.
         */
                     
        default:
        
            break;
    }                 
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_command_complete
|
| Parameters  - context  -> Pointer to an adapter structure.
|               argument -> Pointer to a completion.
|               response -> Return code from the asynchronous operation.
|
| Purpose     - Upcall made by the UDT to notify that an asynchronous
|               command has completed.
|
| Returns     - Nothing.
|
| Notes       - This upcall will be made out of our interrupt handler.
|
---------------------------------------------------------------------------*/

static void mtok_command_complete(
    P_DRIVER_T       context,
    UDT_ARGUMENT_T * argument,
    RETCODE_T        response
    )
{
    MTOK_ADAPTER    * adap  = (MTOK_ADAPTER *) context;
    STATS_T         * stats = &adap->udi_data.res.stats;
    MTOK_COMPLETION * comp  = argument->client_argument;
    char            * mode;
    char            * speed;
    
        
    /*
     * If an open command has just completed then we change the adapter's
     * state.
     */
     
    if (comp->command == MTOK_CMD_OPEN)
    {
        /* 
         * Change the adapter's state. The state changes to open if the 
         * adapter opened successfully and to closed if the adapter did not 
         * open successfully.
         */
         
        if (response == SUCCESS)
        {
            /*
             * If the higer layers have called mtok_open() then we need to 
             * set dev->tbusy to 0 to tell the higher layers that we can 
             * accept transmits.
             */
             
            if (adap->open_count > 0)
            {
				MTOK_FIRST_RESET_TBUSY(adap->dev);
            }

            /*
             * Tell the user that we've opened.
             */
             
            switch (stats->open_info.mode)
            {
                case OPEN_MODE_CLASSIC: 
                
                    mode = "classic";
                    break;

                case OPEN_MODE_DTR: 
                
                    mode = "DTR";
                    break;
                
                case OPEN_MODE_SINGLE: 
                
                    mode = "single-station";
                    break;
                    
                default:
                
                    mode = "an unknown";
                    break;
            }

            speed = mtok_ringspeed_to_string(stats->open_info.ring_speed);
                    
            printk(KERN_NOTICE "%s: The adapter has opened in %s mode at %s\n", 
                adap->dev->name, mode, speed);
            
            adap->state = MTOK_STATE_OPEN;
        }
        else
        {
            /*
             * We only tell the user that the open failed if this is the
             * initial open called by mtok_probe().
             */
             
            if (adap->state == MTOK_STATE_OPENING)
            {
                printk(KERN_WARNING "%s: The adapter has failed to open - "
                       "please check cabling\n", adap->dev->name);
            }           
            
            adap->state = MTOK_STATE_CLOSED;
        }
    }
    else 
    if (comp->command == MTOK_CMD_CLOSE)
    {
        if (response == SUCCESS)
            adap->state = MTOK_STATE_CLOSED;
    }
    
    /*
     * Record the completion details in the completion structure for anyone
     * who is waiting. We set the completed flag last to avoid a race
     * condition with mtok_await_completion() below. (An alternative
     * would be to disable interrupts in mtok_await_completion().)
     */
     
    comp->retc      = response;
    comp->completed = TRUE;

    /*
     * Signal our waiting process in mtok_await_completion that
     * it is OK to wake up now.
     */    

#ifdef SLEEP_WAIT
    wake_up_interruptible(&adap->command_wait);
#endif
}

/*============================================================================
|
| INTERRUPT HANDLING
|
|===========================================================================*/

/*----------------------------------------------------------------------------
|
| Function    - mtok_isr
|
| Parameters  - irq    -> The interrupt number of the device that caused the
|                         interrupt.
|               dev_id -> Pointer to device structure.
|               regs   -> Pointer to the saved processor registers.
|
| Purpose     - Our interrupt service routine.
|
| Returns     - Nothing.
|               
|---------------------------------------------------------------------------*/

void mtok_isr IRQ_ARGS(int irq, void * dev_id,struct pt_regs * regs)
{
    MTOK_ADAPTER * adap;
    UINT_T         req_action;

#ifdef DEBUG 
    QPOKE(PO_YELLOW,"{i");
#endif
    /*
     * Recover our adapter structure pointer (see ver.h).
     */

    adap = ADAPTERFROMIRQ(irq,dev_id);

    /*
     * Grab our ISR lock.
     */
     
    MTOK_ISR_LOCK(&adap->isr_lock);


    /*
     * Flag to the kernel that we've entered our interrupt handler.
     */
         

	MTOK_SET_INTR(adap->dev);

    /*
     * Call the UDI to find out if we caused the interrupt and if so
     * why.
     */
     
    req_action = UDI_READ_ADAPTER_INTERRUPT_STATUS(
                    &adap->udi_data);
    
    /*
     * If some event happened that needs processing then call the UDT
     * to process the event. This can result in a variety of upcalls.
     */
     
    if (req_action == UDT_INTERRUPT_PROCESSING_REQUIRED)
    {
        UDI_PROCESS_ADAPTER_INTERRUPTS(&adap->udi_data);
    }

    
    /*
     * Flag to the kernel that we've left our interrupt handler.
     */
     
	MTOK_RESET_INTR(adap->dev);

    /*
     * Drop our ISR lock.
     */
     
    MTOK_ISR_UNLOCK(&adap->isr_lock);
    
#ifdef DEBUG 
    QPOKE(PO_YELLOW,"}");
#endif
}

/*************************************************************************/
/*
 * RECEIVE SUBSYSTEM
 */
/*************************************************************************/

#define MTOK_NA_EQ(_a1, _a2) \
    ((((WORD_T *) (_a1))[2]       ) == (((WORD_T *) (_a2))[2]       ) && \
     (((WORD_T *) (_a1))[1]       ) == (((WORD_T *) (_a2))[1]       ) && \
     (((BYTE_T *) (_a1))[1]       ) == (((BYTE_T *) (_a2))[1]       ) && \
     (((BYTE_T *) (_a1))[0] & 0x7f) == (((BYTE_T *) (_a2))[0] & 0x7f))

static void mtok_free_rx_descriptors(LINUXDEV *dev)
{
    MTOK_ADAPTER *adap = (MTOK_ADAPTER *)dev->priv;
    
    if (adap->rx_list)
    {
        FRAME_T * Frame = adap->rx_list;
        while(Frame)
        {
            if (Frame->driver_ptr)
            {
                MTOK_DEV_KFREE_SKB(Frame->driver_ptr);
                Frame->driver_ptr = NULL;
            }
            Frame = Frame->next;
        }
        kfree(adap->rx_list);
        adap->rx_list = NULL;
    }
}

#ifdef FIX_SR
/* The minimum is TR_MAXRIFLEN + 8 
 * Make HDR_SPARE_ROOM 32 for maximum alignment efficiency.
 */
#define HDR_SPARE_ROOM  (TR_MAXRIFLEN + 14)
#endif

static int mtok_alloc_rx_skb(LINUXDEV * dev,FRAME_T *frame,int flags)
{
    struct sk_buff * skb;
    FRAG_HANDLE_T  * frag;
    MTOK_ADAPTER  * adap = (MTOK_ADAPTER *)dev->priv;
    DWORD_T max_frame_len = adap->max_frame_size;

#ifdef FIX_SR
    /* 
     * Under Linux 2.0.x we need to pass up frames with fixed length headers.
     * Heres the plan:-
     * Allocate extra space in front of the skb buffer so we can copy the TR
     * header down if necessary.  Allocate space for dummy LLC/SNAP header too.
     */
    skb = alloc_skb(max_frame_len+HDR_SPARE_ROOM,flags);
#else
    skb = alloc_skb(max_frame_len,flags);
#endif

    if (!skb)
    {
        return -ENOMEM;
    }

#ifdef FIX_SR
    skb_reserve(skb,HDR_SPARE_ROOM);
#endif

    frag = frame->frag_array_ptr;
    frag->data = skb->data;
    frag->phys = virt_to_bus(frag->data);
    frag->length = max_frame_len;
    frame->frame_length = max_frame_len;
    frame->driver_ptr = (VOID_T *) skb;
    return 0;
}

static int mtok_alloc_rx_descriptors(LINUXDEV *dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dev->priv;
    FRAG_HANDLE_T * frag_handles;
    int i;
    int num_slots = GET_CONFIG_VALUE(CONFIG_FROM_UDIDS(&adap->udi_data),rx_slots);
    int rx_desc_size = sizeof(FRAME_T) * num_slots + sizeof(FRAG_HANDLE_T) * num_slots + 16;

    if (adap->rx_list)
    {
        return -ENOMEM;
    }

    adap->rx_list = (FRAME_T *)kmalloc(rx_desc_size,GFP_KERNEL);

    if (!adap->rx_list)
        return -ENOMEM;

    memset(adap->rx_list,0,rx_desc_size);
    
    frag_handles = (FRAG_HANDLE_T *) (((char *)adap->rx_list) + (sizeof(FRAME_T) * num_slots));

    for(i=0;i<num_slots;i++)
    {
        adap->rx_list[i].next = (i == num_slots-1) ? NULL : &adap->rx_list[i+1];
        adap->rx_list[i].frag_array_ptr = &frag_handles[i];
        adap->rx_list[i].frag_array_length = 1;
        adap->rx_list[i].xfer_complete = NULL; /* Not used for rx descriptors */

        if (mtok_alloc_rx_skb(dev,&adap->rx_list[i],GFP_KERNEL) < 0)
        {
            if (i==0)
            {
                kfree(adap->rx_list);
                adap->rx_list = NULL;
                return -ENOMEM;
            } else {
                /*
                 * Stop allocating SKB's
                 * Terminate the list by setting the previous slots next ptr to NULL.
                 */
                adap->rx_list[i-1].next = NULL;
                printk(KERN_WARNING "%s: Low memory only using %d RX slots\n",dev->name,i);
                break;
            }
        }
    }
    return 0;
}

#ifdef LINUX20
inline void __skb_trim(struct sk_buff *skb, int len)
{
    skb->len=len;
    skb->tail=skb->data+len;
}
#endif

#ifdef CONFIG_MADGE_TEST_XID
inline void mtok_test_xid_response(MTOK_ADAPTER * adap,const FRAME_T * frame)
{
    BYTE_T         * data;
    int cmd,tmp;
    u8  sr;
    struct trh_hdr * trhdr;
    struct trllc   * llchdr;
    struct sk_buff * skb = frame->driver_ptr;            
    struct sk_buff * newskb;
    
    data = frame->frag_array_ptr->data;
    trhdr = (struct trh_hdr *)data;
    data += sizeof(struct trh_hdr) - TR_MAXRIFLEN;
    
    sr = trhdr->saddr[0] & TR_RII;

    if (sr)
        llchdr = (struct trllc *)(data + ((ntohs(trhdr->rcf) & TR_RCF_LEN_MASK) >> 8));
    else
        llchdr = (struct trllc *)data;

    /* Return if normal data frame */
    if (llchdr->llc == UI_CMD)
        return;

    cmd = llchdr->llc & LLC_TEST_XID_MASK;

    /* Check if it is a Test/XID frame destined for us. */

    if (!((cmd == LLC_TEST || cmd == LLC_XID) && 
        (llchdr->dsap == 0) &&
        (memcmp(trhdr->daddr,adap->dev->dev_addr,TR_ALEN)==0)  ))
        return;     
        
    /* We got one we should reply */

    printk(KERN_INFO "%s: Received Test/XID frame\n",adap->dev->name);

    if (adap->tx_free_list == NULL)
        return;

    __skb_trim(skb,frame->frame_length);

    newskb = skb_copy(skb,GFP_ATOMIC);

    data = newskb->data;
    trhdr = (struct trh_hdr *)data;
    data += sizeof(struct trh_hdr) - TR_MAXRIFLEN;

    if (sr)
        llchdr = (struct trllc *)(data + ((ntohs(trhdr->rcf) & TR_RCF_LEN_MASK) >> 8));
    else
        llchdr = (struct trllc *)data;

    /* Swap header around to make a frame header going back */          
    trhdr->ac = AC;
    trhdr->saddr[0] &= ~TR_RII; /* Turn off source routing bit */
    memcpy(trhdr->daddr,trhdr->saddr,TR_ALEN);
    memcpy(trhdr->saddr,adap->dev->dev_addr,TR_ALEN);
    tmp = llchdr->dsap;
    llchdr->dsap = llchdr->ssap;
    llchdr->ssap = tmp | LLC_RESPONSE;

    if (sr) {
        /* Turn on source routing */
        trhdr->saddr[0] |= TR_RII;
        /* Reverse direction of source routing */
        trhdr->rcf = htons( (ntohs(trhdr->rcf) ^ TR_RCF_DIR_BIT) &
                            ~TR_RCF_BROADCAST_MASK);
    }
            
    if ((llchdr->llc & LLC_TEST_XID_MASK) == LLC_XID) {
        char * data = (char *)llchdr + sizeof(struct trllc) - 5;
        data[0] = LLC_XID_FMTID;
        data[1] = LLC_XID_SERVICES;
        data[2] = 0;
        __skb_trim(newskb, (data - (char*)trhdr) + 3);
    }
    mtok_hardware_send_frame(adap,newskb);
}
#endif

static void mtok_rx_complete(P_DRIVER_T context,P_FRAME_T  frame)
{
    MTOK_ADAPTER   * Adapter  = (MTOK_ADAPTER *) context;
    struct sk_buff * skb;

#ifdef DEBUG 
    QPOKE(PO_CYAN,"[RX");
#endif

    if (frame->frame_length < 14) 
    {
#ifdef DEBUG 
        QPOKE(PO_RED,"TooShort!");
#endif
        Adapter->stats.rx_errors++;
        goto FinishedRX;
    }

#ifdef CONFIG_MADGE_TEST_XID
    mtok_test_xid_response(Adapter,frame);
#endif

    /*
     * For really short frames create an exact sized skb to save memory
     * For long frames we just pass up the skb as is to avoid an additional copy.
     */
#define COPY_THRESHOLD  256  
    if (frame->frame_length < COPY_THRESHOLD)
    {
#ifdef DEBUG 
        QPOKE(PO_GREEN,"Short");
#endif
#ifdef FIX_SR
        if (!(skb = alloc_skb(frame->frame_length+HDR_SPARE_ROOM,GFP_ATOMIC)))
        {
#ifdef DEBUG 
            QPOKE(PO_RED,"NoMem!!");
#endif
            Adapter->stats.rx_dropped++;
            goto FinishedRX;
        }
        /* Under 2.0.x reserve HDR_SPARE_ROOM bytes at start of frame */
        skb_reserve(skb,HDR_SPARE_ROOM);
#else
        if (!(skb = alloc_skb(frame->frame_length,GFP_ATOMIC)))
        {
#ifdef DEBUG 
            QPOKE(PO_RED,"NoMem!!");
#endif
            Adapter->stats.rx_dropped++;
            goto FinishedRX;
        }
#endif
        memcpy(skb_put(skb,frame->frame_length),frame->frag_array_ptr->data,frame->frame_length);
    }
    else
    {
#ifdef DEBUG 
        QPOKE(PO_GREEN,"Long");
#endif
        if (!(skb = frame->driver_ptr))
        {
#ifdef DEBUG 
            QPOKE(PO_RED,"!!!NullSKBInFrame!!!");
            QPOKE(PO_CYAN,"]");
#endif
            return;
        }

#ifdef DEBUG 
        QPOKEW(PO_YELLOW,"Len=",frame->frame_length);
#endif

        __skb_trim(skb,frame->frame_length);

        if (mtok_alloc_rx_skb(Adapter->dev,frame,GFP_ATOMIC) < 0)
        {
#ifdef DEBUG 
            QPOKE(PO_RED,"LowMem!");
#endif

            /* TODO really at this point we should pass the frame up because it is fine
               and try to renew the receive buffer at some later point
               so maintain a list of FRAME_Ts that have been received but could not
               be reallocated.
               */

            frame->frame_length = Adapter->max_frame_size;
            Adapter->stats.rx_dropped++;
            goto FinishedRX;
        }
    }

#ifdef DEBUG
    { int i; for(i=0;i<10;i++) QPOKEB(PO_GREEN," ",skb->data[i]); }
#endif

    if ( (((struct trh_hdr *)skb->data)->daddr[0] & 0x80) != 0)
    {
        Adapter->stats.multicast++;
    }    

    skb->dev      = Adapter->dev;

#ifdef FIX_SR
    mtok_fix_rx_sr(skb);
#endif

    skb->protocol = tr_type_trans(skb, Adapter->dev);

    Adapter->stats.rx_packets++;
#ifdef LINUX21
    Adapter->stats.rx_bytes+=skb->len;
#endif

    /* Note the time of the receive for the power management code. */
     
    Adapter->dev->last_rx = jiffies;

    netif_rx(skb);

FinishedRX:
    frame->next = NULL;
    frame->frame_length = Adapter->max_frame_size;
    UDTI_COMPLETE_RX(&Adapter->udi_data, frame);
#ifdef DEBUG 
    QPOKE(PO_CYAN,"]");
#endif
}

#ifdef FIX_SR

#define HDR ((struct trh_hdr *)skb->data)
/*
 * Pad the source routing information to a fixed number of bytes
 * The skb must have enough reserved space at the start.
 */
inline void mtok_fix_rx_sr(struct sk_buff *skb)
{
    int srlen = 0,is802 = 0;
    struct trllc * wirellc,*fakellc;
    int pad; /* How far down in memory must the headers be moved */
            
#ifdef DEBUG 
    QPOKE(PO_MAGENTA,"{FixSR");
#endif
    
    if (HDR->saddr[0] & TR_RII)
        srlen =  (ntohs(HDR->rcf) & TR_RCF_LEN_MASK) >> 8;

    wirellc = (struct trllc *)(skb->data + sizeof(struct trh_hdr) - TR_MAXRIFLEN + srlen);

    pad = TR_MAXRIFLEN-srlen;

    if (wirellc->dsap != 0xAA || wirellc->ssap != 0xAA)
    {
        is802 = 1;
        pad += 8;
    }
    
    if (pad)
    {
        /* Move the whole lot of header down in memory */
        skb->data-=pad;
        skb->len+=pad;
        /* Use memmove for overlapping areas */
        memmove(skb->data,skb->data+pad,14+srlen);
        memset(skb->data+srlen+14,0,pad);
    }

    if(is802)
    {
        /* Must fake up an LLC and SNAP header before the real LLC for IPX
         * packets.  Use packet type 802.2TR when running ipx_interface.
         *  3  |  5    |  3
         * LLC | SNAP  | LLC
         * Zero| 0x1100| 
         *    Faked    | Wire
         */
        fakellc = (struct trllc *)(skb->data + sizeof(struct trh_hdr));      
        /* Move the LLC header from the wire into position */
        memmove(fakellc+1,wirellc,3);

        /* Fake up LLC and SNAP */
        memset(fakellc,0,sizeof(struct trllc));
        fakellc->ethertype = htons(ETH_P_TR_802_2);
    }
    

    
#ifdef DEBUG 
    QPOKE(PO_MAGENTA,"}");
#endif
}
#undef HDR
#endif

/*************************************************************************
 * TRANSMIT SUBSYSTEM
 *************************************************************************/

/*
 * Allocate a FRAME_T descriptor for each TX slot.
 * Arrange them in a linked list.
 * Allocate an array of FRAG_HANDLE_T.
 * Only one FRAG_HANDLE_T is used for each Frame
 *  
 * This routine does not allocate any memory for the actual
 * frame data because we will transmit straight from the
 * skb given to us
 */

static void mtok_free_tx_descriptors(LINUXDEV * dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dev->priv;

    if (adap->tx_list)
        kfree(adap->tx_list);
    adap->tx_list = NULL;
}

static int mtok_alloc_tx_descriptors(LINUXDEV * dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dev->priv;
    FRAG_HANDLE_T * frag_handles;
    int i;
    int num_slots = GET_CONFIG_VALUE(CONFIG_FROM_UDIDS(&adap->udi_data),tx_slots);
    int tx_desc_size = sizeof(FRAME_T) * num_slots + sizeof(FRAG_HANDLE_T) * num_slots + 16;

    adap->tx_list = (adap->tx_free_list = (FRAME_T *)kmalloc(tx_desc_size,GFP_KERNEL));
    
    if (!adap->tx_free_list)
        return -ENOMEM;

    memset(adap->tx_list,0,tx_desc_size);

    
    frag_handles = (FRAG_HANDLE_T *) ((char *)adap->tx_free_list + (sizeof(FRAME_T) * num_slots));

    for(i=0;i<num_slots;i++)
    {
        adap->tx_free_list[i].next = (i == num_slots-1) ? NULL : &adap->tx_free_list[i+1];
        adap->tx_free_list[i].frag_array_ptr = &frag_handles[i];
        adap->tx_free_list[i].frag_array_length = 1;
        adap->tx_free_list[i].xfer_complete = mtok_transfer_complete;
    }
    return 0;
}

/*----------------------------------------------------------------------------
|
| Function    - MtokTXComplete
|
| Parameters  - context -> Pointer to an adapter structure.
|               fs      -> Frame status.
|               status  -> Was the frame sent successfully?
|               handle  -> Frame handle.
|
| Purpose     - Upcall made by the UDT to notify that a transmit has
|               completed.
|
| Returns     - Nothing.
|
| Notes       - This upcall will be made out of our interrupt handler.
|
---------------------------------------------------------------------------*/

static void mtok_tx_complete(P_DRIVER_T context,BYTE_T fs,BOOLEAN_T status,UINT_T handle)
{
}

static void mtok_transfer_complete(P_DRIVER_T context,P_FRAME_T frame,UINT_T handle)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *) context;


    /*
     * Return the skb to the system now it has been transfered to the
     * adapter.
     */

#ifdef MTOK_SANITY_CHECK
    if (!frame)
    {
        return;
    }
#endif
    
    if (frame->driver_ptr)
        MTOK_DEV_KFREE_SKB_IRQ(frame->driver_ptr);

    /*
     * For Sanity set skb back to NULL
     * (Also in future it may be required to be null if
     *  not transmitting straight out of skb)
     */
    
    frame->driver_ptr = NULL;

    /*
     * Note that the frame is no longer in use by putting it back onto
     * the free list.
     */

    frame->next = adap->tx_free_list;
    adap->tx_free_list = frame;
    
    if (adap->open_count > 0)
    {
        /*
         * Set dev->tbusy to 0 to tell the higher layers that we we are 
         * no-longer busy - assuming they think we're open.
         */
        if (adap->tx_full && (MTOK_TEST_TBUSY(adap->dev)))
        {
            adap->tx_full = 0;

			MTOK_RESET_TBUSY(adap->dev);
			MTOK_MARK_BH();
        }
    }
}


/*
 * This routine is entered from an interrupt context (off a transfer
 * complete) and also from a process context from a send packet call.
 *
 * Must use spin locks before you get here to ensure it is thread safe
 *
 * Give one (at a time) frame to hardware
 */
static void mtok_hardware_send_frame(MTOK_ADAPTER *Adapter,struct sk_buff *skb)
{
    FRAME_T * Frame;

    /*
     * While we have spare Transmit Descriptors and there are
     * frames queued keep going.
     */


    /*
     * Get a hardware transmit slot
     */

    Frame = Adapter->tx_free_list;
    Adapter->tx_free_list = Frame->next;

#ifdef MTOK_SANITY_CHECK
    if (Frame == NULL)
    {
        printk(KERN_ERR "%s: No free TX slots in mtok_hardware_send_frame\n",Adapter->dev->name);
        return;
    }
#endif


    Frame->driver_ptr = (VOID_T *)skb;
    Frame->frag_array_ptr->data = skb->data;
    Frame->frag_array_ptr->phys = virt_to_bus(skb->data);
    Frame->frag_array_ptr->length = skb->len;
    Frame->frame_length = skb->len;
    Frame->next = NULL;
    
    Adapter->dev->trans_start = jiffies;
    
    
    if (SUCCESS == UDTI_TX(&Adapter->udi_data,Frame))
    {
        /*
         * The frame was successfully given to the adapter
         */
        Adapter->stats.tx_packets++;

#ifdef LINUX21
        Adapter->stats.tx_bytes+=skb->len;
#endif
    }
    else
    {

        /*
         * An error occured so give the frame descriptor back
         * to the free list and record the error in the statistics
         */
        Frame->next = Adapter->tx_free_list;
        Adapter->tx_free_list = Frame;
        Adapter->stats.tx_errors++;
    }
}

#define TX_TIMEOUT  (HZ)

/*
 * Return Codes:
 *   0 - driver queued packet, link is unavailable, or skbuff was bad
 *   1 - caller should requeue the sk_buff for later transmission
 *
 * I have been unable to find much definitive information on the use of
 * tbusy flag.  There appears to be conflicting views on the mailing list.
 * But this does seem to be clear:-
 *      tbusy MUST be set if 1 is returned.
 */
static int mtok_transmit(struct sk_buff *skb,LINUXDEV *dev)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *) dev->priv;

#ifdef DEBUG 
    QPOKE(PO_GREEN,"[TX");
    QPOKEP(PO_CYAN,"skb=",skb);

#endif

#ifdef MTOK_SANITY_CHECK
    if (dev == NULL)
    {
        printk(KERN_ERR "mtok_transmit dev is NULL\n");
        if (skb)
            MTOK_DEV_KFREE_SKB(skb);
        adap->stats.tx_dropped++;
        return 0;
    }

    if (skb == NULL)
    {
        adap->stats.tx_dropped++;
        printk(KERN_ERR "%s: mtok_transmit skb is NULL\n",dev->name);
        return 0;
    }
    
    if (adap == NULL)
    {
        MTOK_DEV_KFREE_SKB(skb);
        adap->stats.tx_dropped++;
        printk(KERN_ERR "%s: mtok_transmit adap is NULL\n",dev->name);
        return 0;
    }

    if (adap->open_count == 0)
    {
        MTOK_DEV_KFREE_SKB(skb);
        adap->stats.tx_dropped++;
        printk("%s: mtok_transmit open_count==0\n",dev->name);
        return 0;
    }

    if (skb->len < 14 || skb->len > adap->max_frame_size)
    {
        printk("%s: mtok_transmit SKB Length Error (size = %u max = %d)\n",dev->name,(unsigned int)skb->len,adap->max_frame_size);
        MTOK_DEV_KFREE_SKB(skb);
        adap->stats.tx_errors++;
        return 0;
    }

    if (adap->state != MTOK_STATE_OPEN)
    {
        printk("%s: mtok_transmit called when adapter not open\n",dev->name);
        MTOK_DEV_KFREE_SKB(skb);
        adap->stats.tx_dropped++;
        return 0;
    }
#endif

    /*
     * Set bit zero of tbusy
     * (i.e. dev->tbusy = 1)
     * and see what the value of tbusy was before it was set
     * All done in one atomic operation
     *
     * The atomic operation must be used to block a timer based
     * transmit from overlapping
     */


	MTOK_LOCK(&adap->tx_lock);

	if (MTOK_TEST_TBUSY(dev))
	{
        if (jiffies - dev->trans_start >= TX_TIMEOUT)
        {
#ifdef DEBUG 
            QPOKE(PO_YELLOW,"TXTimeOut");
#endif
        }
#ifdef DEBUG 
        QPOKE(PO_MAGENTA,dev->name);
        QPOKE(PO_CYAN,"TXConflict");
        QPOKE(PO_GREEN,"]");
#endif

		MTOK_UNLOCK(&adap->tx_lock);

        return 1;
	}
	
	MTOK_SET_TBUSY(dev);

#ifdef FIX_SR
    /* Compresses the source route information in the frame */
    skb = mtok_fix_tx_sr(skb);
    if (!skb)
    {
        /*
         * If skb is NULL then the alignment requirements could not
         * be met so we drop the frame.
         */
#ifdef DEBUG 
        QPOKE(PO_RED,"AlignErr");
#endif
        adap->stats.tx_dropped++;
        dev->tbusy = 0;
#ifdef DEBUG 
        QPOKE(PO_GREEN,"]");
#endif
        return 0;
    }
#endif

    MTOK_LOCK(&adap->isr_lock);

#ifdef MTOK_SANITY_CHECK
    /*
     * Check to see if we have any UDT frame descriptors available
     * This shouldn't happen because we leave tbusy set if we have
     * no free tx slots and we reset tbusy to 0 when we get a tx 
     * complete (i.e. a tx slot is freed up)
     */
    
    if (adap->tx_free_list == NULL)
    {
        printk(KERN_ERR "%s: mtok_transmit no free TX slots\n",dev->name);
#ifdef DEBUG 
        QPOKE(PO_RED,"NoTXSlots");
#endif
        MTOK_UNLOCK(&adap->isr_lock);
#ifdef DEBUG 
        QPOKE(PO_GREEN,"]");
#endif
        return 1;
    }
#endif

#ifdef DEBUG 
    QPOKE(PO_WHITE,"(");
#endif
    mtok_hardware_send_frame(adap,skb);
#ifdef DEBUG 
    QPOKE(PO_WHITE,")");
#endif



    if (adap->tx_free_list != NULL)
		MTOK_RESET_TBUSY(dev);
    else
    {
        adap->tx_full = 1;
#ifdef DEBUG 
        QPOKE(PO_MAGENTA,"TXQFull");
#endif
    }

    dev->trans_start = jiffies;
    MTOK_UNLOCK(&adap->isr_lock);
#ifdef DEBUG 
    QPOKE(PO_GREEN,"]");
#endif

	MTOK_UNLOCK(&adap->tx_lock);

    return 0;
}

#ifdef FIX_SR
inline struct sk_buff * mtok_fix_tx_sr(struct sk_buff *skb)
{
    struct trh_hdr *trh = (struct trh_hdr *)skb->data;
    unsigned char * buf = skb->data;
    int len=0,sr=0;

    sr = trh->saddr[0] & TR_RII;

    /*
     * The IPX stack doesn't set the source address in the frame so we
     * need to do it here. (Under older versions of Linux)
     */

    memcpy((void *) trh->saddr, (void *) skb->dev->dev_addr, skb->dev->addr_len);
    
    if(sr) {
        len = (ntohs(trh->rcf) & TR_RCF_LEN_MASK) >> 8;
        trh->saddr[0]|=TR_RII;
        if(len < TR_MAXRIFLEN) {
            memmove(&buf[TR_MAXRIFLEN-len],trh,sizeof(struct trh_hdr)-TR_MAXRIFLEN+len);
            skb->data = &buf[TR_MAXRIFLEN-len];
            skb->len -= (TR_MAXRIFLEN - len);
        }
    }

    /*
     * Some Smart adapters (the MC32 and EISA adapters) require DWORD aligned
     * RAM fragments. Knossos adapters require word alignment. There is no
     * alignment requirement for Blue adapters. It is thus generally best
     * to ensure dword alignment of all the RAM fragments described by the
     * fragment handles in this array.
     * However, this driver does not support MC32 or EISA so its minimal
     * alignment requirement is WORD aligned (for KNOSSOS).
     * Source routing info is always an even number of bytes hence we should
     * never be non word aligned hence only do this check if sanity checking
     * (but SNAP header is odd number of bytes)
     */
#ifdef MTOK_SANITY_CHECK
    if (((unsigned int)skb->data) & 1)
    {
        printk(KERN_ERR "%s: Buffer is not word aligned\n",skb->dev->name);
        MTOK_DEV_KFREE_SKB(skb);
        return NULL;
    }
#endif
    skb->h.raw = skb->mac.raw = skb->data;

    return skb;
}
#endif /* FIX_SR */

#if defined(CONFIG_PROC_FS) && defined(CONFIG_MADGE_BACK_COMPAT)

static int mtok_proc_info(char *buffer, char **start, off_t offset, int length, int dummy)
{
    int len = 0;
    off_t pos = 0;
    off_t begin = 0;
    MTOK_ADAPTER * adap;
    struct dev_list_node * mtok_card = mtok_dev_list;
    

    len += sprintf(buffer,"%s\n",MTOK_VERSION);

    while(mtok_card)
    {
            STATS_T         * stats;
            char buf1[14];
            char buf2[14];
    
            adap = (MTOK_ADAPTER *)mtok_card->dev->priv;
            mtok_card = mtok_card->next;


            stats = &adap->udi_data.res.stats;
            
            len += sprintf(buffer + len,"%s %d %d %s %s %X %X %d %d %d %d %x\n",    
                                                adap->dev->name,
                                                adap->type,
                                                adap->subtype,
                                                mtok_bia_to_string(&adap->bia.byte[0],buf1),
                                                mtok_bia_to_string(adap->dev->dev_addr,buf2),
                                                adap->io_base[0],
                                                (unsigned int)(0x80000000L |
                                                (PCI_BUS_NUMBER(adap->busdevfn) << 8) |
                                                PCI_DEVICE_NUMBER(adap->busdevfn)),
                                                adap->xfer_mode,
                                                adap->irq_number,
                                                stats->open_info.ring_speed,
                                                stats->open_info.mode,
                                                (int)adap->shared_ram
                                            );
            pos = begin + len;
            if (pos < offset) {
                len = 0;
                begin = pos;
            }

            if (pos > offset + length)
                break;          
    }
    
    *start = buffer + (offset - begin);
    len -= (offset - begin);
    if (len > length) len = length;
    return len; 
}
#endif


/*
 * SSI Functions
 *
 */

/*----------------------------------------------------------------------------
|
| Function    - mtok_16bit_pio_from_adapter
|
| Parameters  - io_port -> The I/O location from which to read.
|               buf_ptr -> The buffer in which to put the data.
|               count   -> Number of bytes to PIO.
|               
| Purpose     - Transfer an integral number of 16 bit words from an adapter
|               by programmed I/O.
|
| Returns     - The number of single bytes left to be transferred.
|
|---------------------------------------------------------------------------*/

inline UINT_T
mtok_16bit_pio_from_adapter(
    IO_HANDLE_T   io_port,
    char        * buf_ptr,
    UINT_T        count
    )
{
    if ((count / 2) > 0)
    {
        insw(io_port, (void *) buf_ptr, count / 2);
    }
    
    return count % 1;
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_16bit_pio_to_adapter
|
| Parameters  - io_port -> The I/O location at which to write.
|               buf_ptr -> The buffer from which to get the data.
|               count   -> Number of bytes to PIO.
|               
| Purpose     - Transfer an integral number of 16 bit words to an adapter
|               by programmed I/O.
|
| Returns     - The number of single bytes left to be transferred.
|
|---------------------------------------------------------------------------*/

inline UINT_T
mtok_16bit_pio_to_adapter(
    IO_HANDLE_T   io_port,
    char        * buf_ptr,
    UINT_T        count
    )
{
    if ((count / 2) > 0)
    {
        outsw(io_port, (void *) buf_ptr, count / 2);
    }
    
    return count % 1;
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_32bit_pio_from_adapter
|
| Parameters  - io_port -> The I/O location from which to read.
|               buf_ptr -> The buffer in which to put the data.
|               count   -> Number of bytes to PIO.
|               
| Purpose     - Transfer an integral number of 32 bit words from an adapter
|               by programmed I/O and then transfer any remaining 16bit
|               word.
|
| Returns     - The number of single bytes left to be transferred.
|
|---------------------------------------------------------------------------*/

inline UINT_T
mtok_32bit_pio_from_adapter(
    IO_HANDLE_T   io_port,
    char        * buf_ptr,
    UINT_T        count
    )
{
    if ((count / 4) > 0)
    {
        insl(io_port, (void *) buf_ptr, count / 4);
    }

    if ((count % 4) > 1)
    {
        *((WORD_T *) (buf_ptr + (count & ~3))) = (WORD_T) inw(io_port);
    }
        
    return count % 2;
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_32bit_pio_to_adapter
|
| Parameters  - io_port -> The I/O location at which to write.
|               buf_ptr -> The buffer from which to get the data.
|               count   -> Number of bytes to PIO.
|               
| Purpose     - Transfer an integral number of 32 bit words to an adapter
|               by programmed I/O and then transfer any remaining 16bit
|               word.
|
| Returns     - The number of single bytes left to be transferred.
|
|---------------------------------------------------------------------------*/

inline UINT_T
mtok_32bit_pio_to_adapter(
    IO_HANDLE_T   io_port,
    char        * buf_ptr,
    UINT_T        count
    )
{
    if ((count / 4) > 0)
    {
        outsl(io_port, (void *) buf_ptr, count / 4);
    }

    if ((count % 4) > 1)
    {
        outw((unsigned short) *((WORD_T *) (buf_ptr + (count & ~3))), io_port);
    }
        
    return count % 2;
}




/*----------------------------------------------------------------------------
|
| Function    - mtok_enable_isa_dma
|
| Parameters  - channel -> The DMA channel to be enabled.
|
| Purpose     - Enable an ISA DMA channel for BUS master DMA.
|
| Returns     - TRUE.
|
|---------------------------------------------------------------------------*/

#ifdef ATULA_UDI

BOOLEAN_T
mtok_enable_isa_dma(
    unsigned channel
    )
{
    /*
     * Put the channel into cascade mode.
     */
     
    set_dma_mode(channel, DMA_MODE_CASCADE);
    
    /*
     * Enable the channel.
     */
     
    enable_dma(channel);
    
    return TRUE;
}

#endif /* ATULA_UDI */


/*----------------------------------------------------------------------------
|
| Function    - mtok_disable_isa_dma
|
| Parameters  - channel -> The DMA channel to be disabled.
|
| Purpose     - Disable an ISA DMA channel.
|
| Returns     - TRUE.
|
|---------------------------------------------------------------------------*/

#ifdef ATULA_UDI

void
mtok_disable_isa_dma(
    unsigned channel
    )
{
    /*
     * Enable the channel.
     */
     
    disable_dma(channel);
}

#endif /* ATULA_UDI */

/*============================================================================
|
| PCI HELPER FUNCTIONS USED BY THE UNIVERSAL DRIVER TOOLKIT
|
|===========================================================================*/

/*----------------------------------------------------------------------------
|
| Function    - mtok_pci_readb
|
| Parameters  - bdf    -> Bus/Device/Function for the PCI device for which
|                         configuration space is to be read.
|               offset -> Offset of the data to be read.
|               
| Purpose     - Read a BYTE_T (8 bits) from PCI configuration space.
|
| Returns     - The BYTE_T read.
|
|---------------------------------------------------------------------------*/

inline BYTE_T mtok_pci_readb( 
    PCI_HANDLE_T bdf,
    unsigned     offset
    )
{
    unsigned char value;
#ifdef OLD_PCI
    unsigned char bus;
    unsigned char dev_func;
    bus      = (unsigned char) ((bdf >> 8) & 0xff);
    dev_func = (unsigned char) (bdf & 0xff);
    pcibios_read_config_byte(bus, dev_func, (unsigned char) offset, &value);
#else
    pci_read_config_byte(bdf,offset,&value);
#endif    
    return (BYTE_T) value;
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_pci_readw
|
| Parameters  - bdf    -> Bus/Device/Function for the PCI device for which
|                         configuration space is to be read.
|               offset -> Offset of the data to be read.
|               
| Purpose     - Read a WORD_T (16 bits) from PCI configuration space.
|
| Returns     - The WORD_T read.
|
|---------------------------------------------------------------------------*/

inline WORD_T mtok_pci_readw( 
    PCI_HANDLE_T bdf,
    unsigned     offset
    )
{
    unsigned short value;
#ifdef OLD_PCI
    unsigned char  bus;
    unsigned char  dev_func;
    bus      = (unsigned char) ((bdf >> 8) & 0xff);
    dev_func = (unsigned char) (bdf & 0xff);
    pcibios_read_config_word(bus, dev_func, (unsigned char) offset, &value);
#else
    pci_read_config_word(bdf,offset,&value);
#endif    
    return (WORD_T) value;
}

/*----------------------------------------------------------------------------
|
| Function    - mtok_pci_readd
|
| Parameters  - bdf    -> Bus/Device/Function for the PCI device for which
|                         configuration space is to be read.
|               offset -> Offset of the data to be read.
|               
| Purpose     - Read a DWORD_T (32 bits) from PCI configuration space.
|
| Returns     - The DWORD_T read.
|
|---------------------------------------------------------------------------*/

inline DWORD_T mtok_pci_readd( 
    PCI_HANDLE_T bdf,
    unsigned     offset
    )
{
    unsigned int  value;
#ifdef OLD_PCI
    unsigned char bus;
    unsigned char dev_func;
    bus      = (unsigned char) ((bdf >> 8) & 0xff);
    dev_func = (unsigned char) (bdf & 0xff);
    
    pcibios_read_config_dword(bus, dev_func, (unsigned char) offset, &value);
#else
    pci_read_config_dword(bdf,offset,&value);
#endif    
    return (DWORD_T) value;
}
    
/*----------------------------------------------------------------------------
|
| Function    - mtok_pci_writeb
|
| Parameters  - bdf    -> Bus/Device/Function for the PCI device for which
|                         configuration space is to be written.
|               offset -> Offset of the data to be written.
|               value  -> The value to write.
|               
| Purpose     - Write a BYTE_T (8 bits) to PCI configuration space.
|
| Returns     - Nothing.
|
|---------------------------------------------------------------------------*/
inline void mtok_pci_writeb(PCI_HANDLE_T bdf,unsigned offset,BYTE_T value)
{
#ifdef OLD_PCI
    unsigned char bus;
    unsigned char dev_func;
    
    bus      = (unsigned char) ((bdf >> 8) & 0xff);
    dev_func = (unsigned char) (bdf & 0xff);
    
    pcibios_write_config_byte(
        bus, 
        dev_func, 
        (unsigned char) offset, 
        (unsigned char) value);
#else
    pci_write_config_byte(bdf,offset,value);
#endif
}

/*----------------------------------------------------------------------------
|
| Function    - mtok_pci_writew
|
| Parameters  - bdf    -> Bus/Device/Function for the PCI device for which
|                         configuration space is to be written.
|               offset -> Offset of the data to be written.
|               value  -> The value to write.
|               
| Purpose     - Write a WORD_T (16 bits) to PCI configuration space.
|
| Returns     - Nothing.
|
|---------------------------------------------------------------------------*/

inline void mtok_pci_writew( 
    PCI_HANDLE_T bdf,
    unsigned     offset,
    WORD_T       value
    )
{
#ifdef OLD_PCI
    unsigned char bus;
    unsigned char dev_func;
    
    bus      = (unsigned char) ((bdf >> 8) & 0xff);
    dev_func = (unsigned char) (bdf & 0xff);
    
    pcibios_write_config_word(
        bus, 
        dev_func, 
        (unsigned char) offset, 
        (unsigned short) value);
#else
    pci_write_config_word(bdf,offset,value);
#endif
}


/*----------------------------------------------------------------------------
|
| Function    - mtok_pci_writed
|
| Parameters  - bdf    -> Bus/Device/Function for the PCI device for which
|                         configuration space is to be written.
|               offset -> Offset of the data to be written.
|               value  -> The value to write.
|               
| Purpose     - Write a DWORD_T (32 bits) to PCI configuration space.
|
| Returns     - Nothing.
|
|---------------------------------------------------------------------------*/

inline void mtok_pci_writed( 
    PCI_HANDLE_T bdf,
    unsigned     offset,
    DWORD_T      value
    )
{
#ifdef OLD_PCI
    unsigned char bus;
    unsigned char dev_func;
    
    bus      = (unsigned char) ((bdf >> 8) & 0xff);
    dev_func = (unsigned char) (bdf & 0xff);
    
    pcibios_write_config_dword(
        bus, 
        dev_func, 
        (unsigned char) offset, 
        (unsigned int) value);
#else
    pci_write_config_dword(bdf,offset,value);
#endif
}


/*----------------------------------------------------------------------------
|
| Function    - ssi_read_tuple
|
| Parameters  - dds_ptr-> Driver Data Space Pointer.
|               tuple  -> Which tuple do we want.
|               buffer -> Where to put the tuple.
|               len    -> The length of the tuple data.
|
| Purpose     - Read the BIA from an Olicom PCMCIA adapter.
|
| Returns     - 0    - SUCCESS.
|               1    - Failure.
|
|---------------------------------------------------------------------------*/
#ifdef OLICOM_PCMCIA_UDI
WORD_T ssi_read_tuple(P_DRIVER_T dds_ptr,WORD_T tuple,VOID_T *buffer,int len)
{
    MTOK_ADAPTER * adap = (MTOK_ADAPTER *)dds_ptr;
    switch(tuple)
    {
        case RS_TUPLE:
#ifdef MTOK_SANITY_CHECK
            if (!adap->olicom_rs_tuple)
            {
                return 1;
            }
            if (len < sizeof(RS_TUPLE_T))
            {
                return 1;
            }
#endif /* MTOK_SANITY_CHECK */
            memcpy(buffer,adap->olicom_rs_tuple,sizeof(RS_TUPLE_T));
            break;
        case BIA_TUPLE:
#ifdef MTOK_SANITY_CHECK
            if (!adap->olicom_bia_tuple)
            {
                return 1;
            }
            if (len < sizeof(BIA_TUPLE_T))
            {
                return 1;
            }
#endif /* MTOK_SANITY_CHECK */
            memcpy(buffer,adap->olicom_bia_tuple,sizeof(BIA_TUPLE_T));
            break;
        default:
            return 1;
    }
    return 0;
}
#endif /* OLICOM_PCMCIA_UDI */


#ifndef EMBEDDED_MICRO_CODE
#include "mdgbin.h"
static int errno = 0;
#if LINUX_VERSION_CODE < 0x020100
#define open sys_open
#define close sys_close
#define read sys_read
#endif
static BOOLEAN_T do_mtok_code_open(CODE_TYPE_T type,CODE_HANDLE_T * handlep)
{
    char * fn;
    char * dp;
    IMAGE_HEADER_T *imghdr;
    FILE_HEADER_T filehdr;
    long l;
    int fd;
    WORD_T codetype;
    
    switch (type)
    {

#ifdef SMART_UDI
        case FASTMAC_PLUS_CODE:
            fn = BIN_FILE_PATH "/" FMP_FILENAME;
            break;
#endif /* SMART_UDI */

#ifdef KNOSSOS_UDI
        case KNOSSOS2_ADAP_CODE:
            fn = BIN_FILE_PATH "/" KNOS_FILENAME;
            break;
#endif /* KNOSSOS_UDI */

#ifdef OLICOM_HEBE_UDI
        case HEBE_ADAP_CODE:
            fn = BIN_FILE_PATH "/" HEBE_FILENAME;
            break;
#endif /* OLICOM_HEBE_UDI */

        default:
            return FALSE;
    }

    if ((fd = open(fn,0,0)) < 0) {
        printk(KERN_ERR "Unable to load download image - %s (%d)\n",fn,errno);
        return FALSE;
    }

    if (read(fd,(void*)&filehdr,sizeof(FILE_HEADER_T)) != sizeof(FILE_HEADER_T) || 
        filehdr.sig1 != SIG1 || filehdr.sig2 != SIG2) {
        printk(KERN_ERR "Invalid firmware - %s\n",fn);
        close(fd);
        return FALSE;
    }
    
    l = filehdr.len;
    
    if (NULL == (dp = vmalloc(l))) {
        printk(KERN_ERR "Out of memory loading - %s\n",fn);
        close(fd);
        return FALSE;
    }
    
    if (read(fd,dp,l) != l)
    {
        printk(KERN_ERR "Failed to read %s\n",fn);
        vfree(dp);
        close(fd);
        return FALSE;
    }
    
    close(fd);
    
    imghdr = (IMAGE_HEADER_T *)dp;
    codetype = (((WORD_T)imghdr->code_type[1]) << 8 | imghdr->code_type[0]);
    
    if (codetype != type)
    {
        printk(KERN_ERR "Wrong download image in %s\n",fn);
        vfree(dp);
        return FALSE;   
    }

    handlep->data = (WORD_T *)dp;
    handlep->offset = 0;
    return TRUE;    
}
#endif

#if LINUX_KERNEL_VERSION < 0x020145
typedef unsigned long mm_segment_t;
#endif

/*---------------------------------------------------------------------------
|
| Function    - mtok_code_open
|
| Parameters  - type    -> The type of download image to open.
|               handlep -> Pointer to a code handle.
|
| Purpose     - Open a download image.
|
| Returns     - TRUE on success or FALSE on failure.
|
|--------------------------------------------------------------------------*/
BOOLEAN_T mtok_code_open(CODE_TYPE_T type,CODE_HANDLE_T * handlep)
{
#ifndef EMBEDDED_MICRO_CODE
    mm_segment_t oldfs;
    BOOLEAN_T r;
    oldfs = get_fs();
    set_fs(get_ds());
    r = do_mtok_code_open(type,handlep);
    set_fs(oldfs);
    return r;
#else
    handlep->offset = 0;
    switch (type)
    {
#ifdef SMART_UDI
        case FASTMAC_PLUS_CODE:
            handlep->data = (WORD_T *) &fmplus_image;
            break;
#endif /* SMART_UDI */
#ifdef KNOSSOS_UDI
        case KNOSSOS2_ADAP_CODE:
            handlep->data = (WORD_T *) &knossos_image;
            break;
#endif /* KNOSSOS_UDI */
#ifdef OLICOM_HEBE_UDI
        case HEBE_ADAP_CODE:
            handlep->data = (WORD_T *) &hebe_image;
            break;
#endif /* OLICOM_HEBE_UDI */
        default:
            return FALSE;
    }
    return TRUE;    
#endif /* !EMBEDDED_MICRO_CODE */
}

WORD_T mtok_code_readw(CODE_HANDLE_T * handlep)
{
    WORD_T val =  handlep->data[handlep->offset++];
#ifdef LINUX20
#ifdef __BIG_ENDIAN
#error BIG Endian processors not supported by this driver on old Linux kernels.
#else
    return val;
#endif /* __BIG_ENDIAN */
#endif /* LINUX20 */
#ifdef LINUX21
    return le16_to_cpu(val);    
#endif
}

VOID_T mtok_code_close(CODE_HANDLE_T * handlep)
{
#ifndef EMBEDDED_MICRO_CODE
    vfree(handlep->data);
#endif
}

#if 0
/*-----------------------------------------------------------------
| mtok_ssi_frag_read8
-----------------------------------------------------------------*/

BYTE_T
mtok_ssi_frag_read8(
    FRAG_HANDLE_T    frag_handle,
    UINT_T           offset)
{

    BYTE_T byte;
   
    byte = MTOK_PEEK(frag_handle, offset, BYTE_T);

    return byte;
}

/*-----------------------------------------------------------------
| mtok_ssi_frag_read16
-----------------------------------------------------------------*/

WORD_T
mtok_ssi_frag_read16(
    FRAG_HANDLE_T    frag_handle,
    UINT_T           offset)
{

    WORD_T word;
    word = (WORD_T) MTOK_PEEK(frag_handle, (offset+1), BYTE_T);
    word<<=8;
    word |= MTOK_PEEK(frag_handle, offset, BYTE_T);
    return word;
}

/*-----------------------------------------------------------------
| mtok_ssi_frag_read32
-----------------------------------------------------------------*/

DWORD_T
mtok_ssi_frag_read32(
    FRAG_HANDLE_T    frag_handle,
    UINT_T           offset)
{

    DWORD_T dword;
   
    dword = (((DWORD_T)MTOK_PEEK(frag_handle, offset, BYTE_T)) |
             ((DWORD_T)MTOK_PEEK(frag_handle, (offset+1), BYTE_T) <<  8) |
             ((DWORD_T)MTOK_PEEK(frag_handle, (offset+2), BYTE_T) << 16) |
             ((DWORD_T)MTOK_PEEK(frag_handle, (offset+3), BYTE_T) << 24));

    return dword;
}

/*-----------------------------------------------------------------
| mtok_ssi_frag_write8
-----------------------------------------------------------------*/

VOID_T
mtok_ssi_frag_write8(
    FRAG_HANDLE_T    frag_handle,
    UINT_T           offset,
    BYTE_T           value)
{

    MTOK_PEEK(frag_handle, offset, BYTE_T) = value;
}

/*-----------------------------------------------------------------
| mtok_ssi_frag_write16
-----------------------------------------------------------------*/

VOID_T
mtok_ssi_frag_write16(
    FRAG_HANDLE_T    frag_handle,
    UINT_T           offset,
    WORD_T           value)
{

#ifdef AVOID_UNALIGNED_ACCESS
    MTOK_PEEK(frag_handle, offset, BYTE_T) = (BYTE_T)(value & 0xff);
    MTOK_PEEK(frag_handle, (offset + 1), BYTE_T) = (BYTE_T)((value >> 8) & 0xff);
#else
    *((WORD_T *)&frag_handle.data[offset]) = cpu_to_le16(value);
#endif

}

/*-----------------------------------------------------------------
| mtok_ssi_frag_write32
-----------------------------------------------------------------*/

VOID_T
mtok_ssi_frag_write32(
    FRAG_HANDLE_T    frag_handle,
    UINT_T           offset,
    DWORD_T          value)
{

#ifdef AVOID_UNALIGNED_ACCESS 
    MTOK_PEEK(frag_handle, offset, BYTE_T) = (BYTE_T)(value & 0xff);
    MTOK_PEEK(frag_handle, (offset + 1), BYTE_T) = (BYTE_T)((value >> 8) & 0xff);
    MTOK_PEEK(frag_handle, (offset + 2), BYTE_T) = (BYTE_T)((value >> 16) & 0xff);
    MTOK_PEEK(frag_handle, (offset + 3), BYTE_T) = (BYTE_T)((value >> 24) & 0xff);
#else
    *((DWORD_T *)&frag_handle.data[offset]) = cpu_to_le32(value);
#endif

}

#endif


#ifdef DEBUG
VOID_T poke(BYTE_T color,STRING_T str,BYTE_T type,unsigned long val)
{
#define CASE(col)   case PO_##col: color = MPOKE_##col ; break;    
    switch(color & 0x0F)
    {
        CASE(WHITE)
        CASE(BLACK)
        CASE(BLUE)
        CASE(GREEN)
        CASE(CYAN)
        CASE(RED)
        CASE(MAGENTA)
        CASE(YELLOW)
        CASE(BRIGHT_BLACK)
        CASE(BRIGHT_BLUE)
        CASE(BRIGHT_GREEN)        
        CASE(BRIGHT_CYAN)
        CASE(BRIGHT_RED)
        CASE(BRIGHT_MAGENTA)
        CASE(BRIGHT_YELLOW)
        CASE(BRIGHT_WHITE)
        default:
            color=PO_WHITE;
            break;            
    }
    
    switch(type)
    {
        case PO_TYPE_N:
            mpoke_printf(mp,color,str);
            break;
        case PO_TYPE_C:
            mpoke_printf(mp,color,"%s%c",str,(char)val);
            break;
        case PO_TYPE_B:
            mpoke_printf(mp,color,"%s%02x",str,val&0xFF);
            break;
        case PO_TYPE_W:
            mpoke_printf(mp,color,"%s%04x",str,val&0xFFFF);
            break;
        case PO_TYPE_D:
            mpoke_printf(mp,color,"%s%08x",str,val);
            break;
        case PO_TYPE_S:
            mpoke_printf(mp,color,"%s%s",str,(char *)val);
            break;
        case PO_TYPE_P:
            mpoke_printf(mp,color,"%s%p",str,val);
            break;
        case PO_TYPE_I:
            mpoke_printf(mp,color,"%s%u",str,val);
            break;
        case PO_TYPE_L:
            mpoke_printf(mp,color,"%s%u",str,val);
            break;
    }
}
#endif /* DEBUG */
