/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/*****************************************************************************
*
* madgetr.h
*
* Main header file.
*
* Copyright (c) Madge Networks Ltd
*
*    
*****************************************************************************/

/*----------------------------------------------------------------------------
|
| This file is divided into two sections. The first contains driver
| configuration constants which may be "tuned". The second section contains
| internal structure and constant definitions. The file is split up this way
| as the configuration information is needed by the Universal Driver Toolkit's
| driver configuration file driver.h but the definitions section needs the
| contents of driver.h. The usual way to get around this would be to 
| include driver.h half way through. Unfortunately the UDT includes driver.h
| but not madge.h so we need to do some playing with ifdefs.
|
|--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
|
| SECTION 1 - CONFIGURATION
|
|---------------------------------------------------------------------------*/

#ifndef __MTOK_CONFIG__
#define __MTOK_CONFIG__

#include "mtok.upd"

/*----------------------------------------------------------------------------
|
| Driver names.
|
|---------------------------------------------------------------------------*/

#define MTOK_DRIVER_NAME        "mtok"
#define MTOK_DEVICE_NAME        "tr"


/*----------------------------------------------------------------------------
|
| Maximum number of adapters supported. This should be a number between
| 1 and 8 inclusive.
|
|---------------------------------------------------------------------------*/

#define MTOK_MAX_ADAPTERS       4   


/*----------------------------------------------------------------------------
|
| Maximum numbers of transmit and recieve slots. Transmit and receive are the 
| numbers of pipeline transmit and receive buffers kept on the host. Valid 
| values are between 2 and 16 inclusive.
|
|---------------------------------------------------------------------------*/

#define MTOK_MAX_RX_SLOTS       8
#define MTOK_MAX_TX_SLOTS       4


/*----------------------------------------------------------------------------
|
| Default numbers of transmit and recieve slots. We use different numbers of
| receive slots for adapters with lots of buffering on the adapter and 
| those that stream into host memory.
|
|---------------------------------------------------------------------------*/

#define MTOK_DEF_RX_SLOTS         4
#define MTOK_DEF_STREAM_RX_SLOTS  16
#define MTOK_DEF_TX_SLOTS         4


/*----------------------------------------------------------------------------
|
| Maximum frame size. This is the maximum transfer unit (MTU) size plus
| space for a maximum sized token ring header (14 bytes of MAC header,
| 18 bytes or source routing information, 3 bytes of LLC header and 5 bytes
| of SNAP extension). At the time of writing (Linux 2.0.29) the token 
| ring MTU was hardwired at 2000.
|
|---------------------------------------------------------------------------*/

#define MTOK_DEF_MAX_FRAME_SIZE     (2000 + 14 + 18 + 3 + 5)

#endif /* __MTOK_CONFIG__ */

/*----------------------------------------------------------------------------
|
| SECTION 1 - INTERNAL DEFINITIONS
|
|---------------------------------------------------------------------------*/

#ifndef __MTOK_DEFS__
#define __MTOK_DEFS__

#include "driver.h"

/*----------------------------------------------------------------------------
|
| Product tracking information. This must not be changed.
|
|---------------------------------------------------------------------------*/

#define MTOK_PRODUCT_NAME      "Madge Token-Ring Adapter Driver for Linux"
#define MTOK_PRODUCT_COPYRIGHT "Copyright (c) Madge Networks Ltd 1998"
#define MTOK_PRODUCT_PLATFORM  "Linux " UTS_RELEASE
#define MTOK_PRODUCT_ID        "MadgeLinux " MTOK_VERSION

#ifdef __KERNEL__
#include "ver.h"
#endif


/*----------------------------------------------------------------------------
|
| UDT interaction constants.
|
|---------------------------------------------------------------------------*/

#define MTOK_MAX_CONFIG_LOOPS   8
#define MTOK_WATCHDOG_PERIOD    8
#define MTOK_FRAME_ALIGNMENT    32

#define MTOK_COMP_TIMEOUT       2
#define MTOK_OPEN_TIMEOUT       60
#define MTOK_CLOSE_TIMEOUT		20

#define MTOK_USEC_PER_JIFFIE    (1000000 / HZ)


/*----------------------------------------------------------------------------
|
| Extra return code.
|
|---------------------------------------------------------------------------*/

#define EOK   0


/*----------------------------------------------------------------------------
|
| Interval in jiffies between runs of our background function.
|
|---------------------------------------------------------------------------*/

/*
 * HZ is the number of jiffies per second
 */

#define MTOK_BACKGROUND_INTERVAL  (2 * HZ)  /* Should be 2 seconds. */


/*----------------------------------------------------------------------------
|
| Ring status events we'll report.
|
|---------------------------------------------------------------------------*/

#define MTOK_RING_STATUS_MASK \
    (RING_STATUS_PRESTO_WATCHDOG | \
     RING_STATUS_SINGLE_STATION  | \
     RING_STATUS_REMOVE_RECEIVED | \
     RING_STATUS_AUTO_REMOVAL    | \
     RING_STATUS_LOBE_FAULT      | \
     RING_STATUS_TX_BEACON       | \
     RING_STATUS_HARD_ERROR      | \
     RING_STATUS_SIGNAL_LOSS)

    
/*----------------------------------------------------------------------------
|
| Adapter states. The order of these constants matters.
|
|---------------------------------------------------------------------------*/

#define MTOK_STATE_UNKNOWN       0
#define MTOK_STATE_DEAD          1 /* The adapter hardware has died. */
#define MTOK_STATE_REMOVED       2 /* The adapter has been removed. */
#define MTOK_STATE_INITIALISING  3 /* The adapter is being initialised. */
#define MTOK_STATE_READY		 4 /* Committed */
#define MTOK_STATE_RUNNING       5 /* The adapter is running but not open. */
#define MTOK_STATE_CLOSING		 6
#define MTOK_STATE_CLOSED        7 /* The adapter is not open but should be. */
#define MTOK_STATE_OPENING       8 /* The adapter is being opened. */
#define MTOK_STATE_RE_OPENING    9 /* The adapter is being re-opened. */
#define MTOK_STATE_OPEN          10 /* The adapter is open on the ring. */


/*---------------------------------------------------------------------------
|
| TEST & XID constants.
|
|--------------------------------------------------------------------------*/

#define LLC_TEST_XID_MASK    0xef
#define LLC_TEST             0xe3
#define LLC_XID              0xaf
#define LLC_RESPONSE         0x01
#define LLC_POLL_FINAL       0x10

#define LLC_XID_FMTID        0x81
#define LLC_XID_SERVICES     0x01


/*----------------------------------------------------------------------------
|
| Structure used to coordinate asynchronous UDT command compeletion.
|
|---------------------------------------------------------------------------*/

#define MTOK_CMD_COMMIT     1
#define MTOK_CMD_OPEN       2
#define MTOK_CMD_CLOSE      3
#define MTOK_CMD_ERROR_LOG  4
#define MTOK_CMD_FUNC_ADDR  5
#define MTOK_CMD_PROD_ID    6
#define MTOK_CMD_SET_PROMISC 7

#ifdef __KERNEL__

typedef struct _MTOK_COMPLETION
{
    int       command;
    BOOLEAN_T completed;
    RETCODE_T retc;
}
MTOK_COMPLETION;

/*----------------------------------------------------------------------------
|
| Structure used to simulate a mutex. 
|
| Under old versions of Linux there is no mutex/spin-lock
| as such, however we can acheive the same functionality by disabling
| interrupts.
|
|---------------------------------------------------------------------------*/

#ifdef LINUX20

typedef struct
{
    unsigned long flags;
    
#if MPOKE_DBG_LEVEL > 0
    int           nesting;
#endif
}
MTOK_MUTEX;

#if MPOKE_DBG_LEVEL > 0

#define MTOK_INIT_MUTEX(_lockp) \
{ \
    (_lockp)->nesting = 0; \
}
    
#define MTOK_FREE_MUTEX(_lockp)

#define MTOK_LOCK(_lockp) \
{ \
    save_flags((_lockp)->flags); \
    cli(); \
    \
    if ((_lockp)->nesting++ > 0) \
    { \
        MPOKEF("NESTED LOCK %s %d\n", __FILE__, __LINE__); \
    } \
}

#define MTOK_UNLOCK(_lockp) \
{ \
    if (--(_lockp)->nesting < 0) \
    { \
        MPOKEF("LOCK NOT HELD %s %d\n", __FILE__, __LINE__); \
    } \
    \
    restore_flags((_lockp)->flags); \
}
          
#define MTOK_ISR_LOCK(_lockp) \
{ \
    if (((_lockp)->nesting)++ > 0) \
    { \
        MPOKEF("NESTED LOCK %s %d\n", __FILE__, __LINE__); \
    } \
}

#define MTOK_ISR_UNLOCK(_lockp) \
{ \
    if (--((_lockp)->nesting) < 0) \
    { \
        MPOKEF("LOCK NOT HELD %s %d\n", __FILE__, __LINE__); \
    } \
}
    
#else /* MPOKE_DBG_LEVEL > 0 */

#define MTOK_INIT_MUTEX(_lockp)
    
#define MTOK_FREE_MUTEX(_lockp)

#define MTOK_LOCK(_lockp) \
{ \
    save_flags((_lockp)->flags); \
    cli(); \
}

#define MTOK_UNLOCK(_lockp) \
{ \
    restore_flags((_lockp)->flags); \
}
          
#define MTOK_ISR_LOCK(_lockp)

#define MTOK_ISR_UNLOCK(_lockp)

#endif /* MPOKE_DBG_LEVEL > 0 */

#endif /* LINUX20 */


/*----------------------------------------------------------------------------
|
| The latest Linux Kernels have spin-locks implemented.
|
|---------------------------------------------------------------------------*/

#ifdef LINUX21

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,18)
#include <asm/spinlock.h>
#else
#include <linux/spinlock.h>
#endif

/* BUG: There is a problem here in that the flags are stored inside this structure
   rather than in a local variable.
   For example, we save the flags and acquire the lock. Another process then 
   comes accross the spinlock and saves the flags overwriting the flags for the
   other process.
   Solution: Use a local variable to save the flags.
   Problem:  Makes it hard to keep the driver kernel version independent.
 */

typedef struct {
	spinlock_t	sl;
	u_long flags;
} MTOK_MUTEX;

#define MTOK_INIT_MUTEX(_plock) spin_lock_init(&((_plock)->sl));
#define MTOK_FREE_MUTEX(_plock)

#define MTOK_ISR_LOCK(_plock)   spin_lock((&(_plock)->sl));
#define MTOK_ISR_UNLOCK(_plock) spin_unlock(&(_plock)->sl);
#define MTOK_LOCK(_plock)       spin_lock_irqsave(&((_plock)->sl),(_plock)->flags);
#define MTOK_UNLOCK(_plock)     spin_unlock_irqrestore(&((_plock)->sl),(_plock)->flags);

#endif

/*----------------------------------------------------------------------------
|
| Adapter control structure.
|
|---------------------------------------------------------------------------*/

typedef struct __MTOK_ADAPTER
{
    LINUXDEV * dev;          /* Linux device descriptor. */
    
    /*----------------------------------------------------------------------*/
    
    UDI_DATA_T        udi_data;       /* UDI's private data. */
    UDI_INIT_DATA_T   udi_init_data;  /* UDI initialisation data.*/
    
    /*----------------------------------------------------------------------*/
    
	int busdevfn;

    int type;                     	/* Adapter type. */
    int subtype;					/* Adapter subtype. */

    /*----------------------------------------------------------------------*/

	/* Module Parameters */

	DWORD_T max_frame_size;
	int ringspeed;
	int rxslots;
	int txslots;
	int dtr;
	
    /*----------------------------------------------------------------------*/

    MAC_ADDRESS_T     bia;          /* Our burnt in address. */
    
    /*----------------------------------------------------------------------*/
    
    XFER_METHOD_T xfer_mode;      /* Transfer mode being used. */
    
    /*----------------------------------------------------------------------*/
    
    int state;                    /* The adapter's operational state. */
    int open_count;               /* Number of active opens. */

	int wantmac;				  /* MAC Frames or not ? */

    /*----------------------------------------------------------------------*/
    
    /* 
     * The I/O regions we are using. 
     */
     
    long io_base[MAX_IO_REGIONS]; 
    long io_range[MAX_IO_REGIONS]; 
    
    /*
     * The memory fragments requested by the UDT.
     */
     
    FRAG_HANDLE_T frags[MAX_FRAG_REGIONS]; 

    int               irq_number;   /* Our interrupt number. */
    int               dma_channel;  /* Our DMA channel. */
    DWORD_T			  shared_ram;
    

    /*----------------------------------------------------------------------*/
    
    MTOK_COMPLETION     open_comp;  /* UDT command completion structure */
                                    /* used for open commands. */

    MTOK_MUTEX          isr_lock;   /* Our mutex. */

	MTOK_MUTEX			tx_lock;  

    struct timer_list   bkgd_timer; /* Our background timer. */
    BOOLEAN_T           bkgd_runs;  /* TRUE if the back ground timer */
                                    /* should be running. */    
                                      
    /*----------------------------------------------------------------------*/

    struct net_device_stats stats;     /* Token ring statistics. */

    /*----------------------------------------------------------------------*/
    
	FRAME_T * tx_free_list;		/* List of free transmit descriptors */
	int		  tx_full;			/* Boolean noting if we have no tx slots left */
	VOID_T  * tx_list;			/* List of all transmit descriptors (pending + free) */

	FRAME_T *	rx_list;		/* List of rx descriptors */

#ifdef SLEEP_WAIT
    struct wait_queue * command_wait;
#endif

#if BITS_PER_LONG == 64

    /*
     * For 64 bit architectures this value must be added to any io base
     * addresses given to us by the UDT.
     */

    long io_offset_64bit;

#endif /* BITS_PER_LONG == 64 */

#ifdef OLICOM_PCMCIA_UDI

/*
 * Olicom PCMCIA adapters store their BIA in the CIS.
 * We don't want to access CardServices from the mtok module
 * because that would mean that PCMCIA modules would have
 * to be present on all machines in order for this module to
 * load.  This is a generic module for all adapters so it
 * can be used on non laptop machines.
 *
 * So in the mtok_cs (madgecs.c) module we access card services
 * and read the appropriate CIS tuple.  We then pass this buffer
 * to mtok_probe1 which stores the BIA tuple in this buffer here.
 *
 * The SSI function ssi_read_tuple then just passes back this
 * buffer to the UDT.
 */

    u8  * olicom_rs_tuple;  /* Ring Speed Tuple */
    u8  * olicom_bia_tuple; /* BIA Tuple        */

#endif /* OLICOM_PCMCIA_UDI */

} MTOK_ADAPTER;

/*---------------------------------------------------------------------------
|
| External pointers to embedded downloads.
|
|---------------------------------------------------------------------------*/

#ifdef EMBEDDED_MICRO_CODE

#ifdef KNOSSOS_UDI
extern IMAGE_HEADER_T knossos_image;
#endif

#ifdef SMART_UDI
extern IMAGE_HEADER_T fmplus_image;
#endif

#ifdef OLICOM_HEBE_UDI
extern IMAGE_HEADER_T hebe_image;
#endif

#endif /* EMBEDDED_MICRO_CODE */

#endif  /* __KERNEL__  */

#endif /* __MTOK_DEFS__ */

/**** End of madgetr.h ******************************************************/


