/*
 *
 *    Connectix USB QuickCam VC Video Camera driver
 *
 *    (C) Copyright 2001 De Marchi Daniele
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Jan 2001 This driver develop started on the linux 
 *            kernel 2.4.0.
 *
 */

#include "qcamvc.h"
#ifdef CONFIG_KMOD
#include <linux/kmod.h>
#endif
#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <linux/wrapper.h>

#ifdef MODULE
MODULE_AUTHOR("De Marchi Daniele <demarchidaniele@libero.it>");
MODULE_DESCRIPTION("V4L-driver for QuickCam VC cameras");
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("video");
#endif

#ifdef CONFIG_VIDEO_QCAMVC_PP
extern int qcamvc_pp_init(void);
#endif
#ifdef CONFIG_VIDEO_QCAMVC_USB
extern int qcamvc_usb_init(void);
#endif


static long    qcamvc_vread(struct video_device *dev, char *buf, unsigned long count, int noblock);
static long    qcamvc_vwrite(struct video_device *dev, const char *buf, unsigned long count, int noblock);
static int     qcamvc_vioctl(struct video_device *dev, unsigned int cmd, void *arg);
static int     qcamvc_vopen(struct video_device *dev, int flags);
static void    qcamvc_vclose(struct video_device *dev);
static int     qcamvc_init_done(struct video_device *dev);
#ifdef USE_MMAP
static int qcamvc_vmmap(struct video_device *dev, const char *adr, unsigned long size);
#endif

static struct video_device qcamvc_template = {
  name:         "QuickCam VC",
  type:         VID_TYPE_CAPTURE,
  hardware:     VID_HARDWARE_QCAM_C,
  open:         qcamvc_vopen,
  close:        qcamvc_vclose,
  read:         qcamvc_vread,
  write:        qcamvc_vwrite,
  ioctl:        qcamvc_vioctl,
  initialize:   qcamvc_init_done,
#ifdef USE_MMAP
  mmap:         qcamvc_vmmap,
#endif
};

/*****************************************************************************/
/*                                                                           */
/* QuickCam Video CODEC                                                      */
/*                                                                           */
/*****************************************************************************/
static void hue_reg(struct qcamvc *qcamvc)
{
  float a,b;
  int cnt;
  
  if (qcamvc->hue_calc == 1) return;
  
  a = (qcamvc->blue_hue * 1.12) / 128;
  b = (qcamvc->red_hue * 1.39999999999) / 128;  

  for (cnt = 0 ; cnt < 0xff ; cnt++)
    {
      qcamvc->table_a[cnt] = (float) cnt * b;
      qcamvc->table_b[cnt] = (float) cnt * a;
  
      if(qcamvc->table_a[cnt] > 0xff) qcamvc->table_a[cnt] = 0xff; if(qcamvc->table_a[cnt] < 0   ) qcamvc->table_a[cnt] = 0;
      if(qcamvc->table_b[cnt] > 0xff) qcamvc->table_b[cnt] = 0xff; if(qcamvc->table_b[cnt] < 0   ) qcamvc->table_b[cnt] = 0;
    }
  
  a = qcamvc->santrast / 100;
  
  qcamvc->VAR_A = a * 1.731999999999999;
  qcamvc->VAR_B = a * 0.336;
  qcamvc->VAR_C = a * 0.697999999999999;
  qcamvc->VAR_D = a * 1.370999999999999;
  
  qcamvc->hue_calc = 1;
}

static void get_pixel(struct qcamvc *qcamvc, float pix_a, float pix_b, unsigned char pix_c, \
	      unsigned char pix_d, float pix_e, \
	      unsigned char *pred, unsigned char *pgreen, unsigned char *pblue)
{
  
  float a,b;
  float blue,green,red;

  a = pix_b - qcamvc->table_a[pix_c];
  b = qcamvc->table_b[pix_d] - pix_e;
 
  if(pix_a >= 0) {if(pix_a > 0xff) {pix_a = 0xff;}} else {pix_a = 0;}
  if(a <= 112)   {if(a < -112) {a = -112;}} else {a = 112;}
  if(b <= 112)   {if(b < -112) {b = -112;}} else {b = 112;}
  
  blue =  ( b * qcamvc->VAR_A ) +  pix_a;
  green = ( a * qcamvc->VAR_C ) - (( b * qcamvc->VAR_B) - pix_a );
  red =   ( a * qcamvc->VAR_D ) + pix_a;
   
  if(red >= 0) {if(red > 0xff) red = 0xff;} else {red = 0;}
  if(green >= 0) {if(green > 0xff) green = 0xff;} else {green = 0;}  
  if(blue >= 0) {if(blue > 0xff) blue = 0xff;} else {blue = 0;}
  
  *pred   = (unsigned char) red;  
  *pgreen = (unsigned char) green;  
  *pblue  = (unsigned char) blue;
  
}


static void codec(struct qcamvc *qcamvc, unsigned char *inbuf, unsigned char *outbuf)
{
  int width=qcamvc->width;
  int height=qcamvc->height;
  unsigned char red=0, green=0, blue=0;
  int x,y;
  unsigned char pix1, pix2, pix3, pix4;
  unsigned char *pointer1, *pointer2;
  unsigned char *table = (unsigned char *) kmalloc(width * height, GFP_KERNEL);
  unsigned char *pointer_last_line =(unsigned char *) kmalloc(width, GFP_KERNEL);

  pointer1 = inbuf;
  pointer2 = table;
  
 for (y = 0 ; y < height ; y ++ ){
   for (x = width - 1 ; x >= 0 ; x--){
     *(pointer2 + x) = *(pointer1);
     pointer1 ++;
   }
   pointer2 = pointer2 + width;
 }
  
  pointer1 = inbuf;
  pointer2 = (outbuf + qcamvc->width * qcamvc->height * 3) - 1;
  if (qcamvc->hue_calc == 0 ) hue_reg(qcamvc);
  memcpy( pointer_last_line, pointer1, width);
  for(y = 0 ; y < height ; y++) {  
    if (current->need_resched) schedule();
    
    if((y & 1) == 0) {      
      for (x = 0 ; x < width ; x++){
	if((x & 1) != 0) {
	  pix1 = (( *(width + pointer1)) + (*(x + pointer_last_line))) / 2;	       
	  pix2 = (((*(width + pointer1 + 1)) + (*(x + pointer_last_line)) )) / 2 ;	       
	  pix3 = *pointer1;
	  pix4 = *(pointer1 + 1);	
	} else {	  
	  pix1 = ((*(width + pointer1 + 1)) + (*(x + 1 + pointer_last_line))) / 2;
	  pix2 = ((*(width + pointer1)) + (*(x + pointer_last_line))) / 2;		
	  pix3 = *(pointer1 + 1);
	  pix4 = *pointer1;	  
	}	
	get_pixel(qcamvc, (pix4 + pix3) / 2,  pix4, pix3, pix1, pix2, &blue, &green, &red);
	(unsigned char)*(pointer2)     = blue;  
	(unsigned char)*(pointer2 - 1) = green; 
	(unsigned char)*(pointer2 - 2) = red;	
	pointer2-=3;
	pointer1++;	
      }      
    } else {

      for (x = 0 ; x < width ; x++){
	if((x & 1) != 0) {
	  pix3 = (*(width + pointer1) + *(x + pointer_last_line)) / 2;		
	  pix4 = (*(width + pointer1 + 1) + *(x + 1 + pointer_last_line)) / 2;	       
	  pix1 = *pointer1 & 0xff;
	  pix2 = *(pointer1 + 1);	  
	} else {	  
	  pix3 = (*(width + pointer1 + 1) + *(x + 1 + pointer_last_line)) / 2;
	  pix4 = (*(width + pointer1)     + *(x +     pointer_last_line)) / 2;
	  pix1 = *(pointer1 + 1);
	  pix2 = *pointer1;
	}	
	get_pixel(qcamvc, (pix1 + pix2) / 2,  pix4, pix3, pix1, pix2, &blue, &green, &red);
	(unsigned char)*(pointer2)     = blue;  
	(unsigned char)*(pointer2 - 1) = green; 
	(unsigned char)*(pointer2 - 2) = red;	
	pointer2-=3;
	pointer1++;
      }                  
    }
    memcpy(pointer_last_line, pointer1 - width, width);
  } 
  kfree(table);
  kfree(pointer_last_line);
}

static void qcamvc_set_topmost_row(struct qcamvc *qcamvc,  unsigned char val)
{
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_TOPMOST_ROW, &val, sizeof(val));
}

static void qcamvc_set_bottommost_row(struct qcamvc *qcamvc,  unsigned char val)
{
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_BOTTOMMOST_ROW, &val, sizeof(val));
}

static void qcamvc_set_leftmost_column(struct qcamvc *qcamvc,  unsigned char val)
{
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_LEFTMOST_COLUMN, &val, sizeof(val));
}

static void qcamvc_set_rightmost_column(struct qcamvc *qcamvc,  unsigned char val)
{
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_RIGHTMOST_COLUMN, &val, sizeof(val));
}

static void qcamvc_set_exposure(struct qcamvc *qcamvc,  unsigned char val)
{
  unsigned char buffer[2];
  GET_VALUES(buffer,val);
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_EXPOSURE, buffer, sizeof(buffer));
  if(qcamvc->streaming) qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_GET_FRAME, NULL, 0);
}

static void qcamvc_set_brightness(struct qcamvc *qcamvc,  unsigned char val)
{
  short i,j;
  unsigned char buffer[21]={
    0x58, 0xd8, 0x58, 0xd8, 0x58,
    0xd8, 0x58, 0xd8, 0x58, 0xd8,
    0x58, 0xd8, 0x58, 0xd8, 0x58,
    0xd8, 0x58, 0xd8, 0x58, 0xd8, 0x5c};

  for(i=1, j=18 ; i<=0x80 ; i*=2 , j-=2)
    {
      if( val & i) {
	buffer[j] = buffer[j] | 1;
	buffer[j+1] = buffer[j+1] | 1;}
    }

  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_BRIGHTNESS, buffer, sizeof(buffer));
  if(qcamvc->streaming) qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_GET_FRAME, NULL, 0);
}

static int bpc6_init(struct qcamvc *qcamvc)
{
  int ret=0;
  long i,q;
  unsigned char buffer[0x40];
  unsigned char *bigbuffer;

  FIRST_ROW; LAST_ROW;
  FIRST_COL; LAST_COL;
  MUL_FACTOR;

  bigbuffer=kmalloc(BIGBUFFER_SIZE , GFP_KERNEL);
  if (bigbuffer==NULL) 
    {
      printk("QuickCam VC: Init bigbuffer kmalloc error.\n");
      return -1;
  }
  
  for(i=0; i<BIGBUFFER_SIZE ; i+=2)
    {
    q=i;
     *(bigbuffer + i ) = (unsigned char )(q >> 11);
     *(bigbuffer + i + 1) = (unsigned char) 0xc0;
    }

  buffer[0]=0x78; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  buffer[0]=0x58; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  buffer[0]=0x18; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);

  qcamvc_set_topmost_row     (qcamvc, 0x0e);
  qcamvc_set_leftmost_column (qcamvc, 0x0f);
  qcamvc_set_bottommost_row  (qcamvc, 0x86);
  qcamvc_set_rightmost_column(qcamvc, 0xaf);

  qcamvc_set_brightness(qcamvc, qcamvc->brightness);

  buffer[0]=0x7f;buffer[1]=0x81;
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x02, buffer, 2);

  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0e, buffer, 1);
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0d, bigbuffer, BIGBUFFER_SIZE);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0e, buffer, 1);

  buffer[0]=0x01; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0e, buffer, 1);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  buffer[0]=0x78; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  buffer[0]=0x58; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);

  qcamvc_set_brightness(qcamvc, qcamvc->brightness);
  qcamvc_set_exposure(qcamvc, qcamvc->exposure);

  buffer[0]=0x11; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0a, buffer, 1);
  buffer[0]=0x11; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0a, buffer, 1);

  qcamvc_set_topmost_row     (qcamvc, firstrow[qcamvc->res]);
  qcamvc_set_leftmost_column (qcamvc, firstcolumn[qcamvc->res]);
  qcamvc_set_bottommost_row  (qcamvc, lastrow[qcamvc->res]);
  qcamvc_set_rightmost_column(qcamvc, lastcolumn[qcamvc->res]);

  buffer[0] = mulfactor[qcamvc->res]; 
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0a, buffer, 1);

  kfree(bigbuffer);
  return ret;
}

static int bpc8_init(struct qcamvc *qcamvc)
{
  FIRST_ROW; LAST_ROW;
  FIRST_COL; LAST_COL;
  MUL_FACTOR;
  int q,i;
  unsigned char *bigbuffer;
  unsigned char buffer[0x40];
  unsigned char a=0xff;

  bigbuffer = kmalloc(BIGBUFFER_SIZE , GFP_KERNEL);
  if (bigbuffer==NULL) 
    {
      printk("QuickCam VC: Init bigbuffer kmalloc error.\n");
      return -1;
  }

  for(i=0; i<BIGBUFFER_SIZE ; i+=2)
    {    
      q=i;
      *(bigbuffer + i)   = 0;
      *(bigbuffer + i +1) = (unsigned char )(q >> 11);
    }

  buffer[0]=0x78; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  buffer[0]=0x58; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  buffer[0]=0x18; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  //#ifdef CAM_DEBUG
  qcamvc->ops->qcamvc_get_reg(qcamvc->lowlevel_data, 0x8f, buffer, sizeof(buffer));
  printk("QuickCam VC:8bpp init bpc8 0x8f, len=%d, ",sizeof(buffer));
  for (i=0 ; i < sizeof(buffer) ;  i++) printk (" 0x%02x ", buffer[i]);
  printk("\n");

  qcamvc->ops->qcamvc_get_reg(qcamvc->lowlevel_data, 0x80, buffer, sizeof(buffer));
  printk("QuickCam VC:Set init bcp8 0x80, len=%d, ",sizeof(buffer));
  for (i=0 ; i < sizeof(buffer) ;  i++) printk (" 0x%02x ", buffer[i]);
  printk("\n");
  //#endif
  printk("a\n");
  
  qcamvc_set_topmost_row     (qcamvc, 0x0e);
  qcamvc_set_leftmost_column (qcamvc, 0x0f);
  qcamvc_set_bottommost_row  (qcamvc, 0x86);
  qcamvc_set_rightmost_column(qcamvc, 0xaf);
  printk("b\n");
  qcamvc_set_brightness(qcamvc, 0x80);
  printk("c\n");
  buffer[0]=0xaf;buffer[1]=0xaf;
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x02, buffer, 2);
  printk("d\n");
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0e, buffer, 1);
  printk("e\n");


  
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0d, bigbuffer, BIGBUFFER_SIZE);
  printk("QuickCam VC:8bpc bigbuffer write, len=%d, ",sizeof(buffer));
  for (i=1 ; i < BIGBUFFER_SIZE ;  i+=2) 
    {
      if (a != *(bigbuffer + i)) 
	{ printk (" 0x%02x ", *(bigbuffer + i)); a=*(bigbuffer + i);}
    }
  printk("\n");

  buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0e, buffer, 1);

  qcamvc->ops->qcamvc_get_reg(qcamvc->lowlevel_data, 0x8d, bigbuffer, BIGBUFFER_SIZE);
  printk("QuickCam VC:8bpc bigbuffer read, len=%d, ",sizeof(buffer));
   for (i=1 ; i < BIGBUFFER_SIZE ;  i+=2) 
    {
      if (a != *(bigbuffer + i)) 
	{ printk (" 0x%02x ", *(bigbuffer + i)); a=*(bigbuffer + i);}
    }
  printk("\n");



  //buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0e, buffer, 1);
  //buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  //buffer[0]=0x78; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);
  //buffer[0]=0x00; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_SET_MISC, buffer, 1);
  //buffer[0]=0x58; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0f, buffer, 1);

  //qcamvc_set_brightness(qcamvc, qcamvc->brightness);
  //qcamvc_set_exposure(qcamvc, qcamvc->exposure);

  //buffer[0]=0x11; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0a, buffer, 1);
  //buffer[0]=0x11; qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0a, buffer, 1);

  qcamvc_set_topmost_row     (qcamvc, firstrow[qcamvc->res]);
  qcamvc_set_leftmost_column (qcamvc, firstcolumn[qcamvc->res]);
  qcamvc_set_bottommost_row  (qcamvc, lastrow[qcamvc->res]);
  qcamvc_set_rightmost_column(qcamvc, lastcolumn[qcamvc->res]);

  buffer[0] = mulfactor[qcamvc->res]; 
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, 0x0a, buffer, 1);

  printk("h\n");
  kfree(bigbuffer);
  return 0;
}

static int qcamvc_camera_init(struct qcamvc *qcamvc)
{
  FIRST_ROW; LAST_ROW;
  FIRST_COL; LAST_COL;
  MUL_FACTOR; ISO_LEN;
  int ret=-1;


  switch(qcamvc->bpc)
    {
    case bpc6:
      {
        ret = bpc6_init(qcamvc);
	break;
      }   
 
    case bpc8:
      {
	ret = bpc8_init(qcamvc);
	break;
      }
      
    }

  qcamvc->packet_len = iso_len[qcamvc->bpc][qcamvc->res];
  qcamvc->height =   lastrow[qcamvc->res] -    firstrow[qcamvc->res];
  qcamvc->width = lastcolumn[qcamvc->res] - firstcolumn[qcamvc->res];
 
  if (!(mulfactor[qcamvc->res] & 0x02)) 
    {
      qcamvc->height*=2;
      qcamvc->width*=2;
    }
  qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, QCAM_VC_GET_FRAME, NULL, 0);
  qcamvc->streaming=1;

  qcamvc->hue_calc=0;

#ifdef DBG_CAM
  printk("QuickCam VC:Init at %dx%d\n", qcamvc->width, qcamvc->height);
#endif
  return ret;
}

static void bpc6_parse(struct qcamvc *qcamvc, unsigned char *inbuf, unsigned char *outbuf, size_t count)
{
  size_t i,q;
  unsigned char *temp=kmalloc(qcamvc->width*qcamvc->height, GFP_KERNEL);
  for( i = q = 0 ; i < qcamvc->width * qcamvc->height; i+=4,q+=3 )
    {
      if ((q + 2) < qcamvc->packet_len) *(temp+i+3) = *((inbuf+2) + q + 2) & 0xfc;
      if ((q + 2) < qcamvc->packet_len) *(temp+i+2) = (((*((inbuf+2) + q + 1) >> 4) & 0x0f)|((*((inbuf+2)+ q + 2) <<4) & 0x30))<<2;
      if ((q + 1) < qcamvc->packet_len) *(temp+i+1) = (((*((inbuf+2) + q)>>6)&0x03)|((*((inbuf+2)+ q + 1)&0x0f)<<2))<<2;
      if (q < qcamvc->packet_len)       *(temp+i+0) = (*((inbuf+2) + q)&0x3f)<<2; 
    }
  codec(qcamvc, temp, outbuf);
  kfree(temp);
}

static void bpc8_parse(struct qcamvc *qcamvc, unsigned char *inbuf, unsigned char *outbuf, size_t count)
{
  unsigned char *temp=kmalloc(qcamvc->width*qcamvc->height, GFP_KERNEL);
  memcpy(outbuf, inbuf, qcamvc->packet_len);
  kfree(temp);
}

static void qcamvc_parse(struct qcamvc *qcamvc, unsigned char *inbuf, unsigned char *outbuf, size_t count)
{

  switch(qcamvc->bpc)
    {
    case bpc6:
      {
	bpc6_parse(qcamvc, inbuf, outbuf, count);
	break;
      }   
 
    case bpc8:
      {
	bpc8_parse(qcamvc, inbuf, outbuf, count);
	break;
      }
      
    }

}

static int qcamvc_set_res(struct qcamvc *qcamvc, int res)
{
  if (res == qcamvc->res)
    return -1;
  qcamvc->res = res;
  qcamvc_camera_init(qcamvc);
  return qcamvc->res;
}

/****************************************************************************
 *
 * /proc interface
 *
 ***************************************************************************/

#if defined(CONFIG_PROC_FS) && defined(CONFIG_VIDEO_PROC_FS)

static struct proc_dir_entry *qcamvc_proc_entry = NULL;
extern struct proc_dir_entry *video_proc_entry;

#define YES_NO(x) ((x) ? "yes" : "no")

/**************************************************************************
*
*/
static int qcamvc_read_proc(char *page, char **start, off_t off, int count,
			   int *eof, void *data)
{
	char *out = page;
	int len;
	struct qcamvc *qcamvc = data;
	
	// Stay under PAGE_SIZE or bad things will happen :(
	out+=sprintf(out, "driver_version   : %s\n", QCAMVCVERSION);
	out+=sprintf(out, "model            : %s\n", qcamvc->vcap.name);
        out+=sprintf(out, "max resolution   : %dx%d\n", qcamvc->vcap.maxwidth, qcamvc->vcap.maxheight);
        out+=sprintf(out, "min resolution   : %dx%d\n\n", qcamvc->vcap.minwidth, qcamvc->vcap.minheight);
        out+=sprintf(out, "Settings\n");
        out+=sprintf(out, "resolution       : %dx%d\n", qcamvc->width, qcamvc->height);
	out+=sprintf(out, "bit per component: %d\n", (qcamvc->bpc==bpc6)? 6 : 8);
        out+=sprintf(out, "brightness       : %d\n", qcamvc->brightness);
        out+=sprintf(out, "exposure         : %d\n", (int)qcamvc->exposure);
        out+=sprintf(out, "red hue          : %d\n", (int)qcamvc->red_hue);
        out+=sprintf(out, "blue hue         : %d\n", (int)qcamvc->blue_hue);
        out+=sprintf(out, "santrast         : %d\n", (int)qcamvc->santrast);

	out+=sprintf(out, "\n");
	
	len = out - page;
	len -= off;
	if (len < count) {
		*eof = 1;
		if (len <= 0) return 0;
	} else
		len = count;

	*start = page + off;
	
	return len;	
}

/**************************************************************************
*
*/
static int qcamvc_write_proc(struct file *file, const char *buffer, 
			    unsigned long count, void *data)
{
	return -EINVAL;
}

/**************************************************************************
* This routine is called when a quickcam is registered
*/
static void create_proc_qcamvc (struct qcamvc *qcamvc)
{
	char name[7];
	struct proc_dir_entry *ent;
	
    printk("Quickcam VC: Creating a camera entry in proc quickcam.\n"); 
    printk("Quickcam dev struct in create_proc_qcamvc %p\n",qcamvc);

	if (!qcamvc_proc_entry || !qcamvc)
		return;

	sprintf (name, "video%d", qcamvc->vdev.minor);

	ent = create_proc_entry(name, S_IFREG | S_IRUGO | S_IWUSR,
				qcamvc_proc_entry);

	if (!ent)
		return;

	ent->data = qcamvc;
	ent->read_proc = qcamvc_read_proc;
	ent->write_proc = qcamvc_write_proc;
	qcamvc->proc_entry = ent;
}

/**************************************************************************
* This routine is called when a quickcam is unregistered */

static void destroy_proc_qcamvc (struct qcamvc *qcamvc)
{
	/* One to much, just to be sure :) */
	char name[9];

    printk("Quickcam VC: Destroying a camera entry in proc quickcam.\n"); 
	if (!qcamvc || !qcamvc->proc_entry)
		return;
	
	sprintf(name, "video%d", qcamvc->vdev.minor);
	remove_proc_entry(name, qcamvc_proc_entry);
	qcamvc->proc_entry = NULL;
}

/**************************************************************************
* This routine is called when the driver is placed in memory
* by an insmod.
*/
static void proc_qcamvc_create(void)
{
    printk("QuickCam VC: Making a quickcam directory in /proc.\n"); 
	if (video_proc_entry == NULL) {
		printk("/proc/video/ doesn't exist");
		return;
	}

	qcamvc_proc_entry=create_proc_entry("qcamvc", S_IFDIR, video_proc_entry);

	if (qcamvc_proc_entry)
		qcamvc_proc_entry->owner = THIS_MODULE;
	else
		printk("Unable to initialize /proc/video/qcamvc");
}

/**************************************************************************
* This routine is called when the driver is removed from memory
* by an rmmod command.
*/
static void proc_quickcam_destroy(void)
{
    printk("Quickcam VC: Destroying a quickcam directory in /proc.\n"); 
	if (qcamvc_proc_entry == NULL)
		return;

	remove_proc_entry("qcamvc", video_proc_entry);
}
#endif /* CONFIG_PROC_FS && CONFIG_VIDEO_PROC_FS */

/**************************************************************************
*
*/
static int qcamvc_init_done(struct video_device *dev)
{
    #if defined(CONFIG_PROC_FS) && defined(CONFIG_VIDEO_PROC_FS)
        create_proc_qcamvc(dev->priv);
        printk("qcamvc dev struct in qcamvc_init_done %p\n",dev);
    #endif

    return 0;
}


/*****************************************************************************/
/*                                                                           */
/* Video 4 Linux                                                             */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/


static long qcamvc_vread(struct video_device *dev, char *buf, unsigned long count, int noblock)
{ 
  struct qcamvc *qcamvc = dev->priv; 
  unsigned char *tempbuf = kmalloc(qcamvc->packet_len, GFP_KERNEL);

  if (down_interruptible(&qcamvc->busy))
    return -EINTR;
  
  if (!buf) {
    up(&qcamvc->busy);
    return -EINVAL;
  }
  
  if (!count) {
    up(&qcamvc->busy);
    return 0;
  }
  
  if (!qcamvc->ops) {
    up(&qcamvc->busy);
    return -ENODEV;
  }
#ifdef DBG_CAM
  printk("USB QuickCam quickcam_read count=%ld\n",count);
#endif

  qcamvc->ops->qcamvc_stream_read(qcamvc->lowlevel_data, tempbuf, qcamvc->packet_len);
  qcamvc_parse(qcamvc, tempbuf, buf, count);

  kfree(tempbuf);
  up(&qcamvc->busy);
  return count;
}

static long qcamvc_vwrite(struct video_device *dev, const char *buf, unsigned long count, int noblock)
{
  struct qcamvc *qcamvc = dev->priv;
  if (down_interruptible(&qcamvc->busy))
    return -EINTR;
  up(&qcamvc->busy);
  return -EINVAL;
}

#ifdef USE_MMAP
static int qcamvc_vmmap(struct video_device *dev, const char *adr,
                     unsigned long size)
{
  unsigned long start = (unsigned long) adr;
  unsigned long page, pos;
  int retval;
  struct qcamvc *qcamvc = dev->priv; 

  if (down_interruptible(&qcamvc->busy))
    return -EINTR;

  if (!qcamvc->frame_buf) {
    if ((retval = allocate_frame_buf(qcamvc))) {
      return 0;
    }
  }
  
  pos = (unsigned long)(qcamvc->frame_buf);
  while (size > 0) {
    page = kvirt_to_pa(pos);
    if (remap_page_range(start, page, PAGE_SIZE, PAGE_SHARED)) {
      up(&qcamvc->busy);
      return -EAGAIN;
    }
    start += PAGE_SIZE;
    pos += PAGE_SIZE;
    if (size > PAGE_SIZE)
      size -= PAGE_SIZE;
    else
      size = 0;
  }
  up(&qcamvc->busy);
  return 0;
  
}
/**********************************************************************
 *
 * Memory management
 *
 * This is a shameless copy from the USB-cpia driver (linux kernel
 * version 2.3.29 or so, I have no idea what this code actually does ;).
 * Actually it seems to be a copy of a shameless copy of the bttv-driver.
 * Or that is a copy of a shameless copy of ... (To the powers: is there
 * no generic kernel-function to do this sort of stuff?)
 *
 * Yes, it was a shameless copy from the bttv-driver. IIRC, Alan says
 * there will be one, but apparentely not yet - jerdfelt
 *
 **********************************************************************/

/* Given PGD from the address space's page table, return the kernel
 * virtual mapping of the physical memory mapped at ADR.
 */
static inline unsigned long uvirt_to_kva(pgd_t *pgd, unsigned long adr)
{
  unsigned long ret = 0UL;
  pmd_t *pmd;
  pte_t *ptep, pte;
  
  if (!pgd_none(*pgd)) {
    pmd = pmd_offset(pgd, adr);
    if (!pmd_none(*pmd)) {
			ptep = pte_offset(pmd, adr);
			pte = *ptep;
			if (pte_present(pte)) {
			  ret = (unsigned long) page_address(pte_page(pte));
			  ret |= (adr & (PAGE_SIZE-1));
			}
    }
  }
  return ret;
}

#if LINUX_VERSION_CODE < 0x020400
#define virt_to_page(kaddr) (MAP_NR(kaddr))
#endif

static void *rvmalloc(unsigned long size)
{
	void *mem;
	unsigned long adr, page;
	size += (PAGE_SIZE - 1);
	size &= ~(PAGE_SIZE - 1);

	mem = vmalloc_32(size);
	if (!mem)
		return NULL;

	memset(mem, 0, size);
	adr = (unsigned long) mem;
	while (size > 0) {
		page = kvirt_to_pa(adr);
		mem_map_reserve(virt_to_page(__va(page)));
		adr += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}

	return mem;
}

static void rvfree(void *mem, unsigned long size)
{
  unsigned long adr, page;
  if (!mem)
    return;
  
  size += (PAGE_SIZE - 1);
  size &= ~(PAGE_SIZE - 1);
  
  adr = (unsigned long) mem;
  while (size > 0) {
    page = kvirt_to_pa(adr);
    mem_map_unreserve(virt_to_page(__va(page)));
    adr += PAGE_SIZE;
    if (size > PAGE_SIZE)
      size -= PAGE_SIZE;
    else
      size = 0;
  }
	vfree(mem);
}

static int allocate_frame_buf(struct qcamvc *qcamvc)
{
  int i;
  qcamvc->frame_buf = rvmalloc(QUICKCAM_NUMFRAMES * QUICKCAM_MAX_SIZE);
  if (!qcamvc->frame_buf)
    return -ENOBUFS;
  
  for (i = 0; i < QUICKCAM_NUMFRAMES; i++)
    qcamvc->frame[i].data = qcamvc->frame_buf + i * QUICKCAM_MAX_SIZE;
  
  return 0;
}

static int free_frame_buf(struct  qcamvc *qcamvc)
{
  int i;
	
  rvfree(qcamvc->frame_buf, QUICKCAM_NUMFRAMES * QUICKCAM_MAX_SIZE);
  qcamvc->frame_buf = 0;
  for (i=0; i < QUICKCAM_NUMFRAMES; i++)
    qcamvc->frame[i].data = NULL;
  return 0;
}

static void inline free_frames(struct qcamvc_frame frame[QUICKCAM_NUMFRAMES])
{
	int i;

	for (i=0; i < QUICKCAM_NUMFRAMES; i++)
		frame[i].state = FRAME_UNUSED;
	return;
}

/* Here we want the physical address of the memory.
 * This is used when initializing the contents of the
 * area and marking the pages as reserved.
 */
static inline unsigned long kvirt_to_pa(unsigned long adr)
{
  unsigned long va, kva, ret;
  
  va = VMALLOC_VMADDR(adr);
  kva = uvirt_to_kva(pgd_offset_k(va), va);
  ret = __pa(kva);
  return ret;
}

#endif /* USE_MMAP */

static int qcamvc_vioctl(struct video_device *dev, unsigned int cmd, void *arg)
{
  struct qcamvc *qcamvc = dev->priv;

  if (down_interruptible(&qcamvc->busy))
    return -EINTR;


  switch(cmd)
    {
      
      /* Return capability */
    case VIDIOCGCAP:
      {
	if (copy_to_user(arg, &qcamvc->vcap, sizeof(struct video_capability)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	up(&qcamvc->busy);
	return 0;
      }
      
      /* Video channel ioctl */
    case VIDIOCGCHAN:
      {
	if (copy_to_user(arg, &qcamvc->vchan, sizeof(struct video_channel)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	up(&qcamvc->busy);
	return 0;
      }
      
    case VIDIOCSCHAN:
      {
	int v;
	if(copy_from_user(&v, arg,sizeof(int)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	if(v!=0)
	  {
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }
	up(&qcamvc->busy);
	return 0;
      }
      
      /* Video picture ioctl */
    case VIDIOCGPICT:
      {
	if (copy_to_user(arg, &qcamvc->vpic, sizeof(struct video_picture)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	up(&qcamvc->busy);	
	return 0;
      }
      
    case VIDIOCSPICT:
      {
	struct video_picture p;
	
	if(copy_from_user(&p, arg, sizeof(struct video_picture)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	
	if (p.depth != 24 || p.palette != VIDEO_PALETTE_RGB24)
	  {
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }
	
	if (p.brightness >> 8 != qcamvc->brightness)
	  {
	    qcamvc->brightness = p.brightness >> 8;
	    qcamvc_set_brightness(qcamvc,qcamvc->brightness);
	  }
	
	if (p.whiteness >> 8 != qcamvc->exposure)
	  {
	    qcamvc->exposure = p.whiteness >> 8;
	    qcamvc_set_exposure(qcamvc,qcamvc->exposure);
	  }
	
	if (p.contrast != qcamvc->vpic.contrast)
	  {
	    qcamvc->blue_hue = p.contrast >> 8;
	    qcamvc->hue_calc=0;
	  }

	if (p.hue != qcamvc->vpic.hue)
	  {
	    qcamvc->red_hue = p.hue >> 8;
	    qcamvc->hue_calc=0;
	  }

	if (p.colour  != qcamvc->vpic.colour)
	  {
	    qcamvc->santrast = p.colour >> 8;
	    qcamvc->hue_calc=0;
	  }
	
	qcamvc->vpic = p;

	up(&qcamvc->busy);
	return 0;
      }
      
      /* Capture window ioctl */
   case VIDIOCSWIN:
     {
       struct video_window vw;
	 int res = qcamvc->res;
	   
	   if(copy_from_user(&vw, arg,sizeof(struct video_window)))
	     {
	       up(&qcamvc->busy);
	       return -EFAULT;
	     }
	   
	   if(vw.flags)
	     {
	       printk("1\n");
	       up(&qcamvc->busy);
	       return -EINVAL;
	     }
		   
	   if(vw.clipcount)
	     {
	       printk("2\n");
	       up(&qcamvc->busy);
	       return -EINVAL;
	     }
		       
	if(vw.height<MIN_HEIGHT || vw.height>MAX_HEIGHT)
	  {
	    printk("Want to set %dx%d 3\n",vw.width, vw.height);
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }
 
	if(vw.width<MIN_WIDTH || vw.width>MAX_WIDTH)
	  {
	    printk("Want to set %dx%d 4\n",vw.width, vw.height);
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }

	if(vw.width>=160 && vw.height>=120)
	  {
	    res=R160x120;
	  }
	if(vw.width>=176 && vw.height>=144)
	  {
	    res=R176x144;
	  }
	if(vw.width>=320 && vw.height>=240)
	  {
	    res=R320x240;
	  }
	if(vw.width>=352 && vw.height>=288)
	  {
	    res=R352x288;
	  }
	qcamvc_set_res(qcamvc, res);
	up(&qcamvc->busy);
	return 0;
      }
      
    case VIDIOCGWIN:
      {
	struct video_window vw;
	
	memset(&vw, 0, sizeof(struct video_window));
	vw.width  = qcamvc->width;
	vw.height = qcamvc->height;
	vw.flags  = 0;
	
	if(copy_to_user(arg, &vw, sizeof(struct video_window)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	up(&qcamvc->busy);
	return 0;
      }
#ifdef USE_MMAP      
      /* mmap interface */
    case VIDIOCGMBUF:
      {
        struct video_mbuf vm;
        int i;
	
	memset(&vm, 0, sizeof(struct video_mbuf));
	vm.size = qcamvc->width * qcamvc->height *
	  3 * QUICKCAM_NUMFRAMES;
	vm.frames = QUICKCAM_NUMFRAMES;
	
	for (i = 0; i < QUICKCAM_NUMFRAMES ; i++)
	  vm.offsets[i] = qcamvc->width * qcamvc->height *
	    3  * i;
	
	if (copy_to_user((void *)arg, (void *)&vm, sizeof(struct video_mbuf)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	up(&qcamvc->busy);
	return 0;
      }
      
    case VIDIOCSYNC:
      {
        int frame;
	
        if (copy_from_user((void *)&frame, arg, sizeof(int)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	
	if (frame<0 || frame >= QUICKCAM_NUMFRAMES )
	  {
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }
	
	switch (qcamvc->frame[frame].state) 
	  {
	  case FRAME_UNUSED:
	  case FRAME_READY:
	  case FRAME_GRABBING:
	    {
	      printk("sync to unused frame %d\n", frame);
	      up(&qcamvc->busy);
	      return -EINVAL;
	    }
	    
	  case FRAME_DONE:
	    qcamvc->frame[frame].state = FRAME_UNUSED;
	  }
	up(&qcamvc->busy);
	return 0;
      }
      
    case VIDIOCMCAPTURE:
      {
	struct video_mmap vm;
	
	if (copy_from_user((void *)&vm, (void *)arg, sizeof(struct video_mmap)))
	  {
	    up(&qcamvc->busy);
	    return -EFAULT;
	  }
	
        if (vm.frame<0||vm.frame>QUICKCAM_NUMFRAMES)
	  {
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }
	
        if (vm.format != VIDEO_PALETTE_YUV422)
	  {
	    up(&qcamvc->busy);
	    return -EINVAL;
	  }
	
	if (qcamvc->frame[vm.frame].state == FRAME_UNUSED)
	  {
	    up(&qcamvc->busy);
	    return capture_frame(qcamvc, &vm);
	  }
      }
#else /* USE_MMAP */   
    case VIDIOCGMBUF:
    case VIDIOCSYNC:
    case VIDIOCMCAPTURE:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
#endif/* USE_MMAP */
      
      /* No overlay support */
    case VIDIOCCAPTURE:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      
    case VIDIOCSFBUF:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }

    case VIDIOCGFBUF:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      
    case VIDIOCKEY:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      /* No tuner interface */
    case VIDIOCGTUNER:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }

    case VIDIOCSTUNER:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      
    case VIDIOCGFREQ:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      
    case VIDIOCSFREQ:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      
      /* No audio interface */
    case VIDIOCGAUDIO:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
	
    case VIDIOCSAUDIO:
      {
	up(&qcamvc->busy);
	return -EINVAL;
      }
      
    default:
      {
	up(&qcamvc->busy);
	return -ENOIOCTLCMD;
      }
    }
  up(&qcamvc->busy);
  return 0;
}


static int qcamvc_vopen(struct video_device *dev, int flags)
{
  struct qcamvc *qcamvc = dev->priv;
#ifdef DBG_CAM
   printk("USB QuickCam Open:\n");
#endif

#ifdef USE_MMAP
  /* Set the frames */
  qcamvc->current_frame = 0;
  for (i = 0; i < QUICKCAM_NUMFRAMES; i++) {
    qcamvc->frame[i].state = FRAME_UNUSED;
    qcamvc->frame[i].data = NULL;
  }
#endif
  qcamvc->ops->qcamvc_open(qcamvc->lowlevel_data);
  qcamvc->open_count++;
  return 0;
}


static void qcamvc_vclose(struct video_device *dev)
{
  struct qcamvc *qcamvc = dev->priv;
#ifdef USE_MMAP
  free_frame_buf(qcamvc);
#endif  
  qcamvc->ops->qcamvc_close(qcamvc->lowlevel_data);
  qcamvc->open_count--;
  
  
}

/*****************************************************************************/
/*                                                                           */
/* QuickCam Video Camera layer                                               */
/* This layer want be indipendent from the camera structure (PP/USB)         */
/*                                                                           */
/*****************************************************************************/
static void qcamvc_video_init(struct qcamvc *qcamvc)
{
  if (qcamvc == NULL)
    return;

  /* Set the video capability */
  memset(&qcamvc->vcap, 0, sizeof(struct video_capability));
  strcpy(qcamvc->vcap.name, qcamvc->camtype);

  qcamvc->vcap.type      = VID_TYPE_CAPTURE;
  qcamvc->vcap.channels  = 1;
  qcamvc->vcap.audios    = 0;
  qcamvc->vcap.maxwidth  = MAX_WIDTH;
  qcamvc->vcap.maxheight = MAX_HEIGHT;
  qcamvc->vcap.minwidth  = MIN_WIDTH;
  qcamvc->vcap.minheight = MIN_HEIGHT;
  qcamvc->res            = R352x288;

  /* Set the video channel structure */
  memset(&qcamvc->vchan,  0, sizeof(struct video_channel));
  qcamvc->vchan.flags   = 0;
  qcamvc->vchan.tuners  = 0;
  qcamvc->vchan.channel = 0;
  qcamvc->vchan.type    = VIDEO_TYPE_CAMERA;
  strcpy(qcamvc->vchan.name, "Camera");

  /* Set the video picture structure */
  memset(&qcamvc->vpic,  0, sizeof(struct video_picture));

  qcamvc->bpc=bpc6;
  qcamvc->brightness = 175;
  qcamvc->exposure   = 150;
  qcamvc->hue_calc   = 0;
  qcamvc->blue_hue   = 132;
  qcamvc->red_hue    = 52;
  qcamvc->santrast   = 50;

  qcamvc->vpic.colour     = qcamvc->santrast   * 0xff;
  qcamvc->vpic.hue        = qcamvc->red_hue    * 0xff;
  qcamvc->vpic.brightness = qcamvc->brightness * 0xff;
  qcamvc->vpic.contrast   = qcamvc->blue_hue   * 0xff;
  qcamvc->vpic.whiteness  = qcamvc->exposure   * 0xff;
  qcamvc->vpic.depth      = 24;
  qcamvc->vpic.palette    = VIDEO_PALETTE_RGB24;
  
  init_MUTEX(&qcamvc->busy);

  memcpy(&qcamvc->vdev,  &qcamvc_template,  sizeof(qcamvc_template));
  qcamvc->vdev.priv = qcamvc;

#if LINUX_VERSION_CODE >= 0x020405
  if (video_register_device(&qcamvc->vdev, VFL_TYPE_GRABBER, -1) == -1) {
#else
  if (video_register_device(&qcamvc->vdev, VFL_TYPE_GRABBER) == -1) {
#endif
    printk("Quickcam VC: video_register_device failed.\n"); 
    memset (qcamvc, 0, sizeof(struct qcamvc));
  }

  qcamvc_camera_init(qcamvc);
}

struct qcamvc *qcamvc_register_camera(struct qcamvc_camera_ops *ops, void *lowlevel, char *camtype)
{
  struct qcamvc *qcamvc;

  if ((qcamvc = kmalloc(sizeof(struct qcamvc), GFP_KERNEL)) == NULL) {
    printk("QuickCam VC:Error allocating memory in qcamvc_register_camera\n");
    return NULL;
  }
  qcamvc->lowlevel_data = lowlevel;
  qcamvc->ops = ops;
  qcamvc->open_count = 0;
  qcamvc->camtype=kmalloc(strlen(camtype), GFP_KERNEL);
  strcpy(qcamvc->camtype, camtype);

  qcamvc_video_init(qcamvc);
  MOD_INC_USE_COUNT;
  printk("Camera sucessfull registered\n");
  return qcamvc;
}

void qcamvc_unregister_camera(struct qcamvc *qcamvc)
{
  if (!qcamvc->open_count){
    #if defined(CONFIG_PROC_FS) && defined(CONFIG_VIDEO_PROC_FS)
    destroy_proc_qcamvc (qcamvc);      
    #endif
    printk("QuickCam VC: unregistering video\n");
    video_unregister_device(&qcamvc->vdev);
    kfree(qcamvc->camtype);
    qcamvc->camtype=NULL;
  } else {
    qcamvc->ops=NULL;
  }

  if(!qcamvc->open_count){
    kfree(qcamvc);
  }
  MOD_DEC_USE_COUNT;
}

/*****************************************************************************/
/*                                                                           */
/* QuickCam Video Camera layer                                               */
/* Module subs                                                               */
/*                                                                           */
/*****************************************************************************/

static int __init qcamvc_driver_init(void)
{
  printk("QuickCam VC: (C) 2001 by De Marchi Daniele, <demarchidaniele@libero.it>\n");
  printk("QuickCam VC: v4l level driver version %s registered.\n",QCAMVCVERSION);
#if defined(CONFIG_PROC_FS) && defined(CONFIG_VIDEO_PROC_FS)
  proc_qcamvc_create();
#endif

#ifdef CONFIG_KMOD
#ifdef CONFIG_VIDEO_QCAMVC_USB_MODULE
  request_module("qcamvc_usb");
#endif
#ifdef CONFIG_VIDEO_QCAMVC_PP_MODULE
  request_module("qcamvc_pp");
#endif
#endif

#ifdef CONFIG_VIDEO_QCAMVC_PP
  qcamvc_pp_init();
#endif
#ifdef CONFIG_VIDEO_QCAMVC_USB
  qcamvc_usb_init();
#endif


return 0;
}

static void __exit qcamvc_driver_exit(void)
{
#if defined(CONFIG_PROC_FS) && defined(CONFIG_VIDEO_PROC_FS)
  proc_quickcam_destroy();
#endif
  printk("QuickCam VC: v4l driver unregistered\n");
}

module_init(qcamvc_driver_init);
module_exit(qcamvc_driver_exit);

/* Exported symbols for modules. */

EXPORT_SYMBOL(qcamvc_register_camera);
EXPORT_SYMBOL(qcamvc_unregister_camera);
