/*
    cx88x-audio.c - Conexant CX23880/23881 audio downstream driver driver

     (c) 2001 Michael Eskin, Tom Zakrajsek [Windows version]
     (c) 2002 Yurij Sysoev <yurij@naturesoft.net>
     (c) 2003 Gerd Knorr <kraxel@bytesex.org>

    -----------------------------------------------------------------------

    Lot of voodoo here.  Even the data sheet doesn't help to
    understand what is going on here, the documentation for the audio
    part of the cx2388x chip is *very* bad.

    Some of this comes from party done linux driver sources I got from
    [undocumented].

    Some comes from the dscaler sources, the dscaler driver guy works
    for Conexant ...
    
    -----------------------------------------------------------------------
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/poll.h>
#include <linux/pci.h>
#include <linux/signal.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/vmalloc.h>
#include <linux/init.h>

#include "cx88.h"

static unsigned int audio_debug = UNSET;
MODULE_PARM(audio_debug,"i");
MODULE_PARM_DESC(audio_debug,"enable debug messages [audio]");

#define dprintk(fmt, arg...)	if (audio_debug) \
	printk(KERN_DEBUG "%s: " fmt, dev->name, ## arg)

/* ----------------------------------------------------------- */

static void set_audio_standard_BTSC(struct cx8800_dev *dev, unsigned int sap)
{
	dprintk("set_audio_standard_BTSC()\n");

	// BTSC stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	if (sap) {
		// configure BTSC + SAP
		cx_write(AUD_INIT, 0x0008);
	} else {
		// configure BTSC
		cx_write(AUD_INIT, 0x0001);
	}

	cx_write(AUD_INIT_LD, 0x0001);
	cx_write(AUD_SOFT_RESET, 0x0001);
	
	if (sap) {
		// SAP Rev 2.0 values (damn good -> -32dB)
		cx_write(AUD_DBX_IN_GAIN,0xa5d0);
		cx_write(AUD_DBX_WBE_GAIN,0x4163);
		cx_write(AUD_DBX_SE_GAIN,0x5c0a);
		cx_write(AUD_DEEMPH0_G0,0x0c02);
		cx_write(AUD_PHASE_FIX_CTL,0x0030);
		cx_write(AUD_IIR3_1_SHIFT,0x0000);
		cx_write(AUD_CTL,0x1002);
	} else {
		// BTSC Rev 2.0 values (damn good -> -32dB)
		cx_write(AUD_DBX_IN_GAIN, 0xa5d0);
		cx_write(AUD_DBX_WBE_GAIN, 0x4163);
		cx_write(AUD_DBX_SE_GAIN, 0x5c0a);
		cx_write(AUD_DEEMPH0_G0, 0x0c02);
		cx_write(AUD_PHASE_FIX_CTL, 0x0030);
	}

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// AB: 10/2/01: this register is not being reset appropriately on occasion.
	cx_write(AUD_POLYPH80SCALEFAC,3);
}

#if 0
static void set_audio_standard_I2SPT(struct cx8800_dev *dev)
{
	uint dwFreq,dwDDS;

	dprintk("set_audio_standard_I2SPT()\n");
	cx_andor(MO_GP2_IO, 0x00010101, 0x00010101);

	// xtal = 28.636 MHz
	cx_write(AUD_INIT,         0x0001);
	cx_write(AUD_INIT_LD,      0x0001);
	cx_write(AUD_SOFT_RESET,   0x0001);

	// Enabled I2S to DAC passthrough
	cx_write(AUD_CTL,          0xD000);

	// Set Philips format I2S
	cx_write(AUD_I2SINPUTCNTL, 0x0000);

	// Calc SRC value based on sample frequency
	dwFreq = dev->audio_properties.I2SPTFrequency; //DEFAULT_I2SPT_FREQUENCY
	dwFreq /= 100;

	// Precalculate constant based on:
	// DDS = (output_rate (48162.28081 KHz) * 2^24) / (input_rate * 4)
	// We precalculate the (output_rate (48162.28081 KHz) * 2^24) / 400
	// and divide the frequency by 100 so the whole thing fits in a DWORD

	dwDDS = 2020072471 / dwFreq;

	cx_write(AUD_I2S_RA_DDS,dwDDS);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// Unmute *TEST*
	unmute_audio(dev);
}
#endif

static void set_audio_standard_NICAM_I(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_NICAM_I()\n");

	// NICAM stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// get register definitions

	// initialize NICAM
	cx_write(AUD_INIT,      0x0010);
	cx_write(AUD_INIT_LD,   0x0001);
	cx_write(AUD_SOFT_RESET,0x0001);

	cx_write(AUD_RATE_ADJ1, 0x0010);
	cx_write(AUD_RATE_ADJ2, 0x0040);
	cx_write(AUD_RATE_ADJ3, 0x0100);
	cx_write(AUD_RATE_ADJ4, 0x0400);
	cx_write(AUD_RATE_ADJ5, 0x1000);

	// Rev F and above
	cx_write(AUD_DEEMPHGAIN_R,   0x023C2);
	cx_write(AUD_DEEMPHNUMER1_R, 0x2A7BC);
	cx_write(AUD_DEEMPHNUMER2_R, 0x3023E);
	cx_write(AUD_DEEMPHDENOM1_R, 0x0F3D0);
	cx_write(AUD_DEEMPHDENOM2_R, 0x00000);

	cx_write(AUD_ERRLOGPERIOD_R,    0xFFF);
	cx_write(AUD_ERRINTRPTTHSHLD1_R,0x3FF);
	cx_write(AUD_ERRINTRPTTHSHLD2_R,0x0FF);
	cx_write(AUD_ERRINTRPTTHSHLD3_R,0x03F);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// AB: 10/2/01: this register is not being reset appropriately on occasion.
	cx_write(AUD_POLYPH80SCALEFAC,3);
}

static void set_audio_standard_NICAM_BGDKL(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_NICAM_BGDKL()\n");

	// NICAM stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// get register definitions

	// initialize NICAM
	cx_write(AUD_INIT,      0x0010);
	cx_write(AUD_INIT_LD,   0x0001);
	cx_write(AUD_SOFT_RESET,0x0001);

	cx_write(AUD_RATE_ADJ1, 0x0010);
	cx_write(AUD_RATE_ADJ2, 0x0040);
	cx_write(AUD_RATE_ADJ3, 0x0100);
	cx_write(AUD_RATE_ADJ4, 0x0400);
	cx_write(AUD_RATE_ADJ5, 0x1000);

	cx_write(AUD_PDF_DDS_CNST_BYTE2,        0x0006);
	cx_write(AUD_PDF_DDS_CNST_BYTE1,        0x0082);
	cx_write(AUD_PDF_DDS_CNST_BYTE0,        0x0016);
	cx_write(AUD_QAM_MODE,                  0x0005);
	cx_write(AUD_PHACC_FREQ_8MSB,           0x0034);
        cx_write(0x320d2b,                      0x004c);

#if 0
	cx_write(AUD_ERRLOGPERIOD_R,     0xFFF);
	cx_write(AUD_ERRINTRPTTHSHLD1_R, 0x3FF);
	cx_write(AUD_ERRINTRPTTHSHLD2_R, 0x0FF);
	cx_write(AUD_ERRINTRPTTHSHLD3_R, 0x03F);

	// Rev F and above
	cx_write(AUD_DEEMPHGAIN_R,   0x011E1);
	cx_write(AUD_DEEMPHNUMER1_R, 0x2A7BC);
	cx_write(AUD_DEEMPHNUMER2_R, 0x3023E);
	cx_write(AUD_DEEMPHDENOM1_R, 0x0F3D0);
	cx_write(AUD_DEEMPHDENOM2_R, 0x00000);
#endif

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	cx_write(3,AUD_POLYPH80SCALEFAC);
}

static void set_audio_standard_A1(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_A1()\n");

	// A2 stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// get register definitions

	// initialize A1
	cx_write(AUD_INIT,       0x00000004);
	cx_write(AUD_INIT_LD,    0x00000001);
	cx_write(AUD_SOFT_RESET, 0x00000001);
	cx_write(AUD_DMD_RA_DDS, 0x002a73bd);
	cx_write(AUD_RATE_ADJ1,  0x0100    );
	cx_write(AUD_RATE_ADJ2,  0x0200    );
	cx_write(AUD_RATE_ADJ3,  0x0300    );
	cx_write(AUD_RATE_ADJ4,  0x0400    );
	cx_write(AUD_RATE_ADJ5,  0x0500    );

	// fast-in, fast-out, stereo & dual
	cx_write(AUD_THR_FR,0x0000);

	// implement 3rd order elliptic
	// stopband @ 0.4*Fs -100dB
	cx_write(AUD_PILOT_BQD_1_K0, 0x0000755b);
	cx_write(AUD_PILOT_BQD_1_K1, 0x00551340);
	cx_write(AUD_PILOT_BQD_1_K2, 0x006d30be);
	cx_write(AUD_PILOT_BQD_1_K3, 0xffd394af);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K0, 0x00040000);
	cx_write(AUD_PILOT_BQD_2_K1, 0x002a4841);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K3, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K4, 0x00000000);

	cx_write(AUD_MODE_CHG_TIMER, 0x0040);

	cx_write(AUD_START_TIMER,    0x0200);
	cx_write(AUD_AFE_16DB_EN,    0x0000);
	cx_write(AUD_CORDIC_SHIFT_0, 0x0007);
	cx_write(AUD_CORDIC_SHIFT_1, 0x0007);

	// tweaks in munich 02/16/02
	cx_write(AUD_C1_UP_THR, 0x7000);
	cx_write(AUD_C1_LO_THR, 0x5400);
	cx_write(AUD_C2_UP_THR, 0x5400);
	cx_write(AUD_C2_LO_THR, 0x3000);

	// AB: 9/25/01: to prevent IIR3 overflows, and correct stereo separation.
	cx_write(AUD_DEEMPH0_G0, 0x0380);
	cx_write(AUD_DEEMPH1_G0, 0x0380);

	//AUD_DN0_FREQ = 2^16*(carrier freq/adc sample rate)
	cx_write(AUD_DN0_FREQ, 0x35A3);
	cx_write(AUD_DN2_FREQ, 0x29c7);

	// setup AFC
	cx_write(AUD_DCOC_0_SRC,       0x001a);
	cx_write(AUD_DCOC0_SHIFT,      0x0000);
	cx_write(AUD_DCOC_0_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_0_SHIFT_IN1, 0x0008);
	cx_write(AUD_DCOC_PASS_IN,     0x0003);
	cx_write(AUD_IIR3_0_SEL,       0x0021);
	// setup FM-2
	cx_write(AUD_DN2_AFC,          0x0002);
	cx_write(AUD_DCOC_1_SRC,       0x001b);
	cx_write(AUD_DCOC1_SHIFT,      0x0000);
	cx_write(AUD_DCOC_1_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_1_SHIFT_IN1, 0x0008);
	cx_write(AUD_IIR3_1_SEL,       0x0023);

	//AUD_DN0_FREQ = 2^16*(carrier freq/adc sample rate)
	cx_write(AUD_DN0_FREQ,0x35A3);
	cx_write(AUD_DN2_FREQ,0x29c7);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// force A2 Mono1
	cx_write(AUD_CTL,0x1008);

	cx_write(AUD_POLYPH80SCALEFAC,1);
}

static void set_audio_standard_A2_BG(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_A2_BG()\n");

	// A2 stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// get register definitions

	// initialize A2
	cx_write(AUD_INIT,       0x00000004);
	cx_write(AUD_INIT_LD,    0x00000001);
	cx_write(AUD_SOFT_RESET, 0x00000001);

#if 1
	/* from dscaler */
	cx_write(AUD_DN0_FREQ,             0x0000312b);
	cx_write(AUD_POLY0_DDS_CONSTANT,   0x000a62b4);
	cx_write(AUD_IIR1_0_SEL,           0x00000000);
	cx_write(AUD_IIR1_1_SEL,           0x00000001);
	cx_write(AUD_IIR1_2_SEL,           0x0000001f);	// 0x320160
	cx_write(AUD_IIR1_3_SEL,           0x00000020);	// 0x320168
	cx_write(AUD_IIR1_4_SEL,           0x00000023);	// 0x320170
	cx_write(AUD_IIR1_5_SEL,           0x00000007);	// 0x320180
	cx_write(AUD_IIR1_0_SHIFT,         0x00000000);	// 0x320154
	cx_write(AUD_IIR1_1_SHIFT,         0x00000000);	// 0x32015c
	cx_write(AUD_IIR1_2_SHIFT,         0x00000007);	// 0x320164
	cx_write(AUD_IIR1_3_SHIFT,         0x00000007);	// 0x32016c
	cx_write(AUD_IIR1_4_SHIFT,         0x00000007);	// 0x32017c
	cx_write(AUD_IIR1_5_SHIFT,         0x00000000);	// 0x320184
	cx_write(AUD_IIR2_0_SEL,           0x00000002);	// 0x320190
	cx_write(AUD_IIR2_1_SEL,           0x00000003);	// 0x320198
	cx_write(AUD_IIR2_2_SEL,           0x00000004);	// 0x3201a0
	cx_write(AUD_IIR2_3_SEL,           0x00000005);	// 0x3201a8
	cx_write(AUD_IIR3_0_SEL,           0x00000021);	// 0x3201c0
	cx_write(AUD_IIR3_1_SEL,           0x00000023);	// 0x3201c8
	cx_write(AUD_IIR3_2_SEL,           0x00000016);	// 0x3201d0
	cx_write(AUD_IIR3_0_SHIFT,         0x00000000);	// 0x3201c4
	cx_write(AUD_IIR3_1_SHIFT,         0x00000000);	// 0x3201cc
	cx_write(AUD_IIR3_2_SHIFT,         0x00000000);	// 0x3201d4
	cx_write(AUD_IIR4_0_SEL,           0x0000001d);	// 0x3201e0
	cx_write(AUD_IIR4_1_SEL,           0x00000019);	// 0x3201e8
	cx_write(AUD_IIR4_2_SEL,           0x00000008);	// 0x3201f0
	cx_write(AUD_IIR4_0_SHIFT,         0x00000000);	// 0x3201e4
	cx_write(AUD_IIR4_1_SHIFT,         0x00000000);	// 0x3201ec
	cx_write(AUD_IIR4_2_SHIFT,         0x00000001);	// 0x3201f4
	cx_write(AUD_IIR4_0_CA0,           0x0003e57e);	// 0x320200
	cx_write(AUD_IIR4_0_CA1,           0x00005e11);	// 0x320204
	cx_write(AUD_IIR4_0_CA2,           0x0003a7cf);	// 0x320208
	cx_write(AUD_IIR4_0_CB0,           0x00002368);	// 0x32020c
	cx_write(AUD_IIR4_0_CB1,           0x0003bf1b);	// 0x320210
	cx_write(AUD_IIR4_1_CA0,           0x00006349);	// 0x320214
	cx_write(AUD_IIR4_1_CA1,           0x00006f27);	// 0x320218
	cx_write(AUD_IIR4_1_CA2,           0x0000e7a3);	// 0x32021c
	cx_write(AUD_IIR4_1_CB0,           0x00005653);	// 0x320220
	cx_write(AUD_IIR4_1_CB1,           0x0000cf97);	// 0x320224
	cx_write(AUD_IIR4_2_CA0,           0x00006349);	// 0x320228
	cx_write(AUD_IIR4_2_CA1,           0x00006f27);	// 0x32022c
	cx_write(AUD_IIR4_2_CA2,           0x0000e7a3);	// 0x320230
	cx_write(AUD_IIR4_2_CB0,           0x00005653);	// 0x320234
	cx_write(AUD_IIR4_2_CB1,           0x0000cf97);	// 0x320238
	cx_write(AUD_HP_MD_IIR4_1,         0x00000001);	// 0x320250
	cx_write(AUD_HP_PROG_IIR4_1,       0x00000017);	// 0x320254
	cx_write(AUD_DN1_FREQ,             0x00003618);	// 0x320278
	cx_write(AUD_DN1_SRC_SEL,          0x00000017);	// 0x320284
	cx_write(AUD_DN1_SHFT,             0x00000007);	// 0x320288
	cx_write(AUD_DN1_AFC,              0x00000000);	// 0x320280
	cx_write(AUD_DN1_FREQ_SHIFT,       0x00000000);	// 0x32027c
	cx_write(AUD_DN2_SRC_SEL,          0x00000040);	// 0x320298
	cx_write(AUD_DN2_SHFT,             0x00000000);	// 0x32029c
	cx_write(AUD_DN2_AFC,              0x00000002);	// 0x320294
	cx_write(AUD_DN2_FREQ,             0x0000caaf);	// 0x32028c
	cx_write(AUD_DN2_FREQ_SHIFT,       0x00000000);	// 0x32028c
	cx_write(AUD_PDET_SRC,             0x00000014);	// 0x320370
	cx_write(AUD_PDET_SHIFT,           0x00000000);	// 0x320374
	cx_write(AUD_DEEMPH0_SRC_SEL,      0x00000011);	// 0x320440
	cx_write(AUD_DEEMPH1_SRC_SEL,      0x00000013);	// 0x32045c
	cx_write(AUD_DEEMPH0_SHIFT,        0x00000000);	// 0x320444
	cx_write(AUD_DEEMPH1_SHIFT,        0x00000000);	// 0x320460
	cx_write(AUD_DEEMPH0_G0,           0x000004da);	// 0x320448
	cx_write(AUD_DEEMPH0_A0,           0x0000777a);	// 0x32044c
	cx_write(AUD_DEEMPH0_B0,           0x00000000);	// 0x320450
	cx_write(AUD_DEEMPH0_A1,           0x0003f062);	// 0x320454
	cx_write(AUD_DEEMPH0_B1,           0x00000000);	// 0x320458
	cx_write(AUD_DEEMPH1_G0,           0x000004da);	// 0x320464
	cx_write(AUD_DEEMPH1_A0,           0x0000777a);	// 0x320468
	cx_write(AUD_DEEMPH1_B0,           0x00000000);	// 0x32046c
	cx_write(AUD_DEEMPH1_A1,           0x0003f062);	// 0x320470
	cx_write(AUD_DEEMPH1_B1,           0x00000000);	// 0x320474
	cx_write(AUD_PLL_EN,               0x00000000);	// 0x320400
	cx_write(AUD_DMD_RA_DDS,           0x002a4efb);	// 0x3205bc
	cx_write(AUD_RATE_ADJ1,            0x1000);	// 0x3205d8
	cx_write(AUD_RATE_ADJ2,            0x2000);	// 0x3205dc
	cx_write(AUD_RATE_ADJ3,            0x3000);	// 0x3205e0
	cx_write(AUD_RATE_ADJ4,            0x4000);	// 0x3205e4
	cx_write(AUD_RATE_ADJ5,            0x5000);	// 0x3205e8
	cx_write(AUD_C2_UP_THR,            0xffff);	// 0x3203d8
	cx_write(AUD_C2_LO_THR,            0xe800);	// 0x3203dc
	cx_write(AUD_C1_UP_THR,            0x8c00);	// 0x3203d0
	cx_write(AUD_C1_LO_THR,            0x6c00);	// 0x3203d4
	cx_write(AUD_DCOC_0_SRC,           0x0021);	// 0x320320
	cx_write(AUD_DCOC_1_SRC,           0x001a);	// 0x320330
	cx_write(AUD_DCOC1_SHIFT,          0x0000);	// 0x320334
	cx_write(AUD_DCOC_1_SHIFT_IN0,     0x000a);	// 0x320338
	cx_write(AUD_DCOC_1_SHIFT_IN1,     0x0008);	// 0x32033c
	cx_write(AUD_DCOC_PASS_IN,         0x0000);	// 0x320350
	cx_write(AUD_IIR4_0_SEL,           0x0023);	// 0x3201e0
	cx_write(AUD_DN1_AFC,              0x0000);	// 0x320280
	cx_write(AUD_DCOC_2_SRC,           0x001b);	// 0x320340
	cx_write(AUD_IIR4_1_SEL,           0x0025);	// 0x3201e8
	cx_write(AUD_DN2_SRC_SEL,          0x0001);	// 0x320298
	cx_write(AUD_DN2_FREQ,             0x00003551);	// 0x32028c
	// setup Audio PLL
	//cx_write(AUD_PLL_PRESCALE,         0x0002);	// 0x320600
	//cx_write(AUD_PLL_INT,              0x001f);	// 0x320608
#endif

#if 0	/* old linux driver code */
	cx_write(AUD_DMD_RA_DDS, 0x002a73bd);
	cx_write(AUD_RATE_ADJ1,  0x0100    );
	cx_write(AUD_RATE_ADJ2,  0x0200    );
	cx_write(AUD_RATE_ADJ3,  0x0300    );
	cx_write(AUD_RATE_ADJ4,  0x0400    );
	cx_write(AUD_RATE_ADJ5,  0x0500    );

	// fast-in, fast-out, stereo & dual
	cx_write(AUD_THR_FR,0x0000);

	// implement 3rd order elliptic
	// stopband @ 0.4*Fs -100dB
	cx_write(AUD_PILOT_BQD_1_K0, 0x0000755b);
	cx_write(AUD_PILOT_BQD_1_K1, 0x00551340);
	cx_write(AUD_PILOT_BQD_1_K2, 0x006d30be);
	cx_write(AUD_PILOT_BQD_1_K3, 0xffd394af);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K0, 0x00040000);
	cx_write(AUD_PILOT_BQD_2_K1, 0x002a4841);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K4, 0x00000000);

	cx_write(AUD_MODE_CHG_TIMER,0x0040);

	cx_write(AUD_START_TIMER,    0x0200);
	cx_write(AUD_AFE_16DB_EN,    0x0000);
	cx_write(AUD_CORDIC_SHIFT_0, 0x0007);
	cx_write(AUD_CORDIC_SHIFT_1, 0x0007);

	// tweaks in munich 02/16/02
	cx_write(AUD_C1_UP_THR, 0x7000);
	cx_write(AUD_C1_LO_THR, 0x5400);
	cx_write(AUD_C2_UP_THR, 0x5400);
	cx_write(AUD_C2_LO_THR, 0x3000);

	// AB: 9/25/01: to prevent IIR3 overflows, and correct stereo separation.
	cx_write(AUD_DEEMPH0_G0, 0x0380);
	cx_write(AUD_DEEMPH1_G0, 0x0380);

	// setup AFC
	cx_write(AUD_DCOC_0_SRC,       0x001a);
	cx_write(AUD_DCOC0_SHIFT,      0x0000);
	cx_write(AUD_DCOC_0_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_0_SHIFT_IN1, 0x0008);
	cx_write(AUD_DCOC_PASS_IN,     0x0003);
	cx_write(AUD_IIR3_0_SEL,       0x0021);
	// setup FM-2
	cx_write(AUD_DN2_AFC,          0x0002);
	cx_write(AUD_DCOC_1_SRC,       0x001b);
	cx_write(AUD_DCOC1_SHIFT,      0x0000);
	cx_write(AUD_DCOC_1_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_1_SHIFT_IN1, 0x0008);
	cx_write(AUD_IIR3_1_SEL,       0x0023);
#endif

#if 0	/* cut+paste from 11/2003 specs, probably useless :/ */
	cx_write(AUD_DMD_RA_DDS, 0x002A73BD);
	cx_write(AUD_RATE_ADJ1, 0x0100);
	cx_write(AUD_RATE_ADJ2, 0x0200);
	cx_write(AUD_RATE_ADJ3, 0x0300);
	cx_write(AUD_RATE_ADJ4, 0x0400);
	cx_write(AUD_RATE_ADJ5, 0x0500);
	cx_write(AUD_THR_FR, 0x0000);
	cx_write(AUD_PILOT_BQD_1_K0, 0x0001C000);
	cx_write(AUD_PILOT_BQD_1_K1, 0x00000000);
	cx_write(AUD_PILOT_BQD_1_K2, 0x00000000);
	cx_write(AUD_PILOT_BQD_1_K3, 0x00000000);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K0, 0x00C00000);
	cx_write(AUD_PILOT_BQD_2_K1, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K3, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K4, 0x00000000);
	cx_write(AUD_C2_UP_THR, 0x1C00);
	cx_write(AUD_C2_LO_THR, 0x1000);
	cx_write(AUD_C1_UP_THR, 0x4C00);
	cx_write(AUD_C1_LO_THR, 0x4000);
	cx_write(AUD_MODE_CHG_TIMER, 0x0030);
	cx_write(AUD_START_TIMER, 0x0200);
	cx_write(AUD_AFE_16DB_EN, 0x0001);
	cx_write(AUD_CORDIC_SHIFT_0, 0x0006);	
#endif

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	cx_write(AUD_POLYPH80SCALEFAC,1);
}

static void set_audio_standard_A2_DK(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_A2_DK()\n");

	// A2 stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// initialize A2
	cx_write(AUD_INIT,       0x00000004);
	cx_write(AUD_INIT_LD,    0x00000001);
	cx_write(AUD_SOFT_RESET, 0x00000001);
	cx_write(AUD_DMD_RA_DDS, 0x002a73bd);
	cx_write(AUD_RATE_ADJ1,  0x0100);
	cx_write(AUD_RATE_ADJ2,  0x0200);
	cx_write(AUD_RATE_ADJ3,  0x0300);
	cx_write(AUD_RATE_ADJ4,  0x0400);
	cx_write(AUD_RATE_ADJ5,  0x0500);

	// fast-in, fast-out, stereo & dual
	cx_write(AUD_THR_FR,0x0000);

	// implement 3rd order elliptic
	// stopband @ 0.4*Fs -100dB
	cx_write(AUD_PILOT_BQD_1_K0, 0x0000755b);
	cx_write(AUD_PILOT_BQD_1_K1, 0x00551340);
	cx_write(AUD_PILOT_BQD_1_K2, 0x006d30be);
	cx_write(AUD_PILOT_BQD_1_K3, 0xffd394af);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K0, 0x00040000);
	cx_write(AUD_PILOT_BQD_2_K1, 0x002a4841);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K3, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K4, 0x00000000);

	cx_write(AUD_MODE_CHG_TIMER, 0x0040);

	cx_write(AUD_START_TIMER,    0x0200);
	cx_write(AUD_AFE_16DB_EN,    0x0000);
	cx_write(AUD_CORDIC_SHIFT_0, 0x0007);
	cx_write(AUD_CORDIC_SHIFT_1, 0x0007);

	// tweaks in munich 02/16/02
	cx_write(AUD_C1_UP_THR, 0x7000);
	cx_write(AUD_C1_LO_THR, 0x5400);
	cx_write(AUD_C2_UP_THR, 0x5400);
	cx_write(AUD_C2_LO_THR, 0x3000);

	// AB: 9/25/01: to prevent IIR3 overflows, and correct stereo separation.
	cx_write(AUD_DEEMPH0_G0, 0x0380);
	cx_write(AUD_DEEMPH1_G0, 0x0380);

	// setup AFC
	cx_write(AUD_DCOC_0_SRC,       0x001a);
	cx_write(AUD_DCOC0_SHIFT,      0x0000);
	cx_write(AUD_DCOC_0_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_0_SHIFT_IN1, 0x0008);
	cx_write(AUD_DCOC_PASS_IN,     0x0003);
	cx_write(AUD_IIR3_0_SEL,       0x0021);
	// setup FM-2
	cx_write(AUD_DN2_AFC,          0x0002);
	cx_write(AUD_DCOC_1_SRC,       0x001b);
	cx_write(AUD_DCOC1_SHIFT,      0x0000);
	cx_write(AUD_DCOC_1_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_1_SHIFT_IN1, 0x0008);
	cx_write(AUD_IIR3_1_SEL,       0x0023);

	// for D/K1:
	cx_write(AUD_DN0_FREQ,0x3a1c);
	cx_write(AUD_DN2_FREQ,0xd2e0);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	cx_write(AUD_POLYPH80SCALEFAC,1);
}

static void set_audio_standard_A2_M(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_A2_M()\n");

	// A2 stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// initialize A2
	cx_write(AUD_INIT,       0x00000004);
	cx_write(AUD_INIT_LD,    0x00000001);
	cx_write(AUD_SOFT_RESET, 0x00000001);
	cx_write(AUD_DMD_RA_DDS, 0x002a73bd);
	cx_write(AUD_RATE_ADJ1, 0x0100);
	cx_write(AUD_RATE_ADJ2, 0x0200);
	cx_write(AUD_RATE_ADJ3, 0x0300);
	cx_write(AUD_RATE_ADJ4, 0x0400);
	cx_write(AUD_RATE_ADJ5, 0x0500);

	// fast-in, fast-out, stereo & dual
	cx_write(AUD_THR_FR,0x0000);

	// implement 3rd order elliptic
	// stopband @ 0.4*Fs -100dB
	cx_write(AUD_PILOT_BQD_1_K0, 0x0000755b);
	cx_write(AUD_PILOT_BQD_1_K1, 0x00551340);
	cx_write(AUD_PILOT_BQD_1_K2, 0x006d30be);
	cx_write(AUD_PILOT_BQD_1_K3, 0xffd394af);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K0, 0x00040000);
	cx_write(AUD_PILOT_BQD_2_K1, 0x002a4841);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K3, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K4, 0x00000000);

	cx_write(AUD_MODE_CHG_TIMER,0x0040);

	cx_write(AUD_START_TIMER,    0x0200);
	cx_write(AUD_AFE_16DB_EN,    0x0000);
	cx_write(AUD_CORDIC_SHIFT_0, 0x0007);
	cx_write(AUD_CORDIC_SHIFT_1, 0x0007);

	// tweaks in munich 02/16/02
	cx_write(AUD_C1_UP_THR, 0x7000);
	cx_write(AUD_C1_LO_THR, 0x5400);
	cx_write(AUD_C2_UP_THR, 0x5400);
	cx_write(AUD_C2_LO_THR, 0x3000);

	// AB: 9/25/01: to prevent IIR3 overflows, and correct stereo separation.
	cx_write(AUD_DEEMPH0_G0, 0x0380);
	cx_write(AUD_DEEMPH1_G0, 0x0380);

	// setup AFC
	cx_write(AUD_DCOC_0_SRC,       0x001a);
	cx_write(AUD_DCOC0_SHIFT,      0x0000);
	cx_write(AUD_DCOC_0_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_0_SHIFT_IN1, 0x0008);
	cx_write(AUD_DCOC_PASS_IN,     0x0003);
	cx_write(AUD_IIR3_0_SEL,       0x0021);
	// setup FM-2
	cx_write(AUD_DN2_AFC,          0x0002);
	cx_write(AUD_DCOC_1_SRC,       0x001b);
	cx_write(AUD_DCOC1_SHIFT,      0x0000);
	cx_write(AUD_DCOC_1_SHIFT_IN0, 0x000a);
	cx_write(AUD_DCOC_1_SHIFT_IN1, 0x0008);
	cx_write(AUD_IIR3_1_SEL,       0x0023);

	// for M:
	cx_write(AUD_DN0_FREQ, 0x2836);
	cx_write(AUD_DN2_FREQ, 0x29c7);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// AB: 10/2/01: this register is not being reset appropriately on occasion.
	cx_write(AUD_POLYPH80SCALEFAC,1);
}

static void set_audio_standard_EIAJ(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_EIAJ()\n");
	
	// initialize EIAJ
	cx_write(AUD_INIT,       0x0002);
	cx_write(AUD_INIT_LD,    0x0001);
	cx_write(AUD_SOFT_RESET, 0x0001);
	cx_write(AUD_RATE_ADJ1,  0x0100);
	cx_write(AUD_RATE_ADJ2,  0x0200);
	cx_write(AUD_RATE_ADJ3,  0x0300);
	cx_write(AUD_RATE_ADJ4,  0x0400);
	cx_write(AUD_RATE_ADJ5,  0x0500);

	// fast-in, fast-out, stereo & dual
	cx_write(AUD_THR_FR,         0x0001    );
	cx_write(AUD_PILOT_BQD_1_K0, 0x0000755b);
	cx_write(AUD_PILOT_BQD_1_K1, 0x00551340);
	cx_write(AUD_PILOT_BQD_1_K2, 0x006d30be);
	cx_write(AUD_PILOT_BQD_1_K3, 0xffd394af);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00400000);
	cx_write(AUD_PILOT_BQD_1_K4, 0x00040000);
	cx_write(AUD_PILOT_BQD_2_K1, 0x002a4841);
	cx_write(AUD_PILOT_BQD_2_K2, 0x00400000);
	cx_write(AUD_PILOT_BQD_2_K3, 0x00000000);
	cx_write(AUD_PILOT_BQD_2_K4, 0x00000000);
	cx_write(AUD_START_TIMER,    0x0200    );

	cx_write(AUD_AFE_16DB_EN,    0x0001);
	cx_write(AUD_PHASE_FIX_CTL,  0x0000);
	cx_write(AUD_CORDIC_SHIFT_0, 0x0006);
	cx_write(AUD_CORDIC_SHIFT_1, 0x0006);
	cx_write(AUD_DEEMPH0_G0,     0x0B02);
	cx_write(AUD_DEEMPH1_G0,     0x0DA6);

	// Changes requested by Chris 21 March 2002
	cx_write(AUD_POLY0_DDS_CONSTANT, 0x12010C);
	cx_write(AUD_DN2_FREQ,     0x3700  );
	cx_write(AUD_C1_UP_THR,    0x1300  );
	cx_write(AUD_C1_LO_THR,    0x1000  );
	cx_write(AUD_C2_UP_THR,    0x1000  );
	cx_write(AUD_C2_LO_THR,    0x0800  );
	cx_write(AUD_MODE_CHG_TIMER, 0x00A0  );

	// MAE 26 March 2002 - DDS fix for clicking problem reported in Japan
	cx_write(AUD_DMD_RA_DDS,0xC3E7AA);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	cx_write(AUD_POLYPH80SCALEFAC,3);
}

static void set_audio_standard_System_L_AM(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_System_L_AM\n");

	cx_write(AUD_INIT,0x0003);
	cx_write(AUD_INIT_LD,0x0001);
	cx_write(AUD_SOFT_RESET,0x0001);

	cx_write(AUD_CTL,0x1000);

	cx_write(AAGC_GAIN,   0x0000);
	cx_write(AAGC_HYST,   0x0010);
	cx_write(AUD_DN0_FREQ,0x3a77);
	cx_write(AUD_POLY0_DDS_CONSTANT,0xb715b);
	cx_write(AUD_IIR1_0_SEL,   0x0000);
	cx_write(AUD_IIR1_0_SHIFT, 0x0006);
	cx_write(AUD_IIR1_1_SEL,   0x0002);
	cx_write(AUD_IIR1_1_SHIFT, 0x0007);
	cx_write(AUD_IIR1_2_SEL,   0x0003);
	cx_write(AUD_IIR1_2_SHIFT, 0x0000);
	cx_write(AUD_IIR4_2_SEL,   0x0004);
	cx_write(AUD_IIR4_2_SHIFT, 0x0007);
	cx_write(AUD_PLL_EN,       0x0001);
	cx_write(AUD_PLL_SRC,      0x0019);
	cx_write(AUD_PLL_SHIFT,    0x0000);
	cx_write(AUD_PLL_IF_SEL,   0x0003);
	cx_write(AUD_PLL_IF_SHIFT, 0x0000);
	cx_write(AUD_DCOC_1_SRC,   0x001c);
	cx_write(AUD_DCOC1_SHIFT,  0x0000);
	cx_write(AUD_IIR1_3_SEL,   0x0023);
	cx_write(AUD_IIR1_3_SHIFT, 0x0002);
	cx_write(AUD_IIR1_4_SEL,   0x0005);
	cx_write(AUD_IIR1_4_SHIFT, 0x0007);
	cx_write(AUD_OUT0_SEL,     0x0007);
	cx_write(AUD_OUT1_SEL,     0x0007);
	cx_write(AUD_DMD_RA_DDS, 0x9a1e4f);

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// adjust volume for testing
	cx_write(AUD_VOL_CTL,0x0000);
}

static void set_audio_standard_FM(struct cx8800_dev *dev)
{
	dprintk("set_audio_standard_FM\n");

	// FM stereo -> RAM -> DAC standard setup
	// xtal = 28.636 MHz

	// get register definitions

	// initialize FM Radio
	cx_write(AUD_INIT,0x0020);
	cx_write(AUD_INIT_LD,0x0001);
	cx_write(AUD_SOFT_RESET,0x0001);

#if 0 /* FIXME */
	switch (dev->audio_properties.FM_deemphasis)
	{
		case WW_FM_DEEMPH_50:
			//Set De-emphasis filter coefficients for 50 usec
			cx_write(AUD_DEEMPH0_G0, 0x0C45);
			cx_write(AUD_DEEMPH0_A0, 0x6262);
			cx_write(AUD_DEEMPH0_B0, 0x1C29);
			cx_write(AUD_DEEMPH0_A1, 0x3FC66);
			cx_write(AUD_DEEMPH0_B1, 0x399A);

			cx_write(AUD_DEEMPH1_G0, 0x0D80);
			cx_write(AUD_DEEMPH1_A0, 0x6262);
			cx_write(AUD_DEEMPH1_B0, 0x1C29);
			cx_write(AUD_DEEMPH1_A1, 0x3FC66);
			cx_write(AUD_DEEMPH1_B1, 0x399A);
			
			break;

		case WW_FM_DEEMPH_75:
			//Set De-emphasis filter coefficients for 75 usec
			cx_write(AUD_DEEMPH0_G0, 0x91B );
			cx_write(AUD_DEEMPH0_A0, 0x6B68);
			cx_write(AUD_DEEMPH0_B0, 0x11EC);
			cx_write(AUD_DEEMPH0_A1, 0x3FC66);
			cx_write(AUD_DEEMPH0_B1, 0x399A);

			cx_write(AUD_DEEMPH1_G0, 0xAA0 );
			cx_write(AUD_DEEMPH1_A0, 0x6B68);
			cx_write(AUD_DEEMPH1_B0, 0x11EC);
			cx_write(AUD_DEEMPH1_A1, 0x3FC66);
			cx_write(AUD_DEEMPH1_B1, 0x399A);

			break;
	}
#endif

	// de-assert Audio soft reset
	cx_write(AUD_SOFT_RESET,0x0000);

	// AB: 10/2/01: this register is not being reset appropriately on occasion.
	cx_write(AUD_POLYPH80SCALEFAC,3);
}

/* ----------------------------------------------------------- */

void cx88_set_tvaudio(struct cx8800_dev *dev, unsigned int mode)
{
	cx_write(AUD_CTL, 0x00);

	switch (mode) {
	case WW_NONE:
		break;
		
	case WW_BTSC:
		set_audio_standard_BTSC(dev,0);
		break;
		
	case WW_NICAM_I:
		set_audio_standard_NICAM_I(dev);
		break;

	case WW_NICAM_BGDKL:
		set_audio_standard_NICAM_BGDKL(dev);
		break;

	case WW_A1:
		set_audio_standard_A1(dev);
		break;

	case WW_A2_BG:
		set_audio_standard_A2_BG(dev);
		break;

	case WW_A2_DK:
		set_audio_standard_A2_DK(dev);
		break;

	case WW_A2_M:
		set_audio_standard_A2_M(dev);
		break;

	case WW_EIAJ:
		set_audio_standard_EIAJ(dev);
		break;

	case WW_SYSTEM_L_AM:
		set_audio_standard_System_L_AM(dev);
		break;

	case WW_FM:
		set_audio_standard_FM(dev);
		break;
	}

	// unmute
	cx_set(AUD_CTL, EN_DAC_ENABLE);
	cx_write(AUD_VOL_CTL, 0x00);
	return;
}

void cx88_get_stereo(struct cx8800_dev *dev, u32 std, struct v4l2_tuner *t)
{
	static char *m[] = {"mono", "dual mono", "stereo", "sap"};
	static char *p[] = {"no pilot", "pilot c1", "pilot c2", "?"};
	u32 reg,mode,pilot;

	reg = cx_read(AUD_STATUS);
	mode  = reg & 0x03;
	pilot = (reg >> 2) & 0x03;
	dprintk("AUD_STATUS: %s / %s [status=0x%x,ctl=0x%x,vol=0x%x]\n",
		m[mode], p[pilot], reg,
		cx_read(AUD_CTL), cx_read(AUD_VOL_CTL));

	t->capability = V4L2_TUNER_CAP_STEREO | V4L2_TUNER_CAP_SAP |
		V4L2_TUNER_CAP_LANG1 | V4L2_TUNER_CAP_LANG2;
	t->rxsubchans = V4L2_TUNER_SUB_MONO;
	t->audmode    = V4L2_TUNER_MODE_MONO;

	switch (std) {
	case WW_A2_BG:
 		if (2 == pilot) {
			/* stereo */
			t->rxsubchans = V4L2_TUNER_SUB_MONO | V4L2_TUNER_SUB_STEREO;
			if (2 == mode)
				t->audmode = V4L2_TUNER_MODE_STEREO;
		}
 		if (1 == pilot) {
			/* dual language -- FIXME */
			t->rxsubchans = V4L2_TUNER_SUB_LANG1 | V4L2_TUNER_SUB_LANG2;
			t->audmode = V4L2_TUNER_MODE_LANG1;
		}
		break;
	case WW_NICAM_BGDKL:
		if (2 == mode)
			t->audmode = V4L2_TUNER_MODE_STEREO;
		break;
	}
	return;
}

void cx88_set_stereo(struct cx8800_dev *dev, u32 std, u32 mode)
{
	u32 ctl  = UNSET;
	u32 mask = UNSET;

	switch (std) {
	case WW_A2_BG:
		switch (mode) {
		case V4L2_TUNER_MODE_MONO:   
		case V4L2_TUNER_MODE_LANG1:
			ctl  = EN_A2_FORCE_MONO1;
			mask = 0x3f;
			break;
		case V4L2_TUNER_MODE_LANG2:
			ctl  = EN_A2_AUTO_MONO2;
			mask = 0x3f;
			break;
		case V4L2_TUNER_MODE_STEREO:
			ctl  = EN_A2_AUTO_STEREO | EN_DMTRX_SUMR;
			mask = 0x8bf;
			break;
		}
		break;
	case WW_NICAM_BGDKL:
		switch (mode) {
		case V4L2_TUNER_MODE_MONO:   
			ctl  = EN_NICAM_FORCE_MONO1;
			mask = 0x3f;
			break;
		case V4L2_TUNER_MODE_LANG1:
			ctl  = EN_NICAM_AUTO_MONO2;
			mask = 0x3f;
			break;
		case V4L2_TUNER_MODE_STEREO:
			ctl  = EN_NICAM_FORCE_STEREO | EN_DMTRX_LR;
			mask = 0x93f;
			break;
		}
		break;	
	case WW_FM:
		switch (mode) {
		case V4L2_TUNER_MODE_MONO:   
			ctl  = EN_FMRADIO_FORCE_MONO;
			mask = 0x3f;
			break;
		case V4L2_TUNER_MODE_STEREO:
			ctl  = EN_FMRADIO_AUTO_STEREO;
			mask = 0x3f;
			break;
		}
		break;	
	}

	if (UNSET != ctl) {
		cx_write(AUD_SOFT_RESET, 0x0001);
		cx_andor(AUD_CTL, mask, ctl);
		cx_write(AUD_SOFT_RESET, 0x0000);
		dprintk("cx88_set_stereo: mask 0x%x, ctl 0x%x "
			"[status=0x%x,ctl=0x%x,vol=0x%x]\n",
			mask, ctl, cx_read(AUD_STATUS),
			cx_read(AUD_CTL), cx_read(AUD_VOL_CTL));
	}
	return;
}

/*
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
