/*
 * device driver for philips saa7134 based TV cards
 * gpio interface support
 *
 * By David Atkinson <px-gpio'AT'psykax.com>
 *
 * Based on bttv-if.c (part of bttv - bt848 frame grabber driver)
 * Copyright (C) 1996,97,98 Ralph  Metzler (rjkm@thp.uni-koeln.de)
 *                        & Marcus Metzler (mocm@thp.uni-koeln.de)
 * (c) 1999-2003 Gerd Knorr <kraxel@bytesex.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/init.h>
#include <linux/list.h>
#include <linux/module.h>
#include <linux/kernel.h>

#include "saa7134-reg.h"
#include "saa7134.h"

EXPORT_SYMBOL(saa7134_get_cardinfo);
EXPORT_SYMBOL(saa7134_get_pcidev);
EXPORT_SYMBOL(saa7134_read_gpio);
EXPORT_SYMBOL(saa7134_write_gpio);
EXPORT_SYMBOL(saa7134_get_gpio_queue);

struct saa7134_dev* get_device(unsigned int card)
{
	/* Gets a pointer to a saa7134_dev struct from a card index */

	unsigned int list_count = 0;
	struct list_head* list_ptr;

	list_for_each(list_ptr, &saa7134_devlist) {
		if(list_count == card) break;
		list_count ++;
	}

	if(list_ptr == &saa7134_devlist) {
		return NULL;
	}

	return list_entry(list_ptr, struct saa7134_dev, devlist);
}

int saa7134_get_cardinfo(unsigned int card, unsigned int *board, unsigned int *reserved)
{
	struct saa7134_dev* dev;
	dev = get_device(card);
	if(dev == NULL) return -1;

	*board  = dev->board;
	*reserved = 0; /* reserved is for possible future use in board identification */

	return 0;
}

struct pci_dev* saa7134_get_pcidev(unsigned int card)
{
	struct saa7134_dev* dev;
	dev = get_device(card);
	if(dev == NULL) return NULL;

	return dev->pci;
}

int saa7134_read_gpio(unsigned int card, unsigned long *data)
{
	struct saa7134_dev* dev;
	dev = get_device(card);
	if(dev == NULL) return -EINVAL;
	if(dev->shutdown) return -ENODEV;

	/* rising SAA7134_GPIO_GPRESCAN reads the status */
	saa_andorb(SAA7134_GPIO_GPMODE3,SAA7134_GPIO_GPRESCAN,0);
	saa_andorb(SAA7134_GPIO_GPMODE3,SAA7134_GPIO_GPRESCAN,SAA7134_GPIO_GPRESCAN);
	*data = saa_readl(SAA7134_GPIO_GPSTATUS0 >> 2) & 0xfffffff;
	return 0;
}

int saa7134_write_gpio(unsigned int card, unsigned long mask, unsigned long data)
{
	struct saa7134_dev* dev;
	dev = get_device(card);
	if(dev == NULL) return -EINVAL;

	saa_andorl(SAA7134_GPIO_GPSTATUS0 >> 2, mask, data);
	if (gpio_tracking) saa7134_track_gpio(dev,"extern write");

	return 0;
}

wait_queue_head_t* saa7134_get_gpio_queue(unsigned int card)
{
	struct saa7134_dev* dev;
	dev = get_device(card);
	if(dev == NULL) return NULL;
	if(dev->shutdown) return NULL;

	return &dev->gpio_wq;
}
