#define LM_DATE "20000203"
#define LM_VERSION "1.0.0"

#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/i2c.h>

#include "tw98.h"

#ifdef MODULE
MODULE_PARM(debug,"i");
MODULE_PARM(mode,"i");
MODULE_LICENSE("GPL");
#endif

/* insmod parameter: some programs (e.g. vic) do not allow to
   specify the used video-mode, so you have to tell this to the
   modules by hand, 0 = PAL, 1 = NTSC  */
static int mode = 0;

/* insmod parameter: ... should be quite clear */
static int debug = 0;
#define dprintk	if (debug) printk

#define	I2C_TW98		0x44

/* addresses to scan, found only at 0x88 and/or 0x8a */
static unsigned short normal_i2c[] = {/*I2C_TW98,*/ I2C_TW98+1, I2C_CLIENT_END};
static unsigned short normal_i2c_range[] = {I2C_CLIENT_END};

/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

/* unique ID allocation */
static int tw98_id = 0;

static struct i2c_driver driver;

/* this functions sets partial bits of the register 'subaddress' from the
   i2c-client 'client'. Valid values of 'arg' may range between
   'range_begin' and 'range_end'; the corresponding bits to be set
   range from 'bit_begin' to 'bit_end' */
int	tw98_set_register(struct i2c_client *client, int arg, int subaddress, 
			int range_begin, int range_end,
			int bit_begin, int bit_end) 
{
	s32 byte = 0x0;
	int i = 0, result = 0;

	/* be careful, check arguments :-) */
	if ( arg < range_begin || arg > range_end )
		return -EINVAL;
	
	/* get the old register contents */
	if ( -1 == (byte = i2c_smbus_read_byte_data(client, subaddress))) {
		dprintk(KERN_ERR "tw98: could not read from tw98\n");
		return -EFAULT;
	}

	/* clear out corresponding bits */
	for(i = bit_begin; i <= bit_end; i++)
		byte &= ~(1 << i);

	/* set these bits new */
	byte |= (arg << bit_begin);

	/* write register contents back */
	if ( 0 != (result = i2c_smbus_write_byte_data(client, subaddress, byte))) {
		dprintk(KERN_ERR "tw98: could write to access tw98\n");
		return -EFAULT;
	}
	
	/* everythings fine */
	return 0;
}

static int tw98_command(struct i2c_client *client, unsigned int cmd, void *arg)
{

	switch (cmd)
	{

		case TW98_GET_STATUS:
		{
			u8 	byte = 0x0;
			int	*iarg = arg;
			
			byte = i2c_smbus_read_byte_data(client, TW98_CSTATUS);
			
			/* good means, that colour signal has been detected
			   and that horizontal frequency is locked */
			if ((byte & 0xe0) == 0x80)
				*iarg = TW98_STATUS_GOOD;
			else
				*iarg = TW98_STATUS_BAD;

			dprintk(KERN_ERR "tw98: TW98_GET_STATUS, byte:0x%02x\n", byte);

			return 0;
		}

		case TW98_GET_STATUS_BYTE:
		{
			u8 	byte = 0x0;
			u8	b[16];
			u8	*iarg = arg;
			int i, j;

			byte = i2c_smbus_read_byte_data(client, TW98_CSTATUS);
			
			*iarg = byte;

			dprintk(KERN_ERR "tw98: TW98_GET_STATUS_BYTE, byte:0x%02x\n", byte);

			for (i=0; i<0xff; i=i+16) {
			  for (j=0; j<16; ++j) b[j] = i2c_smbus_read_byte_data(client, i+j);
			  dprintk(KERN_ERR "tw98: TW98_DUMP 0x%02x - %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n", i,
				 b[0], b[1], b[2], b[3], b[4], b[5], b[6], b[7],
				 b[8], b[9], b[10], b[11], b[12], b[13], b[14], b[15]);
			};
			
			return 0;
		}
		
		case TW98_SET_NORM:
		{
			u16 *iarg = arg;
			int result = 0;

			dprintk(KERN_ERR "tw98: TW98_SET_NORM %d\n", *iarg);
			
			switch (*iarg) {

			case TW98_VIDEO_MODE_NTSC: {
				result  = tw98_set_register(client, 0, TW98_INFORM,
						0,1,		/* valid values: 0 to 1 */
						0,1);		/* bits to be used: 0-1 */
	
				return ( 0 != result ? -EFAULT : 0 );
			}

			case TW98_VIDEO_MODE_PAL:
			case TW98_VIDEO_MODE_SECAM: {
				result  = tw98_set_register(client, 1, TW98_INFORM,
						0,1,		/* valid values: 0 to 1 */
						0,1);		/* bits to be used: 0-1 */

				return ( 0 != result ? -EFAULT : 0 );
			}
			
			default:
				return -EINVAL;

			}
		}

		case TW98_SET_MODE: {

			dprintk(KERN_ERR "tw98: TW98_SET_MODE\n");

			return(tw98_set_register(client, *((int*)arg), TW98_INFORM,
					0,3,	/* valid values: 0-3 */
					2,3));	/* bits to be used: 2-3 */
			}

		case TW98_SET_SVIDEO: {

			dprintk(KERN_ERR "tw98: TW98_SET_SVIDEO\n");

			return(tw98_set_register(client, *((int*)arg), TW98_INFORM,
					0,1,	/* valid values: 0-1 */
					4,4));	/* bit to be used: 4 */
			}

		case TW98_SET_OPFORM: {

			dprintk(KERN_ERR "tw98: TW98_SET_OPFORM\n");

			if ( 0 != i2c_smbus_write_byte_data(client, TW98_OPFORM, *((int*)arg) )) {
			  dprintk(KERN_ERR "tw98: could not write to opform\n");
			  return -EFAULT;
			}

			return 0;

			}

		default:
			return -EINVAL;
	}


	return 0;
}

/* this function initalizes a tw98 to useful values. */

static int tw98_initialize(struct i2c_client* client)
{
	int 	i = 0, result = 0;

#define RESET 31
	u8	s[RESET][2] = {	{ TW98_ACNTL,		0x80 }, /* 0x06 */
				{ TW98_INFORM,		0x64 }, /* 0x02 */
				{ TW98_OPFORM,		0x82 }, /* 0x03 */ /* 0x86 for BRS direct */
				{ TW98_HSLEN,		0x80 }, /* 0x04 */
				{ TW98_POLARITY,	0x05 }, /* 0x05 */
				{ TW98_CROP_HI,		0x12 }, /* 0x07 */
				{ TW98_VDELAY_LO,	0x18 }, /* 0x08 */
				{ TW98_VACTIVE_LO,	0xe5 }, /* 0x09 */
				{ TW98_HDELAY_LO,	0x00 }, /* 0x0a */
				{ TW98_HACTIVE_LO,	0xd0 }, /* 0x0b */
				{ TW98_VSCALE_HI,	0x01 }, /* 0x0c */
				{ TW98_VSCALE_LO,	0x00 }, /* 0x0d */
				{ TW98_HSCALE_HI,	0x01 }, /* 0x0e */
				{ TW98_HSCALE_LO,	0x10 }, /* 0x0f */
				{ TW98_BRIGHTNESS,	0xc8 }, /* 0x10 */
				{ TW98_CONTRAST,	0x28 }, /* 0x11 */
				{ TW98_SHARPNESS,	0x14 }, /* 0x12 */
				{ TW98_SAT_U,		0x71 }, /* 0x13 */
				{ TW98_SAT_V,		0x50 }, /* 0x14 */
				{ TW98_HUE,		0x20 }, /* 0x15 */
				{ TW98_CKILL,		0xd0 }, /* 0x16 */
				{ TW98_CORING,		0x00 }, /* 0x17 */
				{ TW98_VBICNTL,		0x90 }, /* 0x19 */
				{ TW98_CC_STATUS,	0x44 }, /* 0x1a */
				{ TW98_AGCGAIN,		0x21 }, /* 0x1c */
				{ TW98_GPIO,		0x00 }, /* 0x1d */
#if 1
				{ 0xd3,			0x14 },
				{ 0xd8,			0x18 },
				{ 0xdf,			0xb4 },
				{ 0xe2,			0x04 },
				{ 0xe6,			0x62 } 
#endif
				};

#if 0
				{ 0xc1,			0x01 },
				{ 0xc2,			0xe2 },
				{ 0xc3,			0xbe },
				{ 0xc4,			0x2f },
				{ 0xc5,			0xe7 },
				{ 0xc7,			0x01 },
				{ 0xc8,			0xe2 },
				{ 0xc9,			0xe2 },
				{ 0xca,			0x46 },
				{ 0xcb,			0x9b },
				{ 0xcd,			0x01 },
				{ 0xce,			0xe2 },
				{ 0xcf,			0x9d },
				{ 0xd0,			0x39 },
				{ 0xd1,			0x97 },
				{ 0xdf,			0xb4 },
#endif

	dprintk(KERN_ERR "tw98: tw98_init\n");

	for(i = 0; i < RESET; i++) {
		if ( 0 != (result = i2c_smbus_write_byte_data(client, s[i][0], s[i][1]))) {
			dprintk(KERN_ERR "tw98: tw98_init error - reset\n");
			return -EFAULT;
		}
	}

	/* check for a valid mode */
	if( mode < 0 || mode > 1 ) {
		printk(KERN_ERR "tw98: invalid mode setting. using PAL as default\n");
		mode = 0;
	}

	/* use the existing tw98_command ioctls */
	i = 0;
	if ( 0 != tw98_command(client, TW98_SET_MODE, &i)) {
		dprintk(KERN_ERR "tw98: tw98_init error - mode\n");
		return -EFAULT;
	}
	
		
	switch(mode) {
		case 1:
			i = 0;
			break;
		default:
			i = 3;
			break;
		}
	
	switch(mode) {
		case 1:
			i = TW98_VIDEO_MODE_NTSC;
			break;
		default:
			i = TW98_VIDEO_MODE_PAL;
			break;
		}
		
	if ( 0 != tw98_command(client, TW98_SET_NORM, &i)) {
		dprintk(KERN_ERR "tw98: tw98_init error #5\n");
		return -EFAULT;
	}

	return 0;
}

/* this function is called by i2c_probe */
int tw98_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	struct	i2c_client *client;
	int err = 0;

	/* let's see whether this adapter can support what we need */
	if ( 0 == i2c_check_functionality(adapter, I2C_FUNC_SMBUS_READ_BYTE_DATA|I2C_FUNC_SMBUS_WRITE_BYTE_DATA)) {
		return 0;
	}

#if 0
	if ( 0 != (result = i2c_smbus_read_byte_data(client, TW98_ID))) {
	  return -EFAULT;
	}
#endif
			
	/* allocate memory for client structure */
	client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL);
        if (0 == client) {
		return -ENOMEM;
	}
	
	/* fill client structure */
	if( I2C_TW98 == address ) {
		sprintf(client->name,"tw98 (0x%02x)", address);
	}

	client->id = tw98_id++;
	client->flags = 0;
	client->addr = address;
	client->adapter = adapter;
	client->driver = &driver;
	client->data = NULL;

	/* tell the i2c layer a new client has arrived */
	if (0 != (err = i2c_attach_client(client))) {
		kfree(client);
		return err;
	}


	if( 0 != tw98_initialize(client) ) {
		printk("tw98.o: could not initialize chipset. continuing anyway.\n");
	}
	
	printk("tw98.o: detected @ 0x%02x on adapter %s\n",2*address,&client->adapter->name[0]);

	return 0;
}

static int tw98_attach(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter,&addr_data,&tw98_detect);
}

static int tw98_detach(struct i2c_client *client)
{
	int err = 0;

	if ( 0 != (err = i2c_detach_client(client))) {
		printk("tw98.o: Client deregistration failed, client not detached.\n");
		return err;
	}
	
	kfree(client);

	return 0;
}

static struct i2c_driver driver = {
	.owner          = THIS_MODULE,
	.name           = "tw98 driver",
	.id             = I2C_DRIVERID_TW98,
	.flags          = I2C_DF_NOTIFY,
	.attach_adapter = tw98_attach,
	.detach_client  = tw98_detach,
	.command        = tw98_command,
};

EXPORT_NO_SYMBOLS;

#ifdef MODULE
static
#else
extern
#endif
	int __init tw98_init(void)
{
	int res = 0;

	printk("tw98.o version %s (%s)\n",LM_VERSION,LM_DATE);

	if( 0 != (res = i2c_add_driver(&driver))) {
		printk("tw98.o: Driver registration failed, module not inserted.\n");
		return res;
	}
	
	return 0;
}

int __init tw98_cleanup(void)
{	
	int res = 0;

	if ( 0 != (res = i2c_del_driver(&driver))) {
		printk("tw98.o: Driver deregistration failed, module not removed.\n");
		return res;
	}
	
	return 0;
}

#ifdef MODULE

MODULE_AUTHOR("Hank Magnuski <hankm@mtinet.com>");
MODULE_DESCRIPTION("Techwell 98 driver");

int init_module(void)
{
	return tw98_init();
}

void cleanup_module(void)
{
	tw98_cleanup();
}

#endif
