/*
 * USB USBVISION Video device driver 0.3.3-test3 (For Kernel > 2.4.6) 
 *
 * Let's call the version 0.... until compression decoding is implemented.
 *
 * This driver is written by Jose Ignacio Gijon and Joerg Heckenbach.
 * It was based on USB CPiA driver written by Peter Pregler,
 * Scott J. Bertin and Johannes Erdfelt
 * Ideas are taken from bttv driver by Ralph Metzler, Marcus Metzler &
 * Gerd Knorr and zoran 36120/36125 driver by Pauline Middelink
 *
 * History:
 *
 * Mar. 2000 - 15.12.2000:  (0.0.0 - 0.2.0)
 *     Several alpha drivers and the first beta. 
 * 	
 * Since Dec. 2000:  (0.2.1) or (v2.1)
 *     Code changes or updates by Dwaine Garden and every other person.
 *
 *     Added: New Hauppauge TV device  Vendor  ID: 0x0573
 *                                     Product ID: 0x4D01
 *            (Thanks to Giovanni Garberoglio)
 *
 *     Added: UK Hauppauge WinTV-USB   Vendor  ID: 0x0573
 *                                     Product ID: 0x4D02
 *            (Thanks to Derek Freeman-Jones)
 *
 * Feb, 2001 - Apr 08, 2001:  (0.3.0)
 *     - Some fixes. Driver is now more stable.
 *     - Scratch is organized as ring-buffer now for better performance
 *     - DGA (overlay) is now supported. 
 *       !!!!Danger!!!! Clipping is not yet implemented. Your system will
 *       crash if your video window leaves the screen!!!
 *     - Max. Framesize is set to 320x240. There isn't more memory on the
 *       nt1003 video device for the FIFO.
 *     - Supported video palettes: RGB565, RGB555, RGB24, RGB32
 *
 * 
 * Apr 15, 2001:  (0.3.1-test...)
 *     - Clipping is implemented
 *     - NTSC is now coloured (Thanks to Dwaine Garden)
 *     - Added SECAM colour detection in saa7111-new
 *     - Added: French Hauppauge WinTV USB  Vendor ID: 0x0573
 *                                          Product ID: 0x4D03
 *              (Thanks to Julius Hrivnac)
 *     - Added: US Hauppauge WINTV USB  Vendor ID: 0x0573
 *                                      Product ID: 0x4D00
 *              (Thanks to  Derrick J Brashear)
 *     - Changes for adding new devices. There's now a table in usbvision.h.
 *       Adding your devices data to the usbvision_device_data table is all
 *       you need to add a new device, I hope.
 *
 * May 11, 2001: (0.3.2-test...) (Thanks to Derek Freeman-Jones)
 *     - Support YUV422 raw format for people with hardware scaling.
 *     - Only power on the device when opened (use option PowerOnAtOpen=0 to disable it).
 *     - Turn off audio so we can listen to Line In.
 *
 * July 5, 2001 - (Patch the driver to run with Kernel 2.4.6)
 *     - Fixed a problem with the number of parameters passed to video_register_device.
 *
 * July 6, 2001 - Added: HAUPPAUGE WINTV-USB FM USA Vendor  ID: 0x0573
 *                                              	Product ID: 0x4D10
 *		(Thanks to Braddock Gaskill)
 *                Added: USBGear USBG-V1 resp. HAMA USB
 *                						Vendor  ID: 0x0573
 *                                      Product ID: 0x0003
 *		(Thanks to Bradley A. Singletary and Juergen Weigert)
 *
 * Jan 24, 2002 - (0.3.3-test...)
 *     - Moved all global variables that are device specific the usb_usbvision struct
 *     - Fixed the 64x48 unchangable image in xawtv when starting it with overlay
 *     - Add VideoNorm and TunerType to the usb_device_data table 
 *     - Checked the audio channels and mute for HAUPPAUGE WinTV USB FM 
 *     - Implemented the power on when opening the device. But some software opens 
 *       the device several times when starting. So the i2c parts are just registered
 *       by an open, when they become deregistered by the next close. You can speed
 *       up tuner detection, when adding "options tuner addr=your_addr" to /etc/modules.conf
 *     - Begin to resize the frame in width and height. So it will be possible to watch i.e. 
 *       384x288 pixels at 23 fps. 
 *
 * Feb 10, 2002 
 *     - Added radio device
 *
 *
 * Jul 30, 2002 - (Thanks Cameron Maxwell)
 *     - Changes to usbvision.h --fixed usbvision device data structure, incorrectly had (0x0573, 0x4d21) for WinTV-USB II, should be 0x4d20.
 *     - Changes for device WinTV-USB II (0x0573. 0x4D21).  It does not have a FM tuner.
 *     - Added the real device HAUPPAUGE WINTV-USB II (PAL) to the device structure in usbvision.h.
 *     - Changes to saa7113-new, the video is 8 bit data for the Phillips SAA7113 not 16bit like SAA7113.
 *     - Tuned lots of setup registers for the Phillips SAA7113 video chipset.
 *     - Changes to the supplied makefile. (Dwaine Garden) Still needs to be fixed so it will compile modules on different distrubutions.
 *
 *
 * Aug 10, 2002 - (Thanks Mike Klinke)
 *     - Changes to usbvision.txt -- Fixed instructions on the location to copy the contents of the tgz file.
 *     - Added device WinTV-USB FM Model 621 (0x0573. 0x4D30). There is another device which carries the same name. Kept that device in the device structure.
 *
 * Aug 12, 2002 - Dwaine Garden
 *     - Added the ability to read the NT100x chip for the MaxISOPacketLength and USB Bandwidth Setting of the video device.
 *     - Adjustments to the SAA7113H code for proper video output.
 *     - Changes to usbvision.h, so all the devices with FM tuners are working.
 *
 * TODO:
 *     - Fix memory settings for nt1004. It is 4 times as big as the
 *       nt1003 memory.
 *     - Implement the decompression algorithm.
 *     - Add audio on endpoint 3 for nt1004 chip.
 *     - Clean up the driver.
 *     - optimization for performance.
 *     - Add Videotext capability (VBI)
 *	   - Add interpolating to the resizing of frames (if resizing is useful)
 *	   - Check audio for other devices
 *     - Check USBVISION_SUPPORTED_PALETTES and the format checking
 *     - Supplied Makefile changes.  Needs to compile on multiple linux distrubutions.
 *     
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/list.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/smp_lock.h>
#include <linux/videodev.h>
#include <linux/vmalloc.h>
#include <linux/wrapper.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/spinlock.h>
#include <linux/usb.h>
#include <asm/io.h>
#include <linux/video_decoder.h>
#include <linux/i2c.h>
#include "i2c-algo-usb.h"
#include "../drivers/media/video/tuner.h"					/* Arhggggggggggggg !!!!!!!!!!! */
#include "../drivers/media/video/audiochip.h"  					/*another Arhggggggggg!!!!!!! */

#ifdef CONFIG_KMOD
#include <linux/kmod.h>
#endif

#include "usbvision.h"

#define	ENABLE_HEXDUMP	0							/* Enable if you need it */

#define DEBUG(x...) 								/* General Debug */
#define IODEBUG(x...) 								/* Debug IO */
#define RIODEBUG(x...) 								/* Debug IO for a radio device */
#define IRQDEBUG(x...) 								/* Debug IRQ */
#define OVDEBUG(x...) 								/* Debug overlay */
#define MDEBUG(x...)								/* Debug memory management */

static int video_nr = -1;							/* Number of Video Devices Initalized */
static int radio_nr = -1;							/* Number of Radio Devices Initalized */

/* Completion states of the data parser */
typedef enum {
	scan_Continue,								/* Just parse next item */
	scan_NextFrame,								/* Frame done, send it to V4L */
	scan_Out,								/* Not enough data for frame */
	scan_EndParse								/* End parsing */
} scan_state_t;

static const int max_imgwidth = MAX_FRAME_WIDTH;
static const int max_imgheight = MAX_FRAME_HEIGHT;
static const int min_imgwidth = MIN_FRAME_WIDTH;
static const int min_imgheight = MIN_FRAME_HEIGHT;

#define FRAMERATE_MIN	0
#define FRAMERATE_MAX	31

static int usbbandwidth = 0;

enum {
	VIDEOFORMAT_YUV422 = 0x03,
	VIDEOFORMAT_YUV420 = 0x14,
	VIDEOFORMAT_COMPRESS = 0x60,
};

/*
 * The value of 'scratch_buf_size' affects quality of the picture
 * in many ways. Shorter buffers may cause loss of data when client
 * is too slow. Larger buffers are memory-consuming and take longer
 * to work with. This setting can be adjusted, but the default value
 * should be OK for most desktop users.
 */
#define DEFAULT_SCRATCH_BUF_SIZE	(0x20000)				/* 128 k Memory Scratch Buffer*/
static const int scratch_buf_size = DEFAULT_SCRATCH_BUF_SIZE;

static int init_brightness = 128;						/* Initalize the brightness of the video device */
static int init_contrast = 192;							/* Initalize the contrast of the video device */
static int init_color = 128;							/* Initalize the color mode of the video device */
static int init_hue = 128;							/* Initalize the Hue settings of the video device */
static int hue_correction = 128;


static int usbvision_restart_isoc(struct usb_usbvision *usbvision);
static int usbvision_begin_streaming(struct usb_usbvision *usbvision);
static int usbvision_muxsel(struct usb_usbvision *usbvision, int channel, int norm);
static int usbvision_i2c_write(void *data, unsigned char addr, char *buf, short len);
static int usbvision_i2c_read(void *data, unsigned char addr, char *buf, short len);

/* Bit flags (options) */
#define FLAGS_RETRY_VIDIOCSYNC		(1 << 0)
#define	FLAGS_MONOCHROME		(1 << 1)
#define FLAGS_DISPLAY_HINTS		(1 << 2)
#define FLAGS_OSD_STATS			(1 << 3)
#define FLAGS_FORCE_TESTPATTERN		(1 << 4)
#define FLAGS_SEPARATE_FRAMES		(1 << 5)
#define FLAGS_CLEAN_FRAMES		(1 << 6)

/* Default Initalization of Device Driver Parameters */
static int flags = 0;								/* Set the default Overlay Display mode of the device driver */
static int debug = 0;								/* Set the default Debug Mode of the device driver */
static int videoformat = VIDEOFORMAT_YUV420; 					/* Set the default video format of the video device */
static int dga = 1;								/* Set the default Direct Graphic Access */
static int PowerOnAtOpen = 1;							/* Set the default device to power on at startup */

/* Grab parameters for the device driver */
MODULE_PARM(flags, "i");							/* Grab the Overlay Display mode of the device driver */
MODULE_PARM(debug, "i");							/* Grab the Debug Mode of the device driver */
MODULE_PARM(videoformat, "i");							/* Grab the video format of the video device */
MODULE_PARM(dga, "i");								/* Grab the Direct Graphic Access */
MODULE_PARM(PowerOnAtOpen, "i");						/* Grab the device to power on at startup */

/* Misc Stuff */
MODULE_AUTHOR("module author");
MODULE_DESCRIPTION("USBVISION USB Video device Driver for Linux");
MODULE_LICENSE("GPL");

#ifdef MODULE
static unsigned int autoload = 1;
#else
static unsigned int autoload = 0;
#endif

#define MAX_USBVISION	4							/* Max number of devices supported - Due to USB bus limits */

struct usb_usbvision vdevs[MAX_USBVISION];



/* Memory management functions */

static struct usb_driver usbvision_driver;
static void usbvision_release(struct usb_usbvision *usbvision);


/* Given PGD from the address space's page table, return the kernel virtual mapping of the physical memory mapped at ADR. */
static inline unsigned long uvirt_to_kva(pgd_t * pgd, unsigned long adr)
{
	unsigned long ret = 0UL;
	pmd_t *pmd;
	pte_t *ptep, pte;
	if (!pgd_none(*pgd)) {
		pmd = pmd_offset(pgd, adr);
		if (!pmd_none(*pmd)) {
			ptep = pte_offset(pmd, adr);
			pte = *ptep;
			if (pte_present(pte))
				ret = (unsigned long)
				    page_address(pte_page(pte));
			ret |= (adr & (PAGE_SIZE - 1));
		}
	}
	MDEBUG(printk("uv2kva(%lx-->%lx)", adr, ret);)
	return ret;
}

static inline unsigned long uvirt_to_bus(unsigned long adr)
{
	unsigned long kva, ret;
	kva = uvirt_to_kva(pgd_offset(current->mm, adr), adr);
	ret = virt_to_bus((void *) kva);
	MDEBUG(printk("uv2b(%lx-->%lx)", adr, ret);)
	return ret;
}

static inline unsigned long kvirt_to_bus(unsigned long adr)
{
	unsigned long va, kva, ret;

	va = VMALLOC_VMADDR(adr);
	kva = uvirt_to_kva(pgd_offset_k(va), va);
	ret = virt_to_bus((void *) kva);
	MDEBUG(printk("kv2b(%lx-->%lx)", adr, ret);)
	return ret;
}

/* Here we want the physical address of the memory. This is used when initializing the contents of the */
/* area and marking the pages as reserved. */
static inline unsigned long kvirt_to_pa(unsigned long adr)
{
	unsigned long va, kva, ret;

	va = VMALLOC_VMADDR(adr);
	kva = uvirt_to_kva(pgd_offset_k(va), va);
	ret = __pa(kva);
	MDEBUG(printk("kv2pa(%lx-->%lx)", adr, ret);)
	return ret;
}

static void *rvmalloc(unsigned long size)
{
	void *mem;
	unsigned long adr, page;
										/* Round it off to PAGE_SIZE */
	size += (PAGE_SIZE - 1);
	size &= ~(PAGE_SIZE - 1);
	mem = vmalloc(size);
	if (!mem)
		return NULL;
	memset(mem, 0, size);							/* Clear the ram out, no junk to the user */
	adr = (unsigned long) mem;
	while (size > 0) {
		page = kvirt_to_pa(adr);
		mem_map_reserve(virt_to_page(__va(page)));
		adr += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}

	return mem;
}

static void rvfree(void *mem, unsigned long size)
{
	unsigned long adr, page;

	if (!mem)
		return;

	size += (PAGE_SIZE - 1);
	size &= ~(PAGE_SIZE - 1);

	adr = (unsigned long) mem;
	while (size > 0) {
		page = kvirt_to_pa(adr);
		mem_map_unreserve(virt_to_page(__va(page)));
		adr += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}
	vfree(mem);
}

#if ENABLE_HEXDUMP
static void usbvision_hexdump(const unsigned char *data, int len)
{
	char tmp[80];
	int i, k;

	for (i = k = 0; len > 0; i++, len--) {
		if (i > 0 && (i % 16 == 0)) {
			printk("%s\n", tmp);
			k = 0;
		}
		k += sprintf(&tmp[k], "%02x ", data[i]);
	}
	if (k > 0)
		printk("%s\n", tmp);
}
#endif


/* These procedures handle the scratch ring buffer */
int scratch_len(struct usb_usbvision *usbvision)				/*This returns the amount of data actually in the buffer */
{
	int len = usbvision->scratch_write_ptr - usbvision->scratch_read_ptr;
	if (len < 0) {
		len += scratch_buf_size;
	}
	if (debug >=3) {
		printk(KERN_DEBUG "scratch_len() = %d\n", len);
	}
	return len;
}


/* This returns the free space left in the buffer */
int scratch_free(struct usb_usbvision *usbvision)
{
	int free = usbvision->scratch_read_ptr - usbvision->scratch_write_ptr;
	if (free <= 0) {
		free += scratch_buf_size;
	}
	if (free) {
		free -= 1;							/* at least one byte in the buffer must */
										/* left blank, otherwise there is no chance to differ between full and empty */
	}
	if (debug >=3) {
		printk(KERN_DEBUG "scratch_free() = %d\n", free);
	}
	return free;
}


void *debug_memcpy(void *dest, void *src, size_t len)
{
	printk(KERN_DEBUG "memcpy(%p, %p, %d);\n", dest, src, len);
	return memcpy(dest, src, len);
}


/* This puts data into the buffer */
int scratch_put(struct usb_usbvision *usbvision, unsigned char *data, int len)
{
	int len_part;

	if (usbvision->scratch_write_ptr + len < scratch_buf_size) {
		memcpy(usbvision->scratch + usbvision->scratch_write_ptr, data, len);
		usbvision->scratch_write_ptr += len;
	}
	else {
		len_part = scratch_buf_size - usbvision->scratch_write_ptr;
		memcpy(usbvision->scratch + usbvision->scratch_write_ptr, data, len_part);
		if (len == len_part) {
			usbvision->scratch_write_ptr = 0;			/* just set write_ptr to zero */
		}
		else {
			memcpy(usbvision->scratch, data + len_part, len - len_part);
			usbvision->scratch_write_ptr = len - len_part;
		}
	}

	if (debug >=3) {
		printk(KERN_DEBUG "scratch_put(), len=%d, new write_ptr=%d\n",
				len, usbvision->scratch_write_ptr);
	}

	return len;
}

/* This marks the write_ptr as position of new frame header */
void scratch_mark_header(struct usb_usbvision *usbvision)
{
	if (debug >=3) {
		printk(KERN_DEBUG "scratch_mark_header() at write_ptr=%d\n",
				usbvision->scratch_headermarker_write_ptr);
	}

	usbvision->scratch_headermarker[usbvision->scratch_headermarker_write_ptr] =
		usbvision->scratch_write_ptr;
	usbvision->scratch_headermarker_write_ptr += 1;
	usbvision->scratch_headermarker_write_ptr %= USBVISION_NUM_HEADERMARKER;
}

/* This gets data from the buffer at the given "ptr" position */
int scratch_get_extra(struct usb_usbvision *usbvision, unsigned char *data, int *ptr, int len)
{
	int len_part;
	if (*ptr + len < scratch_buf_size) {
		memcpy(data, usbvision->scratch + *ptr, len);
		*ptr += len;
	}
	else {
		len_part = scratch_buf_size - *ptr;
		memcpy(data, usbvision->scratch + *ptr, len_part);
		if (len == len_part) {
			*ptr = 0;							/* just set the y_ptr to zero */
		}
		else {
			memcpy(data + len_part, usbvision->scratch, len - len_part);
			*ptr = len - len_part;
		}
	}

	if (debug >=3) {
		printk(KERN_DEBUG "scratch_get_extra(), len=%d, new ptr=%d\n",
				len, *ptr);
	}

	return len;
}


/* This sets the scratch extra read pointer */
void scratch_set_extra_ptr(struct usb_usbvision *usbvision, int *ptr, int len)
{
	*ptr = (usbvision->scratch_read_ptr + len)%scratch_buf_size;

	if (debug >=3) {
		printk(KERN_DEBUG "scratch_get_extra_ptr(): ptr=%d\n",
				*ptr);
	}

}


/*This increments the scratch extra read pointer */
void scratch_inc_extra_ptr(int *ptr, int len)
{
	*ptr = (*ptr + len) % scratch_buf_size;
}


/* This gets data from the buffer */
int scratch_get(struct usb_usbvision *usbvision, unsigned char *data, int len)
{
	int len_part;
	if (usbvision->scratch_read_ptr + len < scratch_buf_size) {
		memcpy(data, usbvision->scratch + usbvision->scratch_read_ptr, len);
		usbvision->scratch_read_ptr += len;
	}
	else {
		len_part = scratch_buf_size - usbvision->scratch_read_ptr;
		memcpy(data, usbvision->scratch + usbvision->scratch_read_ptr, len_part);
		if (len == len_part) {
			usbvision->scratch_read_ptr = 0;				/* just set the read_ptr to zero */
		}
		else {
			memcpy(data + len_part, usbvision->scratch, len - len_part);
			usbvision->scratch_read_ptr = len - len_part;
		}
	}

	if (debug >=3) {
		printk(KERN_DEBUG "scratch_get(), len=%d, new read_ptr=%d\n",
				len, usbvision->scratch_read_ptr);
	}

	return len;
}


/* This sets read pointer to next header and returns it */
int scratch_get_header(struct usb_usbvision *usbvision,struct usbvision_frame_header *header)
{
	int rc=0;

	if (debug >=3) {
		printk(KERN_DEBUG "scratch_get_header() from read_ptr=%d\n",
				usbvision->scratch_headermarker_read_ptr);
	}

	while (usbvision->scratch_headermarker_write_ptr -
		usbvision->scratch_headermarker_read_ptr != 0) {
		usbvision->scratch_read_ptr =
			usbvision->scratch_headermarker[usbvision->scratch_headermarker_read_ptr];
		usbvision->scratch_headermarker_read_ptr += 1;
		usbvision->scratch_headermarker_read_ptr %= USBVISION_NUM_HEADERMARKER;
		scratch_get(usbvision, (unsigned char *)header, USBVISION_HEADER_LENGTH);
		if ((header->vid_frm_patt == USBVISION_MAGIC)
			 && (header->header_length == USBVISION_HEADER_LENGTH)) {
			rc = USBVISION_HEADER_LENGTH;
			break;
		}
	}

	return rc;
}


/*This removes len bytes of old data from the buffer */
void scratch_rm_old(struct usb_usbvision *usbvision, int len)
{

	usbvision->scratch_read_ptr += len;
	usbvision->scratch_read_ptr %= scratch_buf_size;
	if (debug >=3) {
		printk(KERN_DEBUG "scratch_rm_old(), read_ptr is now %d\n",
				usbvision->scratch_read_ptr);
	}
}


/*This resets the buffer - kills all data in it too */
void scratch_reset(struct usb_usbvision *usbvision)
{
	if (debug >=3) {
		printk(KERN_DEBUG "scratch_reset()\n");
	}
	usbvision->scratch_read_ptr = 0;
	usbvision->scratch_write_ptr = 0;
	usbvision->scratch_headermarker_read_ptr = 0;
	usbvision->scratch_headermarker_write_ptr = 0;
	usbvision->isocstate = ISOC_NOFRAME;
}



/* Here comes the OVERLAY stuff */

/* Tell the interrupt handler what to to.  */
static
void usbvision_cap(struct usb_usbvision* usbvision, int on)
{
	DEBUG(printk(KERN_DEBUG "usbvision_cap: overlay was %d, set it to %d\n", usbvision->overlay, on);)

	if (on) {
		usbvision->overlay = 1;
	}
	else {
		usbvision->overlay = 0;
	}
}




/* append a new clipregion to the vector of video_clips */
static
void usbvision_new_clip(struct video_window* vw, struct video_clip* vcp, int x, int y, int w, int h)
{
	vcp[vw->clipcount].x = x;
	vcp[vw->clipcount].y = y;
	vcp[vw->clipcount].width = w;
	vcp[vw->clipcount].height = h;
	vw->clipcount++;
}


#define mark_pixel(x,y)  usbvision->clipmask[((x) + (y) * MAX_FRAME_WIDTH)/32] |= 0x00000001<<((x)%32)
#define clipped_pixel(index) usbvision->clipmask[(index)/32] & (0x00000001<<((index)%32))

static
void usbvision_built_overlay(struct usb_usbvision* usbvision, int count, struct video_clip *vcp)
{
	usbvision->overlay_win = usbvision->overlay_base +
		(signed int)usbvision->vid_win.x * usbvision->vid_buf.depth / 8 +
		(signed int)usbvision->vid_win.y * usbvision->vid_buf.bytesperline;

		IODEBUG(printk(KERN_DEBUG "built_overlay base=%p, win=%p, bpl=%d, clips=%d, size=%dx%d\n",
					usbvision->overlay_base, usbvision->overlay_win,
					usbvision->vid_buf.bytesperline, count,
					usbvision->vid_win.width, usbvision->vid_win.height);)


	/* Add here generation of clipping mask */
{
	int x_start, x_end, y_start, y_end;
	int clip_index, x, y;

	memset(usbvision->clipmask, 0, USBVISION_CLIPMASK_SIZE);

	OVDEBUG(printk(KERN_DEBUG "clips = %d\n", count);)

	for(clip_index = 0; clip_index < count; clip_index++) {
		OVDEBUG(printk(KERN_DEBUG "clip: %d,%d,%d,%d\n", vcp[clip_index].x,
				vcp[clip_index].y,
				vcp[clip_index].width,
				vcp[clip_index].height);)

		x_start = vcp[clip_index].x;
		if(x_start >= (int)usbvision->vid_win.width) {
			OVDEBUG(printk(KERN_DEBUG "x_start=%d\n", x_start);)
			continue; //clipping window is right of overlay window
		}
		x_end	= x_start + vcp[clip_index].width;
		if(x_end <= 0) {
			OVDEBUG(printk(KERN_DEBUG "x_end=%d\n", x_end);)
			continue; //clipping window is left of overlay window
		}
		
		y_start = vcp[clip_index].y;
		if(y_start >= (int)usbvision->vid_win.height) {
			OVDEBUG(printk(KERN_DEBUG "y_start=%d\n", y_start);)
			continue; //clipping window is below overlay window
		}
		y_end   = y_start + vcp[clip_index].height;
		if(y_end <= 0) {
			OVDEBUG(printk(KERN_DEBUG "y_end=%d\n", y_end);)
			continue; //clipping window is above overlay window
		}

		//clip the clipping window
		if (x_start < 0) {
			x_start = 0;
		}
		if (x_end > usbvision->vid_win.width) {
			x_end = usbvision->vid_win.width;
		}
		if (y_start < 0) {
			y_start = 0;
		}
		if (y_end > usbvision->vid_win.height) {
			y_end = usbvision->vid_win.height;
		}

		OVDEBUG(printk(KERN_DEBUG "clip_o: %d,%d,%d,%d\n", x_start,	y_start, x_end, y_end);)



		for(y = y_start; y < y_end; y++) {
			for(x = x_start; x < x_end; x++) {
				mark_pixel(x,y);
			}
		}
	}
}

}



void usbvision_osd_char(struct usb_usbvision *usbvision,
			struct usbvision_frame *frame, int x, int y, int ch)
{
	static const unsigned short digits[16] = {
		0xF6DE,		/* 0 */
		0x2492,		/* 1 */
		0xE7CE,		/* 2 */
		0xE79E,		/* 3 */
		0xB792,		/* 4 */
		0xF39E,		/* 5 */
		0xF3DE,		/* 6 */
		0xF492,		/* 7 */
		0xF7DE,		/* 8 */
		0xF79E,		/* 9 */
		0x77DA,		/* a */
		0xD75C,		/* b */
		0xF24E,		/* c */
		0xD6DC,		/* d */
		0xF34E,		/* e */
		0xF348		/* f */
	};
	unsigned short digit;
	int ix, iy;

	if ((usbvision == NULL) || (frame == NULL))
		return;

	if (ch >= '0' && ch <= '9')
		ch -= '0';
	else if (ch >= 'A' && ch <= 'F')
		ch = 10 + (ch - 'A');
	else if (ch >= 'a' && ch <= 'f')
		ch = 10 + (ch - 'a');
	else
		return;
	digit = digits[ch];

	for (iy = 0; iy < 5; iy++) {
		for (ix = 0; ix < 3; ix++) {
			if (digit & 0x8000) {
			//	USBVISION_PUTPIXEL(frame, x + ix, y + iy,
			//			0xFF, 0xFF, 0xFF);
			}
			digit = digit << 1;
		}
	}
}


void usbvision_osd_string(struct usb_usbvision *usbvision,
			  struct usbvision_frame *frame,
			  int x, int y, const char *str)
{
	while (*str) {
		usbvision_osd_char(usbvision, frame, x, y, *str);
		str++;
		x += 4;		/* 3 pixels character + 1 space */
	}
}

/*
 * usb_usbvision_osd_stats()
 *
 * On screen display of important debugging information.
 *
 */
void usbvision_osd_stats(struct usb_usbvision *usbvision,
			 struct usbvision_frame *frame)
{
	const int y_diff = 8;
	char tmp[16];
	int x = 10;
	int y = 10;

	sprintf(tmp, "%8x", usbvision->frame_num);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->urb_count);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->urb_length);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->data_count);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->header_count);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->scratch_ovf_count);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->iso_skip_count);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8lx", usbvision->iso_err_count);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8x", usbvision->vpic.colour);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8x", usbvision->vpic.hue);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8x", usbvision->vpic.brightness >> 8);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8x", usbvision->vpic.contrast >> 12);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;

	sprintf(tmp, "%8d", usbvision->vpic.whiteness >> 8);
	usbvision_osd_string(usbvision, frame, x, y, tmp);
	y += y_diff;
}

/*
 * usbvision_testpattern()
 *
 * Procedure forms a test pattern (yellow grid on blue background).
 *
 * Parameters:
 * fullframe:   if TRUE then entire frame is filled, otherwise the procedure
 *	        continues from the current scanline.
 * pmode	0: fill the frame with solid blue color (like on VCR or TV)
 *	        1: Draw a colored grid
 *
 */
void usbvision_testpattern(struct usb_usbvision *usbvision, int fullframe,
			int pmode)
{
	static const char proc[] = "usbvision_testpattern";
	struct usbvision_frame *frame;
	unsigned char *f;
	int num_cell = 0;
	int scan_length = 0;
	static int num_pass = 0;

	if (usbvision == NULL) {
		printk(KERN_ERR "%s: usbvision == NULL\n", proc);
		return;
	}
	if ((usbvision->curframe < 0)
	    || (usbvision->curframe >= USBVISION_NUMFRAMES)) {
		printk(KERN_ERR "%s: usbvision->curframe=%d.\n", proc,
		       usbvision->curframe);
		return;
	}

	/* Grab the current frame */
	frame = &usbvision->frame[usbvision->curframe];

	/* Optionally start at the beginning */
	if (fullframe) {
		frame->curline = 0;
		frame->scanlength = 0;
	}

	/* Form every scan line */
	for (; frame->curline < frame->frmheight; frame->curline++) {
		int i;

		f = frame->data + (usbvision->curwidth * 3 * frame->curline);
		for (i = 0; i < usbvision->curwidth; i++) {
			unsigned char cb = 0x80;
			unsigned char cg = 0;
			unsigned char cr = 0;

			if (pmode == 1) {
				if (frame->curline % 32 == 0)
					cb = 0, cg = cr = 0xFF;
				else if (i % 32 == 0) {
					if (frame->curline % 32 == 1)
						num_cell++;
					cb = 0, cg = cr = 0xFF;
				} else {
					cb =
					    ((num_cell * 7) +
					     num_pass) & 0xFF;
					cg =
					    ((num_cell * 5) +
					     num_pass * 2) & 0xFF;
					cr =
					    ((num_cell * 3) +
					     num_pass * 3) & 0xFF;
				}
			} else {
				/* Just the blue screen */
			}

			*f++ = cb;
			*f++ = cg;
			*f++ = cr;
			scan_length += 3;
		}
	}

	frame->grabstate = FRAME_DONE;
	frame->scanlength += scan_length;
	++num_pass;

	/* We do this unconditionally, regardless of FLAGS_OSD_STATS */
	usbvision_osd_stats(usbvision, frame);
}

/*
 * Here comes the data parsing stuff that is run as interrupt
 */

/*
 * usbvision_find_header()
 *
 * Locate one of supported header markers in the scratch buffer.
 */
static scan_state_t usbvision_find_header(struct usb_usbvision *usbvision)
{
	struct usbvision_frame *frame;

	if (usbvision->overlay) {
		frame = &usbvision->overlay_frame;
	}
	else {
		frame = &usbvision->frame[usbvision->curframe];
	}
	
	if (scratch_get_header(usbvision, &frame->usb_header)==USBVISION_HEADER_LENGTH) {
		// found header in scratch
#undef DUMP_HEADER
#ifdef DUMP_HEADER
		printk
		    ("Header: %04x %d %d %d %d %d %02x %u %u\n",
		     frame->usb_header.vid_frm_patt,
		     frame->usb_header.header_length,
		     frame->usb_header.frame_numb,
		     frame->usb_header.frame_phase,
		     frame->usb_header.frame_latency,
		     frame->usb_header.data_format,
		     frame->usb_header.format_param,
		     frame->usb_header.frame_width,
		     frame->usb_header.frame_height);
#endif

		frame->frmwidth = frame->usb_header.frame_width * usbvision->stretch_width;
		frame->frmheight = frame->usb_header.frame_height * usbvision->stretch_height;
		frame->v4l_linesize = frame->frmwidth * usbvision_v4l_format[frame->v4l_format].bytes_per_pixel;

		if (debug > 2) {
			printk(KERN_DEBUG "Header found! w:%d h:%d\n",
			       frame->frmwidth, frame->frmheight);
		}
	}
	else { // no header found
		if (debug > 2) {
			printk(KERN_DEBUG "Skipping scratch data, no header\n");
		}
		scratch_reset(usbvision);
		return scan_EndParse;
	}

	/* Header found */
	usbvision->has_hdr = 1;
	usbvision->header_count++;
	frame->scanstate = STATE_LINES;
	frame->curline = 0;

	if (flags & FLAGS_FORCE_TESTPATTERN) {
		usbvision_testpattern(usbvision, 1, 1);
		return scan_NextFrame;
	}
	return scan_Continue;
}

static scan_state_t usbvision_parse_lines_422(struct usb_usbvision *usbvision,
					   long *pcopylen)
{
	volatile struct usbvision_frame *frame;
	unsigned char *f; 
	int len;
	int i;
	unsigned char yuyv[4]={180, 128, 10, 128}; // YUV components
	unsigned char rv, gv, bv;	// RGB components
	int clipmask_index, bytes_per_pixel;
	int overlay = usbvision->overlay;
	int stretch_bytes, clipmask_add;
	
	if (overlay) {
		frame  = &usbvision->overlay_frame;
		if (usbvision->overlay_base == NULL) {
			//video_buffer is not set yet
			return scan_NextFrame;
		}
		f = usbvision->overlay_win + frame->curline *
			usbvision->vid_buf.bytesperline;
	}
	else {
		frame  = &usbvision->frame[usbvision->curframe];
		f = frame->data + (frame->v4l_linesize * frame->curline);
	}
	
	/* Make sure there's enough data for the entire line */
	len = (frame->usb_header.frame_width * 2)+5;
	if (scratch_len(usbvision) < len) {
		if (debug >= 2) {
			printk(KERN_DEBUG
			       "out of data in line %d, need %u.\n",
			       frame->curline, len);
		}
		return scan_Out;
	}

	if ((frame->curline + 1) >= frame->frmheight) {
		return scan_NextFrame;
	}

	bytes_per_pixel = usbvision_v4l_format[frame->v4l_format].bytes_per_pixel;
	stretch_bytes = (usbvision->stretch_width - 1) * bytes_per_pixel;
	clipmask_index = frame->curline * MAX_FRAME_WIDTH;
	clipmask_add = usbvision->stretch_width;
	
	for (i = 0; i < frame->frmwidth; i+=(2 * usbvision->stretch_width)) {

		scratch_get(usbvision, &yuyv[0], 4);
		
		if((overlay) && (clipped_pixel(clipmask_index))) {
			f += bytes_per_pixel;
		}
		else if (frame->v4l_format == VIDEO_PALETTE_YUV422) {
			*f++ = yuyv[0]; // Y
			*f++ = yuyv[3]; // U 
		}		
		else {

			YUV_TO_RGB_BY_THE_BOOK(yuyv[0], yuyv[1], yuyv[3], rv, gv, bv);
			switch (frame->v4l_format) {
				case VIDEO_PALETTE_RGB565:
					*f++ = (0x1F & (bv >> 3)) | (0xE0 & (gv << 3));
					*f++ = (0x07 & (gv >> 5)) | (0xF8 &  rv);
					break;
				case VIDEO_PALETTE_RGB24:
					*f++ = bv;
					*f++ = gv;
					*f++ = rv;
					break;
				case VIDEO_PALETTE_RGB32:
					*f++ = bv;
					*f++ = gv;
					*f++ = rv;
					f++;
					break;
				case VIDEO_PALETTE_RGB555:
					*f++ = (0x1F & (bv >> 3)) | (0xE0 & (gv << 2));
					*f++ = (0x03 & (gv >> 6)) | (0x7C & (rv >> 1));
					break;				
			}
		}
		clipmask_index += clipmask_add;
		f += stretch_bytes;

		if((overlay) && (clipped_pixel(clipmask_index))) {
			f += bytes_per_pixel;
		}
		else if (frame->v4l_format == VIDEO_PALETTE_YUV422) {
			*f++ = yuyv[2]; // Y
			*f++ = yuyv[1]; // V	
		}
		else {

			YUV_TO_RGB_BY_THE_BOOK(yuyv[2], yuyv[1], yuyv[3], rv, gv, bv);
			switch (frame->v4l_format) {
				case VIDEO_PALETTE_RGB565:
					*f++ = (0x1F & (bv >> 3)) | (0xE0 & (gv << 3));
					*f++ = (0x07 & (gv >> 5)) | (0xF8 &  rv);
					break;
				case VIDEO_PALETTE_RGB24:
					*f++ = bv;
					*f++ = gv;
					*f++ = rv;
					break;
				case VIDEO_PALETTE_RGB32:
					*f++ = bv;
					*f++ = gv;
					*f++ = rv;
					f++;
					break;
				case VIDEO_PALETTE_RGB555:
					*f++ = (0x1F & (bv >> 3)) | (0xE0 & (gv << 2));
					*f++ = (0x03 & (gv >> 6)) | (0x7C & (rv >> 1));
					break;			
			}
		}
		clipmask_index += clipmask_add;
		f += stretch_bytes;
	}
	
	frame->curline += usbvision->stretch_height;
	*pcopylen += frame->v4l_linesize * usbvision->stretch_height;

	if (frame->curline >= frame->frmheight) {
		return scan_NextFrame;
	}
	else {
		return scan_Continue;
	}
}


/*
 * usbvision_parse_compress()
 *
 * Parse compressed frame from the scratch buffer, put 
 * decoded RGB value into the current frame buffer and add the written
 * number of bytes (RGB) to the *pcopylen.
 * 
 */
static scan_state_t usbvision_parse_compress(struct usb_usbvision *usbvision,
					   long *pcopylen)
{
#define USBVISION_STRIP_MAGIC		0x5A
#define USBVISION_STRIP_LEN_MAX		400
#define USBVISION_STRIP_HEADER_LEN	10
	
	struct usbvision_frame *frame;
	unsigned char *f;
	unsigned char strip_data[USBVISION_STRIP_LEN_MAX];
	unsigned char strip_header[USBVISION_STRIP_HEADER_LEN];
	int i, i_end, strip_len, strip_ptr;
	int clipmask_index, bytes_per_pixel;
	int overlay = usbvision->overlay;
	
	if (overlay) {
		frame  = &usbvision->overlay_frame;
		if (usbvision->overlay_base == NULL) {
			//video_buffer is not set yet
			return scan_NextFrame;
		}
		f = usbvision->overlay_win + frame->curline *
			 usbvision->vid_buf.bytesperline;
	}
	else {
		frame  = &usbvision->frame[usbvision->curframe];
		f = frame->data + (frame->v4l_linesize * frame->curline);
	}

	if ((frame->curline + 1) >= frame->frmheight) {
		return scan_NextFrame;
	}

	if (scratch_len(usbvision) < USBVISION_STRIP_HEADER_LEN) {
		return scan_Out;
	}
	
	if (frame->usb_header.format_param & USBVISION_FRAME_FORMAT_PARAM_INTRA) {

		//get strip header without changing the scratch_read_ptr
		scratch_set_extra_ptr(usbvision, &strip_ptr, 0);
		scratch_get_extra(usbvision, &strip_header[0], &strip_ptr, 
				USBVISION_STRIP_HEADER_LEN);
			
		if (strip_header[0] != USBVISION_STRIP_MAGIC) {
			return scan_NextFrame;
		}

		if (strip_header[2] != frame->curline) {
			printk(KERN_DEBUG "usbvision_parse_compress: line number error\n");
		}
			
		strip_len = 2 * (unsigned int)strip_header[1];
		if (scratch_len(usbvision) < strip_len) {
			//there is not enough data for the strip
			return scan_Out;
		}
		
		bytes_per_pixel = usbvision_v4l_format[frame->v4l_format].bytes_per_pixel;
		clipmask_index = frame->curline * MAX_FRAME_WIDTH;

		memset(strip_data, 0, USBVISION_STRIP_LEN_MAX);
		scratch_get(usbvision, strip_data, strip_len);
			
		i_end = frame->frmwidth;
		if (frame->v4l_format == VIDEO_PALETTE_GREY) {
			i_end = USBVISION_STRIP_LEN_MAX;
		}
		
		for(i = 0; i < i_end; i++) { 
			if((overlay) && (clipped_pixel(clipmask_index))) {
				f += bytes_per_pixel;
			}
			else {
				switch (frame->v4l_format) {
					case VIDEO_PALETTE_GREY:
						*f++ = strip_data[i];
						break;
					case VIDEO_PALETTE_RGB565:
					case VIDEO_PALETTE_RGB555:
						*f++ = strip_data[i];
						*f++ = strip_data[i];
						break;
					case VIDEO_PALETTE_RGB24:
						*f++ = strip_data[i];
						*f++ = strip_data[i];
						*f++ = strip_data[i];
						break;
					case VIDEO_PALETTE_RGB32:
						*f++ = strip_data[i];
						*f++ = strip_data[i];
						*f++ = strip_data[i];
						f++;
						break;					
				}
			}
			clipmask_index++;
		}
		*pcopylen += frame->v4l_linesize;

		frame->curline += 1;

	}
	else {
		//it wasn't an INTRA Frame
		return scan_NextFrame;
	}

	if (frame->curline >= frame->frmheight) {
		return scan_NextFrame;
	}
	else {
		return scan_Continue;
	}

}

/*
 * usbvision_parse_lines_420()
 *
 * Parse two lines from the scratch buffer, put 
 * decoded RGB value into the current frame buffer and add the written
 * number of bytes (RGB) to the *pcopylen.
 * 
 */
static scan_state_t usbvision_parse_lines_420(struct usb_usbvision *usbvision,
					   long *pcopylen)
{
	struct usbvision_frame *frame;
	unsigned char *f_even = NULL, *f_odd = NULL;
	unsigned int pixel_per_line;
	int block, pixel, block_split;
	int y_ptr, u_ptr, v_ptr, y_odd_offset;
	const int   y_block_size = 128;
	const int  uv_block_size = 64;
	const int sub_block_size = 32;
	const int y_step[] = { 0, 0, 0, 2 },  y_step_size = 4;
	const int uv_step[]= { 0, 0, 0, 4 }, uv_step_size = 4;
	unsigned char y[2], u, v;	/* YUV components */
	int y_, u_, v_, vb, uvg, ur;
	int r_, g_, b_;			/* RGB components */
	unsigned char g;
	int clipmask_even_index, clipmask_odd_index, bytes_per_pixel;
	int clipmask_add, stretch_bytes;
	int overlay = usbvision->overlay;
	
	if (overlay) {
		frame  = &usbvision->overlay_frame;
		if (usbvision->overlay_base == NULL) {
			//video_buffer is not set yet
			return scan_NextFrame;
		}
		f_even = usbvision->overlay_win + frame->curline *
			 usbvision->vid_buf.bytesperline;
		f_odd  = f_even + usbvision->vid_buf.bytesperline * usbvision->stretch_height;
	}
	else {
		frame  = &usbvision->frame[usbvision->curframe];
		f_even = frame->data + (frame->v4l_linesize * frame->curline);
		f_odd  = f_even + frame->v4l_linesize * usbvision->stretch_height;
	}
	
	/* Make sure there's enough data for the entire line */
	/* In this mode usbvision transfer 3 bytes for every 2 pixels */
	/* I need two lines to decode the color */
	bytes_per_pixel = usbvision_v4l_format[frame->v4l_format].bytes_per_pixel;
	stretch_bytes = (usbvision->stretch_width - 1) * bytes_per_pixel;
	clipmask_even_index = frame->curline * MAX_FRAME_WIDTH;
	clipmask_odd_index  = clipmask_even_index + MAX_FRAME_WIDTH;
	clipmask_add = usbvision->stretch_width;
	pixel_per_line = frame->usb_header.frame_width;
	
	if (scratch_len(usbvision) < pixel_per_line * 3) {
		//printk(KERN_DEBUG "out of data, need %d\n", len);
		return scan_Out;
	}

	if ((frame->curline + 1) >= frame->frmheight) {
		return scan_NextFrame;
	}

	block_split = (pixel_per_line%y_block_size) ? 1 : 0;	//are some blocks splitted 
															//into different lines?
	
	y_odd_offset = (pixel_per_line / y_block_size) * (y_block_size + uv_block_size)
			+ block_split * uv_block_size;
	
	scratch_set_extra_ptr(usbvision, &y_ptr, y_odd_offset);
	scratch_set_extra_ptr(usbvision, &u_ptr, y_block_size);
	scratch_set_extra_ptr(usbvision, &v_ptr, y_odd_offset 
			+ (4 - block_split) * sub_block_size);
	
	for (block = 0; block < (pixel_per_line / sub_block_size);
	     block++) {
		
	
		for (pixel = 0; pixel < sub_block_size; pixel +=2) {
			scratch_get(usbvision, &y[0], 2);
			scratch_get_extra(usbvision, &u, &u_ptr, 1);
			scratch_get_extra(usbvision, &v, &v_ptr, 1);

			//I don't use the YUV_TO_RGB macro for better performance
			v_ = v - 128;
			u_ = u - 128;
			vb =              132252 * v_;
			uvg= -53281 * u_ - 25625 * v_;
			ur = 104595 * u_;
			
			if((overlay) && (clipped_pixel(clipmask_even_index))) {
				f_even += bytes_per_pixel;
			}
			else {
				y_ = 76284 * (y[0] - 16);
			
				b_ = (y_ + vb) >> 16;
				g_ = (y_ + uvg)>> 16;
				r_ = (y_ + ur) >> 16;
		
				switch (frame->v4l_format) {
					case VIDEO_PALETTE_RGB565:
						g = LIMIT_RGB(g_);
						*f_even++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 3));
						*f_even++ = (0x07 & (          g   >> 5)) | (0xF8 & LIMIT_RGB(r_));
						break;
					case VIDEO_PALETTE_RGB24:
						*f_even++ = LIMIT_RGB(b_);
						*f_even++ = LIMIT_RGB(g_);
						*f_even++ = LIMIT_RGB(r_);
						break;
					case VIDEO_PALETTE_RGB32:
						*f_even++ = LIMIT_RGB(b_);
						*f_even++ = LIMIT_RGB(g_);
						*f_even++ = LIMIT_RGB(r_);
						f_even++;
						break;
					case VIDEO_PALETTE_RGB555:
						g = LIMIT_RGB(g_);
						*f_even++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 2));
						*f_even++ = (0x03 & (          g   >> 6)) | 
							    (0x7C & (LIMIT_RGB(r_) >> 1));
						break;				
				}
			}
			clipmask_even_index += clipmask_add;
			f_even += stretch_bytes;
			
			if((overlay) && (clipped_pixel(clipmask_even_index))) {
				f_even += bytes_per_pixel;
			}
			else {
				y_ = 76284 * (y[1] - 16);
			
				b_ = (y_ + vb) >> 16;
				g_ = (y_ + uvg)>> 16;
				r_ = (y_ + ur) >> 16;
			
				switch (frame->v4l_format) {
					case VIDEO_PALETTE_RGB565:
						g = LIMIT_RGB(g_);
						*f_even++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 3));
						*f_even++ = (0x07 & (          g   >> 5)) | (0xF8 & LIMIT_RGB(r_));
						break;
					case VIDEO_PALETTE_RGB24:
						*f_even++ = LIMIT_RGB(b_);
						*f_even++ = LIMIT_RGB(g_);
						*f_even++ = LIMIT_RGB(r_);
						break;
					case VIDEO_PALETTE_RGB32:
						*f_even++ = LIMIT_RGB(b_);
						*f_even++ = LIMIT_RGB(g_);
						*f_even++ = LIMIT_RGB(r_);
						f_even++;
						break;
					case VIDEO_PALETTE_RGB555:
						g = LIMIT_RGB(g_);
						*f_even++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 2));
						*f_even++ = (0x03 & (          g   >> 6)) | 
							    (0x7C & (LIMIT_RGB(r_) >> 1));
						break;					
				}
			}		
			clipmask_even_index += clipmask_add;
			f_even += stretch_bytes;
			
			scratch_get_extra(usbvision, &y[0], &y_ptr, 2);
			
			if ((overlay) && (clipped_pixel(clipmask_odd_index))) {
				f_odd += bytes_per_pixel;
			}
			else {
				y_ = 76284 * (y[0] - 16);
			
				b_ = (y_ + vb) >> 16;
				g_ = (y_ + uvg)>> 16;
				r_ = (y_ + ur) >> 16;
			
				switch (frame->v4l_format) {
					case VIDEO_PALETTE_RGB565:
						g = LIMIT_RGB(g_);
						*f_odd++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 3));
						*f_odd++ = (0x07 & (          g   >> 5)) | (0xF8 & LIMIT_RGB(r_));
						break;
					case VIDEO_PALETTE_RGB24:
						*f_odd++ = LIMIT_RGB(b_);
						*f_odd++ = LIMIT_RGB(g_);
						*f_odd++ = LIMIT_RGB(r_);
						break;
					case VIDEO_PALETTE_RGB32:
						*f_odd++ = LIMIT_RGB(b_);
						*f_odd++ = LIMIT_RGB(g_);
						*f_odd++ = LIMIT_RGB(r_);
						f_odd++;
						break;
					case VIDEO_PALETTE_RGB555:
						g = LIMIT_RGB(g_);
						*f_odd++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 2));
						*f_odd++ = (0x03 & (          g   >> 6)) | 
							   (0x7C & (LIMIT_RGB(r_) >> 1));
						break;					
				}
			}
			clipmask_odd_index += clipmask_add;
			f_odd += stretch_bytes;
			
			if((overlay) && (clipped_pixel(clipmask_odd_index))) {
				f_odd += bytes_per_pixel;
			}
			else {
				y_ = 76284 * (y[1] - 16);
			
				b_ = (y_ + vb) >> 16;
				g_ = (y_ + uvg)>> 16;
				r_ = (y_ + ur) >> 16;
		
				switch (frame->v4l_format) {
					case VIDEO_PALETTE_RGB565:
						g = LIMIT_RGB(g_);
						*f_odd++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 3));
						*f_odd++ = (0x07 & (          g   >> 5)) | (0xF8 & LIMIT_RGB(r_));
						break;
					case VIDEO_PALETTE_RGB24:
						*f_odd++ = LIMIT_RGB(b_);
						*f_odd++ = LIMIT_RGB(g_);
						*f_odd++ = LIMIT_RGB(r_);
						break;
					case VIDEO_PALETTE_RGB32:
						*f_odd++ = LIMIT_RGB(b_);
						*f_odd++ = LIMIT_RGB(g_);
						*f_odd++ = LIMIT_RGB(r_);
						f_odd++;
						break;
					case VIDEO_PALETTE_RGB555:
						g = LIMIT_RGB(g_);
						*f_odd++ = (0x1F & (LIMIT_RGB(b_) >> 3)) | (0xE0 & (g << 2));
						*f_odd++ = (0x03 & (          g   >> 6)) | 
							   (0x7C & (LIMIT_RGB(r_) >> 1));
						break;					
				}
			}
			clipmask_odd_index += clipmask_add;
			f_odd += stretch_bytes;
		}
		
		scratch_rm_old(usbvision,y_step[block % y_step_size] * sub_block_size); 
		scratch_inc_extra_ptr(&y_ptr, y_step[(block + 2 * block_split) % y_step_size]
				* sub_block_size);
		scratch_inc_extra_ptr(&u_ptr, uv_step[block % uv_step_size]
				* sub_block_size);
		scratch_inc_extra_ptr(&v_ptr, uv_step[(block + 2 * block_split) % uv_step_size]
				* sub_block_size);
	}

	scratch_rm_old(usbvision, pixel_per_line * 3 / 2 
			+ block_split * sub_block_size);
	
	frame->curline += 2 * usbvision->stretch_height;
	*pcopylen += frame->v4l_linesize * 2 * usbvision->stretch_height;

	if (frame->curline >= frame->frmheight)
		return scan_NextFrame;
	else
		return scan_Continue;
}

/*
 * usbvision_parse_data()
 *
 * Generic routine to parse the scratch buffer. It employs either
 * usbvision_find_header() or usbvision_parse_lines() to do most
 * of work.
 *
 */
static void usbvision_parse_data(struct usb_usbvision *usbvision)
{
	struct usbvision_frame *frame;
	scan_state_t newstate;
	long copylen = 0;

	/* Grab the current frame and the previous frame */
	if (usbvision->overlay) {
		frame = &usbvision->overlay_frame;
	}
	else {
		frame = &usbvision->frame[usbvision->curframe];
	}
	
//	printk(KERN_DEBUG "frame=%p\n", frame);
	if (debug >=1) {
		printk(KERN_DEBUG "parsing len=%d\n", scratch_len(usbvision));
	}


	
	while (1) {

		newstate = scan_Out;
		if (scratch_len(usbvision)) {
			if (frame->scanstate == STATE_SCANNING) {
				newstate = usbvision_find_header(usbvision);
			}
			else if (frame->scanstate == STATE_LINES) {
				if (usbvision->videoformat == VIDEOFORMAT_YUV420) {
					newstate = usbvision_parse_lines_420(usbvision, &copylen);
				}
				else if (usbvision->videoformat == VIDEOFORMAT_YUV422) {
					newstate = usbvision_parse_lines_422(usbvision, &copylen);
				}
				else if (usbvision->videoformat == VIDEOFORMAT_COMPRESS) {
					newstate = usbvision_parse_compress(usbvision, &copylen);
				}

			}
		}
		if (newstate == scan_Continue) {
			continue;
		}
		else if ((newstate == scan_NextFrame) || (newstate == scan_Out)) {
			break;
		}
		else {
			return;	/* scan_EndParse */
		}
	}

	if (newstate == scan_NextFrame) {
		frame->grabstate = FRAME_DONE;
		if (usbvision->overlay) {
			frame->grabstate = FRAME_GRABBING;
			frame->scanstate = STATE_SCANNING;
			frame->scanlength = 0;
			copylen = 0;
		}
		else {
			usbvision->curframe = -1;
		}
		usbvision->frame_num++;

		/* Optionally display statistics on the screen */
		if (flags & FLAGS_OSD_STATS)
			usbvision_osd_stats(usbvision, frame);

		/* This will cause the process to request another frame. */
//	if (usbvision->overlay == 0) {
			if (waitqueue_active(&frame->wq)) {
				wake_up_interruptible(&frame->wq);
			}
//		}
	}

	/* Update the frame's uncompressed length. */
	frame->scanlength += copylen;
}


/*
 * Make all of the blocks of data contiguous
 */
static int usbvision_compress_isochronous(struct usb_usbvision *usbvision,
				       struct urb *urb)
{
	unsigned char *packet_data;
	int i, totlen = 0;

	for (i = 0; i < urb->number_of_packets; i++) {
		int packet_len = urb->iso_frame_desc[i].actual_length;
		int packet_stat = urb->iso_frame_desc[i].status;

		packet_data =
		    urb->transfer_buffer + urb->iso_frame_desc[i].offset;

		/* Detect and ignore errored packets */
		if (packet_stat) {	// packet_stat != 0 ?????????????
			if (debug >= 1) {
				printk(KERN_ERR
				       "usbvision data error: [%d] len=%d, status=%X\n",
				       i, packet_len, packet_stat);
			}
			usbvision->iso_err_count++;
			continue;
		}

		/* Detect and ignore empty packets */
		if (packet_len < 0) {
			if (debug >= 2) {
				printk(KERN_DEBUG
				       "usbvision error packet [%d]\n", i);
			}
			usbvision->iso_skip_count++;
			continue;
		} 
		else if (packet_len == 0) {	/* Frame end ????? */
			if (debug >= 2) {
				printk(KERN_DEBUG
				       "usbvision null packet [%d]\n", i);
			}
			usbvision->isocstate=ISOC_NOFRAME;
			usbvision->iso_skip_count++;
			continue;
		}
		else if (packet_len > usbvision->iso_packet_len) {
			printk(KERN_DEBUG "usbvision packet[%d] > iso_packet_len\n", i);
			usbvision->iso_skip_count++;
			continue;
		}
		
		if (debug >= 2) {
			printk(KERN_DEBUG "usbvision packet ok [%d] len=%d\n", i, packet_len);
		}
		
		if (usbvision->isocstate==ISOC_NOFRAME) { //new frame begins
			usbvision->isocstate=ISOC_INFRAME;
			scratch_mark_header(usbvision);
			if (debug >= 2) {
				printk(KERN_DEBUG "usbvision packet with header\n");
			}
		}
			
		/*
		 * If usbvision continues to feed us with data but there is no
		 * consumption (if, for example, V4L client fell asleep) we
		 * may overflow the buffer. We have to move old data over to
		 * free room for new data. This is bad for old data. If we
		 * just drop new data then it's bad for new data... choose
		 * your favorite evil here.
		 */
		if (scratch_free(usbvision) < packet_len) {

			usbvision->scratch_ovf_count++;
			if (debug >= 2) {
				printk(KERN_ERR
				       "usbvision: scratch buf overflow! "
				       "scr_len: %d, n: %d\n",
				       scratch_len(usbvision), packet_len);
			}
			scratch_rm_old(usbvision, packet_len - scratch_free(usbvision));
		}

		/* Now we know that there is enough room in scratch buffer */
		scratch_put(usbvision, packet_data, packet_len);
		totlen += packet_len;

	}
#if ENABLE_HEXDUMP
	if (totlen > 0) {
		static int foo = 0;
		if (foo < 1) {
			printk(KERN_DEBUG "+%d.\n", usbvision->scratchlen);
			usbvision_hexdump(data0, (totlen > 64) ? 64 : totlen);
			++foo;
		}
	}
#endif
	return totlen;
}

static void usbvision_isoc_irq(struct urb *urb)
{
	IRQDEBUG(static const char proc[]="usbvision_isoc_irq: ";)
	int len;
	struct usb_usbvision *usbvision = urb->context;
	struct usbvision_sbuf *sbuf;
	int i;

	/* We don't want to do anything if we are about to be removed! */
	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return;

	if (!usbvision->streaming) {
		IRQDEBUG(printk(KERN_DEBUG "%soops, not streaming, but interrupt\n", proc);)
		return;
	}

	sbuf = &usbvision->sbuf[usbvision->cursbuf];

	/* Copy the data received into our scratch buffer */
	len = usbvision_compress_isochronous(usbvision, urb);

	usbvision->urb_count++;
	usbvision->urb_length = len;
	usbvision->data_count += len;

	/* If we collected enough data let's parse! */
	if (scratch_len(usbvision) > USBVISION_HEADER_LENGTH) {	/* 12 == header_length */
		/*If we don't have a frame we're current working on, complain */
		if ((usbvision->curframe >= 0) || (usbvision->overlay))
			usbvision_parse_data(usbvision);
		else {
			IRQDEBUG(printk(KERN_DEBUG "%sreceived data, but no frame\n", proc);)
			scratch_reset(usbvision);
		}
	}

	for (i = 0; i < FRAMES_PER_DESC; i++) {
		sbuf->urb->iso_frame_desc[i].status = 0;
		sbuf->urb->iso_frame_desc[i].actual_length = 0;
	}

	/* Move to the next sbuf */
	usbvision->cursbuf = (usbvision->cursbuf + 1) % USBVISION_NUMSBUF;

	return;
}

/*************************************/
/* Low level usbvision access functions */
/*************************************/

/*
 * usbvision_read_reg()
 *
 * return  < 0 -> Error
 *        >= 0 -> Data
 */

static int usbvision_read_reg(struct usb_usbvision *usbvision, unsigned char reg)
{
	static const char proc[] = "usbvision_read_reg";
	int rc;
	unsigned char buffer[1];

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return -1;

	rc = usb_control_msg(usbvision->dev, usb_rcvctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,
			     USB_DIR_IN | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0, (__u16) reg, buffer, 1,
			     HZ);

	if (rc < 0) {
		printk(KERN_ERR "%s: ERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
		return rc;
	}
	return buffer[0];
}

/*
 * usbvision_write_reg()
 *
 * return 1 -> Reg written
 *        0 -> usbvision is not yet ready
 *       -1 -> Something went wrong
 */

static int usbvision_write_reg(struct usb_usbvision *usbvision, unsigned char reg,
			    unsigned char value)
{
	static const char proc[] = "usbvision_write_reg";
	int rc;

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;

	rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0, (__u16) reg, &value, 1,
			     HZ);

	if (rc < 0) {
		printk(KERN_ERR "%s: ERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
	}
	return rc;
}

/* ----------------------------------------------------------------------- */
/* I2C functions                                                           */
/* ----------------------------------------------------------------------- */

static void call_i2c_clients(struct usb_usbvision *usbvision, unsigned int cmd,
			     void *arg)
{
	int i;

	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
		if (NULL == usbvision->i2c_clients[i])
			continue;
		if (NULL == usbvision->i2c_clients[i]->driver->command)
			continue;
		usbvision->i2c_clients[i]->driver->command(usbvision->
							i2c_clients[i],
							cmd, arg);
	}
}


static int
usbvision_i2c_read_max4(struct usb_usbvision *usbvision, unsigned char addr,
		     char *buf, short len)
{
	int rc, retries;

	for (retries = 5;;) {
		rc = usbvision_write_reg(usbvision, USBVISION_SER_ADRS, addr);
		if (rc < 0)
			return rc;

		/* Initiate byte read cycle                    */
		/* USBVISION_SER_CONT <- d0-d2 n. of bytes to r/w */
		/*                    d3 0=Wr 1=Rd             */
		rc = usbvision_write_reg(usbvision, USBVISION_SER_CONT,
				      (len & 0x07) | 0x18);
		if (rc < 0)
			return rc;

		/* Test for Busy and ACK */
		do {
			/* USBVISION_SER_CONT -> d4 == 0 busy */
			rc = usbvision_read_reg(usbvision, USBVISION_SER_CONT);
		} while (rc > 0 && ((rc & 0x10) != 0));	/* Retry while busy */
		if (rc < 0)
			return rc;

		/* USBVISION_SER_CONT -> d5 == 1 Not ack */
		if ((rc & 0x20) == 0)	/* Ack? */
			break;

		/* I2C abort */
		rc = usbvision_write_reg(usbvision, USBVISION_SER_CONT, 0x00);
		if (rc < 0)
			return rc;

		if (--retries < 0)
			return -1;
	}

	switch (len) {
	case 4:
		buf[3] = usbvision_read_reg(usbvision, USBVISION_SER_DAT4);
	case 3:
		buf[2] = usbvision_read_reg(usbvision, USBVISION_SER_DAT3);
	case 2:
		buf[1] = usbvision_read_reg(usbvision, USBVISION_SER_DAT2);
	case 1:
		buf[0] = usbvision_read_reg(usbvision, USBVISION_SER_DAT1);
		break;
	default:
		printk(KERN_ERR
		       "usbvision_i2c_read_max4: buffer length > 4\n");
	}

	return len;
}


static int usbvision_i2c_write_max4(struct usb_usbvision *usbvision,
				 unsigned char addr, const char *buf,
				 short len)
{
	int rc, retries;
	int i;
	unsigned char value[6];
	unsigned char ser_cont;

	ser_cont = (len & 0x07) | 0x10;

	value[0] = addr;
	value[1] = ser_cont;
	for (i = 0; i < len; i++)
		value[i + 2] = buf[i];

	for (retries = 5;;) {
		rc = usb_control_msg(usbvision->dev,
				     usb_sndctrlpipe(usbvision->dev, 1),
				     USBVISION_OP_CODE,
				     USB_DIR_OUT | USB_TYPE_VENDOR |
				     USB_RECIP_ENDPOINT, 0,
				     (__u16) USBVISION_SER_ADRS, value,
				     len + 2, HZ);

		if (rc < 0)
			return rc;

		rc = usbvision_write_reg(usbvision, USBVISION_SER_CONT,
				      (len & 0x07) | 0x10);
		if (rc < 0)
			return rc;

		/* Test for Busy and ACK */
		do {
			rc = usbvision_read_reg(usbvision, USBVISION_SER_CONT);
		} while (rc > 0 && ((rc & 0x10) != 0));	/* Retry while busy */
		if (rc < 0)
			return rc;

		if ((rc & 0x20) == 0)	/* Ack? */
			break;

		/* I2C abort */
		usbvision_write_reg(usbvision, USBVISION_SER_CONT, 0x00);

		if (--retries < 0)
			return -1;

	}

	return len;
}

static int usbvision_i2c_write(void *data, unsigned char addr, char *buf,
			    short len)
{
	char *bufPtr = buf;
	int retval;
	int wrcount = 0;
	int count;
	int maxLen = 2;
	struct usb_usbvision *usbvision = (struct usb_usbvision *) data;

	/* When writing in the SAA7111 have to write pairs
	 * sub-addr, value. If not the usbvision does not write
	 * in the right address */
	/*if ((addr == USBVISION_SAA7111_ADDR)||(addr == USBVISION_BT819_ADDR))
		maxLen = 2;
	else
		maxLen = 4;*/	/* 4 is the max USBVISION is able to write */

	while (len > 0) {
		count = (len > maxLen) ? maxLen : len;
		retval = usbvision_i2c_write_max4(usbvision, addr, bufPtr, count);
		if (retval > 0) {
			len -= count;
			bufPtr += count;
			wrcount += count;
		} else
			return (retval < 0) ? retval : -EFAULT;
	}
	return wrcount;
}

static int usbvision_i2c_read(void *data, unsigned char addr, char *buf,
			   short len)
{
	char temp[4];
	int retval, i;
	int rdcount = 0;
	int count;
	struct usb_usbvision *usbvision = (struct usb_usbvision *) data;

	while (len > 0) {
		count = (len > 3) ? 4 : len;
		retval = usbvision_i2c_read_max4(usbvision, addr, temp, count);
		if (retval > 0) {
			for (i = 0; i < len; i++)
				buf[rdcount + i] = temp[i];
			len -= count;
			rdcount += count;
		} else
			return (retval < 0) ? retval : -EFAULT;
	}
	return rdcount;
}

static struct i2c_algo_usb_data i2c_algo_template = {
	.inb	= usbvision_i2c_read,
	.outb	= usbvision_i2c_write,
	.udelay	= 10,
	.mdelay	= 10,
	.timeout	= 100,
};

static struct i2c_adapter i2c_adap_template = {
	.owner	= THIS_MODULE,
	.name	= "usbvision",
	.id	= I2C_HW_B_BT848,
};

static struct i2c_client i2c_client_template = {
	.name	= "usbvision internal",
	.id	= -1,
};

static int usbvision_init_i2c(struct usb_usbvision *usbvision)
{
	memcpy(&usbvision->i2c_adap, &i2c_adap_template,
	       sizeof(struct i2c_adapter));
	memcpy(&usbvision->i2c_algo, &i2c_algo_template,
	       sizeof(struct i2c_algo_usb_data));
	memcpy(&usbvision->i2c_client, &i2c_client_template,
	       sizeof(struct i2c_client));

	sprintf(usbvision->i2c_adap.name + strlen(usbvision->i2c_adap.name),
		" #%d", usbvision->nr);
	usbvision->i2c_algo.data = usbvision;
	usbvision->i2c_adap.data = usbvision;
	usbvision->i2c_adap.algo_data = &usbvision->i2c_algo;
	usbvision->i2c_client.adapter = &usbvision->i2c_adap;

	if (usbvision_write_reg(usbvision, USBVISION_SER_MODE, USBVISION_IIC_LRNACK) < 0) {
		printk(KERN_ERR "usbvision_init_i2c: can't wirte reg\n");
		return -EBUSY;
	}

	/* Request the load of the i2c modules we need */
	if (autoload) {
		switch (usbvision_device_data[usbvision->DevModel].Codec) {
			case CODEC_SAA7113:
				request_module("saa7113-new");
				break;
			case CODEC_SAA7111:
				request_module("saa7111-new");
				break;
			case CODEC_BT819:
				request_module("bt819-new");
				break;
		}
		if (usbvision_device_data[usbvision->DevModel].Tuner == 1) {
	    	request_module("tuner");
		}
	}
	
	usbvision->i2c_ok = i2c_usb_add_bus(&usbvision->i2c_adap);

	return usbvision->i2c_ok;

}


/****************************/
/* usbvision utility functions */
/****************************/

static int usbvision_power_off(struct usb_usbvision *usbvision)
{
	printk(KERN_DEBUG "usbvision_power_off\n");
	return (usbvision_write_reg(usbvision, USBVISION_PWR_REG, USBVISION_SSPND_EN));
}

/*
 * usbvision_set_video_format()
 *
 */

static int usbvision_set_video_format(struct usb_usbvision *usbvision, int format)
{
	static const char proc[] = "usbvision_set_video_format";
	int rc;
	unsigned char value[2];

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;

	if ((format != VIDEOFORMAT_YUV422)
	    && (format != VIDEOFORMAT_YUV420)
	    && (format != VIDEOFORMAT_COMPRESS)) {
		printk(KERN_ERR "usbvision: unknown video format %02x, using default YUV420",
		       format);
		format = VIDEOFORMAT_YUV420;
	}
	value[0] = 0x0A;  //TODO: See the effect of the filter
	value[1] = format;
	rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0,
			     (__u16) USBVISION_FILT_CONT, value, 2, HZ);

	if (rc < 0) {
		printk(KERN_ERR "%s: ERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
	}
	usbvision->videoformat = format;
	return rc;
}

/*
 * usbvision_set_output()
 *
 */

static int usbvision_set_output(struct usb_usbvision *usbvision, int width,
				  int height)
{
	static const char proc[] = "usbvision_set_output: ";
	int rc;
	int UsbWidth, UsbHeight;
	unsigned int framerate=0, frame_drop_factor=0;
	unsigned char value[4];

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;
	
	if (width > MAX_USB_WIDTH) {
		UsbWidth = width / 2;
		usbvision->stretch_width = 2;
	}
	else {
		UsbWidth = width;
		usbvision->stretch_width = 1;
	}
	
	if (height > MAX_USB_HEIGHT) {
		UsbHeight = height / 2;
		usbvision->stretch_height = 2;
	}
	else {
		UsbHeight = height;
		usbvision->stretch_height = 1;
	}

	RESTRICT_TO_RANGE(UsbWidth, MIN_FRAME_WIDTH, MAX_USB_WIDTH);
	UsbWidth &= ~(MIN_FRAME_WIDTH-1);
	RESTRICT_TO_RANGE(UsbHeight, MIN_FRAME_HEIGHT, MAX_USB_HEIGHT);
	UsbHeight &= ~(1);
	
	DEBUG(printk(KERN_DEBUG "%susb %dx%d; screen %dx%d; stretch %dx%d\n", 
							proc, UsbWidth, UsbHeight, width, height,
							usbvision->stretch_width, usbvision->stretch_height);)
		
	/* I'll not rewrite the same values */
	if ((UsbWidth != usbvision->curwidth) || (UsbHeight != usbvision->curheight)) {
		value[0] = UsbWidth & 0xff;		//LSB
		value[1] = (UsbWidth >> 8) & 0xff;		//MSB
		value[2] = UsbHeight & 0xff;		//LSB
		value[3] = (UsbHeight >> 8) & 0xff;	//MSB

		rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0,
			     (__u16) USBVISION_LXSIZE_O, value, 4, HZ);

		if (rc < 0) {
			printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
		    		   "reconnect or reload driver.\n", proc, rc);
			return rc;
		}
		usbvision->curwidth = usbvision->stretch_width * UsbWidth;
		usbvision->curheight = usbvision->stretch_height * UsbHeight;
	}
	
	if (usbvision->videoformat == VIDEOFORMAT_YUV422) {
		framerate = (usbvision->iso_packet_len * 1000) / (UsbWidth * UsbHeight * 2);
	}
	else if (usbvision->videoformat == VIDEOFORMAT_YUV420) {
		framerate = (usbvision->iso_packet_len * 1000) / ((UsbWidth * UsbHeight * 12) / 8);
	}
	else {
		framerate = 31;
	}

	if ((usbvision->vchan.norm == VIDEO_MODE_PAL) ||
	    (usbvision->vchan.norm == VIDEO_MODE_SECAM)) {
 		frame_drop_factor = framerate * 32 / 25 - 1;
	}
	else if (usbvision->vchan.norm == VIDEO_MODE_NTSC) {
		frame_drop_factor = framerate * 32 / 30 - 1;
	}

	RESTRICT_TO_RANGE(frame_drop_factor, FRAMERATE_MIN, FRAMERATE_MAX);
	
	DEBUG(printk(KERN_DEBUG "%sframe rate %d fps\n", proc, framerate);) 
			
	frame_drop_factor = FRAMERATE_MAX; 	// We can allow the maximum here, because dropping is controlled 
										// by BUF_THR automatically
	
	return usbvision_write_reg(usbvision, USBVISION_FRM_RATE, frame_drop_factor);
}


/*
 * usbvision_set_compress_params()
 *
 */

static int usbvision_set_compress_params(struct usb_usbvision *usbvision)
{
	static const char proc[] = "usbvision_set_compresion_params: ";
	int rc;
	unsigned char value[6];

	value[0] = 0x02; //by hec 0x0F;	// Intra-Compression cycle
	value[1] = 0x01;    // Reg.45 Strip-Size
	value[2] = 0x01;    // Reg.46 Force intra mode on all new frames
	value[3] = 0x00;    // Reg.47 FORCE_UP <- 0 normal operation (not force)
	value[4] = 0xA2;    // Reg.48 BUF_THR I'm not sure if this does something in not compressed mode.
//      value[5] = 0x00;    // Reg.49 DVI_YUV This has nothing to do with compression

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;

	rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0,
			     (__u16) USBVISION_INTRA_CYC, value, 5, HZ);

	if (rc < 0) {
		printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
		return rc;
	}
	
	
	value[0] = 0x04;	// PCM Threshold 1
	value[1] = 0x04;	// PCM Threshold 2
	value[2] = 0x02;	// Distorsion Threshold d7-d0
	value[3] = 0x00;	// Distorsion Threshold d11-d8
	value[4] = 0x00;	// Max Distorsion d7-d0
	value[5] = 0x00;	// Max Distorsion d8

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;

	rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0,
			     (__u16) USBVISION_PCM_THR1, value, 6, HZ);

	if (rc < 0) {
		printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
		return rc;
	}


	return rc;

}

/*
 * usbvision_set_input()
 *
 * Set the input (saa7111) size x y and other misc input params
 * I've no idea if this parameters are right
 *
 */

static int usbvision_set_input(struct usb_usbvision *usbvision)
{
	static const char proc[] = "usbvision_set_input: ";
	int rc;
	unsigned char value[8];

        if (usbvision_device_data[usbvision->DevModel].Vin_Reg1 >= 0) {
                value[0]=usbvision_device_data[usbvision->DevModel].Vin_Reg1 & 0xff;
		rc = usbvision_write_reg(usbvision,USBVISION_VIN_REG1,value[0]);
		if (rc < 0) {
		        printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
			       "reconnect or reload driver.\n", proc, rc);
			return rc;
		}
	}

	if (usbvision_device_data[usbvision->DevModel].Vin_Reg2 >= 0) {
	        value[0]=usbvision_device_data[usbvision->DevModel].Vin_Reg2 & 0xff;
		rc = usbvision_write_reg(usbvision,USBVISION_VIN_REG2,value[0]);
		if (rc < 0) {
		        printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
			       "reconnect or reload driver.\n", proc, rc);
			return rc;
		}
	}

	if (usbvision->vchan.norm == VIDEO_MODE_PAL) {
		value[0] = 0xC0;
		value[1] = 0x02;	//0x02C0 -> 704 Input video line length
		value[2] = 0x20;
		value[3] = 0x01;	//0x0120 -> 288 Input video n. of lines
		value[4] = 0x60;
		value[5] = 0x00;	//0x0060 -> 96 Input video h offset
		value[6] = 0x16;
		value[7] = 0x00;	//0x0016 -> 22 Input video v offset
	} else if (usbvision->vchan.norm == VIDEO_MODE_SECAM) {
		value[0] = 0xC0;
		value[1] = 0x02;	//0x02C0 -> 704 Input video line length
		value[2] = 0x20;
		value[3] = 0x01;	//0x0120 -> 288 Input video n. of lines
		value[4] = 0x01;
		value[5] = 0x00;	//0x0001 -> 01 Input video h offset
		value[6] = 0x01;
		value[7] = 0x00;	//0x0001 -> 01 Input video v offset
	} else {	/* VIDEO_MODE_NTSC */
		value[0] = 0xD0;
		value[1] = 0x02;	//0x02D0 -> 720 Input video line length
		value[2] = 0xF0;
		value[3] = 0x00;	//0x00F0 -> 240 Input video number of lines
		value[4] = 0x50;
		value[5] = 0x00;	//0x0050 -> 80 Input video h offset
		value[6] = 0x10;
		value[7] = 0x00;	//0x0010 -> 16 Input video v offset
	}

	if (usbvision_device_data[usbvision->DevModel].X_Offset >= 0) {
	        value[4]=usbvision_device_data[usbvision->DevModel].X_Offset & 0xff;
		value[5]=(usbvision_device_data[usbvision->DevModel].X_Offset & 0xff00) >> 8;
	}

	if (usbvision_device_data[usbvision->DevModel].Y_Offset >= 0) {
	        value[6]=usbvision_device_data[usbvision->DevModel].Y_Offset & 0xff;
		value[7]=(usbvision_device_data[usbvision->DevModel].Y_Offset & 0xff00) >> 8;
	}

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;

	rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,	/* USBVISION specific code */
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0,
			     (__u16) USBVISION_LXSIZE_I, value, 8, HZ);

	if (rc < 0) {
		printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
		return rc;
	}

	if(usbvision_device_data[usbvision->DevModel].Dvi_yuv >= 0){
	        return (usbvision_write_reg(usbvision, USBVISION_DVI_YUV, usbvision_device_data[usbvision->DevModel].Dvi_yuv & 0xff));
	}
	else if(usbvision_device_data[usbvision->DevModel].Codec == CODEC_SAA7113)
        {
        	/* This changes as the fine sync control changes. Further investigation necessary */
		return (usbvision_write_reg(usbvision, USBVISION_DVI_YUV, 0x06));
        }
        else
        {
		/* U comes after V, Ya comes after U/V, Yb comes after Yb */
		return (usbvision_write_reg(usbvision, USBVISION_DVI_YUV, 0x00));
        }

}

/*
 * usbvision_set_dram_settings()
 *
 * Set the buffer address needed by the usbvision dram to operate
 * This values has been taken with usbsnoop.
 *
 */

static int usbvision_set_dram_settings(struct usb_usbvision *usbvision)
{
	static const char proc[] = "usbvision_set_dram_settings: ";
	int rc;
	unsigned char value[8];
	
	if (usbvision->videoformat == VIDEOFORMAT_COMPRESS) {
		value[0] = 0x42;
		value[1] = 0x71;
		value[2] = 0xff;
		value[3] = 0x00;
		value[4] = 0x98;
		value[5] = 0xe0;
		value[6] = 0x71;
		value[7] = 0xff;
		// UR:  0x0E200-0x3FFFF = 204288 Byte
		// FDL: 0x00000-0x0E099 =  57498 Byte
		// VDW: 0x0E3FF-0x3FFFF
		printk(KERN_DEBUG "%scompressed\n", proc);
	}
	else {
		value[0] = 0x42;
		value[1] = 0x00;
		value[2] = 0xff;
		value[3] = 0x00;
		value[4] = 0x00;
		value[5] = 0x00;
		value[6] = 0x00;
		value[7] = 0xff;
	}
	/* These are the values of the address of the video buffer,
	 * they have to be loaded into the USBVISION_DRM_PRM1-8
	 *
	 * Start address of video output buffer for read: 	drm_prm1-2 -> 0x00000
	 * End address of video output buffer for read: 	drm_prm1-3 -> 0x1ffff
	 * Start address of video frame delay buffer: 		drm_prm1-4 -> 0x20000
	 *    Only used in compressed mode
	 * End address of video frame delay buffer: 		drm_prm1-5-6 -> 0x3ffff
	 *    Only used in compressed mode
	 * Start address of video output buffer for write: 	drm_prm1-7 -> 0x00000
	 * End address of video output buffer for write: 	drm_prm1-8 -> 0x1ffff
	 */

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return 0;

	rc = usb_control_msg(usbvision->dev, usb_sndctrlpipe(usbvision->dev, 1),
			     USBVISION_OP_CODE,	/* USBVISION specific code */
			     USB_DIR_OUT | USB_TYPE_VENDOR |
			     USB_RECIP_ENDPOINT, 0,
			     (__u16) USBVISION_DRM_PRM1, value, 8, HZ);

	if (rc < 0) {
		printk(KERN_ERR "%sERROR=%d. USBVISION stopped - "
		       "reconnect or reload driver.\n", proc, rc);
		return rc;
	}

	/* Restart the video buffer logic */
	if ((rc = usbvision_write_reg(usbvision, USBVISION_DRM_CONT, USBVISION_RES_UR |
				   USBVISION_RES_FDL | USBVISION_RES_VDW)) < 0)
		return rc;
	rc = usbvision_write_reg(usbvision, USBVISION_DRM_CONT, 0x00);

	return rc;
}

/*
 * usbvision_power_on()
 *
 * Power on the device, enables suspend-resume logic 
 * &  reset the isoc End-Point
 *
 */

static int usbvision_power_on(struct usb_usbvision *usbvision)
{
	usbvision_write_reg(usbvision, USBVISION_PWR_REG, USBVISION_SSPND_EN);
	usbvision_write_reg(usbvision, USBVISION_PWR_REG,
			 USBVISION_SSPND_EN | USBVISION_RES2);
	/* Set input format expected from decoder, should this be moved to usbvision_set_input() ?? */
        if(usbvision_device_data[usbvision->DevModel].Codec == CODEC_SAA7113)
        {
		/* SAA7113 uses 8 bit output */
		usbvision_write_reg(usbvision, USBVISION_VIN_REG1, USBVISION_8_422_SYNC);
        }
        else
        {
		/* I'm sure only about d2-d0 [010] 16 bit 4:2:2 usin sync pulses
		 * as that is how saa7111 is configured */
		usbvision_write_reg(usbvision, USBVISION_VIN_REG1, USBVISION_16_422_SYNC);
		/* | USBVISION_VSNC_POL | USBVISION_VCLK_POL);*/
        }

	usbvision_write_reg(usbvision, USBVISION_PWR_REG,
			 USBVISION_SSPND_EN | USBVISION_PWR_VID);
	printk(KERN_DEBUG "usbvision_power_on\n");
	return (usbvision_write_reg
		(usbvision, USBVISION_PWR_REG,
		 USBVISION_SSPND_EN | USBVISION_PWR_VID | USBVISION_RES2));
}

/*
 * usbvision_begin_streaming()
 * Sure you have to put bit 7 to 0, if not incoming frames are droped, but no
 * idea about the rest
 */

static int usbvision_begin_streaming(struct usb_usbvision *usbvision)
{

	return (usbvision_write_reg(usbvision, USBVISION_VIN_REG2, USBVISION_NOHVALID));

	return 0;
}

/*
 * usbvision_restart_isoc()
 * Not sure yet if touching here PWR_REG make loose the config
 */

static int usbvision_restart_isoc(struct usb_usbvision *usbvision)
{
	int ret;

	if (
	    (ret =
	     usbvision_write_reg(usbvision, USBVISION_PWR_REG,
			      USBVISION_SSPND_EN | USBVISION_PWR_VID)) < 0)
		return ret;
	if (
	    (ret =
	     usbvision_write_reg(usbvision, USBVISION_PWR_REG,
			      USBVISION_SSPND_EN | USBVISION_PWR_VID |
			      USBVISION_RES2)) < 0)
		return ret;
	if (
	    (ret =
	     usbvision_write_reg(usbvision, USBVISION_VIN_REG2,
			      USBVISION_KEEP_BLANK|USBVISION_NOHVALID)) < 0) return ret;

	/* TODO: schedule timeout */
	while ((usbvision_read_reg(usbvision, USBVISION_STATUS_REG) && 0x01) != 1);

	return 0;
}

static int usbvision_audio_on(struct usb_usbvision *usbvision)
{
	if (usbvision_write_reg(usbvision, USBVISION_IOPIN_REG, usbvision->AudioChannel) < 0) {
		printk(KERN_ERR "usbvision_audio_on: can't wirte reg\n");
		return -1;
	}
	DEBUG(printk(KERN_DEBUG "usbvision_audio_on: channel %d\n", usbvision->AudioChannel));
	usbvision->AudioMute = 0;
	return 0;
}

static int usbvision_audio_mute(struct usb_usbvision *usbvision)
{
	if (usbvision_write_reg(usbvision, USBVISION_IOPIN_REG, 0x03) < 0) {
		printk(KERN_ERR "usbvision_audio_mute: can't wirte reg\n");
		return -1;
	}
	DEBUG(printk(KERN_DEBUG "usbvision_audio_mute: audio mute\n"));
	usbvision->AudioMute = 1;
	return 0;
}

static int usbvision_audio_off(struct usb_usbvision *usbvision)
{
	if (usbvision_write_reg(usbvision, USBVISION_IOPIN_REG, USBVISION_AUDIO_MUTE) < 0) {
		printk(KERN_ERR "usbvision_audio_off: can't wirte reg\n");
		return -1;
	}
	DEBUG(printk(KERN_DEBUG "usbvision_audio_off: audio off\n"));
	usbvision->AudioMute = 0;
	usbvision->AudioChannel = USBVISION_AUDIO_MUTE;
	return 0;
}

static int usbvision_set_audio(struct usb_usbvision *usbvision, int AudioChannel)
{
	if (!usbvision->AudioMute) {
		if (usbvision_write_reg(usbvision, USBVISION_IOPIN_REG, AudioChannel) < 0) {
			printk(KERN_ERR "usbvision_set_audio: can't write iopin register for audio switching\n");
			return -1;
		}
	}
	DEBUG(printk(KERN_DEBUG "usbvision_set_audio: channel %d\n", AudioChannel));
	usbvision->AudioChannel = AudioChannel;
	return 0;
}

static int usbvision_setup(struct usb_usbvision *usbvision)
{
	usbvision_set_video_format(usbvision, videoformat);
	usbvision_set_dram_settings(usbvision);
	usbvision_set_compress_params(usbvision);
	usbvision_set_input(usbvision);
	usbvision_set_output(usbvision, MAX_USB_WIDTH, MAX_USB_HEIGHT);
	usbvision_restart_isoc(usbvision);
	usbvision_begin_streaming(usbvision);

	/* cosas del PCM */

	return USBVISION_IS_OPERATIONAL(usbvision);
}


/*
 * usbvision_init_isoc()
 *
 */
static int usbvision_init_isoc(struct usb_usbvision *usbvision)
{
	IRQDEBUG(static const char proc[]="usbvision_init_isoc: ";)
	struct usb_device *dev = usbvision->dev;
	int i, err;

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return -EFAULT;

	usbvision->curframe = -1;
	usbvision->cursbuf = 0;
	scratch_reset(usbvision);

	/* Alternate interface 1 is is the biggest frame size */
	i = usb_set_interface(dev, usbvision->iface, usbvision->ifaceAltActive);
	if (i < 0) {
		printk(KERN_ERR "usb_set_interface error\n");
		usbvision->last_error = i;
		return -EBUSY;
	}

	/* We double buffer the Iso lists */

	for (i = 0; i < USBVISION_NUMSBUF; i++) {
		int j, k;
		struct urb *urb;

		urb = usb_alloc_urb(FRAMES_PER_DESC);
		if (urb == NULL) {
			printk(KERN_ERR
			       "usbvision_init_isoc: usb_init_isoc() failed.\n");
			return -ENOMEM;
		}
		usbvision->sbuf[i].urb = urb;
		urb->dev = dev;
		urb->context = usbvision;
		urb->pipe = usb_rcvisocpipe(dev, usbvision->video_endp);
		urb->transfer_flags = USB_ISO_ASAP;
		urb->transfer_buffer = usbvision->sbuf[i].data;
		urb->complete = usbvision_isoc_irq;
		urb->number_of_packets = FRAMES_PER_DESC;
		urb->transfer_buffer_length =
		    usbvision->iso_packet_len * FRAMES_PER_DESC;
		for (j = k = 0; j < FRAMES_PER_DESC; j++,
		     k += usbvision->iso_packet_len) {
			urb->iso_frame_desc[j].offset = k;
			urb->iso_frame_desc[j].length =
			    usbvision->iso_packet_len;
		}
	}

	/* Link URBs into a ring so that they invoke each other infinitely */
	for (i = 0; i < USBVISION_NUMSBUF; i++) {
		if ((i + 1) < USBVISION_NUMSBUF)
			usbvision->sbuf[i].urb->next =
			    usbvision->sbuf[i + 1].urb;
		else
			usbvision->sbuf[i].urb->next = usbvision->sbuf[0].urb;
	}

	/* Submit all URBs */
	for (i = 0; i < USBVISION_NUMSBUF; i++) {
		err = usb_submit_urb(usbvision->sbuf[i].urb);
		if (err)
			printk(KERN_ERR
			       "usbvision_init_isoc: usb_run_isoc(%d) ret %d\n",
			       i, err);
	}


	usbvision->streaming = 1;
	IRQDEBUG(printk(KERN_DEBUG "%sstreaming=1 usbvision->video_endp=$%02x\n", 
			proc, usbvision->video_endp);)
	return 0;
}

/*
 * usbvision_stop_isoc()
 *
 * This procedure stops streaming and deallocates URBs. Then it
 * activates zero-bandwidth alt. setting of the video interface.
 *
 */
static void usbvision_stop_isoc(struct usb_usbvision *usbvision)
{
	static const char proc[] = "usbvision_stop_isoc: ";
	int i, j;

	if (!usbvision->streaming || (usbvision->dev == NULL))
		return;

	/* Unschedule all of the iso td's */
	for (i = 0; i < USBVISION_NUMSBUF; i++) {
		j = usb_unlink_urb(usbvision->sbuf[i].urb);
		if (j < 0)
			printk(KERN_ERR "%susb_unlink_urb() error %d.\n", proc, j);
	}
	DEBUG(printk(KERN_DEBUG "%sstreaming=0\n", proc);)
	usbvision->streaming = 0;

	/* Delete them all */
	for (i = 0; i < USBVISION_NUMSBUF; i++)
		usb_free_urb(usbvision->sbuf[i].urb);

	if (!usbvision->remove_pending) {

		/* Set packet size to 0 */
		j = usb_set_interface(usbvision->dev, usbvision->iface,
				      usbvision->ifaceAltInactive);
		if (j < 0) {
			printk(KERN_ERR "%s: usb_set_interface() error %d.\n", proc, j);
			usbvision->last_error = j;
		}
	}
}

/*
 * usbvision_new_frame()
 *
 */
static int usbvision_new_frame(struct usb_usbvision *usbvision, int framenum)
{
	struct usbvision_frame *frame;
	int n; //byhec  , width, height;

	/* If we're not grabbing a frame right now and the other frame is */
	/*  ready to be grabbed into, then use it instead */
	if (usbvision->curframe != -1)
		return 0;

	n = (framenum - 1 + USBVISION_NUMFRAMES) % USBVISION_NUMFRAMES;
	if (usbvision->frame[n].grabstate == FRAME_READY)
		framenum = n;

	frame = &usbvision->frame[framenum];

	frame->grabstate = FRAME_GRABBING;
	frame->scanstate = STATE_SCANNING;
	frame->scanlength = 0;	/* Accumulated in usbvision_parse_data() */
	usbvision->curframe = framenum;

	/*
	 * Normally we would want to copy previous frame into the current one
	 * before we even start filling it with data; this allows us to stop
	 * filling at any moment; top portion of the frame will be new and
	 * bottom portion will stay as it was in previous frame. If we don't
	 * do that then missing chunks of video stream will result in flickering
	 * portions of old data whatever it was before.
	 *
	 * If we choose not to copy previous frame (to, for example, save few
	 * bus cycles - the frame can be pretty large!) then we have an option
	 * to clear the frame before using. If we experience losses in this
	 * mode then missing picture will be black (flickering).
	 *
	 * Finally, if user chooses not to clean the current frame before
	 * filling it with data then the old data will be visible if we fail
	 * to refill entire frame with new data.
	 */
	if (!(flags & FLAGS_SEPARATE_FRAMES)) {
		/* This copies previous frame into this one to mask losses */
		memmove(frame->data, usbvision->frame[1 - framenum].data,
			MAX_FRAME_SIZE);
	} else {
		if (flags & FLAGS_CLEAN_FRAMES) {
			/*This provides a "clean" frame but slows things down */
			memset(frame->data, 0, MAX_FRAME_SIZE);
		}
	}
	return 0;
}

static int usbvision_muxsel(struct usb_usbvision *usbvision, int channel, int norm)
{
	int mode[] = {0, 1, 7}; //modes of saa7111  fix me (this is only for Hauppauge WinTV)
	int audio[]= {1, 0, 0};	//channel 0 is TV with audiochannel 1 (tuner mono)
							//channel 1 is Composite with audio channel 0 (line in)
							//channel 2 is S-Video with audio channel 0 (line in)

	RESTRICT_TO_RANGE(channel, 0, usbvision->video_inputs);
	/* set the new video norm */
	if (usbvision->vchan.norm != norm) {
		int video_command = norm; //VIDEO_MODE_AUTO;
		call_i2c_clients(usbvision, DECODER_SET_NORM, &video_command);	//set norm in video-codec
		call_i2c_clients(usbvision, VIDIOCSCHAN, &usbvision->vchan);	//set norm in tuner
		video_command = 1;
		call_i2c_clients(usbvision, DECODER_ENABLE_OUTPUT,
				 &video_command);
		usbvision->vchan.norm = norm;
	}

	/* set the new channel */
	call_i2c_clients(usbvision, DECODER_SET_INPUT, &mode[channel]);
	usbvision->channel = channel;
	usbvision_set_audio(usbvision, audio[channel]);
	return 0;
}


/*
 * usbvision_open()
 *
 * This is part of Video 4 Linux API. The driver can be opened by one
 * client only (checks internal counter 'usbvision->user'). The procedure
 * then allocates buffers needed for video processing.
 *
 */
static int usbvision_open(struct video_device *dev, int flags)
{
	struct usb_usbvision *usbvision = (struct usb_usbvision *) dev;
	const int sb_size = FRAMES_PER_DESC * usbvision->iso_packet_len;
	/* sb_size should be FR_PER_DESC * max_packet_len if wanna change 
	 * packet len dinamically for bw management
	 */
	int i, err = 0;

	MOD_INC_USE_COUNT;
	
	if(PowerOnAtOpen) {
		usbvision_power_on(usbvision);
		usbvision_init_i2c(usbvision);
	}

	IODEBUG(printk(KERN_DEBUG "usbvision_open: usbvision = %p\n", usbvision);)
	down(&usbvision->lock);

	if (usbvision->user)
		err = -EBUSY;
	else {
		/* Clean pointers so we know if we allocated something */
		for (i = 0; i < USBVISION_NUMSBUF; i++)
			usbvision->sbuf[i].data = NULL;

		/* Allocate memory for the frame buffers */
		usbvision->fbuf_size = USBVISION_NUMFRAMES * MAX_FRAME_SIZE;
		usbvision->fbuf = rvmalloc(usbvision->fbuf_size);
		usbvision->scratch = kmalloc(scratch_buf_size, GFP_KERNEL);
		scratch_reset(usbvision);
		if ((usbvision->fbuf == NULL) || (usbvision->scratch == NULL)) {
			printk(KERN_ERR "usbvision_open: unable to allocate memory for frame buffer and scratch\n");
			err = -ENOMEM;
		}
		else {
			/* Allocate all buffers */
			for (i = 0; i < USBVISION_NUMFRAMES; i++) {
				usbvision->frame[i].grabstate = FRAME_UNUSED;
				usbvision->frame[i].data = usbvision->fbuf +
				    i * MAX_FRAME_SIZE;
				/*
				 * Set default sizes in case IOCTL
				 * (VIDIOCMCAPTURE)
				 * is not used (using read() instead).
				 */
				usbvision->stretch_width = 1;
				usbvision->stretch_height = 1;
				usbvision->frame[i].width = usbvision->curwidth;
				usbvision->frame[i].height = usbvision->curheight;
				usbvision->frame[i].bytes_read = 0;
			}
			if (dga) { //set default for DGA
				usbvision->overlay_frame.grabstate = FRAME_UNUSED;
				usbvision->overlay_frame.scanstate = STATE_SCANNING;
				usbvision->overlay_frame.data = NULL;
				usbvision->overlay_frame.width = usbvision->curwidth;
				usbvision->overlay_frame.height = usbvision->curheight;
				usbvision->overlay_frame.bytes_read = 0;
			}
			
			for (i = 0; i < USBVISION_NUMSBUF; i++) {
				usbvision->sbuf[i].data =
				    kmalloc(sb_size, GFP_KERNEL);
				if (usbvision->sbuf[i].data == NULL) {
					err = -ENOMEM;
					break;
				}
			}
		}
		if (err) {
			/* Have to free all that memory */
			if (usbvision->fbuf != NULL) {
				rvfree(usbvision->fbuf, usbvision->fbuf_size);
				usbvision->fbuf = NULL;
			}
			if (usbvision->scratch != NULL) {
				kfree(usbvision->scratch);
				usbvision->scratch = NULL;
			}
			for (i = 0; i < USBVISION_NUMSBUF; i++) {
				if (usbvision->sbuf[i].data != NULL) {
					kfree(usbvision->sbuf[i].data);
					usbvision->sbuf[i].data = NULL;
				}
			}
			if (PowerOnAtOpen) {
				i2c_usb_del_bus(&usbvision->i2c_adap);
				usbvision_power_off(usbvision);
				usbvision->initialized = 0;
			}
		}
	}

	/* If so far no errors then we shall start the camera */
	if (!err) {
		/* Send init sequence only once, it's large! */
		if (!usbvision->initialized) {
			int setup_ok = 0;
			setup_ok = usbvision_setup(usbvision);
			if (setup_ok)
				usbvision->initialized = 1;
			else
				err = -EBUSY;
		}
		if (!err) {
			usbvision_begin_streaming(usbvision);
			err = usbvision_init_isoc(usbvision);
			usbvision->user++;
		}
	}
	up(&usbvision->lock);
	if (err)
		MOD_DEC_USE_COUNT;
	return err;
}

/*
 * usbvision_close()
 *
 * This is part of Video 4 Linux API. The procedure
 * stops streaming and deallocates all buffers that were earlier
 * allocated in usbvision_open().
 *
 */
static void usbvision_close(struct video_device *dev)
{
	struct usb_usbvision *usbvision = (struct usb_usbvision *) dev;
	int i;

	down(&usbvision->lock);

	usbvision_audio_off(usbvision);
	usbvision_restart_isoc(usbvision);
	usbvision_stop_isoc(usbvision);
//      usbvision->initialized=0;

	rvfree(usbvision->fbuf, usbvision->fbuf_size);
	kfree(usbvision->scratch);
	for (i = 0; i < USBVISION_NUMSBUF; i++)
		kfree(usbvision->sbuf[i].data);

	usbvision->user--;

	if (usbvision->remove_pending) {
		printk(KERN_INFO "usbvision_close: Final disconnect.\n");
		usbvision_release(usbvision);
	}
	
	if (PowerOnAtOpen) {
		i2c_usb_del_bus(&usbvision->i2c_adap);
		usbvision_power_off(usbvision);
		usbvision->initialized = 0;
	}
	
	up(&usbvision->lock);
	
	IODEBUG(printk(KERN_DEBUG "usbvision_close: usbvision = %p\n", usbvision);)

	MOD_DEC_USE_COUNT;
}

static int usbvision_init_done(struct video_device *dev)
{
	return 0;
}

static long
usbvision_write(struct video_device *dev, const char *buf,
	     unsigned long count, int noblock)
{
	return -EINVAL;
}

/*
 * usbvision_ioctl()
 *
 * This is part of Video 4 Linux API. The procedure handles ioctl() calls.
 *
 */
static int
usbvision_ioctl(struct video_device *dev, unsigned int cmd, void *arg)
{
	static const char proc[] = "usbvision_ioctl: ";
	struct usb_usbvision *usbvision = (struct usb_usbvision *) dev;

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return -EFAULT;

	switch (cmd) {
	case VIDIOCGCAP:
	{
		if (copy_to_user(arg, &usbvision->vcap,
				 sizeof(usbvision->vcap)))
			    return -EFAULT;
		return 0;
	}
	case VIDIOCGCHAN:
	{
		int chan;
		struct video_channel v;

		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		v.flags = usbvision->vchan.flags;
		v.tuners = usbvision->vchan.tuners;
		v.type = usbvision->vchan.type;
		v.norm = usbvision->vchan.norm;
		if (v.channel >= usbvision->video_inputs)
			return -EINVAL;
		if (usbvision->have_tuner) {
			chan = v.channel;
		}
		else {
			chan = v.channel + 1; //skip Television string
		}
		switch(chan) {
			case 0:
				strcpy(v.name, "Television");
				v.flags |= VIDEO_VC_TUNER;
				v.type = VIDEO_TYPE_TV;
				v.tuners = usbvision->have_tuner;
				break;
			case 1:
				strcpy(v.name, "Composite 1");
				break;
			case 2:
				strcpy(v.name, "S-Video");
				break;			
		}

		if (copy_to_user(arg, &v, sizeof(v)))
			return -EFAULT;
		
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGCHAN name=%s\n", proc, v.name);)
		return 0;
	}
	case VIDIOCSCHAN:
	{
		struct video_channel v;
		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;

		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSCHAN channel=%d\n", proc, v.channel);)

		if (v.channel > usbvision->video_inputs)
			return -EINVAL;

		usbvision->vchan.channel = v.channel;
		//usbvision->vchan.norm = v.norm; <- this will be done in usbvision_muxsel()
		down(&usbvision->lock);
		
		usbvision_muxsel(usbvision, v.channel, v.norm);
		usbvision_set_input(usbvision);
		usbvision_set_output(usbvision, usbvision->curwidth, usbvision->curheight); 
		up(&usbvision->lock);
		
		return 0;
	}
	case VIDIOCGPICT:
	{
		if (copy_to_user(arg, &usbvision->vpic,
				 sizeof(usbvision->vpic)))
			    return -EFAULT;
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGPICT depth=%d, palette=%d\n", proc, 
					usbvision->vpic.depth, usbvision->vpic.palette);)
		return 0;
	}
	case VIDIOCSPICT:
	{
		struct video_picture p;
		if (copy_from_user(&p, arg, sizeof(p)))
			return -EFAULT;
		call_i2c_clients(usbvision, DECODER_SET_PICTURE, &p);
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSPICT depth=%d, palette=%d\n",
					proc, p.depth, p.palette);)
		return 0;
	}
	case VIDIOCGMBUF:
	{
		struct video_mbuf vm;

		memset(&vm, 0, sizeof(vm));
		vm.size = MAX_FRAME_SIZE * 2;
		vm.frames = 2;
		vm.offsets[0] = 0;
		vm.offsets[1] = MAX_FRAME_SIZE;

		if (copy_to_user ((void *) arg, (void *) &vm, sizeof(vm)))
			return -EFAULT;

		return 0;
	}
	case VIDIOCMCAPTURE:
	{
		struct video_mmap vm;
		struct usbvision_frame *frame;

		if (copy_from_user ((void *) &vm, (void *) arg, sizeof(vm)))
			return -EFAULT;

		IODEBUG(printk(KERN_DEBUG "%sVIDIOCMCAPTURE frame=%d, size=%dx%d, format=%d\n",
			       proc, vm.frame, vm.width, vm.height, vm.format);)

		if ((vm.format >= USBVISION_SUPPORTED_PALETTES) || 
		    (usbvision_v4l_format[vm.format].supported == 0)) {	
		    printk(KERN_ERR "%sVIDIOCMCAPTURE format %d not supported\n", proc, vm.format);
			return -EINVAL;
		}
		else
			
		if (vm.frame >= USBVISION_NUMFRAMES)
			return -EINVAL;
		
		frame = &usbvision->frame[vm.frame];
		
		if (frame->grabstate == FRAME_GRABBING)
			return -EBUSY;

		usbvision_set_output(usbvision, vm.width, vm.height);

		frame->width = vm.width;
		frame->height = vm.height;
		frame->v4l_format = vm.format;
		/* Mark it as ready */
		frame->grabstate = FRAME_READY;

		return usbvision_new_frame(usbvision, vm.frame);
	}
	case VIDIOCSYNC:
	{
		int frame, ret = 0;

		DECLARE_WAITQUEUE(wait, current);

		if (copy_from_user ((void *) &frame, arg, sizeof(int)))
			return -EFAULT;

		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSYNC frame=%d, grabstate=%d\n",
			       proc, frame, usbvision->frame[frame].grabstate);)

		switch (usbvision->frame[frame].grabstate) {
		case FRAME_UNUSED:
			ret = -EINVAL;
			break;
		case FRAME_GRABBING:
			add_wait_queue(&usbvision->frame [frame].wq, &wait);
			current->state = TASK_INTERRUPTIBLE;
			while (usbvision->frame[frame].
			       grabstate == FRAME_GRABBING) {
				schedule();
				if (signal_pending(current)) {
					remove_wait_queue
						(&usbvision->frame[frame].wq,
						 &wait);
					current->state = TASK_RUNNING;
					return -EINTR;
				}
			}
			remove_wait_queue(&usbvision->frame [frame].wq, &wait);
			current->state = TASK_RUNNING;
		case FRAME_READY:
		case FRAME_ERROR:
		case FRAME_DONE:
			ret = (usbvision->frame[frame].grabstate == 
					FRAME_ERROR) ? -EIO : 0;
			usbvision->frame[frame].grabstate = FRAME_UNUSED;
			break;
		}

		usbvision->frame[frame].grabstate = FRAME_UNUSED;

		return ret;
	}
	case VIDIOCGFBUF:
	{
//		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGFBUF\n", proc);)
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGFBUF(%p,%d,%d,%d,%d)\n",
				proc, usbvision->vid_buf.base, 
				usbvision->vid_buf.width,usbvision->vid_buf.height,
				usbvision->vid_buf.depth,usbvision->vid_buf.bytesperline);)


		if (dga) {
			if (copy_to_user ((void *) arg, (void *) &usbvision->vid_buf,
					sizeof(usbvision->vid_buf))) {
				return -EFAULT;
			}
		}
		else {
			struct video_buffer vb;

			memset(&vb, 0, sizeof(vb));
			vb.base = NULL;	//dga not supported, not used
			if (copy_to_user ((void *) arg, (void *) &vb, sizeof(vb))) {
				return -EFAULT;
			}
		}		
		return 0;
	}
	case VIDIOCSFBUF:
	{
		struct video_buffer vb;
		int format_index;
		
		if (dga == 0) {
			return -EINVAL;
		}
	
		if(!capable(CAP_SYS_ADMIN) && !capable(CAP_SYS_ADMIN))
			return -EPERM;
		
		if (copy_from_user(&vb,arg,sizeof(vb)))
			return -EFAULT;
		
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSFBUF(%p,%d,%d,%d,%d)\n",
				proc, vb.base, vb.width,vb.height,vb.depth,vb.bytesperline);)
		
		for (format_index=0; format_index <= USBVISION_SUPPORTED_PALETTES; format_index++) {
			if (format_index == USBVISION_SUPPORTED_PALETTES) {
				return -EINVAL; // no matching video_format
			}
			if ((vb.depth == usbvision_v4l_format[format_index].depth) &&
			    (usbvision_v4l_format[format_index].supported)) {
				break; //found matching video_format
			}
		}
		
		if (vb.bytesperline<1)
			return -EINVAL;
		if (usbvision->overlay)
			return -EBUSY;
		
		down(&usbvision->lock);
		if (usbvision->overlay_base) {
			vfree(usbvision->overlay_base);
		}
		usbvision->overlay_base = ioremap((ulong)vb.base, vb.height * vb.bytesperline);
		
		memcpy(&usbvision->vid_buf, &vb, sizeof(vb));
		usbvision->vid_buf.base = vb.base;
		usbvision->overlay_frame.v4l_format = format_index;
		up(&usbvision->lock);
		return 0;
	}
	case VIDIOCGWIN:
	{
		struct video_window vw;
	
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGWIN\n", proc);)
		
		if (dga == 0) {
			return -EINVAL;
		}
	
		down(&usbvision->lock);
		vw.x      = usbvision->vid_win.x;
		vw.y      = usbvision->vid_win.y;
		vw.width  = usbvision->vid_win.width;
		vw.height = usbvision->vid_win.height;
		vw.chromakey= usbvision->vid_win.chromakey;
		vw.flags  = usbvision->vid_win.flags;
		up(&usbvision->lock);
		
		if (copy_to_user(arg, &vw, sizeof(vw))) {
			return -EFAULT;
		}
		break;
	 }
	 case VIDIOCSWIN:
	 {
		struct video_window vw;
		struct video_clip *vcp;
		int on;
		
		if (dga == 0) {
			return -EINVAL;
		}
		
		if (copy_from_user(&vw,arg,sizeof(vw)))
			return -EFAULT;
		
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSWIN(%d,%d,%d,%d,%x,%x,%d)\n", proc, 
				vw.x, vw.y, vw.width, vw.height, vw.chromakey, 
				vw.flags, vw.clipcount);)

		if (vw.flags)
			return -EINVAL;

		if (vw.clipcount>256)
			return -EDOM;   /* Too many! */

		/*
		 *      Do any clips.
		 */
		vcp = vmalloc(sizeof(struct video_clip)*(vw.clipcount+4));
		if (vcp==NULL)
			return -ENOMEM;
		if (vw.clipcount && copy_from_user(vcp,vw.clips,sizeof(struct video_clip)*vw.clipcount))
			return -EFAULT;

		on = usbvision->overlay;
		if (on)
			usbvision_cap(usbvision, 0);

		// strange, it seems xawtv sometimes calls us with 0
		// width and/or height. Ignore these values
		 
		if (vw.x == 0)
			vw.x = usbvision->vid_win.x;
		if (vw.y == 0)
			vw.y = usbvision->vid_win.y;

		// by now we are committed to the new data...
		down(&usbvision->lock);
		usbvision->vid_win.x = vw.x;
		usbvision->vid_win.y = vw.y;
		RESTRICT_TO_RANGE(vw.width, MIN_FRAME_WIDTH, MAX_FRAME_WIDTH);
		RESTRICT_TO_RANGE(vw.height, MIN_FRAME_HEIGHT, MAX_FRAME_HEIGHT);
		usbvision_set_output(usbvision, vw.width, vw.height);
		
		usbvision->vid_win.width = vw.width;
		usbvision->vid_win.height = vw.height;
		usbvision->overlay_frame.width = vw.width;
		usbvision->overlay_frame.height = vw.height;
		usbvision->vid_win.chromakey = vw.chromakey;
		usbvision->vid_win.flags = vw.flags;
		up(&usbvision->lock);


		/*
		 *      Impose display clips
		 */
		if (vw.x+vw.width > usbvision->vid_buf.width)
			usbvision_new_clip(&vw, vcp, usbvision->vid_buf.width-vw.x, 0,
					vw.width-1, vw.height-1);
		if (vw.y+vw.height > usbvision->vid_buf.height)
			usbvision_new_clip(&vw, vcp, 0, usbvision->vid_buf.height-vw.y, 
					vw.width-1, vw.height-1);

		/* built the requested clipping zones */
		usbvision_built_overlay(usbvision, vw.clipcount, vcp);
		vfree(vcp);

		/* if we were on, restart the video engine */
		if (on)
			usbvision_cap(usbvision, 1);
		break;
	}
	case VIDIOCKEY:
	{
		return 0;
	}
	case VIDIOCCAPTURE:
	{
		int v;
	
		if (dga == 0) {
			return -EINVAL;
		}
		
		if (get_user(v, (int *)arg)) {
			return -EFAULT;
		}
		
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCCAPTURE(%d)\n",proc, v);)

		if (v==0) {
			usbvision_cap(usbvision, 0);
		}
		else {
			// is VIDIOCSFBUF, VIDIOCSWIN done?
			if (usbvision->vid_buf.base  ==0 || 
			    usbvision->vid_win.width ==0 || 
			    usbvision->vid_win.height==0 ||
			    usbvision->overlay_base  ==0) {
				return -EINVAL;
			}
			usbvision_cap(usbvision, 1);
		}
		return 0;

	}
	case VIDIOCGTUNER:
	{
		struct video_tuner v;
		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;

		if (!usbvision->have_tuner || v.tuner)	// Only tuner 0
			return -EINVAL;

		strcpy(v.name, "Television");
		v.rangelow = 0;
		v.rangehigh = ~0;
		v.flags = VIDEO_TUNER_PAL | VIDEO_TUNER_NTSC |
			VIDEO_TUNER_SECAM;
		v.mode = usbvision->vchan.norm;
		v.signal = 0xFFFF;	/* unknown */

		call_i2c_clients(usbvision, cmd, &v);

		if (copy_to_user(arg, &v, sizeof(v)))
			return -EFAULT;

		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGTUNER\n",proc);)
		return 0;
	}
	case VIDIOCSTUNER:
	{
		struct video_tuner v;
		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		/* Only no or one tuner for now */
		if (!usbvision->have_tuner || v.tuner)
			return -EINVAL;

		/* and it only has certain valid modes */
		if (v.mode != VIDEO_MODE_PAL && v.mode != VIDEO_MODE_NTSC &&
		    v.mode != VIDEO_MODE_SECAM)
			return -EOPNOTSUPP;

		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSTUNER tuner=%d, mode=%d\n",proc, v.tuner, v.mode);)
		return usbvision_muxsel(usbvision, v.tuner, v.mode);

	}
	case VIDIOCGFREQ:
	{
		unsigned long v = usbvision->freq;
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCGFREQ freq=0x%lX\n",proc, v);)
		RIODEBUG(printk(KERN_DEBUG "usbvision_radio_ioctl: VIDIOCGFREQ freq=0x%lX\n", v);)
		if (copy_to_user(arg, &v, sizeof(v)))
			return -EFAULT;
		return 0;
	}
	case VIDIOCSFREQ:
	{
		unsigned long v;
		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		usbvision->freq = v;
		call_i2c_clients(usbvision, cmd, &v);
		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSFREQ freq=0x%lX\n",proc, v);)
		RIODEBUG(printk(KERN_DEBUG "usbvision_radio_ioctl: VIDIOCSFREQ freq=0x%lX\n", v);)
		return 0;
	}
	case VIDIOCGAUDIO:
	{
		struct video_audio v;
		memset(&v,0, sizeof(v));
		v.flags|=VIDEO_AUDIO_MUTABLE;
		v.mode=VIDEO_SOUND_MONO;
		v.volume=1;
		v.step=1;
		strcpy(v.name, "TV");
		if(copy_to_user(arg,&v, sizeof(v)))
			return -EFAULT;

		return 0;
	}
	case VIDIOCSAUDIO:
	{
		struct video_audio v;
		if(copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		if(v.audio)
			return -EINVAL;

		IODEBUG(printk(KERN_DEBUG "%sVIDIOCSAUDIO(flags:0x%x)\n",proc, v.flags);)
		
		if(v.flags&VIDEO_AUDIO_MUTE) {
			if (usbvision_audio_mute(usbvision))
				return -EFAULT;
		}
		else
			if (usbvision_audio_on(usbvision))
				return -EFAULT;
		return 0;
	}

	/* Should I do CGCAPTURE and CSCAPTURE */

	default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

static long
usbvision_read(struct video_device *dev, char *buf, unsigned long count,
	    int noblock)
{
	struct usb_usbvision *usbvision = (struct usb_usbvision *) dev;
	int frmx = -1;
	volatile struct usbvision_frame *frame;

	if (debug >= 1)
		printk(KERN_DEBUG "usbvision_read: %ld bytes, noblock=%d\n",
		       count, noblock);

	if (!USBVISION_IS_OPERATIONAL(usbvision) || (buf == NULL))
		return -EFAULT;

	//code for testing compression
	if (usbvision->videoformat == VIDEOFORMAT_COMPRESS) {
		usbvision->frame[0].v4l_format = VIDEO_PALETTE_GREY;
		usbvision->frame[1].v4l_format = VIDEO_PALETTE_GREY;
	}
	
	/* See if a frame is completed, then use it. */
	if (usbvision->frame[0].grabstate >= FRAME_DONE) /* _DONE or _ERROR */
		frmx = 0;
	else if (usbvision->frame[1].grabstate >= FRAME_DONE)/*_DONE or _ERROR*/
		frmx = 1;

	if (noblock && (frmx == -1))
		return -EAGAIN;

	/* If no FRAME_DONE, look for a FRAME_GRABBING state. */
	/* See if a frame is in process (grabbing), then use it. */
	if (frmx == -1) {
		if (usbvision->frame[0].grabstate == FRAME_GRABBING)
			frmx = 0;
		else if (usbvision->frame[1].grabstate == FRAME_GRABBING)
			frmx = 1;
	}

	/* If no frame is active, start one. */
	if (frmx == -1)
		usbvision_new_frame(usbvision, frmx = 0);

	frame = &usbvision->frame[frmx];
	
      restart:
	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return -EIO;
	while (frame->grabstate == FRAME_GRABBING) {
		interruptible_sleep_on((void *) &frame->wq);
		if (signal_pending(current))
			return -EINTR;
	}

	if (frame->grabstate == FRAME_ERROR) {
		frame->bytes_read = 0;
		if (usbvision_new_frame(usbvision, frmx))
			printk(KERN_ERR
			       "usbvision_read: usbvision_new_frame error\n");
		goto restart;
	}

	if (debug >= 1)
		printk(KERN_DEBUG
		       "usbvision_read: frmx=%d, bytes_read=%ld, scanlength=%ld\n",
		       frmx, frame->bytes_read, frame->scanlength);

	/* copy bytes to user space; we allow for partials reads */
	if ((count + frame->bytes_read) > frame->scanlength)
		count = frame->scanlength - frame->bytes_read;

	if (copy_to_user(buf, frame->data + frame->bytes_read, count))
		return -EFAULT;

	frame->bytes_read += count;
	if (debug >= 1)
		printk(KERN_DEBUG
		       "usbvision_read: {copy} count used=%ld, new bytes_read=%ld\n",
		       count, frame->bytes_read);

	if (frame->bytes_read >= frame->scanlength) {/*All data has been read */
		frame->bytes_read = 0;

		/* Mark it as available to be used again. */
		usbvision->frame[frmx].grabstate = FRAME_UNUSED;
		if (usbvision_new_frame(usbvision, frmx ? 0 : 1))
			printk(KERN_ERR
			       "usbvision_read: usbvision_new_frame returned error\n");
	}

	return count;
}

static int
usbvision_mmap(struct video_device *dev, const char *adr, unsigned long size)
{
	struct usb_usbvision *usbvision = (struct usb_usbvision *) dev;
	unsigned long start = (unsigned long) adr;
	unsigned long page, pos;

	if (!USBVISION_IS_OPERATIONAL(usbvision))
		return -EFAULT;

	if (size >
	    (((2 * MAX_FRAME_SIZE) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1)))
		    return -EINVAL;

	pos = (unsigned long) usbvision->fbuf;
	while (size > 0) {
		page = kvirt_to_pa(pos);
		if (remap_page_range(start, page, PAGE_SIZE, PAGE_SHARED))
			return -EAGAIN;

		start += PAGE_SIZE;
		pos += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}

	return 0;
}

static struct video_device usbvision_template = {
	name:           "USBVision USB Video",
	type:           VID_TYPE_TUNER | VID_TYPE_CAPTURE,
	hardware:       VID_HARDWARE_USBVISION,    
	open:           usbvision_open,
	close:          usbvision_close,
	read:           usbvision_read,
	write:          usbvision_write,
	ioctl:          usbvision_ioctl,
	mmap:           usbvision_mmap,
	initialize:     usbvision_init_done,
};


/* 
 * Here comes the stuff for radio on usbvision based devices
 *
 */

static int usbvision_radio_open(struct video_device *dev, int flags)
{
	struct usb_usbvision *usbvision = (struct usb_usbvision *)dev->priv;
	unsigned long v;
	int err = 0;

	RIODEBUG(printk(KERN_DEBUG "usbvision_radio_open\n");)
			
	down(&usbvision->lock);
	MOD_INC_USE_COUNT;
	
	if(PowerOnAtOpen) {
		usbvision_power_on(usbvision);
		usbvision_init_i2c(usbvision);
	}

	


	if (usbvision->user) {
		err = -EBUSY;
		if (PowerOnAtOpen) {
			i2c_usb_del_bus(&usbvision->i2c_adap);
			usbvision_power_off(usbvision);
			usbvision->initialized = 0;
		}
	}

	/* If so far no errors then we shall start the radio */
	if (!err) {
		usbvision->radio = 1;
		v = 400*16;
		call_i2c_clients(usbvision,VIDIOCSFREQ,&v);
		call_i2c_clients(usbvision,AUDC_SET_RADIO,&usbvision->tuner_type);
		usbvision_set_audio(usbvision, USBVISION_AUDIO_RADIO);
		usbvision->user++;
	}
	up(&usbvision->lock);
	if (err)
		MOD_DEC_USE_COUNT;
	return err;
}


static void usbvision_radio_close(struct video_device *dev)
{
	struct usb_usbvision *usbvision = (struct usb_usbvision *)dev->priv;

	RIODEBUG(printk(KERN_DEBUG "usbvision_radio_close\n");)
			
	down(&usbvision->lock);
	usbvision_audio_off(usbvision);
    usbvision->radio=0;
	usbvision->user--;

	if (usbvision->remove_pending) {
		printk(KERN_INFO "usbvision_radio_close: Final disconnect.\n");
		usbvision_release(usbvision);
	}
	
	if (PowerOnAtOpen) {
		i2c_usb_del_bus(&usbvision->i2c_adap);
		usbvision_power_off(usbvision);
		usbvision->initialized = 0;
	}
	
	up(&usbvision->lock);
	MOD_DEC_USE_COUNT;
}

static long usbvision_radio_write(struct video_device *dev, const char *buf, unsigned long count, int nonblock)
{
	RIODEBUG(printk(KERN_DEBUG "usbvision_radio_write\n");)
	return -EINVAL;
}


static long usbvision_radio_read(struct video_device *dev, char *buf, unsigned long count, int nonblock)
{
	RIODEBUG(printk(KERN_DEBUG "usbvision_radio_read\n");)
	return -EINVAL;
}


static int usbvision_radio_ioctl(struct video_device *dev, unsigned int cmd, void *arg)
{
	RIODEBUG(char proc[]="usbvision_radio_ioctl: ";)
	struct usb_usbvision *usbvision=(struct usb_usbvision *)dev->priv;
	switch (cmd) {	
	case VIDIOCGCAP:
	{
		struct video_capability v;
		strcpy(v.name,usbvision->vcap.name);
		v.type = VID_TYPE_TUNER;
		v.channels = 1;
		v.audios = 1;
		/* No we don't do pictures */
		v.maxwidth = 0;
		v.maxheight = 0;
		v.minwidth = 0;
		v.minheight = 0;
		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCGCAP\n",proc);)
		if (copy_to_user(arg, &v, sizeof(v)))
			return -EFAULT;
		return 0;
		break;
	}
	case VIDIOCGTUNER:
	{
		struct video_tuner v;
		if(copy_from_user(&v,arg,sizeof(v))!=0)
			return -EFAULT;
		if(v.tuner||usbvision->channel)	/* Only tuner 0 */
			return -EINVAL;
		strcpy(v.name, "Radio");
		/* japan:          76.0 MHz -  89.9 MHz
		   western europe: 87.5 MHz - 108.0 MHz
		   russia:         65.0 MHz - 108.0 MHz */
		v.rangelow=(int)(65*16);
		v.rangehigh=(int)(108*16);
		v.flags= 0;
		v.mode = 0;
		call_i2c_clients(usbvision,cmd,&v);
		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCGTUNER signal=%d\n",proc, v.signal);)
		if(copy_to_user(arg,&v,sizeof(v)))
			return -EFAULT;
		return 0;
	}
	case VIDIOCSTUNER:
	{
		struct video_tuner v;
		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCSTUNER\n",proc);)
		if(copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		/* Only channel 0 has a tuner */
		if(v.tuner || usbvision->channel)
			return -EINVAL;

		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCSTUNER ok\n",proc);)
		return 0;
	}
	case VIDIOCGAUDIO:
	{
		struct video_audio v;
		memset(&v,0, sizeof(v));
		call_i2c_clients(usbvision, cmd, &v);
		v.flags|=VIDEO_AUDIO_MUTABLE;
		v.volume=1;
		v.step=1;
		strcpy(v.name, "Radio");
		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCGAUDIO\n",proc);)
		if(copy_to_user(arg,&v, sizeof(v)))
			return -EFAULT;

		return 0;
	}
	case VIDIOCSAUDIO:
	{
		struct video_audio v;
		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCSAUDIO\n",proc);)
		if(copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		if(v.audio)
			return -EINVAL;

		if(v.flags&VIDEO_AUDIO_MUTE) {
			if (usbvision_audio_mute(usbvision))
				return -EFAULT;
		}
		else
			if (usbvision_audio_on(usbvision))
				return -EFAULT;
		RIODEBUG(printk(KERN_DEBUG "%sVIDIOCSAUDIO(flags:0x%x)\n",proc, v.flags);)
		return 0;
	}
	case VIDIOCGFREQ:
	case VIDIOCSFREQ:
		usbvision_ioctl((struct video_device *)usbvision,cmd,arg);
		break;
	default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

static struct video_device usbvision_radio_template=
{
	owner:		THIS_MODULE,
	name:		"USBVision USB Radio",
	type:		VID_TYPE_TUNER,
	hardware:	VID_HARDWARE_USBVISION, 		
	open:		usbvision_radio_open,
	close:		usbvision_radio_close,
	read:		usbvision_radio_read,	/* just returns -EINVAL */
	write:		usbvision_radio_write,	/* just returns -EINVAL */
	ioctl:		usbvision_radio_ioctl,
	minor:		-1,
};


static void usbvision_configure_video(struct usb_usbvision *usbvision)
{
	int model = usbvision->DevModel;
	
	if (usbvision == NULL)
		return;

	RESTRICT_TO_RANGE(init_brightness, 0, 255);
	RESTRICT_TO_RANGE(init_contrast, 0, 255);
	RESTRICT_TO_RANGE(init_color, 0, 255);
	RESTRICT_TO_RANGE(init_hue, 0, 255);
	RESTRICT_TO_RANGE(hue_correction, 0, 255);

	memset(&usbvision->vpic, 0, sizeof(usbvision->vpic));

	usbvision->vpic.colour = init_color << 8;
	usbvision->vpic.hue = init_hue << 8;
	usbvision->vpic.brightness = init_brightness << 8;
	usbvision->vpic.contrast = init_contrast << 8;
	usbvision->vpic.whiteness = 105 << 8;	/* This one isn't used */
	usbvision->vpic.depth = 24;
	usbvision->vpic.palette = VIDEO_PALETTE_RGB24;
	
	memset(&usbvision->vcap, 0, sizeof(usbvision->vcap));
	strcpy(usbvision->vcap.name, "USBVision USB Video");
	usbvision->vcap.type = VID_TYPE_CAPTURE | 
		(dga ? (VID_TYPE_CLIPPING | VID_TYPE_OVERLAY) : 0) |
		(usbvision->have_tuner ? VID_TYPE_TUNER : 0);
	
	usbvision->vcap.channels = usbvision_device_data[model].VideoChannels;
	usbvision->vcap.audios = usbvision_device_data[model].AudioChannels;
	usbvision->vcap.maxwidth = max_imgwidth;
	usbvision->vcap.maxheight = max_imgheight;
	usbvision->vcap.minwidth = min_imgwidth;
	usbvision->vcap.minheight = min_imgheight;

	memset(&usbvision->vchan, 0, sizeof(usbvision->vchan));
	usbvision->vchan.flags = VIDEO_VC_AUDIO;
	usbvision->vchan.tuners = 0;
	usbvision->vchan.channel = 0;
	usbvision->vchan.type = VIDEO_TYPE_CAMERA;
	usbvision->vchan.norm = usbvision_device_data[model].VideoNorm;

	usbvision->video_inputs = usbvision->vcap.channels;

	/* This should be here to make i2c clients to be able to register */
	usbvision_audio_off(usbvision);	//first switch off audio
	if (!PowerOnAtOpen) {
		usbvision_power_on(usbvision);	//and then power up the noisy tuner
		usbvision_init_i2c(usbvision);
	}
}

/*
 * usbvision_find_struct()
 *
 * This code searches the array of preallocated (static) structures
 * and returns index of the first one that isn't in use. Returns -1
 * if there are no free structures.
 *
 */
static int usbvision_find_struct(void)
{
	int i, u;

	for (u = 0; u < MAX_USBVISION; u++) {
		struct usb_usbvision *usbvision = &vdevs[u];
		if (!usbvision->usbvision_used) {	/* This one is free */
			usbvision->usbvision_used = 1;	/* In use now */
			for (i = 0; i < USBVISION_NUMFRAMES; i++) {
				init_waitqueue_head(&usbvision->frame[i].wq);
			}
			init_waitqueue_head(&usbvision->overlay_frame.wq);
			init_MUTEX(&usbvision->lock);	/* to 1 == available */
			usbvision->dev = NULL;
			memcpy(&usbvision->vdev, &usbvision_template, sizeof(usbvision_template));
			memcpy(&usbvision->rdev, &usbvision_radio_template, sizeof(usbvision_radio_template));
			
			return u;
		}
	}
	return -1;
}

/*
 * usbvision_probe()
 *
 * This procedure queries device descriptor and accepts the interface
 * if it looks like USBVISION video device
 *
 */
static void *usbvision_probe(struct usb_device *dev, unsigned int ifnum,
				const struct usb_device_id *id)
{
	struct usb_usbvision *usbvision = NULL;
	const struct usb_interface_descriptor *interface;
	const struct usb_endpoint_descriptor *endpoint;
	int devnum;
	int model;
	
	if (debug >= 1)
		printk(KERN_DEBUG "usbvision_probe(%p,%u.)\n", dev, ifnum);

	/* We don't handle multi-config */
	/*
	if (dev->descriptor.bNumConfigurations != 1)
		return NULL;
	*/

	/* Is it an USBVISION video dev? */
	model = 0;
	for(model = 0; usbvision_device_data[model].idVendor; model++) {
		if (dev->descriptor.idVendor  != usbvision_device_data[model].idVendor) {
			continue;
		}
		if (dev->descriptor.idProduct != usbvision_device_data[model].idProduct) {
			continue;
		}
		
		printk(KERN_INFO "usbvision: %s found\n", 
			usbvision_device_data[model].ModelString);
		break;
	}
	if (usbvision_device_data[model].idVendor == 0) {
		return NULL; //no matching device
	}
	
	if (usbvision_device_data[model].Interface >= 0) {
		interface = &dev->actconfig->interface[usbvision_device_data[model].Interface].altsetting[0];
	}
	else {
		interface = &dev->actconfig->interface[ifnum].altsetting[0];
	}
	/* The nt1004 have 3 or 4 endpoints
	if (interface->bNumEndpoints != 2) {
		printk(KERN_ERR
		       "USBVISION: interface %d. has %u. endpoints!\n", ifnum,
		       (unsigned) (interface->bNumEndpoints));
		return NULL;
	}
	*/

	endpoint = &interface->endpoint[1];
	if ((endpoint->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) != USB_ENDPOINT_XFER_ISOC) {
		printk(KERN_ERR "USBVISION: interface %d. has non-ISO endpoint!\n", ifnum);
		printk(KERN_ERR "USBVISION: Endpoint attribures %d\n", endpoint->bmAttributes);
		return NULL;
	}
	if ((endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK) == USB_DIR_OUT) {
		printk(KERN_ERR
		       "USBVISION: interface %d. has ISO OUT endpoint!\n",
		       ifnum);
		return NULL;
	}

	/* Code below may sleep, need to lock module while we are here */
	MOD_INC_USE_COUNT;

	devnum = usbvision_find_struct();
	if (devnum == -1) {
		printk(KERN_INFO "USBVISION USB driver: Too many devices!\n");
		MOD_DEC_USE_COUNT;
		return NULL;
	}
	usbvision = &vdevs[devnum];
	usbvision->nr = devnum;

	usbvision->have_tuner = usbvision_device_data[model].Tuner;
	if (usbvision->have_tuner) {
		usbvision->tuner_type = usbvision_device_data[model].TunerType;
	}

	down(&usbvision->lock);
	usbvision->DevModel = model;
	usbvision->remove_pending = 0;
	usbvision->last_error = 0;
	usbvision->dev = dev;
	usbvision->iface = ifnum;
	usbvision->ifaceAltInactive = 0;
	usbvision->ifaceAltActive = 1;
	usbvision->video_endp = endpoint->bEndpointAddress;

	usbvision->iso_packet_len = ((16 - usbvision_read_reg(usbvision, USBVISION_ALTER_REG)) * 64) - 1;
	printk(KERN_INFO "USBVision Maxium ISO Packet Length:%d\n", usbvision->iso_packet_len);

	usbbandwidth = (16 - usbvision_read_reg(usbvision, USBVISION_ALTER_REG)) * 0.5;
	printk(KERN_INFO "USBVision USB Bandwidth Usage: %dMbit/Sec\n", usbbandwidth);


	usbvision->user = 0;

	usbvision_configure_video(usbvision);
	up(&usbvision->lock);

	usbvision->vdev.priv = usbvision;
	if (video_register_device(&usbvision->vdev, VFL_TYPE_GRABBER, video_nr) < 0) {
		printk(KERN_ERR "video_register_device() failed\n");
		usbvision = NULL;
	}
	if (debug > 1)
		printk(KERN_DEBUG "video_register_device() successful\n");

	if (usbvision_device_data[model].Radio) {
		usbvision->rdev.priv = usbvision;	
		if (video_register_device(&usbvision->rdev, VFL_TYPE_RADIO, radio_nr) < 0) {
			printk(KERN_ERR "video_register_device(radio) failed\n");
			video_unregister_device(&usbvision->vdev);
			usbvision = NULL;
		}
		if (debug > 1)
			printk(KERN_DEBUG "video_register_device(radio) successful\n");
	}

	MOD_DEC_USE_COUNT;
	return usbvision;
}

/*
 * usbvision_release()
 *
 * This code does final release of struct usb_usbvision. This happens
 * after the device is disconnected -and- all clients closed their files.
 *
 */
static void usbvision_release(struct usb_usbvision *usbvision)
{
	if (usbvision_device_data[usbvision->DevModel].Radio) {
		video_unregister_device(&usbvision->rdev);
	}
	video_unregister_device(&usbvision->vdev);

	if (!PowerOnAtOpen) {
		i2c_usb_del_bus(&usbvision->i2c_adap);
		usbvision_power_off(usbvision);
	}
	
	if (debug > 0)
		printk(KERN_DEBUG
		       "usb_usbvision_release: Video unregistered.\n");
	usbvision->usbvision_used = 0;
	usbvision->initialized = 0;
}

/*
 * usbvision_disconnect()
 *
 * This procedure stops all driver activity, deallocates interface-private
 * structure (pointed by 'ptr') and after that driver should be removable
 * with no ill consequences.
 *
 */
static void usbvision_disconnect(struct usb_device *dev, void *ptr)
{
	static const char proc[] = "usbvision_disconnect";
	struct usb_usbvision *usbvision = (struct usb_usbvision *) ptr;

	MOD_INC_USE_COUNT;

	if (debug > 0)
		printk(KERN_DEBUG "%s(%p,%p.)\n", proc, dev, ptr);

	down(&usbvision->lock);
	usbvision->remove_pending = 1;	/* Now all ISO data will be ignored */

	/* At this time we ask to cancel outstanding URBs */
	usbvision_stop_isoc(usbvision);

	usbvision->dev = NULL;	/* USB device is no more */

	if (usbvision->user)
		printk(KERN_INFO "%s: In use, disconnect pending.\n",
		       proc);
	else
		usbvision_release(usbvision);
	up(&usbvision->lock);
	printk(KERN_INFO "USBVISION USB video disconnected.\n");

	MOD_DEC_USE_COUNT;
}

MODULE_DEVICE_TABLE (usb, usbvision_table);

static struct usb_driver usbvision_driver = {
	name:		"USBVision Video Capture",
	probe:		usbvision_probe,
	disconnect:	usbvision_disconnect,
	id_table:	usbvision_table
};

/*
 * usbvision_init()
 *
 * This code is run to initialize the driver.
 *
 */
static int __init usbvision_init(void)
{
	unsigned u;

	/* Initialize struct */
	for (u = 0; u < MAX_USBVISION; u++) {
		struct usb_usbvision *usbvision = &vdevs[u];
		memset(usbvision, 0, sizeof(struct usb_usbvision));
	}
	return usb_register(&usbvision_driver);
}

static void usbvision_cleanup(void)
{
	unsigned int dev_index;
	
	printk(KERN_DEBUG "usbvision_cleanup\n");

	/* Power off devices */
	for (dev_index = 0; dev_index < MAX_USBVISION; dev_index++) {
		struct usb_usbvision *usbvision = &vdevs[dev_index];
	if (!PowerOnAtOpen) usbvision_power_off(usbvision);
	}
	
	usb_deregister(&usbvision_driver);
}

module_init(usbvision_init);
module_exit(usbvision_cleanup);




















