#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/file.h>
#include <linux/net.h>
#include <net/sock.h>
#include <linux/grsecurity.h>
#include <linux/grinternal.h>
#include <linux/gracl.h>

void
gr_attach_curr_ip(const struct sock *sk)
{
#ifdef CONFIG_GRKERNSEC
	struct task_struct *p;
	unsigned int i;
	struct inode *inode;
	struct file *filp;
	struct socket *connect_sock;

	if (unlikely(sk->protocol != IPPROTO_TCP))
		return;

	read_lock(&tasklist_lock);
	for_each_task(p) {
		if (!p->used_connect)
			continue;
		task_lock(p);
		if (unlikely(!p->files)) {
			task_unlock(p);
			continue;
		}
		read_lock(&p->files->file_lock);
		for (i = 0; i < p->files->max_fds; i++) {
			filp = fcheck_files(p->files, i);
			if (likely(!filp))
				continue;
			inode = filp->f_dentry->d_inode;
			if (likely(!inode || !inode->i_sock))
				continue;
			connect_sock = &inode->u.socket_i;
			if (unlikely(!connect_sock ||
				     connect_sock->sk->protocol != IPPROTO_TCP))
				continue;
			if (unlikely(sk->rcv_saddr == connect_sock->sk->daddr &&
				     sk->daddr == connect_sock->sk->rcv_saddr &&
				     ntohs(sk->sport) ==
				     ntohs(connect_sock->sk->dport)
				     && ntohs(sk->dport) ==
				     ntohs(connect_sock->sk->sport))) {
				current->curr_ip = p->curr_ip;
				current->used_accept = 1;
				read_unlock(&p->files->file_lock);
				task_unlock(p);
				read_unlock(&tasklist_lock);
				return;
			}
		}
		read_unlock(&p->files->file_lock);
		task_unlock(p);
	}
	read_unlock(&tasklist_lock);

	current->curr_ip = sk->daddr;
	current->used_accept = 1;
#endif
	return;
}

int
gr_handle_sock_all(const int family, const int type, const int protocol)
{
#ifdef CONFIG_GRKERNSEC_SOCKET_ALL
	if (grsec_enable_socket_all && in_group_p(grsec_socket_all_gid) &&
	    (family != AF_UNIX) && (family != AF_LOCAL)) {
		security_alert(GR_SOCK_MSG, family, type, protocol,
			       DEFAULTSECARGS);
		return -EACCES;
	}
#endif
	return 0;
}

int
gr_handle_sock_server(const struct sockaddr *sck)
{
#ifdef CONFIG_GRKERNSEC_SOCKET_SERVER
	if (grsec_enable_socket_server &&
	    in_group_p(grsec_socket_server_gid) &&
	    sck && (sck->sa_family != AF_UNIX) &&
	    (sck->sa_family != AF_LOCAL)) {
		security_alert(GR_BIND_MSG, DEFAULTSECARGS);
		return -EACCES;
	}
#endif
	return 0;
}

int
gr_handle_sock_client(const struct sockaddr *sck)
{
#ifdef CONFIG_GRKERNSEC_SOCKET_CLIENT
	if (grsec_enable_socket_client && in_group_p(grsec_socket_client_gid) &&
	    sck && (sck->sa_family != AF_UNIX) &&
	    (sck->sa_family != AF_LOCAL)) {
		security_alert(GR_CONNECT_MSG, DEFAULTSECARGS);
		return -EACCES;
	}
#endif
	return 0;
}

__u32
gr_cap_rtnetlink(void)
{
#ifdef CONFIG_GRKERNSEC
	if (!gr_acl_is_enabled())
		return current->cap_effective;
	else
		return (current->cap_effective & ~(current->acl->cap_lower));
#else
	return current->cap_effective;
#endif
}
