
#ifndef _LINUX_SWSUSP_DEBUG_H
#define _LINUX_SWSUSP_DEBUG_H

#include <linux/suspend.h>

#define name_suspend "Suspend Machine:  "
#define name_resume  "Resume Machine:   "
#define swsusp_version "1.0.3"
#define name_swsusp  "Swsusp " swsusp_version ": "
#define console_suspend " S U S P E N D   T O   D I S K " /* Same length to ensure one overwrites the other */
#define console_resume  "R E S U M E   F R O M   D I S K"

#define TEST_RESULT_STATE(bit) (test_bit(bit, &swsusp_result))
#define SET_RESULT_STATE(bit) (test_and_set_bit(bit, &swsusp_result))
#define CLEAR_RESULT_STATE(bit) (test_and_clear_bit(bit, &swsusp_result))

#define TEST_ACTION_STATE(bit) (test_bit(bit, &swsusp_action))
#define SET_ACTION_STATE(bit) (test_and_set_bit(bit, &swsusp_action))
#define CLEAR_ACTION_STATE(bit) (test_and_clear_bit(bit, &swsusp_action))

#define TEST_DEBUG_STATE(bit) (test_bit(bit, &swsusp_debug_state))
#define SET_DEBUG_STATE(bit) (test_and_set_bit(bit, &swsusp_debug_state))
#define CLEAR_DEBUG_STATE(bit) (test_and_clear_bit(bit, &swsusp_debug_state))

extern int suspend_console;

/* first status register - this is swsusp's return code. */
#define SUSPEND_ABORTED			0
#define SUSPEND_ABORT_REQUESTED		1
#define SUSPEND_NOSWAP_AVAILABLE	2
#define SUSPEND_INSUFFICIENT_SWAP	3
#define SUSPEND_FREEZING_FAILED		4
#define SUSPEND_UNEXPECTED_ALLOC	5

/* second status register */
#define SUSPEND_REBOOT		0
#define SUSPEND_NO_OUTPUT	1
#define SUSPEND_PAUSE		2
#define SUSPEND_SLOW		3
#define SUSPEND_BEEP		4
#define SUSPEND_NO_ASYNC_READS	5
#define SUSPEND_NO_ASYNC_WRITES	6
#define SUSPEND_NOPAGESET2	7
#define SUSPEND_LOGALL		8
/* Set to disable compression when compiled in */
#define SUSPEND_NO_COMPRESSION	9
//#define SUSPEND_ENABLE_KDB	10
#define SUSPEND_CAN_CANCEL	11

/* debug sections  - if debugging compiled in */
#define SUSPEND_ANY_SECTION	0
#define SUSPEND_FREEZER		1
#define SUSPEND_EAT_MEMORY 	2
#define SUSPEND_PAGESETS	3
#define SUSPEND_MARK_PAGESET2 	4
#define SUSPEND_COPY_PAGESET	5
#define SUSPEND_WRITE_PAGESETS 	6
#define SUSPEND_READ_PAGESETS 	7
#define SUSPEND_CHECKSUMS	8
#define SUSPEND_SWAP		9
#define SUSPEND_MEMORY		10
#define SUSPEND_RANGES		11
/* debugging levels - if debugging compiled in.
 * These are straight integer values, not bit flags.
 * (Stored as LEVEL << 16 + section_flags).
 */
#define SUSPEND_ERROR		3
#define SUSPEND_LOW	 	4
#define SUSPEND_MEDIUM	 	5
#define SUSPEND_HIGH	  	6
#define SUSPEND_VERBOSE		7

#define NO_OUTPUT_OR_PAUSING ((fg_console != suspend_console) || \
		(TEST_ACTION_STATE(SUSPEND_NO_OUTPUT)))

#define CHECKMASK(mask) (((!mask) || TEST_DEBUG_STATE(mask)) && (!NO_OUTPUT_OR_PAUSING))
/* 
 *   #defs that are dependant on CONFIG_SOFTWARE_SUSPEND_DEBUG
 *
 */
#if defined(CONFIG_SOFTWARE_SUSPEND_DEBUG)
void printnolog(int mask, int level, int restartline, const char *fmt, ...);
void printlog(int mask, int level, const char *fmt, ...);
#define MDELAY(a)	if (TEST_ACTION_STATE(SUSPEND_SLOW)) { mdelay(a); } else { do { } while (0); }
#define beepOK \
	if (TEST_ACTION_STATE(SUSPEND_BEEP)) { \
		currentbeep += 200; \
		kd_mksound(currentbeep,HZ/8); \
		mdelay(150); \
	} else { \
		do { } while (0); \
	}
#define beepERR \
	if (TEST_ACTION_STATE(SUSPEND_BEEP)) { \
		kd_mksound(300,HZ/4); \
		mdelay(300); \
	} else { \
		do { } while (0); \
	} 

#else // #ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG
#define printnolog(...) do { } while(0)
#define printlog(...) do { } while(0)
#define MDELAY(a) do { } while (0)
#define beepOK do { } while (0)
#define beepERR do { } while (0)
#endif // #ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG

#endif
