#ifndef _LINUX_SWSUSP_H
#define _LINUX_SWSUSP_H

#if defined(SUSPEND_C) || defined(ACPI_C)
#include <asm/suspend.h>
#endif
#include <linux/swap.h>
#include <linux/notifier.h>
#include <linux/config.h>
#include <linux/kdev_t.h>

#define NORESUME	 1
#define RESUME_SPECIFIED 2
#define COMMANDLINE_ERROR 4

#ifdef CONFIG_SOFTWARE_SUSPEND
extern unsigned long swsusp_action;
extern unsigned long swsusp_result;
extern unsigned char trap_page_allocs; /* code to catch a hard-to-find bug */

#ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG
extern unsigned long swsusp_debug_state;
#define PRINTFREEMEM(desn) printlog(SUSPEND_MEMORY, SUSPEND_MEDIUM, \
		"Free memory %s: %d.\n", desn, nr_free_pages())
#ifdef CONFIG_PREEMPT
#define PRINTPREEMPTCOUNT(desn) printlog(SUSPEND_FREEZER, SUSPEND_MEDIUM, \
		"Preempt count: %s: %d.\n", desn, current->preempt_count)
#endif
#else /* CONFIG_SOFTWARE_SUSPEND_DEBUG */
#define PRINTFREEMEM(desn) do { } while(0)
#endif /* CONFIG_SOFTWARE_SUSPEND_DEBUG */

#ifndef PRINTPREEMPTCOUNT
#define PRINTPREEMPTCOUNT(desn) do { } while(0)
#endif

struct rangechain {
	struct range * first;
	int size; /* size of the range ie sum (max-min+1) */
	int allocs;
	int frees;
	int debug;
	int timesusedoptimisation;
	char * name;
	struct range * lastaccessed, *prevtolastaccessed, *prevtoprev;
};

struct range {
	unsigned long minimum;
	unsigned long maximum;
	struct range * next;
};

/* page backup entry */
typedef struct pbe {
	struct page * origaddress;	/* 4: original address of pageset entry */
	struct page * address;		/* 4: address of copy of pageset entry (pageset1 only) */
	swp_entry_t swap_address;	/* 4: */
	unsigned long checksum;
	unsigned long checksum_error;
	kdev_t swap_bdev;		/* Physical location of swap entry */
	unsigned long block;
	swp_entry_t dummy;		/* we need at least one sizeof(long) scratch space at 
					 * end of page (see link, diskpage) and
					 * sizeof(struct pbe) must be multiple of 4*sizeof(long)
					 */
} suspend_pbe_t;

typedef struct pagedir {
	int pageset_size;
	int pagedir_size;
	int pagedir_capacity;
	int alloc_from;
	struct pbe *data;
	struct pbe *first_allocated;	/* Shortcut to first pbe in pageset 2 where extra pages
					   were allocated */
	struct rangechain origranges;
	struct rangechain destranges;
	struct rangechain swapranges;
	struct rangechain blockranges;
	struct rangechain allocdranges;
} suspend_pagedir_t;

struct pbelink {
	char dummy[sizeof(struct pbe) - sizeof(struct pbe *)];
	struct pbe * next;
};

#define SWAP_FILENAME_MAXLENGTH	32

struct suspend_header {
	__u32 version_code;
	unsigned long num_physpages;
	char machine[65];
	char version[65];
	int num_cpus;
	int page_size;
	unsigned long base_mem_free;
	unsigned long swapforimage;
	unsigned int expected_size1;
	unsigned int expected_size2;
	struct pagedir pagedir;
	int param0;
	int param1;
	int param2;
	int param3;
	int param4;
	int progress0;
	int progress1;
	int progress2;
	int progress3;
	int progress4;
	kdev_t kdevs[MAX_SWAPFILES];
	struct swap_location {
		char filename[SWAP_FILENAME_MAXLENGTH];
	} swap_location[MAX_SWAPFILES];
};

#define PBES_PER_PAGE	(PAGE_SIZE/sizeof(struct pbe))
#define SUSPEND_PD_PAGES(x)	(((x)+PBES_PER_PAGE-1)/PBES_PER_PAGE)
#define PAGEDIR_CAPACITY(x)     (((x)*PBES_PER_PAGE))
//#define PAGEDIR_ENTRY(pagedir, i) ((pagedir)->data[((i)/PBES_PER_PAGE)] + ((i)%PBES_PER_PAGE))
   
extern struct tq_struct suspend_tq;

/* mm/vmscan.c */
extern int shrink_mem(void);

/* kernel/suspend.c */
extern void software_suspend_pending(void);
extern void software_resume(void);

extern int register_suspend_notifier(struct notifier_block *);
extern int unregister_suspend_notifier(struct notifier_block *);
extern int register_resume_notifier(struct notifier_block *);
extern int unregister_resume_notifier(struct notifier_block * nb);
extern void refrigerator(unsigned long);
extern struct buffer_head * suspend_swap_bh;

extern struct page * last_suspend_cache_page;
extern unsigned int suspend_task;

#ifdef CONFIG_MTRR
#define SOFTWARE_SUSPEND_MTRR 1
#else
#undef SOFTWARE_SUSPEND_MTRR
#endif

#define TASK_NOT_SUSPENDED(p)	((!suspend_task) || \
				 (p->flags & PF_IOTHREAD) || \
				 (!p->pid) || \
				 (suspend_task == p->pid) || \
				 ((p->flags & PF_REFRIGERATE) && (!(p->flags & PF_FROZEN))))
#else
#define software_suspend_pending()	do { } while(0)
#define software_resume()		do { } while(0)
#define register_suspend_notifier(a)	do { } while(0)
#define unregister_suspend_notifier(a)	do { } while(0)
#define refrigerator(a)			do { BUG(); } while(0)
#define TASK_NOT_SUSPENDED(p) (1)
#define suspend_task (0)
#endif

#endif /* _LINUX_SWSUSP_H */
