/*
 * Software Suspend.
 *
 * Module level definitions.
 *
 */

#ifndef SWSUSP_COMMON_H
#define SWSUSP_COMMON_H

#include <linux/version.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/locks.h>
#include <linux/swapctl.h>
#if defined(CONFIG_FBCON_SPLASHSCREEN)
#include <linux/fb.h>
#include <video/fbcon.h>
#else
#include <linux/console_struct.h>
#endif
#include <linux/suspend.h>
#include <linux/proc_fs.h>
#include <linux/smp_lock.h>
#include <linux/file.h>
#include <linux/utsname.h>
#include <linux/init.h>
#include <linux/compile.h>
#include <linux/delay.h>
#include <linux/reboot.h>
#include <linux/vt_kern.h>
#include <linux/bitops.h>
#include <linux/interrupt.h>
#include <linux/kbd_kern.h>
#include <linux/keyboard.h>
#include <linux/spinlock.h>
#include <linux/genhd.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/swap.h>
#include <linux/pm.h>
#include <linux/raid/md.h>
#include <linux/selection.h>
#include <linux/console.h>
#include <linux/fs.h>
#include <linux/string.h>
#include <linux/suspend-debug.h>
#include <linux/zlib.h>

#include <asm/uaccess.h>
#include <asm/mmu_context.h>
#include <asm/pgtable.h>
#include <asm/io.h>
#include <asm/mtrr.h>

#define MAX(a,b)	((signed long)(a)>(signed long)(b)?(a):(b))
#define MIN(a,b)	((signed long)(a)<(signed long)(b)?(a):(b))

#ifndef SWSUSP_MAIN_C
extern unsigned char software_suspend_enabled;
extern unsigned int suspend_task;
extern unsigned long swsusp_debug_state;
extern struct sysinfo swapinfo;
extern spinlock_t suspend_irq_lock __nosavedata;
extern unsigned long pagedirflags __nosavedata;

extern int max_async_ios;
extern int image_size_limit;

extern int currentbeep;
extern int orig_loglevel;
extern int now_resuming;
extern struct pagedir pagedir1, pagedir2;
#endif

struct swaplink {
	char dummy[PAGE_SIZE - sizeof(swp_entry_t)];
	swp_entry_t next;
};

union diskpage {
	union swap_header swh;	/* swh.magic is the only member used */
	struct swaplink link;
	struct suspend_header sh;
};

union p_diskpage {
	union diskpage *pointer;
	char *ptr;
        unsigned long address;
};

typedef struct pageset_sizes_result {
	int size1; /* Can't be unsigned - breaks MAX function */
	int size2;
	int size2low;
} pageset_sizes_t;

/* */
extern int C_A_D;

/*
 * Poll the swsusp state every second
 */
#define SWSUSP_CHECK_TIMEOUT	(HZ)
#define SWSUSP_COMMIT_TIMEOUT	(HZ)

#if defined(CONFIG_VT) && defined(CONFIG_VT_CONSOLE)
#define DEFAULT_SUSPEND_CONSOLE	(MAX_NR_CONSOLES-1)
#endif

#define TIMEOUT	(6 * HZ)	/* Timeout for stopping processes */
#define __ADDRESS(x)  ((unsigned long) phys_to_virt(x))
#define ADDRESS(x) __ADDRESS((x) << PAGE_SHIFT)

#define MB(x) ((x) >> (20 - PAGE_SHIFT))

#define MARK_SWAP_SUSPEND 0
#define MARK_SWAP_RECOVER 1
#define MARK_SWAP_RESUME 2

/* References to section boundaries */
extern char _text, _etext, _edata, __bss_start, _end;
extern char __nosave_begin, __nosave_end;

extern int console_loglevel;
extern inline void signal_wake_up(struct task_struct *t);
asmlinkage long sys_sched_yield(void);
asmlinkage void sys_sync(void);	/* it's really int */
extern void ide_disk_unsuspend(int);
extern void ide_disk_suspend(void);
extern kdev_t name_to_kdev_t(char *line) __init;
extern void vt_console_print(struct console *co, const char * b, unsigned count);
extern void free_suspend_cache_page(struct page * page);
extern inline pte_t *lookup_address(unsigned long address);
extern int rw_swap_page_base(int rw, swp_entry_t entry, struct page *page);
extern int try_to_free_pages_zone_swsusp(zone_t *classzone, unsigned int gfp_mask, int priority, int nr_pages);
extern int try_to_free_pages_swsusp(unsigned int gfp_mask, int amount_needed);
extern char log_buf[];
extern void si_swapinfo_devsonly(struct sysinfo *val);

#ifdef CONFIG_PREEMPT
#define swsusp_spin_lock_irq(lock) { \
	int count = current->preempt_count; \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinlock %p. %d->%d.\n", lock, count, count+1); \
	spin_lock_irq(lock); \
}

#define swsusp_spin_lock_irqsave(lock, flags) { \
	int count = current->preempt_count; \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinlock IRQSave %p. %d->%d.\n", lock, count, count+1); \
	spin_lock_irqsave(lock, flags); \
}

#define swsusp_spin_unlock_irq(lock) { \
	int count = current->preempt_count; \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinunlock %p. %d->%d.\n", lock, count, count-1); \
	spin_unlock_irq(lock); \
}

#define swsusp_spin_unlock_irqrestore(lock, flags) { \
	int count = current->preempt_count; \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinunlock IRQRestore %p. %d->%d\n", lock, count, count-1); \
	spin_unlock_irqrestore(lock, flags); \
}
#else /* No preempt */
#define swsusp_spin_lock_irq(lock) { \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinlock %p.\n", lock); \
	spin_lock_irq(lock); \
}

#define swsusp_spin_lock_irqsave(lock, flags) { \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinlock IRQSave %p.\n", lock); \
	spin_lock_irqsave(lock, flags); \
}

#define swsusp_spin_unlock_irq(lock) { \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinunlock %p.\n", lock); \
	spin_unlock_irq(lock); \
}

#define swsusp_spin_unlock_irqrestore(lock, flags) { \
	printlog(SUSPEND_FREEZER, SUSPEND_VERBOSE, "Spinunlock IRQRestore %p.\n", lock); \
	spin_unlock_irqrestore(lock, flags); \
}
#endif

/* Variables to be preserved over suspend */
extern char resume_file[256];	/* For resume= kernel option */
/* Local variables that should not be affected by save */
#define pageset1_size (pagedir1.pageset_size)
#define pageset2_size (pagedir2.pageset_size)

#define BITS_PER_PAGE (PAGE_SIZE * 8)
#define PAGES_PER_BITMAP ((max_mapnr + BITS_PER_PAGE - 1) / BITS_PER_PAGE)
#define BITMAP_ORDER (get_bitmask_order(PAGES_PER_BITMAP))

/*
 * XXX: We try to keep some more pages free so that I/O operations succeed
 * without paging. Might this be more?
 */
#define MIN_FREE_RAM 1024

void prepare_status(int printalways, int clearbar, const char *fmt, ...);
void abort_suspend(const char *fmt, ...);

/* Exported variables */

int read_secondary_pagedir(int);
int read_primary_suspend_image(char * specialfile, int noresume);
void thaw_processes(void);

extern int nr_unused_buffer_heads;

/* ------ io.c ------ */
int write_pageset(struct pagedir * pagedir, int whichtowrite);
int read_pageset(struct pagedir * pagedir, int whichtoread, int overwrittenpagesonly);
int write_pagedir_and_suspend_header(void);
void mark_swapfiles(swp_entry_t prev, int mode);
void sync_swap_partitions(void); 
int noresume_fix_signature(union p_diskpage cur);
void do_suspend_sync(void);
void read_swapfiles(void);
void finish_all_io(void);
int bdev_read_page(kdev_t bdev, long pos, struct page * page);
extern kdev_t resume_device;
int bdev_read_swap(struct pbe * pbe, int asyncio);
__inline__ int fill_suspend_header(struct suspend_header *sh);

/* ------ ui.c ------ */
void check_shift_keys(int pause, char * message);
int update_status(unsigned long value, unsigned long maximum, const char *fmt, ...);
#if defined(CONFIG_FBCON_SPLASHSCREEN)
extern int fb_splash_set_progress(int consolenr, int value, int maximum, int reset);
extern int fb_splash_progress[5]; /* Needed while progress bar posn/size set via proc interface */
int splash_is_on(int consolenr);
#define SPLASH_IS_ON(consolenr) (splash_is_on(consolenr))
#else
#define SPLASH_IS_ON(consolenr) (0)
#define fb_splash_set_progress(a, b, c, d) do { } while (0)
#endif
int prepare_suspend_console(void);
void restore_console(void);

#define PBEPAGE_START(pbe) (((unsigned long) (pbe)) & PAGE_MASK)
inline void set_next_pagedir_page(struct pbe * pbe, struct pbe * next);
inline struct pbe * get_next_pagedir_page(struct pbe * pbe);
inline struct pbe * get_next_pbe(struct pbe * pbe);
#endif

#define REAL_MAX_ASYNC (MIN(10 * (MAX_BUF_PER_PAGE + 2), (max_async_ios ? 100 : max_async_ios)))

#define SWAP_NEEDED (pageset1_size + pageset2_size + pagedir1_size + 1)
#define RAM_TO_SUSPEND (pagedir1_size + pagedir2_size + 1 + MAX((pageset1_size - pageset2_sizelow), 0) + MIN_FREE_RAM + REAL_MAX_ASYNC)
#define RAM_TO_RESUME  ((pageset1_size + pagedir1_size) * 2 + 8 + MIN_FREE_RAM + (long) max_mapnr - orig_mem_free + REAL_MAX_ASYNC)

/* --------- Range chains ----------- */

#define RANGES_PER_PAGE (PAGE_SIZE / (sizeof(struct range)))
#define RANGE_PAGES_NEEDED(x) (((x)+RANGES_PER_PAGE-1)/RANGES_PER_PAGE)

#define range_for_each(rangechain, rangepointer, value) \
	for (rangepointer = rangechain->first, value = rangepointer->minimum; \
	     (rangepointer && (rangepointer->next || value <= rangepointer->maximum)); \
	     ((value == rangepointer->maximum) ? \
		(rangepointer = rangepointer->next, value = (rangepointer ? rangepointer->minimum : 0)) : \
	     	value++))

struct range_entry {
	unsigned long thisvalue;
	struct range * rangeptr;
};

extern int max_ranges_used;
extern int num_range_pages;
int add_range_pages(int number_requested);
int add_to_range_chain(struct rangechain * chain, unsigned long value);
void put_range_chain(struct rangechain * chain);
void print_chain(int debuglevel, struct rangechain * chain, int printasswap);
void set_chain_names(struct pagedir * p);
int free_ranges(void);
void init_range_entry(struct range_entry * range_entry, struct rangechain rangechain);
unsigned long get_next_range_entry(struct range_entry * range_entry);
int append_range_to_range_chain(struct rangechain * chain, unsigned long minimum, unsigned long maximum);

/* We are putting offset in the low bits so consecutive swap entries make consecutive range values */
#define swap_entry_to_range_val(swp_entry) ((swp_entry.val >> 8) | ((swp_entry.val & 0x3f) << 24))
#define range_val_to_swap_entry(val) (swp_entry_t) { ((val >> 24) | ((val & 0xffffff) << 8)) }
