
#define SWSUSP_IO_C

#include "common.h"

extern void reset_terminal(int currcons, int do_clear);
extern void hide_cursor(int currcons);

#ifdef DEFAULT_SUSPEND_CONSOLE
static int barwidth = 0, barposn = -1, newbarposn = 0;
extern int swsusp_default_console_level;
#endif
#ifdef CONFIG_VT
static int orig_fgconsole;
static char print_buf[1024];	/* Same as printk - should be safe */
#endif
#ifdef DEFAULT_SUSPEND_CONSOLE
static int orig_kmsg;
#endif
#define cond_console_print(chars, count) \
	if (suspend_console == fg_console) vt_console_print((struct console *) NULL, chars, count)
#undef video_num_columns
#define video_num_columns	(vc_cons[suspend_console].d->vc_cols)
#undef video_num_lines
#define video_num_lines		(vc_cons[suspend_console].d->vc_rows)

extern struct kbd_struct kbd_table[MAX_NR_CONSOLES];

/* 
 * Console being used for suspend - may not be default if splash screen 
 * displayed 
 */
int suspend_console = 0;	

static char lastheader[512];
static int nice_display = 0;
static int lastloglevel = 0;
static int lastmessagelen = 0;
static int lastshiftstate = 0;

/* ------------------  Splash screen defines  -------------------------- */
char * currentmessage;
#if defined(CONFIG_FBCON_SPLASHSCREEN)
extern struct display fb_display[MAX_NR_CONSOLES];
int splash_is_on(int consolenr)
{
	struct display *p = &fb_display[consolenr];
	if (p && p->splash_data)
		return ((p->splash_data->splash_state & 1) == 1);
	return 0;
}
int bar_granularity_limit = 0; /* 0 = no limit */
#endif

void abort_suspend(const char *fmt, ...)
{
	va_list args;
	int printed_len = 0;

	if (!TEST_RESULT_STATE(SUSPEND_ABORTED)) {
		if ((!TEST_RESULT_STATE(SUSPEND_ABORT_REQUESTED)) && (!NO_OUTPUT_OR_PAUSING)) {
			int locked = (spin_is_locked(&suspend_irq_lock));
			if (locked)
				swsusp_spin_unlock_irqrestore(&suspend_irq_lock, pagedirflags);
			va_start(args, fmt);
			printed_len = vsnprintf(print_buf, sizeof(print_buf), fmt, args);
			va_end(args);
			printed_len = sprintf(print_buf + printed_len,
					" (Press SHIFT to continue)");
			prepare_status(1, 1, print_buf);
			beepERR;
			/* 
			 * Make sure message seen - wait for shift to be
			 * released if being pressed 
			 */
			while ((shift_state & (1 << KG_SHIFT)))
				yield();
			while (!(shift_state & (1 << KG_SHIFT)))
				yield();
			lastshiftstate = shift_state;
			if (locked)
				swsusp_spin_lock_irqsave(&suspend_irq_lock, pagedirflags);
		}
		/* Turn on aborting flag */
		SET_RESULT_STATE(SUSPEND_ABORTED);
	}
}

void say_if_loglevel_changed(void)
{
	if (console_loglevel != lastloglevel) {
		lastloglevel = console_loglevel;
		printnolog(0, console_loglevel, 0, "Switched to console loglevel %d.\n", console_loglevel);
	}
}


#if defined(DEFAULT_SUSPEND_CONSOLE) && defined(CONFIG_SOFTWARE_SUSPEND_DEBUG)
/*
 * This function is intended to do the same job as printk, but without actually
 * logging what is printed. The point is to be able to get debugging info on
 * screen without filling the logs with "1/534. ^M 2/534^M. 3/534^M"
 *
 * It may be called from an interrupt context - can't sleep!
 */
//void printnolog(int restartline, const char *fmt, ...)
void printnolog(int mask, int level, int restartline, const char *fmt, ...)
{
	va_list args;
	int printed_len = 0;

	if (console_loglevel < level)
		return;

	if (!CHECKMASK(mask))
		return;

	if (nice_display) {
		nice_display = 0;
		reset_terminal(suspend_console, 1);
	}
	
	say_if_loglevel_changed();

	if (restartline) {
		if (SUSPEND_LOGALL)
			printk("\n");
		else
			printed_len = vsnprintf(print_buf + printed_len, sizeof(print_buf) - printed_len, "\r", NULL);
	}

	va_start(args, fmt);
	printed_len += vsnprintf(print_buf + printed_len, sizeof(print_buf) - printed_len, fmt, args);
	va_end(args);

	if (SUSPEND_LOGALL) 
		printk(print_buf);
	else
		cond_console_print(print_buf, printed_len);
}

/*
 * This function is a wrapper around printk. It adds the log level of
 * the message to the front, so we can use Sys-Rq to change the detail
 * displayed at runtime.
 */
void printlog(int mask, int level, const char *fmt, ...)
{
	va_list args;
	int printed_len = 0;

	if (console_loglevel < level)
		return;

	if (!CHECKMASK(mask))
		return;

	if (nice_display) {
		nice_display = 0;
		reset_terminal(suspend_console, 1);
	}
	
	say_if_loglevel_changed();

	va_start(args, fmt);
	printed_len += vsnprintf(print_buf + printed_len, sizeof(print_buf) - printed_len, fmt, args);
	va_end(args);

	printk(print_buf);
}
#endif

void prepare_status(int printalways, int clearbar, const char *fmt, ...)
{
#if defined(DEFAULT_SUSPEND_CONSOLE)
	unsigned char posn[2];
	va_list args;
	int message_len = 0;

	if (NO_OUTPUT_OR_PAUSING)
		return;

	if (fmt) {
		va_start(args, fmt);
		message_len = vsnprintf(lastheader, 512, fmt, args);
		va_end(args);
	}

	if (console_loglevel >= SUSPEND_ERROR) {
		if (nice_display) {
			/* Clear the screen */
			reset_terminal(suspend_console, 1);
			hide_cursor(suspend_console);
			nice_display = 0;
		}
		say_if_loglevel_changed();

		if (printalways)
			printk("** %s\n", lastheader);
		return;
	}
	
	if (!nice_display) {
		reset_terminal(suspend_console, 1);
		nice_display = 1;
		lastloglevel = console_loglevel;
	}

	hide_cursor(suspend_console);

	/* Print version */
	posn[0] = (unsigned char) (0);
	posn[1] = (unsigned char) (video_num_lines);
	putconsxy(suspend_console, posn);
	cond_console_print(swsusp_version, strlen(swsusp_version));

	/* Print header */
	posn[0] = (unsigned char) ((video_num_columns - 29) / 2);
	posn[1] = (unsigned char) ((video_num_lines / 3) -4);
	putconsxy(suspend_console, posn);

	if (now_resuming) {
		cond_console_print(console_resume, strlen(console_resume));
	} else {
		cond_console_print(console_suspend, strlen(console_suspend));
	}
		
	/* Print action */
	posn[1] = (unsigned char) (video_num_lines / 3);
	if (lastmessagelen) {
		posn[0] = (unsigned char) ((video_num_columns - lastmessagelen) / 2);
		putconsxy(suspend_console, posn);
	
		/* Blank out previous contents of line - assumes text length <= bar width */
		for (barposn = 0; barposn < lastmessagelen; barposn++) 
			cond_console_print(" ", 1);
		}

	posn[0] = (unsigned char) ((video_num_columns - message_len) / 2);
	putconsxy(suspend_console, posn);
	cond_console_print(lastheader, message_len);
	lastmessagelen = message_len;
	
#if defined(CONFIG_FBCON_SPLASHSCREEN)
	if (SPLASH_IS_ON(suspend_console)) {
		/* proc interface ensures bar_granularity_limit >= 0 */
		if (bar_granularity_limit)
			barwidth = MIN(fb_splash_progress[2], bar_granularity_limit);
		else
			barwidth = fb_splash_progress[2];
	} else 
#endif
	{
		/* Draw left bracket of progress bar. */
		posn[0] = (unsigned char) (video_num_columns / 4);
		posn[1]++;
		putconsxy(suspend_console, posn);
		cond_console_print("[", 1);

		/* Draw right bracket of progress bar. */
		posn[0] = (unsigned char) (video_num_columns - (video_num_columns / 4) - 1);
		putconsxy(suspend_console, posn);
		cond_console_print("]", 1);
	
		barwidth = (video_num_columns - 2 * (video_num_columns / 4) - 2);
	}

	if (clearbar) {
		if (SPLASH_IS_ON(suspend_console))
			fb_splash_set_progress(suspend_console, 0, 1, 1);
		else {
			/* Position at start of progress */
			posn[0] = (unsigned char) (video_num_columns / 4 + 1);
			putconsxy(suspend_console, posn);

			/* Clear bar */
			for (barposn = 0; barposn < barwidth; barposn++)
				cond_console_print(" ", 1);
			putconsxy(suspend_console, posn);
		}
	}

	barposn = 0;
	hide_cursor(suspend_console);
#endif
}

/* NULL message does not mean that any previous message is erased! */
/* Returns next value where status needs to be updated */
int update_status(unsigned long value, unsigned long maximum, const char *fmt, ...)
{
	unsigned long next_update = 0;
#if defined(DEFAULT_SUSPEND_CONSOLE)
	unsigned char posn[2];
	va_list args;
	int message_len = 0;

	if (NO_OUTPUT_OR_PAUSING || (!maximum) || (!barwidth))
		return maximum;

	newbarposn = (int) (value * barwidth / maximum);
	
	if (newbarposn < barposn) {
		barposn = 0;
		fb_splash_set_progress(suspend_console, value, maximum, 0);
	}

	next_update = (newbarposn + 1) * maximum / barwidth;

	if (value < 0)
		value = 0;
	if (value > maximum)
		value = maximum;

	if (console_loglevel >= SUSPEND_ERROR) {
		if (nice_display) {
			/* Clear the screen */
			reset_terminal(suspend_console, 1);
			hide_cursor(suspend_console);
			nice_display = 0;
		}
		say_if_loglevel_changed();
		if ((SPLASH_IS_ON(suspend_console)) && (newbarposn != barposn)) {
			fb_splash_set_progress(suspend_console, value, maximum, 0);
			barposn = newbarposn;
		}
		return next_update;
	}
	
	if (!nice_display)
		prepare_status(0, 0, lastheader);

	lastloglevel = console_loglevel;

	if (SPLASH_IS_ON(suspend_console)) {
		if (newbarposn != barposn)
			fb_splash_set_progress(suspend_console, value, maximum, 0);
	} else {
		posn[1] = (unsigned char) ((video_num_lines / 3) + 1);

		/* Clear bar if at start */
		if (!barposn) {
			posn[0] = (unsigned char) (video_num_columns / 4 + 1);
			putconsxy(suspend_console, posn);
			for (; barposn < barwidth; barposn++)
				cond_console_print(" ", 1);
			barposn = 0;
		}
		posn[0] = (unsigned char) (video_num_columns / 4 + 1 + barposn);
		putconsxy(suspend_console, posn);

		for (; barposn < newbarposn; barposn++)
			cond_console_print("-", 1);
	}

	/* Don't print string in progress bar on loglevel 1 */
	if ((fmt) && (console_loglevel)) {
		va_start(args, fmt);
		message_len = vsnprintf(print_buf, sizeof(print_buf), fmt, args);
		va_end(args);

		if (message_len) {
			posn[0] = (unsigned char) ((video_num_columns - message_len) / 2);
			posn[1] = (unsigned char) ((video_num_lines / 3) + 1);
			putconsxy(suspend_console, posn);
			cond_console_print(print_buf, message_len);
		}
	}

	barposn = newbarposn;
	hide_cursor(suspend_console);
#else
	next_update = maximum;
#endif
	return next_update;
}

void request_abort_suspend(void)
{
	if (TEST_RESULT_STATE(SUSPEND_ABORTED) || now_resuming) 
		return;

	prepare_status(1, 1, "   --- ESCAPE PRESSED : ABORTING PROCESS ---");
	SET_RESULT_STATE(SUSPEND_ABORTED);
	SET_RESULT_STATE(SUSPEND_ABORT_REQUESTED);
	abort_suspend("Alt key pressed.\n");
}
#define SHIFT_AND_ALT (1 << KG_ALT | 1 << KG_SHIFT)

void check_shift_keys(int pause, char * message)
{
#ifdef DEFAULT_SUSPEND_CONSOLE 
	int was_locked = (spin_is_locked(&suspend_irq_lock));
	static struct kbd_struct * kbd;
	
	kbd = kbd_table + fg_console;

	if (was_locked)
		swsusp_spin_unlock_irqrestore(&suspend_irq_lock, pagedirflags);

	yield();

	if (NO_OUTPUT_OR_PAUSING)
		goto done_checking;

	/* Don't pause if just toggled */
	if (((TEST_ACTION_STATE(SUSPEND_PAUSE) && pause) || (vc_kbd_led(kbd, VC_CAPSLOCK))) 
			&& (console_loglevel > 1)) { 
		prepare_status(1, 0, "%sPress SHIFT to continue.\n", message ? message : "");
		while ((shift_state & (1 << KG_SHIFT)) &&
			(TEST_ACTION_STATE(SUSPEND_PAUSE) || vc_kbd_led(kbd, VC_CAPSLOCK)) &&
			(!TEST_RESULT_STATE(SUSPEND_ABORTED)))
		 yield();
		while ((!shift_state & (1 << KG_SHIFT)) &&
			(TEST_ACTION_STATE(SUSPEND_PAUSE) || vc_kbd_led(kbd, VC_CAPSLOCK)) &&
			(!TEST_RESULT_STATE(SUSPEND_ABORTED)))
		 yield();
	}
done_checking:
	lastshiftstate = shift_state;
	if (was_locked)
		swsusp_spin_lock_irqsave(&suspend_irq_lock, pagedirflags);
#endif
}

int prepare_suspend_console(void)
{

	if (TEST_ACTION_STATE(SUSPEND_NO_OUTPUT))
		/* No output should be produced. */ 
		return 0;

	orig_loglevel = console_loglevel;
	if (!now_resuming)
		console_loglevel = swsusp_default_console_level;

#ifdef CONFIG_VT_CONSOLE
	orig_fgconsole = fg_console;

	if ((orig_fgconsole != DEFAULT_SUSPEND_CONSOLE) && (!SPLASH_IS_ON(fg_console))) {
		suspend_console = DEFAULT_SUSPEND_CONSOLE;
		if (vc_allocate(DEFAULT_SUSPEND_CONSOLE))
		  /* we can't have a free VC for now. Too bad,
		   * we don't want to mess the screen for now. */
			return 1;

		set_console(DEFAULT_SUSPEND_CONSOLE);
		if (vt_waitactive(DEFAULT_SUSPEND_CONSOLE)) {
			printk("Can't switch virtual consoles.");
			return 1;
		}
	} else 
		suspend_console = fg_console;
	reset_terminal(suspend_console, 1);
	orig_kmsg = kmsg_redirect;
	kmsg_redirect = suspend_console;
	prepare_status(0, 0, "");
#endif
	return 0;
}

void restore_console(void)
{

	if (TEST_ACTION_STATE(SUSPEND_NO_OUTPUT))
		return;

	console_loglevel = orig_loglevel;
	reset_terminal(suspend_console, 1);
	
	if ((orig_fgconsole != suspend_console) && (!SPLASH_IS_ON(suspend_console))) {
		set_console(orig_fgconsole);
		if (vt_waitactive(orig_fgconsole)) {
			printk("Can't switch virtual consoles.");
			return;
		}
	}
	
	kmsg_redirect = orig_kmsg;
	return;
}
